Package: doBy
Version: 4.7.1
Title: Groupwise Statistics, LSmeans, Linear Estimates, Utilities
Authors@R: c(
    person(given = "Ulrich", family = "Halekoh",
        email = "uhalekoh@health.sdu.dk", role = c("aut", "cph")),
    person(given = "Søren", family = "Højsgaard",
        email = "sorenh@math.aau.dk", role = c("aut", "cre", "cph"))
  )
Description: Utility package containing: Main categories: Working with grouped data: 'do' something to data when  stratified 'by' some variables.  General linear estimates. Data handling utilities. Functional programming, in particular restrict functions to a smaller domain. Miscellaneous functions for data handling. Model stability in connection with model selection. Miscellaneous other tools. 
Encoding: UTF-8
VignetteBuilder: knitr
LazyData: true
LazyDataCompression: xz
URL: https://github.com/hojsgaard/doBy
License: GPL (>= 2)
Depends: R (>= 4.2.0), methods
Imports: boot, broom, cowplot, Deriv, dplyr, forecast, ggplot2, MASS,
        Matrix, modelr, microbenchmark, rlang, purrr, tibble, tidyr,
Suggests: geepack, knitr, lme4, markdown, rmarkdown, multcomp, pbkrtest
        (>= 0.5.2), survival, testthat (>= 2.1.0)
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-01 22:47:07 UTC; sorenh
Author: Ulrich Halekoh [aut, cph],
  Søren Højsgaard [aut, cre, cph]
Maintainer: Søren Højsgaard <sorenh@math.aau.dk>
Repository: CRAN
Date/Publication: 2025-12-02 06:10:39 UTC
Built: R 4.6.0; ; 2026-04-25 14:00:36 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(LE_matrix,default)
S3method(LSmeans,default)
S3method(LSmeans,lmerMod)
S3method(as_lhs_chr,character)
S3method(as_lhs_chr,formula)
S3method(as_lhs_frm,character)
S3method(as_lhs_frm,formula)
S3method(as_rhs_chr,character)
S3method(as_rhs_chr,formula)
S3method(as_rhs_frm,character)
S3method(as_rhs_frm,formula)
S3method(coef,esticon_class)
S3method(coef,linest_class)
S3method(coef,lmBy)
S3method(coef,summary_lmBy)
S3method(confint,esticon_class)
S3method(confint,linest_class)
S3method(esticon,coxph)
S3method(esticon,geeglm)
S3method(esticon,glm)
S3method(esticon,gls)
S3method(esticon,lm)
S3method(esticon,lme)
S3method(esticon,merMod)
S3method(firstobs,default)
S3method(firstobs,formula)
S3method(fitted,lmBy)
S3method(formula,model_stability_glm_class)
S3method(get_X,default)
S3method(get_X,merMod)
S3method(get_contrasts,default)
S3method(get_contrasts,merMod)
S3method(get_xlevels,default)
S3method(get_xlevels,mer)
S3method(get_xlevels,merMod)
S3method(head,splitByData)
S3method(lastobs,default)
S3method(lastobs,formula)
S3method(linest,geeglm)
S3method(linest,glm)
S3method(linest,lm)
S3method(linest,lmerMod)
S3method(linest,merMod)
S3method(popMeans,default)
S3method(popMeans,lmerMod)
S3method(print,esticon_class)
S3method(print,linest_class)
S3method(print,linest_matrix_class)
S3method(print,lmBy)
S3method(print,model_stability_glm_class)
S3method(print,model_stability_glm_summary_class)
S3method(print,section_function)
S3method(print,splitByData)
S3method(print,summary_lmBy)
S3method(residuals,lmBy)
S3method(sigma,lmBy)
S3method(simplify_rhs,character)
S3method(simplify_rhs,formula)
S3method(summary,esticon_class)
S3method(summary,linest_class)
S3method(summary,linest_matrix_class)
S3method(summary,lmBy)
S3method(summary,model_stability_glm_class)
S3method(summary,section_function)
S3method(tail,splitByData)
S3method(tidy,esticon_class)
S3method(tidy,linest_class)
S3method(vcov,esticon_class)
export(LE_matrix)
export(LSmeans)
export(add)
export(add_int)
export(add_pred)
export(add_resid)
export(aggregate_linest_list)
export(align_coefs)
export(as_lhs_chr)
export(as_lhs_frm)
export(as_rhs_chr)
export(as_rhs_frm)
export(binomial_to_bernoulli_data)
export(bquote_fun_list)
export(cv_glm_fitlist)
export(descStat)
export(divide)
export(doby.xtabs)
export(esticon)
export(expr_to_fun)
export(firstobs)
export(formula_add)
export(formula_add_str)
export(formula_chr_to_form)
export(formula_nth)
export(formula_poly)
export(formula_to_interaction_matrix)
export(generate_data_list)
export(getBy)
export(get_X)
export(get_contrasts)
export(get_formulas)
export(get_fun)
export(get_linest_list)
export(get_section)
export(get_vartypes)
export(get_xlevels)
export(head2)
export(interaction_plot)
export(is_estimable)
export(lapplyBy)
export(lapply_by)
export(lastobs)
export(linest)
export(lmBy)
export(lm_by)
export(matrix2set_list)
export(mb_summary)
export(model_stability_glm)
export(mult)
export(orderBy)
export(order_by)
export(parseGroupFormula)
export(pick1)
export(pick2)
export(plot_lm)
export(popMeans)
export(pow)
export(rbind_list)
export(reciprocal)
export(recodeVar)
export(recode_var)
export(recover_pca_data)
export(renameCol)
export(response)
export(response_plot)
export(sampleBy)
export(sample_by)
export(sapplyBy)
export(sapply_by)
export(scaleBy)
export(scale_by)
export(scale_df)
export(section_fun)
export(section_fun_env)
export(section_fun_sub)
export(set_covariate_val)
export(set_default)
export(set_list2matrix)
export(set_xlevels)
export(simplify_rhs)
export(splitBy)
export(splitBy.legacy)
export(split_by)
export(split_by.legacy)
export(split_bycol)
export(split_byrow)
export(subSeq)
export(sub_seq)
export(subsetBy)
export(subset_by)
export(subtract)
export(summaryBy)
export(summary_by)
export(summary_mb)
export(tail2)
export(taylor)
export(terms_labels)
export(timeSinceEvent)
export(time_since_event)
export(to_str)
export(transformBy)
export(transform_by)
export(transform_forecast)
export(truncate0)
export(unique_formula)
export(v)
export(vcheck)
export(vmap)
export(vparse)
export(vrename)
export(vselect)
export(which.maxn)
export(which.minn)
import(dplyr)
import(ggplot2)
import(methods)
importClassesFrom(Matrix,Matrix)
importFrom(Deriv,Deriv)
importFrom(MASS,Null)
importFrom(MASS,ginv)
importFrom(Matrix,Matrix)
importFrom(Matrix,colMeans)
importFrom(Matrix,colSums)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(boot,cv.glm)
importFrom(broom,tidy)
importFrom(cowplot,plot_grid)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(forecast,Arima)
importFrom(forecast,auto.arima)
importFrom(forecast,autoplot)
importFrom(forecast,forecast)
importFrom(methods,setOldClass)
importFrom(microbenchmark,microbenchmark)
importFrom(modelr,rmse)
importFrom(purrr,imap_dfr)
importFrom(rlang,abort)
importFrom(rlang,as_name)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,eval_tidy)
importFrom(rlang,f_rhs)
importFrom(rlang,is_formula)
importFrom(rlang,quo_get_expr)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,delete.response)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,ftable)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,rstandard)
importFrom(stats,rstudent)
importFrom(stats,sd)
importFrom(stats,sigma)
importFrom(stats,step)
importFrom(stats,summary.lm)
importFrom(stats,terms)
importFrom(stats,tsp)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(tibble,as_tibble)
importFrom(tidyr,pivot_longer)
importFrom(utils,head)
importFrom(utils,help)
importFrom(utils,str)
importFrom(utils,tail)
importMethodsFrom(Matrix,"%*%")
importMethodsFrom(Matrix,diag)
importMethodsFrom(Matrix,isSymmetric)
importMethodsFrom(Matrix,solve)
importMethodsFrom(Matrix,t)

## doBy

A utility package: Groupwise Statistics, LSmeans, Linear Contrasts, Utilities

1. Facilities for working with grouped data: 'do' something to data stratified 'by' some variables.
2. Linear estimates and LSmeans (marginal means)
3. Functional  programming
4. Pipe arithmetic
5. Flexible variable selection
6. Miscellaneous datasets

Author: Søren Højsgaard,  Ulrich Halekoh

### Online documentation

https://hojsgaard.github.io/doBy

### Development site

https://github.com/hojsgaard/doBy

### CRAN page:

https://cran.r-project.org/web/packages/doBy/index.html

#' @title Calculate "time since event" in a vector.
#' 
#' @description Calculate "time since event" in a vector.
#'
#' @details Events are coded as 1 in numeric vector (and non-events are
#'     coded with values different from 1). \code{timeSinceEvent} will give the
#'     time since event (with and without sign). In a logical vector, events are
#'     coded as TRUE and all non-events as FALSE.
#' 
#' @param yvar A numerical or logical vector specifying the events
#' @param tvar An optional vector specifying time
#' @return A dataframe with columns 'yvar', 'tvar', 'abs.tse' (absolute time
#'     since nearest event), 'sign.tse' (signed time since nearest event) and
#'     'run' (indicator of the time window around each event).
#'
#' @note NA's in yvar are converted to zeros.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{subSeq}}, \code{\link{rle}}
#' @keywords utilities
#' @examples
#' 
#' ## Events:
#' yvar <- c(0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0,
#'           0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0)
#' 
#' ## Plot results:
#' tse <- timeSinceEvent(yvar)
#' plot(sign.tse ~ tvar, data=tse, type="b")
#' grid()
#' rug(tse$tvar[tse$yvar==1], col=4, lwd=4)
#' points(scale(tse$run), col=tse$run, lwd=2)
#' lines(abs.tse + .2 ~ tvar, data=tse, type="b", col=3)
#' 
#' ## Find times for which time since an event is at most 1:
#' tse$tvar[tse$abs <= 1]
#' 
#' yvar <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 
#' 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
#' )
#' 
#' tvar <- c(207, 208, 208, 208, 209, 209, 209, 209, 210, 210, 211, 211, 
#' 211, 212, 213, 213, 214, 214, 215, 216, 216, 216, 216, 217, 217, 
#' 217, 218, 218, 219, 219, 219, 219, 220, 220, 221, 221, 221, 221, 
#' 222, 222, 222)
#' 
#' timeSinceEvent(yvar, tvar)
#' 
#' 
#' 
#' @export timeSinceEvent
timeSinceEvent <- function(yvar, tvar=seq_along(yvar)) {

    if (!(is.numeric(yvar) | is.logical(yvar))) {
        stop("yvar must be either numeric or logical")
    }
    
    yvar[is.na(yvar)] <- 0
    
    event.idx <- which(yvar == 1)
    
    if (length(event.idx) == 0) {
        return(NULL)
    }
    
    n.event <- length(event.idx)

    ## find event times
    event.time <- tvar[event.idx]

    ## get time difference to each event
    rrr <-  do.call(rbind, lapply(event.idx, function(ii) tvar-tvar[ii]))
    abs.tse <- apply(abs(rrr), 2, min)


    ## get the event windows (~ symmetrical around event time)
    ewin<-rep.int(NA, length(yvar))
    if (n.event > 1) {
        ff <- event.time[1:(n.event-1)] + diff(event.time) / 2
        ewin[tvar <= ff[1]] <- 1
        for (ii in 2:(length(ff) - 0)){
            ewin[tvar > ff[ii - 1] & tvar <= ff[ii] ] <- ii
        }
        ewin[tvar > ff[length(ff)]] <- n.event
    } else {
        ewin[] <- n.event
    }

  ## get the signs
    ggg <- list()
    for (ii in 1:(length(event.idx))){
        ggg[[ii]] <- rrr[ii, ewin == ii]
    }
    ggg <- unlist(ggg)
    sign.tse <- sign(ggg) * abs.tse

    run <- cumsum(yvar)

    un <- unique(run)
    tlist <- list()
    for (ii in 1:length(un)){
        vv <- un[ii]
        yy <- yvar[run == vv]
        tt <- tvar[run == vv]
        tt <- tt - tt[1]
        tlist[[ii]] <- tt
    }
    tae <- unlist(tlist)
    tae[run == 0] <- NA

    yvar2 <- rev(yvar)
    tvar2 <- rev(tvar)

    run2 <- cumsum(yvar2)
    un2 <- unique(run2)
    tlist2 <- list()
    for (ii in 1:length(un2)){
        vv <- un2[ii]
        yy <- yvar2[run2 == vv]
        tt <- tvar2[run2 == vv]
        tt <- tt - tt[1]
        tlist2[[ii]] <- tt
  }
    tbe <- unlist(tlist2)
    tbe[run2==0] <- NA

    tbe <- rev(tbe)
    run[run==0]<-NA
    
    ans <- cbind(data.frame(yvar=yvar, tvar=tvar), abs.tse, sign.tse, ewin=ewin,
                 run, tae=tae, tbe=tbe)
    ans
}













## .timeSinceEvent <- function(yvar, tvar=seq_along(yvar)){

##   if (!(is.numeric(yvar) | is.logical(yvar))){
##     stop("yvar must be either numeric or logical")
##   }

##   yvar[is.na(yvar)] <- 0

##   event.idx <- which(yvar==1)
##   if (length(event.idx)==0){
##     return(NULL)
##   }

##   res <- vector("list", length(event.idx))
##   for (kk in seq_along(event.idx)){
##     jj <- event.idx[kk]
##     res[[kk]] <- tvar-tvar[jj]
##   }

##   res      <-do.call(rbind, res)
##   #print(res)
##   abs.tse  <-apply(abs(res),2,min)
##   #print(abs.tse)

##   sgn 	<- rep.int(0, length(yvar))
##   sss 	<- subSeq(yvar,item=0)
##   for (ii in 1:nrow(sss)){
##     idx  <- sss[ii,1]:sss[ii,2]
##     idx1 <- seq_along(idx)
##     uuu  <- abs.tse[idx]
##     fff <- .find.inc.dec(uuu)
##     sgn[idx] <- fff
##   }



##   #print(sgn)
##   sign.tse <- abs.tse * sgn
##   eventWindow <- rep(NA, length(abs.tse))

##   #print(sign.tse)

##   curr.state <- 1
##   eventWindow[1] <- curr.state
##   for (jj in 2:(length(eventWindow)-1)){
##     if (sign.tse[jj] <= 0 & sign.tse[jj-1] >0 ){
##       curr.state <- curr.state + 1
##     }
##     eventWindow[jj] <- curr.state
##   }
##   eventWindow[length(eventWindow)] <- curr.state
##   #print(eventWindow)

##   run <- cumsum(yvar)
##   un <- unique(run)
##   tlist <- list()
##   for (ii in 1:length(un)){
##     vv <- un[ii]
##     yy <- yvar[run==vv]
##     tt <- tvar[run==vv]
##     tt <- tt - tt[1]
##     tlist[[ii]] <- tt
##   }
##   timeAfterEvent <- unlist(tlist)
##   timeAfterEvent[run==0] <- NA

##   yvar2 <- rev(yvar)
##   tvar2 <- rev(tvar)

##   run2 <- cumsum(yvar2)
##   un2 <- unique(run2)
##   tlist2 <- list()
##   for (ii in 1:length(un2)){
##     vv <- un2[ii]
##     yy <- yvar2[run2==vv]
##     tt <- tvar2[run2==vv]
##     tt <- tt - tt[1]
##     tlist2[[ii]] <- tt
##   }
##   timeBeforeEvent <- unlist(tlist2)
##   timeBeforeEvent[run2==0] <- NA

##   timeBeforeEvent <- rev(timeBeforeEvent)
##   run[run==0]<-NA

##   #aux <- cbind(yvar,tvar, run, timeAfterEvent, timeBeforeEvent)
##   aux <- cbind(run, tae=timeAfterEvent, tbe=timeBeforeEvent)
##   ans <- cbind(data.frame(yvar=yvar, tvar=tvar), abs.tse, sign.tse, ewin=eventWindow, aux)
##   return(ans)
## }






## .find.inc.dec <- function(ttt){
##   ans <- rep.int(0,length(ttt))
##   max.ttt <- max(ttt)
##   wm <- which(ttt==max.ttt)
##   if (max(wm)==length(ttt)){ ## The last element is maximal
##     mmm <- max(wm)
##     ans[] <- 1
##   } else {
##     if (min(wm)==1){ ## The first element is maximal
##       mmm <- 1
##       ans[] <- -1
##     } else {
##       mmm <- ceiling(mean(wm))
##       ans[1:(mmm-1)] <- 1
##       ans[mmm:length(ans)] <- -1
##     }
##   }
##   ans
## }
## # ttt <- c(1,1,1,2,2,3,4,4,4,5,5,5,3,2,2,2,1,1)
## # ttt <- c(1,1,1,2,2,3,4,4,4,5,5,4)
## # ttt <- rev(ttt)

#' @import methods
#'
#' @importFrom MASS ginv Null
#'
#' @importFrom Matrix rowMeans colMeans rowSums colSums
#'
#' @importFrom stats model.response model.weights rstandard rstudent residuals tsp is.ts time window
#'
#' @importFrom forecast auto.arima Arima autoplot forecast
#' 
#' @importClassesFrom Matrix Matrix
#'
#' @importFrom Matrix Matrix sparseMatrix
#' 
#' @importMethodsFrom Matrix t isSymmetric "%*%" solve diag

#' @importFrom stats as.formula coef delete.response family fitted formula ftable lm logLik median model.frame model.matrix pchisq pnorm pt qnorm qt residuals sd summary.lm terms vcov aggregate printCoefmat confint predict resid rstandard step update sigma frequency quantile simulate start ts
#'     
#' @importFrom modelr rmse

## #' @importFrom microbenchmark microbenchmark
#' @importFrom Deriv Deriv
#' @importFrom utils str help
#' @importFrom methods setOldClass
#' @importFrom cowplot plot_grid
#' @importFrom rlang enquo
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr mutate select
#' @importFrom tibble as_tibble
#' @importFrom broom tidy

NULL

#' @title Extract components from a formula with "conditioning bar"
#' 
#' @description Extract components from a formula with the form
#'     \code{y ~ x1 + ... + xn | g1 + ... + gm}
#' 
#' @param form A formula of the form \code{y ~ x1 + ... + xn | g1 + ... + gm}
#' @return If the formula is \code{y ~ x1 + x2 | g1 + g2} the result is
#'     \item{model}{\code{y ~ x1 + x2}} \item{groups}{\code{ g1 + g2}}
#'     \item{groupFormula}{\code{~ g1 + g2}}
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @keywords utilities
#' @examples
#' 
#' gf1 <- parseGroupFormula(y ~ x1 + x2 | g1 + g2)
#' gf1
#'
#' gf2 <- parseGroupFormula( ~ x1 + x2 | g1 + g2)
#' gf2
#' 
#' @export parseGroupFormula
parseGroupFormula <- function(form)
{

    if (!inherits(form, "formula"))
        stop("formula must be a two-sided formula object")


    rhs <- form[[length(form)]]

    if (!inherits(rhs, "call") || rhs[[1]] != as.symbol('|'))
        stop("rhs of formula must be a conditioning expression")
    new_form <- form 
    new_form[[length(form)]] <- rhs[[2]]
    groups <- rhs[[3]]
    grpFormula <- as.formula(paste("~", deparse(groups)))
    list(model = new_form, groups = groups, groupFormula=grpFormula)
}

#' @title Convert right hand sided formula to a list
#' @description Convert right hand sided formula to a list
#' @param f A right hand sided formula

.rhsf2list <- function (f) {
    if (is.character(f))    list(f) 
    else if (is.numeric(f)) lapply(list(f), "as.character")
    else if (is.list(f))    lapply(f, "as.character")
    else {
        .xxx. <- f[[length(f)]]
        f1 <- unlist(strsplit(paste(deparse(.xxx.), collapse = ""), 
                              " *\\+ *"))
        f2 <- unlist(lapply(f1, strsplit, " *\\* *| *: *| *\\| *"), 
                     recursive = FALSE)
        f2
    }
}


#' Where are the n largest or n smallest elements in a numeric vector ?
#' 
#' Determines the locations, i.e., indices of the n largest or n smallest
#' elements of a numeric vector.

#' 
#' @aliases which.maxn which.minn
#' @param x numeric vector
#' @param n integer >= 1
#' @return A vector of length at most n with the indices of the n largest /
#'     smaller elements. NAs are discarded and that can cause the vector to be
#'     smaller than n.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{which.max}}, \code{\link{which.min}}
#' @keywords utilities
#' @examples
#' 
#' x <- c(1:4, 0:5, 11, NA, NA)
#' ii <- which.minn(x, 5)
#' 
#' x <- c(1, rep(NA,10), 2)
#' ii <- which.minn(x, 5)
#' 
#' @export which.maxn
which.maxn <- function(x, n = 1) {
  if (n==1)
    which.max(x)
  else
    {
      if (n > 1){
        ii <- order(x,decreasing = TRUE)[1:min(n, length(x))]
        ii[!is.na(x[ii])]
      }
      else {
       stop("n must be >= 1")
      }
    }
}

#' @export
which.minn <- function(x, n = 1) {
  if (n == 1)
    which.min(x)
  else
    {
      if (n > 1) {
        ii <- order(x,decreasing=FALSE)[1:min(n, length(x))]
        ii[!is.na(x[ii])]
      }
      else {
       stop("n must be >= 1")
      }
    }
}




#' Truncate values in a matrix / vector to zero if they are below a certain threshold.
#' 
#' @param x matrix / vector
#' @param tol threshold
#' @param sparse logical; if TRUE and `x` is a matrix, return a sparse matrix
#' 
#' @export
truncate0 <- function(x, tol=0.6, sparse=TRUE){ ## doBy candidate
    x[abs(x) <= tol] <- 0
    if (is.matrix(x)){
      if (sparse) as(x, "dgCMatrix") else x
    }
}



#' @name taylor
#'
#' @title Taylor expansion (one dimension)
#'
#' @description Returns Taylor polynomial approximating a function fn(x)
#'
#' @param fn A function of one variable and that variable must be named 'x'.
#' @param x0 The point in which to to the Taylor expansion.
#' @param ord The order of the Taylor expansion.
#'
#' @return function.
#'
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#'
#' @keywords utilities
#'
#' @examples
#' fn <- function(x) log(x)
#' ord <- 2
#' x0 <- 2
#'
#' xv  <- seq(.2, 5, .1)
#' plot(xv, fn(xv), type="l")
#' lines(xv, taylor(fn, x0=x0, ord=ord)(xv), lty=2)
#' abline(v=x0)
#'
#' fn <- function(x)sin(x)
#' ord <- 4
#' x0 <- 0
#' xv <- seq(-2*pi, 2*pi, 0.1)
#' plot(xv, fn(xv), type="l")
#' lines(xv, taylor(fn, x0=x0, ord=ord)(xv), lty=2)
#' abline(v=x0)
#'
#'
#'

#' @export
taylor <- function(fn, x0, ord=1){

    if (!is.function(fn))
        stop("'fn' is not a function")

    aname <- names(formals(fn))
    if (length(aname) != 1)
        stop("'fn' must take a single argument")
    if (!identical(aname, "x"))
        stop("argument to 'fn' must be called 'x'")
    
    df <- Deriv(fn, nderiv=1:ord)

    quad <- paste0("(x-x0)^")
    aa  <- paste0(quad, 1:ord)

    bb  <- gsub("x0", x0, aa)
    bb
    fac <- factorial(1:ord)
    ff  <- unlist(df(x0)) / fac
    bb  <- paste0(ff, "*", bb)
    bb

    expr <- parse(text=(paste0(bb, collapse="+")))
    fun <- expr_to_fun(expr)

    fun
    
}



#' Recode values of a vector
#' 
#' Recodes a vector with values, say 1,2 to a variable with values, say 'a',
#' 'b'
#' 
#' @param x A vector; the variable to be recoded.
#' @param src The source values: a subset of the present values of x
#' @param tgt The target values: the corresponding new values of x
#' @param default Default target value for those values of x not listed in
#'     `src`. When default=NULL, values of x which are not given in `src` will
#'     be kept in the output.
#' @param keep.na If TRUE then NA's in x will be retained in the output
#' @return A vector
#' @section Warning : Care should be taken if x is a factor. A safe approach may
#'     be to convert x to a character vector using as.character.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link[base]{cut}}, \code{\link[base]{factor}},
#'     \code{\link[doBy]{recodeVar}}
#' @keywords utilities
#' @examples
#' 
#' x <- c("dec", "jan", "feb", "mar", "apr", "may")
#' src1 <- list(c("dec", "jan", "feb"), c("mar", "apr", "may"))
#' tgt1 <- list("winter", "spring")
#' recodeVar(x, src=src1, tgt=tgt1)
#' #[1] "winter" "winter" "winter" "spring" "spring" "spring"
#' 
#' x <- c(rep(1:3, 3))
#' #[1] 1 2 3 1 2 3 1 2 3
#' 
#' ## Simple usage:
#' recodeVar(x, src=c(1, 2), tgt=c("A", "B"))
#' #[1] "A" "B" NA  "A" "B" NA  "A" "B" NA 
#' 
#' ## Here we need to use lists
#' recodeVar(x, src=list(c(1, 2)), tgt=list("A"))
#' #[1] "A" "A" NA  "A" "A" NA  "A" "A" NA 
#' recodeVar(x, src=list(c(1, 2)), tgt=list("A"), default="L")
#' #[1] "A" "A" "L" "A" "A" "L" "A" "A" "L"
#' recodeVar(x, src=list(c(1, 2), 3), tgt=list("A", "B"), default="L")
#' #[1] "A" "A" "B" "A" "A" "B" "A" "A" "B"
#' 
#' ## Dealing with NA's in x
#' x<-c(NA,rep(1:3, 3),NA)
#' #[1] NA  1  2  3  1  2  3  1  2  3 NA
#' recodeVar(x, src=list(c(1, 2)), tgt=list("A"))
#' #[1] NA  "A" "A" NA  "A" "A" NA  "A" "A" NA  NA 
#' recodeVar(x, src=list(c(1, 2)), tgt=list("A"), default="L")
#' #[1] NA  "A" "A" "L" "A" "A" "L" "A" "A" "L" NA 
#' recodeVar(x, src=list(c(1, 2)), tgt=list("A"), default="L", keep.na=FALSE)
#' #[1] "L" "A" "A" "L" "A" "A" "L" "A" "A" "L" "L"
#' 
#' x <- c("no", "yes", "not registered", "no", "yes", "no answer")
#' recodeVar(x, src = c("no", "yes"), tgt = c("0", "1"), default = NA)
#' 
#' 
#' @export recodeVar
recodeVar <- function(x, src, tgt, default=NULL, keep.na=TRUE) {

  if (length(src)!=length(tgt)) {
    stop("length of src not equal to length of tgt")
  }
    mtc <- lapply(src,
                  function(zzz) {
                      which(x %in% zzz)
                  })
    
  idx <- seq_along(x)
  unmatch <- setdiff(idx, unlist(mtc))
  
  if (is.factor(x)) {
    val <- as.character(x)
  } else {
    val <- x
  }
  for (ii in 1:length(tgt))
    val[mtc[[ii]]] <- tgt[[ii]]

  if (!is.null(default)) {
    if (keep.na) {
      iii <- intersect(which(!is.na(x)), unmatch)
      val[iii] <- default
    } else {
      val[unmatch] <- default
    }
  }
  
  if (is.factor(x))
    val <- as.factor(val)
  val

  return(val)
}

#' @rdname recodeVar
#' @export
recode_var <- recodeVar


#' Rename columns in a matrix or a dataframe.
#' 
#' Rename columns in a matrix or a dataframe.
#' 
#' 
#' @param indata A dataframe or a matrix
#' @param src Source: Vector of names of columns in `indata` to be
#'     renamed. Can also be a vector of column numbers.
#' @param tgt Target: Vector with corresponding new names in the
#'     output.
#' @return A dataframe if `indata` is a dataframe; a matrix in
#'     `indata` is a matrix.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @keywords utitlities
#' @examples
#' renameCol(CO2, 1:2, c("kk", "ll"))
#' renameCol(CO2, c("Plant", "Type"), c("kk", "ll"))
#' 
#' # These fail - as they should:
#' # renameCol(CO2, c("Plant", "Type", "conc"), c("kk", "ll"))
#' # renameCol(CO2, c("Plant", "Type", "Plant"), c("kk", "ll"))
#' 
#' @export renameCol
renameCol <- function(indata, src, tgt){

  if (inherits(indata, "data.frame")) {
    isDF <- TRUE
    dfnames <- names(indata)
  } else {
    if (inherits(indata, "matrix")) {
      isDF <- FALSE
      dfnames <- colnames(indata)
    } else {
      stop("'indata' must be either a dataframe or a matrix")
    }
  }
  
  if (length(src)!=length(unique(src))){
    stop("A src name has been repeated")
  }

  if (length(tgt)!=length(unique(tgt))){
    stop("A tgt name has been repeated")
  }

  if (length(src)!=length(tgt)){
    stop("length of src not equal to length of tgt")
  }
    
  if (is.numeric(src)){
    idx <- src
    iii <- intersect(seq_along(dfnames), src)
    iii <- setdiff(src, iii)
    if (length(iii)>0){
      sss <- sprintf("Column(s) %s are not in 'indata'", toString(iii))
      stop(sss)
    }
  } else {
    idx <- match(src, dfnames)
    if (any(is.na(idx))){
      sss <- sprintf("Column names %s are not in 'indata'", toString(src[is.na(idx)]))
      stop(sss)
    }
  }
  
  ans <- indata
  if (isDF){
    names(ans)[idx] <- tgt
  } else {
    colnames(ans)[idx] <- tgt
  }

  return(ans)
}



###############################################################################
#' @title Computing simple descriptive statistics of a numeric vector.
#' @description Computing simple descriptive statistics of a numeric
#'     vector - not unlike what proc means of SAS does
###############################################################################
#' @param x A numeric vector
#' @param na.rm Should missing values be removed
#' @return A vector with named elements.
#' @author Gregor Gorjanc; \email{gregor.gorjanc@@bf.uni-lj.si}
#' @seealso \code{\link{summaryBy}}, \code{\link{summary_by}}
#' @keywords utilities
#' @examples
#' 
#' x <- c(1, 2, 3, 4, NA, NaN)
#' descStat(x)
#' 
#' @export descStat
descStat <- function (x, na.rm = TRUE)
{
  if(!is.numeric(x))
    stop("'x' must be numeric")
  m <- mean(x, na.rm = na.rm)
  s <- sd(x, na.rm = na.rm)
  c(n = length(x),
    obs = sum(!is.na(x)),
    mean = m,
    median = median(x, na.rm = na.rm),
    sd = s,
    cv = s/m,
    min = min(x, na.rm = na.rm),
    max = max(x, na.rm = na.rm))
}

#' Plot the response variable against the predictor variables.
#'
#' @param formula. A formula of the form y ~ x1 + x2 + ... + xn, where
#'     y is the response variable and x1, x2, ..., xn are the
#'     predictor variables. A dot as right hand side is allowed.
#' 
#' @param data. A data frame containing the variables in the formula.
#' @param geoms A list of ggplot2 geoms to be added to the plot.
#' @param global_aes A list of global aesthetics to be added to the plot.
#' @param plot A logical value indicating whether the plot should be displayed.
#' @param nrow,ncol Number of rows / columns in plot. 
#'
#' @return A list of ggplot2 plots.
#' @export
#'
#' @examples
#' library(ggplot2)
#' response_plot(iris, Sepal.Width ~ ., geoms=geom_point())
#' response_plot(iris, Sepal.Width ~ ., geoms=geom_point(), global_aes=list(color="Species"))
#' personality |> response_plot(easygon~., geoms=geom_point(), global_aes=NULL)
#' 
response_plot <- function(data., formula., geoms=NULL, global_aes=NULL, plot=TRUE, nrow=NULL, ncol=NULL) {  
    trms <- terms(formula., data=data.)
    ## trms
    yy <- as.character(formula.[[2]])
    xx <- setdiff(attr(trms, "term.labels"), yy)
    ##    list(x=xx, y=yy) |> str()

    aes_template <- lapply(xx, function(x_){
        c(list(x=x_, y=yy), global_aes) 
    })
    ## aes_template

    aes_list <- aes_template |> lapply(ggpubr_create_aes)
    ## aes_list
    
    plot_basic <- lapply(aes_list, function(z_){
        data.  |> ggplot(mapping = z_)    
    })
    ## plot_basic
    
    plot_list <-lapply(plot_basic, function(pl_) {
        pl_ + geoms
    } )
    if (plot){
        s <- cowplot::plot_grid(plotlist = plot_list, nrow=nrow, ncol=ncol)        
        print(s)
    }
    
    return(invisible(plot_list))
}


### Taken from ggpubr::create_aes to avoid circular dependencies

ggpubr_create_aes <- function (.list, parse = TRUE) 
{
  # if (missing(parse)) {
  #   parse <- base::getOption("ggpubr.parse_aes", default = TRUE)
  # }
#  if (parse) {
    return(create_aes.parse(.list))
#  }
#  else {
#    return(create_aes.name(.list))
#  }
}

create_aes.parse <- function (.list) 
{
#  .list <- .list %>% purrr::map(function(x) parse_expression(x))
  .list <- .list |> lapply(function(x) parse_expression(x))
  do.call(ggplot2::aes, .list)
}

parse_expression <- function (x) 
{
  if (is_parsable_aes(x)) {
    if (contains_space(x)) {
      if (!is_math_string(x)) 
        return(as.name(x))
    }
    x <- parse(text = x)[[1]]
  }
  x
}

is_numeric_char <- function (x) 
{
  if (is.character(x)) 
    res <- grepl("^[[:digit:]]+$", x)
  else res <- FALSE
  res
}

is_parsable_aes <- function (x) 
{
  is.character(x) & (!is_numeric_char(x)) & (length(x) == 1)
}

contains_space <- function (x) 
{
  grepl("\\s", x)
}

is_math_string <- function (x) 
{
  operators <- c("+", "-", "*", "^", "%%", "%/%", "/", "==", 
                 ">", "<", "!=", "<=", ">=")
  contains_math_operators <- unlist(lapply(operators, grepl, 
                                           x, fixed = TRUE))
  contains_parentheses <- grepl(pattern = "\\(.*\\)", x)
  any(c(contains_math_operators, contains_parentheses))
}




# parse_as_expression <- function (text) 
# {
#   stopifnot(is.character(text))
#   out <- vector("expression", length(text))
#   for (i in seq_along(text)) {
#     expr <- parse(text = text[[i]])
#     out[[i]] <- if (length(expr) == 0) 
#       NA
#     else expr[[1]]
#   }
#   out
# }

#' Bind list of data frames and add list names as a column
#'
#' @description
#' Binds a named list of data frames (or tibbles) into a single data frame.
#' Adds the list name as a new column (first column).
#'
#' @param lst A named list of data frames or tibbles.
#' @param name A character scalar: name of the column to hold the list names (default "name").
#' @concept data_handling
#' @return A data frame or tibble, depending on the class of the input.
#'
#' @examples
#' lst <- list(a = data.frame(x = 1:2), b = data.frame(x = 3:4))
#' rbind_list(lst)
#'
#' lst <- split(iris, iris$Species)
#' rbind_list(lst)
#' 
#'
#' @export
rbind_list <- function(lst, name = "name") {
  stopifnot(
    is.list(lst),
    !is.null(names(lst)),
    all(nzchar(names(lst))),
    is.character(name),
    length(name) == 1
  )

  use_tibble <- requireNamespace("tibble", quietly = TRUE) &&
                any(vapply(lst, inherits, logical(1), "tbl_df"))

  out <- Map(function(df, nm) {
    df <- as.data.frame(df)  # ensures even tibbles behave nicely
    df[[name]] <- nm
    df[, c(name, setdiff(names(df), name)), drop = FALSE]
  }, lst, names(lst))

  result <- do.call(rbind, out)
  row.names(result) <- NULL

  if (use_tibble) {
    result <- tibble::as_tibble(result)
  }

  result
}

## Navn | Fordel | Ulempe
## rbind_list | Beskrivende, enkel, matcher base R-logik | En smule generisk
## stack_list | Mere "verbalt" og beskriver handlingen | Kan forveksles med stack()
## combine_list | Letforståeligt, signalerer sammensmeltning | Ikke specifikt om rbind
## bind_rows_list | Tydeligt og i stil med dplyr::bind_rows() | Længere navn
## join_list | Knap så præcist – lyder som en slags merge | Forvirrende i statistisk kontekst
## collapse_list | Tydeligt at det handler om at lave én ud af mange | Kan tolkes som vektor-collapse

#' @title Find sub-sequences of identical elements in a vector. 
#' @description Find sub-sequences of identical elements in a vector.
#' @name sub_seq
#' 
#' @param x An atomic vector or a factor.
#' @param item Optionally a specific value to look for in `x`.
#' @return A dataframe.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{rle}}
#'
#' @details
#'
#' * `sub_seq` is synonymous with `subSeq`
#' 
#' * `rle2` is identical to `rle` (from base) but `rle2` works on
#'     factors as input (a factor is coerced to character).
#'
#' * `is_grouped` checks if the values in `x` are clustered into the
#' smallest number of clusters.
#' 
#' @keywords utilities
#' @examples
#' 
#' x <- c(1, 1, 1, 0, 0, 1, 1, 1, 2, 2, 2, 1, 2, 2, 2, 3)
#' (ans <- subSeq(x))
#' ans$value
#' 
#' ## Notice: Same results below
#' subSeq(x, item=1)
#' subSeq(x, item="1")
#' 
#' xc <- as.character(x)
#' (ans<-subSeq(xc))
#' ans$value
#'
#' ## Notice: Same results below
#' subSeq(xc, item="1")
#' subSeq(xc, item=1)
#'
#' is_grouped(x)
#' is_grouped(sort(x))
#' is_grouped(xc)
#' is_grouped(sort(xc))


#' @rdname sub_seq
#' @export subSeq
subSeq <- function (x, item = NULL) {
    rrr <- rle2(x)
    len <- rrr$lengths
    val <- rrr$values

    first <- last <- rep.int(NA, length(val))
    first[1] <- 1
    last [1] <- len[1]
    if (length(val) > 1) {
	for (kk in 2:length(val)) {
            first[kk] <- last[kk - 1] + 1
            last [kk] <- last[kk - 1] + len[kk]
	}
    }
    midp <- floor(first + len / 2)

    ans <- cbind(first=first, last=last, slength=len, midpoint=midp)

    if (!is.null(item)) {
        iii <- val == item
        ans <- as.data.frame.matrix(ans[iii,, drop=FALSE], stringsAsFactors=FALSE)
        ans$value <- val[iii]
    } else {
        ans <- as.data.frame.matrix(ans, stringsAsFactors=FALSE)
        ans$value <- val
    }
    ans
}

#' @rdname sub_seq
#' @export
sub_seq <- subSeq

#' @rdname sub_seq
#' @export
is_grouped <- function(x) {
    !any(table(rle2(x)$values) > 1)    
}

#' @rdname sub_seq
#' @export
rle2 <- function(x) {
    if (is.factor(x)) {        
        rle(as.character(x))
    } else {
        rle(x)
    }
}

#' @export
doby.xtabs <-
function (formula = ~., data = parent.frame(),
          subset, na.action, exclude = c(NA,NaN), drop.unused.levels = FALSE,
          FUN=sum) 
{

  ep      <- FALSE;
  sh.eprint<- function(x){
    if (ep==TRUE){str <- deparse(substitute(x)); cat(paste("E>>", str," =", x),fill=TRUE)}}
  
  if (!missing(formula) && !inherits(formula, "formula")) 
    stop("formula is incorrect")
  if (any(attr(terms(formula), "order") > 1)) 
    stop("interactions are not allowed")
  if (missing(na.action)) 
    na.action <- getOption("na.action")
  
  m <- match.call(expand.dots = FALSE)
  if (is.matrix(eval(m$data, parent.frame()))) 
    m$data <- as.data.frame(data)
  m$... <- m$exclude <- m$drop.unused.levels <- NULL ## FIXME ...
  m$... <- m$exclude <- m$FUN <- NULL
  m[[1]] <- as.name("model.frame")
  mf <- eval(m, parent.frame())
  if (length(formula) == 2) {
    by <- mf
    y <- NULL
  }
  else {
    i <- attr(attr(mf, "terms"), "response")
    by <- mf[-i]
    y <- mf[[i]]
  }
  
  by <- lapply(by, function(u) {
    if (!is.factor(u)) 
      u <- factor(u, exclude = exclude)
    u[, drop = drop.unused.levels]
  })

  x <- if (is.null(y)) 
    do.call("table", by)
  else if (NCOL(y) == 1){
    tapply(y, by, FUN) 
  }
  else {
    z <- lapply(as.data.frame(y), tapply, by, FUN)    
    array(unlist(z), dim = c(dim(z[[1]]), length(z)), dimnames = c(dimnames(z[[1]]), 
                                                        list(names(z))))
  }
  x[is.na(x)] <- 0
  class(x) <- c("xtabs", "table")
  attr(x, "call") <- match.call()
  x
  value <- x
  return(value)
}

######################################################################

###############################################################################
#' @title Locate the index of the first/last unique value
#' @description Locate the index of the first/last unique value in i) a vector
#'     or of a variable in a data frame.
#' @name firstlastobs
###############################################################################
#' @details If writing ~a + b + c as formula, then only a is considered.
#' 
#' @aliases firstobs lastobs firstobs.default lastobs.default firstobs.formula
#'     lastobs.formula
#' @param x A vector
#' @param formula A formula (only the first term is used, see 'details').
#' @param data A data frame
#' @param \dots Currently not used
#' @concept data_handling
#' @return A vector.
#' 
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @keywords utilities
#' @examples
#' x <- c(rep(1, 5), rep(2, 3), rep(3, 7), rep(1, 4))
#' firstobs(x)
#' lastobs(x)
#' data(dietox)
#' firstobs(~Pig, data=dietox)
#' lastobs(~Pig, data=dietox)
#' 

#' @concept data_handling
#' @export
#' @rdname firstlastobs
lastobs <- function(x, ...)  UseMethod("lastobs")

#' @concept data_handling
#' @export
#' @rdname firstlastobs
firstobs <- function(x, ...) UseMethod("firstobs")

#' @export
#' @rdname firstlastobs
lastobs.default <- function(x, ...) {
  ux <- unique(x)
  m <-match(ux, x)
  sort(sapply(ux, function(i) {max(which(i==x))}))
}

#' @export
#' @rdname firstlastobs
firstobs.default <- function(x, ...) {
  ux <- unique(x)
  m <- match(ux,x)
  sort(sapply(ux, function(i) {min(which(i==x))}))
}

#' @export
#' @rdname firstlastobs
lastobs.formula <- function(formula, data=parent.frame(), ...) {
  rhs <- gsub(" +","",strsplit(paste(formula[2]),"\\+")[[1]][1])
  lastobs(data[,rhs])
}

#' @export
#' @rdname firstlastobs
firstobs.formula <- function(formula, data=parent.frame(), ...) {
   rhs <- gsub(" +","",strsplit(paste(formula[2]),"\\+")[[1]][1])
   firstobs(data[,rhs])
}

#' @title Extract (pick) elements without using brackets
#'
#' @description Extract (pick) elements without using brackets so that elements can be picked out as part of a pipe workflow.
#'
#' @details
#' 
#' These two helper functions extract elements from lists, data frames, or vectors.
#' They are simple wrappers for the standard bracket operators in R:
#'
#' - `pick1()` uses single brackets (`[`) and returns a **subset**.
#' - `pick2()` uses double brackets (`[[`) and returns the **element itself**.
#'
#' These are safer and more flexible than `$`, especially when used with the base R pipe (`|>`)
#' or in functional programming.
#'
#' @param x A list, data frame, or vector.
#' @param which The index or name of the element(s) to extract.
#'
#' @return
#' - `pick1()` returns a subset of `x`.
#' - `pick2()` returns a single element from `x`.
#'
#' @examples
#' lst <- list(a = 1:3, b = 4:6)
#'
#' # Without pipe
#' pick1(lst, "a")      # List with one element
#' pick2(lst, "a")      # Just the vector 1:3
#'
#' # With base R pipe
#' lst |> pick1("a")
#' lst |> pick2("a")
#'
#' df <- data.frame(x = 1:5, y = letters[1:5])
#'
#' df |> pick1("y")     # Returns a data frame with column 'y'
#' df |> pick2("y")     # Returns column 'y' as a character vector
#'
#' @name pick_elements
#' @aliases pick1 pick2
#' @export
pick1 <- function(x, which) {
  x[which]
}

#' @rdname pick_elements
#' @export
pick2 <- function(x, which) {
  x[[which]]
}



#' head and tail for matrices
#' @param x matrix
#' @param n,m number of rows and columns
#' @name head_matrix
#' @examples
#' M <- matrix(1:20, nrow=4)
#' head2(M)
#' head2(M, 2)
#' @export
head2 <- function(x, n=6, m=n) {
    if (!inherits(x, c("matrix","sparseMatrix")))
        stop("'x' must be a matrix or a sparse matrix\n")
    if (length(dim(x)) != 2)
        stop("'x' must be have dimension with length two. \n")        
    di <- dim(x)
    nr <- di[1]
    nc <- di[2]

    rr <- 1:min(n, nr)
    cc <- 1:min(m, nc)  
    x[rr, cc]
}

#' @rdname head_matrix
#' @export
tail2 <- function(x, n=6, m=n) {
    if (length(dim(x)) != 2)
        stop("'x' must be have dimension with length two. \n")        
    di <- dim(x)
    nr <- di[1]
    nc <- di[2]
    n  <- min(n, nr)
    m  <- min(m, nc)
    rr <- (nr-n+1):nr
    cc <- (nc-m+1):nc
    x[rr, cc]
}

#########################################################################
#' @title Sampling from a data frame
#' @description A data frame is split according to some variables in a
#'     formula, and a sample of a certain fraction of each is drawn.
#' @name by-sample
#########################################################################
#' @details If systematic=FALSE (default) then frac gives the fraction
#'     of data sampled. If systematic=TRUE and frac=.2 then every 1/.2
#'     i.e. every 5th observation is taken out.
#'
#' @param formula A formula defining the grouping of the data frame.
#' @param frac The part of data to be sampled.
#' @param replace Is the sampling with replacement.
#' @param data A data frame.
#' @param systematic Should sampling be systematic.
#'
#' @concept byfunction
#' @return A dataframe.
#' @seealso \code{\link{orderBy}}, \code{\link{order_by}},
#'    \code{\link{splitBy}}, \code{\link{split_by}}, 
#'    \code{\link{summaryBy}}, \code{\link{summary_by}}, 
#'    \code{\link{transformBy}}, \code{\link{transform_by}}
#'
#' @keywords utilities
#' 
#' @examples
#' data(dietox)
#' sampleBy(formula = ~ Evit + Cu, frac=.1, data = dietox)


#' @export
#' @rdname by-sample
sample_by <- function(data, formula,
                      frac=0.1, replace=FALSE, systematic=FALSE){
    arg <- list(formula=formula, frac=frac, replace=replace, data=data,
                systematic=systematic) 
    do.call(sampleBy, arg)
}


#' @export
#' @rdname by-sample
sampleBy <- function(formula, frac=0.1, replace=FALSE, data=parent.frame(),
                     systematic=FALSE){

    out <- splitBy(formula, data=data)

    out <- lapply(out, function(dat){
        if (systematic){
            idx <- seq(1, nrow(dat), 1 / frac)
        } else {  
            idx <- sort(sample(1:nrow(dat), size=round(frac * nrow(dat)),
                               replace=replace))
        }
        dat[idx, ]
    }) 
    out <- do.call("rbind", out)
    out
}

#########################################################################
#'
#' @title Ordering (sorting) rows of a data frame
#' @description Ordering (sorting) rows of a data frame by the certain
#'     variables in the data frame. This function is essentially a
#'     wrapper for the \code{order()} function - the important
#'     difference being that variables to order by can be given by a
#'     model formula.
#' @name by-order
#' 
#########################################################################
#'
#' @details The sign of the terms in the formula determines whether
#'     sorting should be ascending or decreasing; see examples below
#' 
#' @param formula The right hand side of a formula
#' @param data A dataframe
#' @return The ordered data frame
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk} and Kevin
#'     Wright
#' @seealso \code{\link{transformBy}}, \code{\link{transform_by}}, \code{\link{splitBy}}, \code{\link{split_by}}
#' @concept byfunction
#' @keywords utilities
#' @examples
#'
#' orderBy(~ conc + Treatment, CO2)
#' ## Sort decreasingly by conc
#' orderBy(~ - conc + Treatment, CO2)
#' ## Same as:
#' order_by(CO2, c("conc", "Treatment"))
#' order_by(CO2, c("-conc", "Treatment"))


#' @export
#' @rdname by-order
order_by <- function(data, formula){
    cl   <- match.call(expand.dots = TRUE)
    cl[[2]] <- formula
    cl[[3]] <- data
    names(cl)[2:3] <- c("formula", "data")
    cl[[1]] <- as.name("orderBy")
    eval(cl)
}

#' @export
#' @rdname by-order
orderBy <- function (formula, data){

    if (!inherits(data, "tbl_df")) is.tib = FALSE
    else {is.tib = TRUE; data = as.data.frame(data)}
    
    myrank <- function(x){
        rv  <- rep(NA, length(x))
        r   <- rank(cout[!is.na(cout)])
        rv[!is.na(cout)]  <- r
        rv[is.na(rv)]     <- max(r) + 1
        rv
    }


    if (is.vector(formula)) ## E.g. c("Temp", "Month")
        formula <- as.formula(paste0("~ ", paste(formula, collapse=" + ")))
    
    form <- formula
    out  <- data

    
    
    if(form[[1]] != "~")
        stop("Error: Formula must be one-sided.")
    
    formc <- as.character(form[2])
    formc <- gsub(" ", "", formc)
    if(!is.element(substring(formc, 1, 1),c("+", "-")))
        formc <- paste("+", formc, sep="")
    
    vars <- unlist(strsplit(formc, "[\\+\\-]"))
    vars <- vars[vars != ""] # Remove spurious "" terms
    
    signs <- formc  
    for (i in 1:length(vars)){
        signs <- gsub(vars[i], "", signs)
    }
    signs <- unlist(strsplit(signs, ""))
    
    orderlist <- list()
    for(i in 1:length(vars)){
        csign <- signs[i]
        cvar  <- vars[i]
        cout  <- out[, cvar]
        if (is.factor(cout)){
            orderlist[[i]] <-
                if (csign == "-") -myrank(cout) else myrank(cout) 
        } else {
            orderlist[[i]] <-
                if (csign == "-") -cout else cout            
        }
    }
    
    out <- out[do.call("order", orderlist),, drop=FALSE]

    if (is.tib) as_tibble(out) else out
    ##out
}

#' Backquote a list of functions
#'
#' @param fun_list List of functions
#' @concept functional 
#' @seealso [base::bquote()], [doBy::set_default()], [doBy::section_fun()]
#' @examples
#'
#' ## Evaluate a list of functions
#' f1 <- function(x){x + 1}
#' f2 <- function(x){x + 8}
#'
#' f1_ <- set_default(f1, list(x=10))
#' f2_ <- set_default(f2, list(x=10))
#'
#' f1_(); f2_()
#' 
#' fn_list  <- list(f1_, f2_)
#' fn_list_ <- bquote_fun_list(fn_list)
#' 
#' eval(fn_list[[1]])     ## No
#' sapply(fn_list, eval)  ## No
#' 
#' eval(fn_list_[[1]])    ## Yes
#' sapply(fn_list_, eval) ## Yes
#' 
#' @export 
bquote_fun_list <- function(fun_list) {
    if (!inherits(fun_list, "list")) 
        stop("'fun_list' is not a list.")
    cls <- c("function", "section_function")
    if (!all(sapply(fun_list, inherits, cls)))
        stop("Not all elements in 'fun_list' are functions or section_function objects.")
    lapply(fun_list, function(g) {
        bquote(.(g)())
    }
    )
}

## Note to self: The 'modern' packages tibble and broom are only used
## in this file

#' @title Tidy a linest object
#' @description Tidy summarizes information about the components of the object.
#' @name tidy-linest
#'
#' @param x A 'linest_class' object (produced by \code{linest} methods).
#' @param conf.int Should confidence intervals be added.
#' @param conf.level Desired confidence level.
#' @param \dots Additional arguments; currently not used.
#' @export
tidy.linest_class <- function(x, conf.int = FALSE, conf.level = 0.95, ...) {
    co <- stats::coef(x)
    rownames(co) <- NULL
    
    if (ncol(co) == 5) { ## There are degrees of freedom in the output.
        co <- co[,c(1,2,3,5,4)]
        nn <- c("estimate", "std.error", "statistic", "p.value", "df")
    } else
        nn <- c("estimate", "std.error", "statistic", "p.value")
    
    names(co) <- nn
    
    if (conf.int) {
        ci <- .ci_fun(co, level=conf.level)
        colnames(ci) <- c("conf.low", "conf.high")    
        co <- cbind(co, ci)
    }
    as_tibble(co)
}

#' @title Tidy an esticon object
#' @description Tidy summarizes information about the components of the object.
#' @name tidy-esticon
#'
#' @param x A 'esticon_class' object (produced by \code{esticon} methods).
#' @param conf.int Should confidence intervals be added.
#' @param conf.level Desired confidence level.
#' @param \dots Additional arguments; currently not used.
#' @export
tidy.esticon_class <- function(x, conf.int = FALSE, conf.level = 0.95, ...) {
    co <- x[,1:6]
    rownames(co) <- NULL

    if (conf.int){
        ci <- confint(x, level=conf.level)
        colnames(ci) <- c("conf.low", "conf.high")    
        co <- cbind(co, ci)
    }
    as_tibble(co)
}

#' @title Determines if contrasts are estimable.
#' 
#' @description Determines if contrasts are estimable, that is, if the contrasts
#'     can be written as a linear function of the data.
#'
#' @name is_estimable
#' 
#' @details Consider the setting \eqn{E(Y)=Xb}. A linear function of \eqn{b},
#'     say \eqn{l'b} is estimable if and only if there exists an \eqn{r} such
#'     that \eqn{r'X=l'} or equivalently \eqn{l=X'r}. Hence \eqn{l} must be in
#'     the column space of \eqn{X'}, i.e. in the orthogonal complement of the
#'     null space of \eqn{X}. Hence, with a basis \eqn{B} for the null space,
#'     \code{is_estimable()} checks if each row \eqn{l} of the matrix \eqn{K} is
#'     perpendicular to each column basis vector in \eqn{B}.
#' 
#' @param K A matrix.
#' @param null.basis A basis for a null space (can be found with
#'     \code{null_basis()}). 
#' @return A logical vector. 
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @references \url{http://web.mit.edu/18.06/www/Essays/newpaper_ver3.pdf}
#' @keywords utilities
#' 
#' @export is_estimable
is_estimable <- function(K, null.basis){
    if (is.null(null.basis) ||
        (is.matrix(null.basis) && ncol(null.basis)==1) ){
        rep(TRUE, nrow(K))
    } else {
        out <- lapply(1:nrow(K),
                      function(i){
                          k <- K[i,]
                          all(abs(apply(null.basis, 2, function(x) sum(k * x))) < 1e-04)
                      })
        unlist( out )
    }
}


null_basis <- function(M){
    MASS::Null(t(M))
}

##' Recover data from principal component analysis
##' 
##' Recover data from principal component analysis based on the first (typically few) components.
##' 
##' @param object An object of class `prcomp`.
##' @param comp The number of components to be used. Must be smaller
##'     than the number of variables.
##' 
##' @return A dataframe
##' 
##' @examples
##' 
##' crime <- doBy::crimeRate
##' rownames(crime) <- crime$state
##' crime$state <- NULL
##' 
##' o <- order(apply(scale(crime), 1, sum))
##' dat <- crime[o,]
##' head(dat)
##' tail(dat)
##' matplot(scale(dat), type="l")
##' 
##' pc1 <- prcomp(dat, scale. = TRUE)
##' summary(pc1)
##' rec2 <- recover_pca_data(pc1, 2)
##' 
##' pairs(rec2)
##' 
##' par(mfrow=c(1,2))
##' matplot(scale(dat), type="l")
##' matplot(scale(rec2), type="l")
##' 
##' j <- merge(dat, rec2, by=0)
##' pairs(j[,-1])
##'
##' @export
recover_pca_data <- function (object, comp = 1) 
{
    if (!inherits(object, "prcomp"))
        stop("'object' must be of class 'prcomp'.")
    
    if (length(comp) == 1) 
        comp <- 1:comp
    
    if (is.null(object$x)){
        stop("Can not 'recover' data.\n")
    }

    ## x <- object$x
    rot <- object$rot
    vn <- row.names(rot)
    if (object$scale[1]) {
        rot <- diag(object$scale) %*% rot
    }
    pred <- object$x[, comp, drop = FALSE] %*% t.default(rot[, comp, drop = FALSE])

    if (object$center[1]) {
        pred <- t(t(pred) + object$center)
    }
    colnames(pred) <- vn
    return(pred)
}

#' Convert expression into function object.
#'
#' @param expr_ R expression.
#' @param order desired order of function argument.
#' @param vec_arg should the function take vector valued argument.
#' @concept functional 
#' @examples
#'
#' ee  <- expression(b1 + (b0 - b1)*exp(-k*x) + b2*x)
#' ff1 <- expr_to_fun(ee)
#' formals(ff1)
#'
#' ff2 <- expr_to_fun(ee, vec_arg=TRUE)
#' formals(ff2)
#' formals(ff2)$length_parm
#' formals(ff2)$names_parm |> eval()
#' 
#' ee <- expression(matrix(c(x1+x2, x1-x2, x1^2+x2^2, x1^3+x2^3), nrow=2))
#' ff1 <- expr_to_fun(ee)
#' ff2 <- expr_to_fun(ee, vec_arg=TRUE)
#'
#' formals(ff2)
#' formals(ff2)$length_parm
#' formals(ff2)$names_parm |> eval()
#' 
#' @export
expr_to_fun <- function(expr_, order=NULL, vec_arg=FALSE) {
    if (vec_arg) {
        expr_to_one_param_fun(expr_, order=order)
    } else {
        expr_to_multi_param_fun(expr_, order=order)
    }    
}

handle_order <- function(e, order) {
    if (is.null(order))
        return(sort(all.vars(e)))
    
    nms <- all.vars(e)
    ss <- setdiff(nms, order)
    if (length(ss) > 0) {        
        stop("some arguments are not given in 'order'")
    }
    return(order)
}
  
    
expr_to_one_param_fun <- function(e, order=NULL) {

    nms <- handle_order(e, order) 
    e_str <- expr_to_string(e)
    
    if (length(nms)) {
        aux <- sapply(1:length(nms),
                      function(i) {
                          nm <- nms[i]
                          paste0(nm, " = parm[", i, "]")
                      }
                      )
    } else {
        aux <- NULL
    }

    comb <- c(aux, e_str)    

    uuu <- paste("c(",paste0(sQuote(nms, q="'"), collapse=", "), ")")
    
    fun_str <- sprintf("function(parm, length_parm=%d, names_parm=%s)", length(nms), uuu)
    
    bd <- paste0("\n{ \n ", paste0(comb, collapse=";\n "), "\n}")

    ff <- paste0(fun_str, bd)
    fun <- eval(parse(text=ff))
    return(fun)
}



expr_to_multi_param_fun <- function(ee, order=NULL) {

    nms <- handle_order(ee, order)   
    ee_str <- expr_to_string(ee)
    
    fun_str <- paste0("function(", paste0(nms, collapse=", "), ")")
        
    bd <- paste0("\n{ \n", paste0(ee_str, collapse=";\n "), "\n}")
    ff <- paste0(fun_str, bd)
    fun <- eval(parse(text=ff))
    return(fun)
}

expr_to_string <- function(ee) {
    ee_str <- lapply(ee, deparse)
    
    ee_str <-
        lapply(ee_str,
               function(e_) {
                   paste0(e_, collapse="\n")               
               })
    ee_str
}

#' @title Convert binomial data to bernoulli data
#'
#' @description Convert binomial data to bernoulli data by expanding dataset.
#'
#' @param data. A dataframe
#' @param y     Column with 'successes' in binomial distribution `y~bin(size, p)`
#' @param size  Column with 'failures', i.e. size-y or 'total', i.e. size. 
#' @param type  Whether `size` is rest (i.e. 'failures') or 'total'
#' @param response_name Name of response variable in output dataset.
#' @param rest_name Name of 'failures' in column `response_name`. 
#'
#' @examples
#'
#' dat <- budworm
#' dat <- dat[dat$dose %in% c(1,2), ]
#' dat$ntotal <- 5
#' dat
#' dat.a <- dat |>
#'   binomial_to_bernoulli_data(ndead, ntotal, type="total")
#' dat.b <- dat |>
#'   dplyr::mutate(nalive=ntotal-ndead) |> dplyr::select(-ntotal) |>
#'   binomial_to_bernoulli_data(ndead, nalive, type="rest")
#'
#' m0 <- glm(cbind(ndead, ntotal-ndead) ~ dose + sex, data=dat, family=binomial())
#' m1 <- glm(ndead / ntotal ~ dose + sex, data=dat, weight=ntotal, family=binomial())
#' ma <- glm(response ~ dose + sex, data=dat.a, family=binomial())
#' mb <- glm(response ~ dose + sex, data=dat.b, family=binomial())
#'
#' dat.a$response
#' dat.b$response ## Not same and therefore the following do not match
#'
#' all.equal(coef(m0), coef(ma))
#' all.equal(coef(m0), coef(mb))
#' all.equal(coef(m1), coef(ma))
#' all.equal(coef(m1), coef(mb))
#' @export

binomial_to_bernoulli_data <- function(data., y, size, type=c("rest", "total"), 
                                  response_name="response", rest_name=NULL){
  
    type <- match.arg(type)
    
    yy <- rlang::enquo(y) |> rlang::as_name() ## dead
    xx <- rlang::enquo(size) |> rlang::as_name() ## total (or rest)
    if (is.null(rest_name)){
        rest_name <- paste0("not_", yy)
    }
    `:=` = NULL
    if (identical(type, "total")){        
        data. <- data. |> 
            mutate(!!sym(rest_name) := !!sym(xx) - !!sym(yy)) |>
            select(-!!sym(xx))
        tidy_data <- data. |>
            tidyr::pivot_longer(c(!!sym(yy), !!sym(rest_name)),
                                names_to  = response_name,
                                values_to = "n")
    } else {
        tidy_data <- data. |>
            tidyr::pivot_longer(c(!!sym(yy), !!sym(xx)),
                                names_to  = response_name,
                                values_to = "n")
    }
    out <- tidy_data |> tidyr::uncount(n)
    resp <- out[,response_name] |> pull()
    lev <- rev(c(yy, setdiff(unique(resp), yy)))
    
    
    out[,response_name] <- out[,response_name] |> pull() |> factor(levels=lev)
    return(out)
}

## ######################################################################
##
## Auxillary functions for calculating contrasts, lsmeans etc
##
## Banff, August 2013, Søren Højsgaard
##
## ######################################################################

#' @title Auxillary functions for computing lsmeans, contrasts etc
#'
#' @description Auxillary functions for computing lsmeans, contrasts etc.
#'
#' @name linest-get
#'
#' @param obj An R object
#' @param xlev FIXME: to be described
#' @param at FIXME: to be described
#' @param covariateVal FIXME: to be described
#' @param newdata FIXME: to be described

#' @export
#' @rdname linest-get
get_xlevels <- function(obj){
  UseMethod("get_xlevels")
}

#' @export
#' @rdname linest-get
get_xlevels.default <- function(obj){
  obj$xlevels
}

#' @export
#' @rdname linest-get
get_xlevels.mer <- function(obj){
  ans <- lapply(obj@frame,levels)
  ans <- ans[unlist(lapply(ans, length))>0]

  nn <- names(attr(terms(obj), "dataClasses"))
  ii <- match(names(ans), nn)
  ii <- ii[!is.na(ii)]
  ans <- ans[nn[ii]]
  ans
}

#' @export
#' @rdname linest-get
get_xlevels.merMod <- function(obj){
  ##cat("get_xlevels.lmerMod\n")
  ans <- lapply(obj@frame,levels)
  ans <- ans[unlist(lapply(ans, length))>0]

  nn <-attr(terms(obj), "term.labels")
  ii <- match(names(ans), nn)
  ii <- ii[!is.na(ii)]
  ans <- ans[nn[ii]]
  ##print(ans)
  ans
}

#' @export
#' @rdname linest-get
get_contrasts <- function(obj){
  UseMethod("get_contrasts")
}

#' @export
#' @rdname linest-get
get_contrasts.default <- function(obj){
  obj$contrasts ## FIXME: check RL code
}

#' @export
#' @rdname linest-get
get_contrasts.merMod <- function(obj){
  attr(model.matrix(obj), "contrasts")
}

#' @export
#' @rdname linest-get
set_xlevels <- function(xlev, at){
  nam     <- names(xlev)
  nn      <- match(nam, names(at))
  nn      <- nn[!is.na(nn)]
  at.fact <- at[nn]
  xlev[names(at[nn])]   <- at.fact
  attr(xlev, "at.fact") <- at.fact
  xlev
}

#' @export
#' @rdname linest-get
get_vartypes <- function(obj){
    #cat("get_vartypes:\n")
    ## Common!!
    trms <- terms( model.frame( obj ))
    trms <- delete.response( trms )
    att  <- attributes( trms )
    #cat("factors:\n"); print(att$factors)
    if (length(att$factors) > 0){
        rhs.terms <- rownames(att$factors)[rowSums(att$factors) > 0]
        rhs.class <- att$dataClass[match(rhs.terms, names(att$dataClass))]
        nums      <- rhs.terms[rhs.class == "numeric"]
        ##print(list(rhs.terms=rhs.terms, rhs.class=rhs.class, nums=nums))
        fact      <- rhs.terms[rhs.class == "factor"]
    } else {
        nums <- character(0)
        rhs.class <- character(0)
        fact <- character(0)
    }

    ## END

    ## print("here")
    ## print(list(numeric=nums, factor=fact))
    list(numeric=nums, factor=fact)
}

#' @export
#' @rdname linest-get
set_covariate_val <- function(xlev, covariateVal){
  nam     <- names(xlev)
  nn      <- match(nam, names(covariateVal))
  nn      <- nn[!is.na(nn)]
  con.con <- covariateVal[nn]
  xlev[names(covariateVal[nn])] <- con.con
  xlev
}

.get_covariate_ave <- function(obj, at=NULL, tt=terms(obj)){
    ##cat(".get_covariate_ave:\n")
    ## Common!!
    trms <- terms( model.frame( obj ))
    trms <- delete.response( trms )
    att  <- attributes( trms )
    ##ff <<- att$factors
    if (length(att$factors)>0){
        rhs.terms <- rownames(att$factors)[rowSums(att$factors)>0]
        rhs.class <- att$dataClass[match(rhs.terms, names(att$dataClass))]
        nums      <- rhs.terms[rhs.class=="numeric"]
    } else {
        nums <- character(0)
        rhs.class <- character(0)
    }
    ##print(nums)
    ## END

    ## calculate average:
    ans  <- lapply(model.frame(obj)[,nums, drop=FALSE], mean)
    ##cat("ans:\n"); print(ans)
    nn   <- match(names(ans), names(at))
    nn   <- nn[!is.na(nn)]
    at.num <- at[nn]
    ans[names(at[nn])]  <- at.num
    ##cat("ans (after insertion of 'at' values):\n"); print(ans)
    attr(ans, "at.num") <- at.num
    ans
}



#' @export
#' @rdname linest-get
get_X <- function(obj, newdata, at=NULL){
  UseMethod("get_X")
}

#' @export
#' @rdname linest-get
#' 
get_X.default <- function(obj, newdata, at=NULL){
    ##cat("get_X.default\n"); print(newdata)
    tt <- terms(obj)
    Terms  <- delete.response(tt)

    offval <- NULL
    M <- attr(terms(formula(obj)), "factors")
    idx <- grep("offset\\(", rownames(M))

    if (length(idx) > 0){
        off    <- rownames(M)[idx]
        offvar <- gsub("offset\\((.*)\\)", "\\1\\", off)
        offval <-
            if (!is.null(at) && !is.null(at[offvar]))
                at[offvar]
            else 1
        d <- data.frame(rep(offval, nrow(newdata)))
        names(d) <- offvar
        newdata <- cbind(d, newdata)
    }

    mf  <- model.frame(Terms, newdata, xlev = get_xlevels(obj))
    X   <- model.matrix(Terms, mf, contrasts.arg = get_contrasts(obj))
    attr(X, "assign" )    <- NULL
    attr(X, "contrasts" ) <- NULL
    attr(X, "offset" )    <- offval
    X
}

#' @export
#' @rdname linest-get
get_X.merMod <- function(obj, newdata, at=NULL){
   tt <- terms(obj)
   Terms  <- delete.response(tt)
   mf  <- model.frame(Terms, newdata, xlev = get_xlevels(obj))
   X   <- model.matrix(Terms, mf, contrasts.arg = get_contrasts(obj))

   attr(X,"assign")     <- NULL
   attr(X, "contrasts") <- NULL
   X
}

#' @title Model stability for glm objects
#' @description A way of assessing variability in selected models
#'     across different datasets.
#' @param data. A data frame
#' @param model A glm object
#' @param M Number of searches
#' @param method Method for generating data. Either `subgroup` like in
#'     bootstrapping or `resample` by resampling the entire datasets
#'     with replications.
#' @param mc.cores Number of cores, defaults to getOption("mc.cores", 2L)
#' @param ... Additional arguments to be passed to \code{\link{step}}
#' @note This is a recent addition. Suggestions and comments are
#'     welcome. The addition may be removed without warning.
#' @details From `data.`, `M` datasets are generated. Options are
#'
#' * "resamples": M datasets with resamples of data with replacement, i.e. bootstrap samples.
#'
#' * "subgroups": M datasets with subgroups each consisting of n-M randomly selected observations.
#'
#' On each of these datasets, a stepwise selection
#'     (using `step`) is performed.
#' @return A list with several slots of information. The most
#'     important ones:
#'
#' * `rhs_matrix` : A matrix representatation of
#'     formulas for the different models. As many rows as there are
#'     predictors, as many columns as there are different models (the
#'     same model might be selected on several datasets.)
#'
#' * `model_freq` : How many time each model is selected.
#'
#' * `pred_freq` : How many time does each predictor appear in a
#'     model.
#'
#' * `model_parm` : How many parameters are in each model.
#'
#' @examples
#'
#' set.seed(1234)
#' dat <- personality[,1:20]
#' idx <- sample(1:nrow(dat), 0.6*nrow(dat))
#' train <- dat[idx, ]
#' test <- dat[-idx, ]
#' fit <- stats::glm(agreebl ~ ., data = train)
#'
#' M <- 8
#' 
#' #stab <- doBy::model_stability_glm(data=train, model=fit, M=M, method="resample", mc.cores=1)
#' stab <- doBy::model_stability_glm(data=train, model=fit, M=M, method="subgroups", mc.cores=1)
#' stab
#'
#' formula_list <- formula(stab)
#' fit_list <- formula(stab, fit=TRUE)
#'
#' formula_list
#' stab
#'
#' (cv.train <- doBy::cv_glm_fitlist(train, fit_list, K=5))
#'
#' (cv.test <- doBy::cv_glm_fitlist(test, fit_list, K=5))
#' ylim <- range(c(cv.train, cv.test))
#' par(mfrow=c(1,2))
#' plot(stab$model_parm, cv.train, ylim=ylim)
#' plot(stab$model_parm, cv.test, ylim=ylim)

#'
#' 
#' @export
model_stability_glm <- function(data., model, M=10, method=c("resample", "subgroups"), mc.cores= getOption("mc.cores", 2L), ...){
    method <- match.arg(method)
    data_list <- generate_data_list(data., M=M, method=method)
    
    lhs <- as.character(model$formula[[2]])
        
    fit_list <- data_list |>
        parallel::mclapply(function(dat..){
            model <- update(model, data=dat..)
            step(model, ...)
        }, mc.cores=mc.cores)
    
    fit_list2 <- parallel::mclapply(
                               fit_list, function(x){
                                   update(x, data=data.)
                               }, mc.cores=mc.cores
                           )
    rhs_nms  <- get_rhs_nms(model)    
    rhs_list <- get_predictor_list(fit_list2)
    rhs_matrix <- set_list2matrix(rhs_list, aggregate=TRUE)
    rhs_raw    <- set_list2matrix(rhs_list, aggregate=FALSE)    
    rhs_matrix <- as.data.frame(rhs_matrix)
    freq <- rhs_matrix$Freq__
    rhs_matrix$Freq__ <- NULL
    
    rhs_matrix = as(t(rhs_matrix), "dgCMatrix")
    rhs_raw = as(t(rhs_raw), "dgCMatrix")
    
    loc <- get_location(rhs_raw, rhs_matrix)    
    
    out <- list(
        call = match.call(),
        model = model,
        data. = data.,
        data_list = data_list,
        formula = formula(model),
        fit = fit_list2,        
        rhs_raw = rhs_raw,
        M = M,
        loc = loc,
        lhs = lhs,
        ##aic = sapply(fit_list2, function(x)AIC(x)),
        pred_freq = unname(rowSums(rhs_matrix)),
        model_parm = colSums(rhs_matrix),
        rhs_matrix = rhs_matrix,
        model_freq = freq)
    
    class(out) <- "model_stability_glm_class"
    out
}


#' @export
print.model_stability_glm_class <- function(x, ...){
  x$data. <- NULL
  print.default(x[c("rhs_matrix", "model_freq", "pred_freq", "model_parm")])    
}

#' @export
formula.model_stability_glm_class <- function(x, fit=FALSE, unique=TRUE, text=FALSE, ...){

    if (fit){
        text <- FALSE
    }
    
    f <- get_formulas(x, unique=unique, text=text)
    if (!fit){
        f
    } else {
        cl <- x$model$call
        lapply(f, function(z) {
            cl$formula <- z
            out <- update(eval(cl), data=x$data.)
            out
        }
        )
    }
}


#' @export
summary.model_stability_glm_class <- function(object, ...){

    freq_pred <- object$rhs_raw |> rowSums()
    num_pred <- object$rhs_matrix |> colSums()

    out <- list(number_of_models=ncol(object$rhs_raw),
                number_of_unique_models=ncol(object$rhs_matrix),                
                number_of_predictors=num_pred, 
                frequency_of_predictors=freq_pred)
    
    class(out) <- "model_stability_glm_summary_class"
    out
}



#' @export
print.model_stability_glm_summary_class <- function(x, ...){
    print.default(unclass(x))
}

#' @title Get formulas from model_stability_glm_class object
#' @param object A model_stability_glm_class object
#' @param unique If TRUE, return unique models
#' @param text If TRUE, return text (rather than formula).
#' @note This is a recent addition. Suggestions and comments are
#'     welcome. The addition may be removed without warning.
#' 
#' @export
get_formulas <- function(object, unique=TRUE, text=FALSE){

    if (unique){
        mmm <- object$rhs_matrix        
    } else {
        mmm <- object$rhs_raw        
    }

    
    rhs2formula <- function(object, i, text=FALSE) {
        out <- paste0(object$lhs, "~", paste0(names(which(mmm[,i]==1)), collapse="+"))
        if (!text)
            as.formula(out)
        else
            out
    }

    out <- lapply(1:ncol(mmm),
                  function(i){
                      rhs2formula(object, i, text=text)
                  })
    return(out)
}





#' @title Cross-validation for list of glm objects
#' 
#' @param data. A data frame
#' @param fit_list A list of glm objects
#' @param K Number of folds in cross validation.
#'
#' @details A wrapper for calling boot::cv.glm for each model in
#'     fit_list.
#' @importFrom boot cv.glm
#' @export
cv_glm_fitlist <- function(data., fit_list, K=10){
    fit_list <-
        lapply(fit_list,
               function(fit){
                   if(!inherits(fit, "glm"))
                       stop("fit must be an glm object")
                   update(fit, data=data.)
               })

    cv.error <-
        sapply(fit_list,
               function(fit){
                   boot::cv.glm(data., fit, K=K)$delta[1] 
               })
    
    return(cv.error)    
}




get_rhs_nms <- function(model){
  nms <- names(eval(model$call$data))
  lhs <- as.character(formula(model)[[2]])
  rhs_nms <- setdiff(nms, lhs)
  rhs_nms      
}

get_location <- function(rhs_raw, rhs_matrix){
    u <- apply(rhs_raw, 1, paste0, collapse='')
    z <- apply(rhs_matrix, 1, paste0, collapse='')

    loc <- sapply(u, function(i){
        (i == z)  |> which()
    })
    names(loc) <- NULL
    loc
}


get_predictor_list <- function(x){
  rhs_list <-
    lapply(x,
           function(s){
             s |> terms() |> attr("term.labels")  
           })
  rhs_list
}


get_rhs_matrix <- function(x, aggregate=FALSE){
  if (!inherits(x, "model_stability_glm_class"))
    stop("'x' is not model_stability_glm_class\n")
  rhs_list <- get_predictor_list(x$fit)
  rhs_raw <- set_list2matrix(rhs_list, aggregate=aggregate)
  t(rhs_raw)
}


#' @title Matrix representatation of list of vectors and vice versa
#' @param set_list list of vectors
#' @param aggregate should the vectors be aggregated
#' @param set_matrix matrix representatation
#' @note This is a recent addition. Suggestions and comments are
#'     welcome. The addition may be removed without warning.
#' 
#' @name set_list_set_matrix
#'
#' @examples
#' l <- list(c(1,2,3), c(3,2,4), c(3,2,4))
#' m1 <- set_list2matrix(l)
#' m1
#' matrix2set_list(m1)
#' 
#' m2 <- set_list2matrix(l, aggregate=TRUE)
#' m2
#' matrix2set_list(m2)
#' 
#' 
#' @export
set_list2matrix <- function(set_list, aggregate=FALSE){  
    aggregate_matrix <- function(mat){
        
        if (ncol(mat) > 0){
            mat <- mat |> as.data.frame()  |> table() |> as.data.frame.table()
            mat <- dplyr::filter(mat, .data$Freq>0)
            mat <- mat[order(mat$Freq, decreasing=TRUE),]
            mat <- sapply(mat, 
                          function(o){
                              as.numeric(as.character(o))
                          })
            
            if (is.null(dim(mat)))
                mat <- as.data.frame(t(mat))
            else
                mat
        } else { 
            as.data.frame(cbind(Freq=nrow(mat)))        
        }
    }
    
    ## Get unique terms across all lists
    unique_terms <- unique(unlist(set_list))
  
    ## Create an empty matrix M with 3 rows and columns equal to the number of unique terms
    M <- matrix(0, nrow = length(set_list), ncol = length(unique_terms), 
                dimnames = list(NULL, unique_terms))
    
    ## Iterate over each list and mark the presence of terms in the matrix
    for (i in seq_along(set_list)) {
        ##print(set_list[[i]])
        M[i, set_list[[i]]] <- 1
    }

    if (aggregate) {
        M2 <- aggregate_matrix(M)
        colnames(M2) <- c(colnames(M), "Freq__")
        return(M2)
    }
    else
        return(M)
}


#' @rdname set_list_set_matrix
#' @param set_matrix matrix representatation
#' @export
matrix2set_list<- function(set_matrix){
    has_freq <- "Freq__" %in% colnames(set_matrix)
    if (has_freq){
        set_matrix <- exclude_column_by_name(repeat_rows(set_matrix, set_matrix[,"Freq__"]), "Freq__")
    }
      nr <- nrow(set_matrix)
    out <- vector("list", nr)
    nms <- colnames(set_matrix)
    nms <- if(is.null(nms)) 1:ncol(set_matrix) else nms
    ##print(out); cat("nms:\n"); print(nms)
    
    for (i in 1:nrow(set_matrix)){
        ri <- set_matrix[i,]
      ##  print(ri)
        idx <- which(abs(ri)> 1e-5)
        ##cat("idx:\n");print(idx);print(nms[idx])
        out[[i]] <- nms[idx]
    }
    return(out)
}

repeat_rows <- function(M, v) {
  # Check that the length of v matches the number of rows in M
  if (length(v) != nrow(M)) {
    stop("The length of vector v must match the number of rows in matrix M.")
  }
  
  # Create an empty list to store the repeated rows
  repeated_rows <- vector("list", length = sum(v))
  
  # Fill the list with repeated rows
  index <- 1
  for (i in seq_along(v)) {
    for (j in seq_len(v[i])) {
      repeated_rows[[index]] <- M[i, , drop = FALSE]
      index <- index + 1
    }
  }
  
  # Combine the list into a new matrix
  new_matrix <- do.call(rbind, repeated_rows)
  return(new_matrix)
}

exclude_column_by_name <- function(M, colname) {
  # Check if the column name exists in the matrix
  col_index <- which(colnames(M) == colname)
  
  if (length(col_index) == 0) {
    stop(paste("Column", colname, "does not exist in the matrix."))
  }
  
  # Exclude the column by name
  M <- M[, -col_index, drop = FALSE]
  
  return(M)
}

## modelr::crossv_kfold(dat, k=4)[[1]][[1]] |> as.data.frame()

#' @title Generate data list
#' @param data. A data frame
#' @param M Number of folds
#' @param method Method for generating data
#' @details There are n rows in data. The function returns a list of M
#'     datasets. If method="subgroups" each dataset in the list has
#'     (approximately) n-M randomly selected rows from data. If
#'     method="resample" each dataset has n rows obtained by sampling
#'     with replacement from the original data. The function is
#'     heavily inspired by the boot::cv.glm function.
#' @note This is a recent addition. Suggestions and comments are
#'     welcome. The addition may be removed without warning.
#' 
#'
#' @examples
#' dat <- cars[1:15,]
#' set.seed(1411)
#' dl1 <- generate_data_list(dat, 4)
#' dl1[[1]]
#' dl1 |> sapply(nrow)
#' dl2 <- generate_data_list(dat, 4, method="resample")
#' dl2 |> sapply(nrow)
#' 
#' @export
generate_data_list <- function(data., M, method=c("subgroups", "resample")){
  
  method <- match.arg(method)
  
  sample0 <- function (x, ...) {
    x[sample.int(length(x), ...)]
  }
  
  out <- as.list(rep(NA, M))
  
  switch (method,
          "subgroups" = {
            n <- nrow(data.)
            f <- ceiling(n/M)
            s <- sample0(rep(1L:M, f), n)
            ms <- max(s)
            ##str(list(n=n, f=f, s=s, ms=ms))

            for (i in seq_len(ms)) {
              j.in <- seq_len(n)[(s != i)]
              di <- data.[j.in, , drop = FALSE]
              out[[i]] <- di
            }
          },
          "resample" = {
            for (i in 1:M){
              j.in <- sample(nrow(data.), replace=TRUE) |> sort()
              di <- data.[j.in, , drop = FALSE]
              out[[i]] <- di              
            }
          }
  )
  out
}

#' @title Find sub-sequences of identical elements in a vector. 
#' @description Find sub-sequences of identical elements in a vector.
#' @name sub_seq
#' 
#' @param x An atomic vector or a factor.
#' @param item Optionally a specific value to look for in `x`.
#' @return A dataframe.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{rle}}
#'
#' 
#' @keywords utilities
#' @examples
#' 
#' x <- c(1, 1, 1, 0, 0, 1, 1, 1, 2, 2, 2, 1, 2, 2, 2, 3)
#' sub_seq(x)
#' sub_seq(x, item=1)


#' @rdname sub_seq
#' @export 
sub_seq <- function (x, item = NULL) {
    rrr <- rle(as.character(x))
    len <- rrr$lengths
    val <- rrr$values

    first <- last <- rep.int(NA, length(val))
    first[1] <- 1
    last [1] <- len[1]
    if (length(val) > 1) {
	for (kk in 2:length(val)) {
            first[kk] <- last[kk - 1] + 1
            last [kk] <- last[kk - 1] + len[kk]
	}
    }
    midp <- floor(first + len / 2)

    ans <- cbind(first=first, last=last, slength=len, midpoint=midp)

    if (!is.null(item)) {
        iii <- val == item
        ans <- as.data.frame.matrix(ans[iii,, drop=FALSE], stringsAsFactors=FALSE)
        ans$value <- val[iii]
    } else {
        ans <- as.data.frame.matrix(ans, stringsAsFactors=FALSE)
        ans$value <- val
    }
    ans
}

#' @rdname sub_seq
#' @export
subSeq <- sub_seq



## #' @rdname sub_seq
## #' @export
## is_grouped <- function(x) {
##     !any(table(rle2(x)$values) > 1)    
## }

## #' @rdname sub_seq
## #' @export
## rle2 <- function(x) {
##     if (is.factor(x)) {        
##         rle(as.character(x))
##     } else {
##         rle(x)
##     }
## }



#'
#' Calculate "time since event" in a vector
#'
#' Calculates the time since the nearest event in a sequence,
#' optionally using a custom time scale.
#'
#' Events are coded as 1 (or TRUE). Non-events are anything else. The
#' result includes absolute and signed distances to events.
#'
#' @param yvar A numeric or logical vector indicating events.
#' @param tvar An optional numeric vector specifying time values. Defaults to the index.
#' @return A data frame with columns 'yvar', 'tvar', 'abs.tse' (absolute time since event), 
#'         'sign.tse' (signed time since event), and other helper columns.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{subSeq}}, \code{\link{rle}}
#' @examples
#' 
#' ## Example 1: Basic usage with default time index
#' y <- c(0, 0, 1, 0, 0, 1, 0)
#' tse <- time_since_event(y)
#' print(tse)
#'
#' ## Example 2: Custom (non-integer) time variable
#' y <- c(0, 0, 1, 0, 0, 0, 1, 0)
#' t <- seq(0.5, 3.5, length.out = length(y))
#' tse <- time_since_event(y, t)
#' print(tse)
#'
#' ## Example 3: Plotting the signed time since event
#' plot(sign.tse ~ tvar, data = tse, type = "b",
#'      main = "Signed time since event",
#'      xlab = "Time", ylab = "Signed time since event")
#' grid()
#' abline(h = 0, col = "red", lty = 2)
#' 
#' @export 
time_since_event <- function(yvar, tvar=seq_along(yvar)) {

    if (!(is.numeric(yvar) | is.logical(yvar))) {
        stop("yvar must be either numeric or logical")
    }
    
    yvar[is.na(yvar)] <- 0
    
    event.idx <- which(yvar == 1)
    
    if (length(event.idx) == 0) {
        return(NULL)
    }
    
    n.event <- length(event.idx)

    ## find event times
    event.time <- tvar[event.idx]

    ## get time difference to each event
    rrr <-  do.call(rbind, lapply(event.idx, function(ii) tvar-tvar[ii]))
    abs.tse <- apply(abs(rrr), 2, min)


    ## get the event windows (~ symmetrical around event time)
    ewin<-rep.int(NA, length(yvar))
    if (n.event > 1) {
        ff <- event.time[1:(n.event-1)] + diff(event.time) / 2
        ewin[tvar <= ff[1]] <- 1
        for (ii in 2:(length(ff) - 0)){
            ewin[tvar > ff[ii - 1] & tvar <= ff[ii] ] <- ii
        }
        ewin[tvar > ff[length(ff)]] <- n.event
    } else {
        ewin[] <- n.event
    }

  ## get the signs
    ggg <- list()
    for (ii in 1:(length(event.idx))){
        ggg[[ii]] <- rrr[ii, ewin == ii]
    }
    ggg <- unlist(ggg)
    sign.tse <- sign(ggg) * abs.tse

    run <- cumsum(yvar)

    un <- unique(run)
    tlist <- list()
    for (ii in 1:length(un)){
        vv <- un[ii]
        yy <- yvar[run == vv]
        tt <- tvar[run == vv]
        tt <- tt - tt[1]
        tlist[[ii]] <- tt
    }
    tae <- unlist(tlist)
    tae[run == 0] <- NA

    yvar2 <- rev(yvar)
    tvar2 <- rev(tvar)

    run2 <- cumsum(yvar2)
    un2 <- unique(run2)
    tlist2 <- list()
    for (ii in 1:length(un2)){
        vv <- un2[ii]
        yy <- yvar2[run2 == vv]
        tt <- tvar2[run2 == vv]
        tt <- tt - tt[1]
        tlist2[[ii]] <- tt
  }
    tbe <- unlist(tlist2)
    tbe[run2==0] <- NA

    tbe <- rev(tbe)
    run[run==0]<-NA
    
    ans <- cbind(data.frame(yvar=yvar, tvar=tvar), abs.tse, sign.tse, ewin=ewin,
                 run, tae=tae, tbe=tbe)
    ans
}






## yvar, tvar=seq_along(yvar)

#' @rdname time_since_event
#' @param ... Arguments pased on to time_since_event
#' @export
timeSinceEvent <- function(...) {
    .Deprecated("time_since_event")
    time_since_event(...)
}

#' @title Scale numeric variables in a data frame
#'
#' @description Applies `base::scale()` to numeric, integer, or
#'     logical columns in a data frame. Non-numeric columns are left
#'     unchanged.
#' @concept data_handling
#' @param x A data frame or matrix.
#' @param center Logical; if TRUE, center the variables.
#' @param scale Logical; if TRUE, scale the variables.
#'
#' @details
#' If `x` is not a data frame, `base::scale()` is applied directly.
#'
#' @return An object of the same class as `x`.
#'
#' @examples
#' scale_df(iris)
#'
#' @name scale_df
#' @export
scale_df <- function(x, center = TRUE, scale = TRUE){

    if (!is(x, "data.frame")){
        return(scale(x, center=center, scale=scale))
    } else { ## x is dataframe
        
        b <- sapply(x,
                    function(z){is(z, c("numeric")) || is(z, c("integer")) || is(z, c("logical")) })
        
        if (!any(b)){ ## x only has numeric values
            return(scale(x, center=center, scale=scale))
        } else { ## x is dataframe with non-numerics
            
            x2 <- x[,b, drop=FALSE]
            x2 <- scale(x2, center=center, scale=scale)
            x[, b] <- x2
            
            if (!is.null(a <- attributes(x2)$"scaled:center"))
                attr(x, "scaled:center") <- a
            
            if (!is.null(a <- attributes(x2)$"scaled:scale"))
                attr(x, "scaled:scale") <- a
            
            return(x)            
        }            
    }
}

###############################################################################
#'
#' @title Function to make groupwise transformations
#' @description Function to make groupwise transformations of data by applying
#'     the transform function to subsets of data.
#' @name by-transform
#' @concept byfunction
###############################################################################
#'
#' @details The ... arguments are tagged vector expressions, which are evaluated
#'     in the data frame data. The tags are matched against names(data), and for
#'     those that match, the value replace the corresponding variable in data,
#'     and the others are appended to data.
#' 
#' @param formula A formula with only a right hand side, see examples below
#' @param data A data frame
#' @param \dots Further arguments of the form tag=value
#' @return The modified value of the dataframe data.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' 
#' @seealso \code{\link{orderBy}}, \code{\link{order_by}}, \code{\link{summaryBy}}, \code{\link{summary_by}},
#'     \code{\link{splitBy}}, \code{\link{split_by}}
#' @keywords univar
#' @examples
#'  
#' data(dietox)
#' transformBy(~Pig, data=dietox, minW=min(Weight), maxW=max(Weight), 
#'     gain=diff(range(Weight)))
#' 

#' @export 
#' @rdname by-transform
transform_by <- function (data, formula, ...) {
    cl <- match.call(expand.dots = TRUE)
    cl[[2]] <- formula
    cl[[3]] <- data
    names(cl)[2:3] <- c("formula", "data")
    cl[[1]] <- as.name("transformBy")
    eval(cl)
}

#' @export
#' @rdname by-transform 
transformBy <- function (formula, data, ...) {

    if (!inherits(data, "tbl_df")) is.tib = FALSE
    else {is.tib = TRUE; data = as.data.frame(data)}
    
    transform2<- function (data, ...) 
    {
        e <- eval(substitute(list(...)), data, parent.frame())
        tags <- names(e)
        inx <- match(tags, names(data))
        matched <- !is.na(inx)
        if (any(matched)) {
            data[inx[matched]] <- e[matched]
            data <- data.frame(data)
        }
        if (!all(matched)) {
            for (i in 1:length(e))
                data[, names(e)[i]] <- e[i]
        }
        return(data)
    }
    
    ddd <- splitBy(formula, data=data, omit=TRUE)
    ee <- lapply(ddd, function(d){
        transform2(d, ...)
    })
    
    out  <- do.call("rbind", ee)
    rownames(out) <- NULL

    if (is.tib) as_tibble(out) else out
    ##out
}

#' Construct lagged regressors and aligned response
#'
#' @description
#' Helper for building lagged design matrices for time series or
#' longitudinal data. The function aligns a response vector `y`
#' and a single regressor `x` (possibly with several lags) and
#' returns a list with the cleaned response, the design matrix and
#' the row indices used.
#'
#' You can either:
#' * supply `y` and `x` directly, or
#' * use a formula of the form `y ~ x` together with `data`.
#'
#' @param formula An optional model formula. The left-hand side is
#'   taken as the response and the right-hand side as the regressor(s).
#'   Only one non-intercept regressor is currently allowed when
#'   lagging is used.
#' @param data Optional data frame in which to evaluate `formula`.
#' @param y Optional numeric or time series object with the response.
#'   Ignored if `formula` is supplied.
#' @param x Optional numeric vector with the regressor to be lagged.
#'   Ignored if `formula` is supplied. If `x` is not `NULL`, it must
#'   have the same length as `y` (when `y` is supplied).
#' @param lags Integer vector of lags to include. The default `lags = 0`
#'   means that only the contemporaneous regressor is used. For example,
#'   `lags = 0:2` gives columns corresponding to (x(t), x(t-1), x(t-2)).
#' @param include_intercept Logical; if `TRUE` (default) an intercept
#'   column named `"Intercept"` is prepended to the design matrix.
#' @param preserve_ts Logical; if `TRUE` and `y` is a `ts` object, the
#'   returned `y` will also be a `ts` object obtained via
#'   \code{\link[stats]{window}} starting at the first retained time
#'   point. Otherwise, `y` is returned as a numeric vector.
#'
#' @details
#' When using the formula interface, the function creates a model frame
#' internally. The first column is taken as the response. All remaining
#' columns are considered regressors. If the right-hand side contains
#' only an intercept, `x` is set to `NULL` and no lagged regressors are
#' constructed.
#'
#' If multiple regressors are present on the right-hand side, they are
#' combined into a matrix, but in the current implementation this is
#' only allowed when no lagging is requested (i.e., `lags` is `0`).
#' Attempting to lag multiple regressors will result in an error.
#'
#' For a given vector of lags, a lag matrix is built with one column
#' per lag. The column names are `"lag0"`, `"lag1"`, etc., corresponding
#' to the entries in `lags`. Rows which involve undefined values (due to
#' lagging) are dropped from both `y` and `X`. The indices of the rows
#' kept from the original series are returned in the `rows` component.
#'
#' @return
#' A list with components
#' \describe{
#'   \item{y}{The aligned response vector. If `preserve_ts = TRUE`
#'     and `y` was a `ts` object, then this is a `ts` object; otherwise
#'     a numeric vector.}
#'   \item{X}{The design matrix of lagged regressors, including an
#'     intercept column if `include_intercept = TRUE`. If `x` is `NULL`,
#'     then `X` is `NULL`.}
#'   \item{rows}{Integer vector of row indices from the original data
#'     that are retained after lagging.}
#'   \item{max_lag}{The maximum lag used, i.e. \code{max(lags)}.}
#' }
#'
#' @seealso
#' \code{\link[stats]{lag}}, \code{\link[stats]{embed}},
#' and \code{\link[stats]{ts}} for related time series utilities.
#'
#' @examples
#' ## ------------------------------------------------------------
#' ## 1. Basic usage with y/x interface
#' ## ------------------------------------------------------------
#' set.seed(123)
#' n  <- 10
#' y  <- rnorm(n)
#' x  <- rnorm(n)
#'
#' ## Use current and one lag of x
#' ld1 <- lag_data(y = y, x = x, lags = 0:1)
#' ld1$y                # aligned response
#' ld1$X                # design matrix (Intercept, lag0, lag1)
#' ld1$rows             # indices retained
#'
#'
#' ## ------------------------------------------------------------
#' ## 2. Formula interface with a data frame
#' ## ------------------------------------------------------------
#' dat <- data.frame(
#'   y = rnorm(20),
#'   x = rnorm(20)
#' )
#'
#' ## Use y ~ x with one lag
#' ld2 <- lag_data(y ~ x, data = dat, lags = 0:2)
#' head(ld2$X)
#' length(ld2$y)
#'
#'
#' ## ------------------------------------------------------------
#' ## 3. Using the result in a regression / ARX setting
#' ## ------------------------------------------------------------
#' ## Here we regress y_t on an intercept and lagged x's
#' ld3 <- lag_data(y = y, x = x, lags = 0:2)
#'
#' ## Remove intercept column when passing to lm()
#' fit_lm <- lm(ld3$y ~ ld3$X[, -1])
#' coef(fit_lm)
#'
#'
#' ## ------------------------------------------------------------
#' ## 4. Time series example with preserve_ts = TRUE
#' ## ------------------------------------------------------------
#' y_ts <- Nile
#'
#' ## Regress Nile flow on its own lag-1 (simple ARX-like setup)
#' ld4 <- lag_data(y = y_ts, x = y_ts, lags = 1, preserve_ts = TRUE)
#'
#' ## y is now a ts object starting at the appropriate time
#' start(y_ts)
#' start(ld4$y)
#'
#' ## Fit an AR(1) model with lagged Nile as xreg
#' ## (intercept in X, lag in the second column)
#' fit_arx <- stats::arima(ld4$y,
#'                         order = c(1, 0, 0),
#'                         xreg  = ld4$X[, -1, drop = FALSE])
#' fit_arx
#'
#' @export
lag_data <- function(formula = NULL,
                          data = NULL,
                          y = NULL,
                          x = NULL,
                          lags = 0,
                          include_intercept = TRUE,
                          preserve_ts = FALSE) {

  # --- Handle formula interface ---
  if (!is.null(formula)) {

    # Get model frame
    mf <- model.frame(formula, data = data)

    # Extract y and RHS variables
    y <- mf[[1]]
    Xvars <- mf[-1]

    # If only intercept on RHS: no xreg
    if (ncol(Xvars) == 0 || all(sapply(Xvars, function(v) all(v == 1)))) {
      x <- NULL
    } else {
      # If multiple regressors: combine into a single matrix
      x <- as.matrix(Xvars)
      if (ncol(x) > 1 && !is.null(lags) && any(lags != 0)) {
        stop("Lagging multiple regressors not implemented in this version.")
      }
      # For now: require only 1 regressor for lagging
      if (ncol(x) > 1) stop("Only one RHS variable allowed when using lags.")
      x <- drop(x)  # ensure a vector
    }
  }

  # --- Now fallback to previous behaviour if formula NULL ---
  if (is.null(y) && is.null(x)) {
    stop("Need either formula=..., or y=..., or x=...")
  }

  # -----------------------------------------------------------
  # --- Build X matrix (single regressor) ----------------------
  # -----------------------------------------------------------
  if (!is.null(x)) {
    x <- as.numeric(x)
    n_x <- length(x)
    max_lag <- max(lags)

    # Build full lag matrix
    lagmat <- sapply(lags, function(k) {
      if (k == 0L) x else c(rep(NA, k), x[1:(n_x - k)])
    })
    colnames(lagmat) <- paste0("lag", lags)

    # Add intercept if requested
    if (include_intercept) {
      lagmat <- cbind(Intercept = 1, lagmat)
    }

    valid_idx <- (max_lag + 1):n_x
    X_clean <- lagmat[valid_idx, , drop = FALSE]

  } else {
    X_clean  <- NULL
    max_lag  <- 0
    valid_idx <- NULL
  }

  # -----------------------------------------------------------
  # --- Align y -----------------------------------------------
  # -----------------------------------------------------------
  if (!is.null(y)) {

    n_y <- length(y)

    if (max_lag >= n_y) {
      stop("max_lag too large: no y left after truncation.")
    }

    if (!is.null(x)) {
      row_idx <- valid_idx
    } else {
      row_idx <- seq_len(n_y)  # no lagging needed
    }

    if (preserve_ts && is.ts(y)) {
      t_start <- time(y)[row_idx[1]]
      y_clean <- window(y, start = t_start)
    } else {
      y_clean <- as.numeric(y)[row_idx]
    }

  } else {
    y_clean <- NULL
    row_idx <- valid_idx
  }

  # -----------------------------------------------------------
  # --- Return result -----------------------------------------
  # -----------------------------------------------------------
  list(
    y       = y_clean,
    X       = X_clean,
    rows    = row_idx,
    max_lag = max_lag
  )
}

###########################################################################
#'
#' @title Split a data frame into groups defined by variable(s)
#' 
#' @description Split a dataframe according to the levels of variables
#'     in the dataframe. Uses vparse() to interpret flexible input.
#' @name by-split
#' @concept byfunction
#' 
#' @param data. A data frame (or tibble) to split
#' @param ... Variables defining the groups
#' @param omit If TRUE (default), group-defining variables are omitted in each split group
#' @param formula A right hand sided formula (for the old interface)
#' @param data A data frame  (for the old interface)
#'
#' @return An object of class \"splitByData\" (a named list with group attributes)
#' @export
#' 
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{orderBy}}, \code{\link{order_by}},
#'     \code{\link{summaryBy}}, \code{\link{summary_by}},
#'     \code{\link{transformBy}}, \code{\link{transform_by}} 
#' @keywords utilities
#' @examples
#'
#' split_by(CO2, ~Treatment+Type)
#' split_by(CO2, Treatment, Type)
#' split_by(CO2, c("Treatment", "Type"))
#' split_by(CO2, "Treatment", "Type")
#'
#' x <- split_by(CO2, "Treatment", "Type")
#' head(x, 3)
#' tail(x, 3)
#'
#' ## Via wrapper:
#' foo2 <- function(x) {
#'  x <- rlang::enquo(x)
#'  split_by(CO2, !!x)
#' }
#' foo2(~Treatment)
#'
#' ## The "Old" interface
#' splitBy(~Treatment + Type, CO2)
#' splitBy(~Treatment + Type, data=CO2)
#' splitBy(c("Treatment", "Type"), data=CO2)
#'
#' @rdname by-split
#' @export 
split_by <- function(data., ..., omit = TRUE) {
  rhs <- vparse(...)
  split_by_worker(data., rhs, omit = omit)
}



#' @export 
#' @rdname by-split
splitBy <- function(formula, data, omit=TRUE){
    if (inherits(formula, "formula")){
        rhs       <- all.vars(formula[[2]])
    } else {
        rhs <- formula
    }
    split_by_worker(data, rhs, omit=omit)
}

split_by_worker <- function(data., rhs, omit=TRUE){ 

    is.tib <- inherits(data., "tbl_df")
    if (is.tib)
        data. = as.data.frame(data.)

    grps <- apply(data.[, rhs, drop=FALSE], 1, paste0, collapse="|")

    ## print(grps)
    out <- split(data., grps)
    
    if (omit){
        rhs.idx <- match(rhs, names(data.))
        out <- lapply(out, function(d){
            d[, -rhs.idx]            
        })
    }
        
    if (is.tib) {
        out <- lapply(out, function(d) {
            as_tibble(d)
        })
    }
        
    groupid <- unique(data.[, rhs, drop=FALSE])
    idxvec <- split(1:nrow(data.), grps)
    
    attr(out, "groupid") <- groupid
    attr(out, "idxvec")  <- idxvec
    attr(out, "grps")    <- grps

    class(out) <- c("splitByData", "list")    
    return(out)
}


#' @export
print.splitByData <- function(x, ...){
#  print(attr(x,"groupid"))
    print(cbind(listentry=names(x), attr(x,"groupid")))
    return(invisible(x))
}

#' @importFrom utils head
#' @param x A splitByData object.
#' @param n An integer vector.
#' @export
#' @rdname by-split
head.splitByData  <- function(x, n=6L, ...){
    lapply(x, head, n=n, ...)
}

#' @importFrom utils tail
#' @export
#' @rdname by-split
tail.splitByData  <- function(x, n=6L, ...){
    lapply(x, tail, n=n, ...)
}





#' @export 
#' @rdname by-split
#' @param drop Obsolete
split_by.legacy <- function(data, formula, drop=TRUE){
    arg <- list(formula=formula, data=data, drop=drop)
    do.call(splitBy.legacy, arg)
}

#' @export 
#' @rdname by-split
splitBy.legacy <-function (formula, data = parent.frame(), drop=TRUE) {
                                        #, return.matrix=FALSE){

    if (!inherits(data, "tbl_df")) is.tib = FALSE
    else {is.tib = TRUE; data = as.data.frame(data)}
    
    data.var  <- names(data)
    
    if (!(inherits(formula, c("formula", "character"))))        
        stop("'formula' must be a right hand sided formula or a character vector")
    
    if (inherits(formula, "formula")){
        rhs       <- formula[[2]]
        rhs.var   <- all.vars(rhs)
    } else {
        rhs.var <- formula
    }
    
    cls <- lapply(data, class)
    factor.columns <- which(unlist(lapply(cls, function(x) any( x %in% "factor"))))
                                        #print(factor.columns)

    cls       <- lapply(data, class)
    num.idx   <- cls %in% c("numeric","integer")
    fac.var   <- data.var[ !num.idx ]
    
    rhs.fac   <- intersect( rhs.var, data.var )

    if ("." %in% rhs.var){
        ## need all factors not mentioned elsewhere as grouping factors
        rhs.fac <- union( fac.var, rhs.fac)
    }

    
    ## str(list(rhs.var=rhs.var, rhs.fac=rhs.fac, fac.var=fac.var))
    
    rh.trivial <- length( rhs.var ) == 0 
    
    ## FIXME rhs.fac, rhs.var -- clean up!!!
    ## Use: data, rhs.fac, rh.trivial
    if ( rh.trivial ){
        grps <- rep.int(1, nrow(data))
        unique.grps <- 1
        rh.idx    <- 1
    } else {
        grps <- .get_rhs_string( data, rhs.fac, sep.string="|")
        unique.grps <- unique(grps)
        rh.idx    <- match(unique.grps, grps)
    }
    
    out_list <- vector("list", length(unique.grps))

    names(out_list) <- unique.grps

    for (ii in 1:length(unique.grps)){
        dd <- data[unique.grps[ ii ] == grps, ,drop=FALSE]
        if (drop && length(factor.columns)>0){
            for (jj in 1:length(factor.columns)){
                dd[, factor.columns[jj]] <- factor(dd[, factor.columns[jj]])
            }
        }
        if (is.tib) dd <- as_tibble(dd)
        out_list[[ ii ]] <- dd
    }
    
    idxvec <- vector("list", length(unique.grps))
    names(idxvec) <- unique.grps
    for (ii in 1:length(unique.grps)){
        idxvec[[ii]] <- which(grps == unique.grps[ii])
    }
    
    groupid <- data[rh.idx, rhs.fac, drop=FALSE]
    rownames(groupid) <- 1:nrow(groupid)
    
    attr(out_list,"groupid") <- groupid
    attr(out_list,"idxvec")  <- idxvec
    attr(out_list,"grps")    <- grps
    
    class(out_list) <- c("splitByData", "list")    
    out_list
}

test_that("split_by works with different input types", {
  data(CO2)

  # unquoted
  out1 <- split_by(CO2, Treatment)
  expect_s3_class(out1, "splitByData")
  expect_true(all(sapply(out1, function(df) !("Treatment" %in% names(df)))))

  # formula
  out2 <- split_by(CO2, ~Treatment + Type)
  expect_equal(length(out2), length(unique(paste(CO2$Treatment, CO2$Type))))

  # character vector
  out3 <- split_by(CO2, c("Treatment", "Type"))
  expect_equal(length(out3), length(out2))

  # do not omit group vars
  out4 <- split_by(CO2, Treatment, omit = FALSE)
  expect_true("Treatment" %in% names(out4[[1]]))
})

###############################################################################
#'
#' @title Finds subsets of a dataframe which is split by variables in
#'     a formula.
#' @description A data frame is split by a formula into groups. Then
#'     subsets are found within each group, and the result is
#'     collected into a data frame.
#' @name by-subset
#' @concept byfunction
###############################################################################
#' @param formula A right hand sided formula or a character vector of
#'     variables to split by.
#' @param subset logical expression indicating elements or rows to
#'     keep: missing values are taken as false.
#' @param data A data frame.
#' @param select expression, indicating columns to select from a data
#'     frame.
#' @param drop passed on to \code{[} indexing operator.
#' @param join If FALSE the result is a list of data frames (as
#'     defined by 'formula'); if TRUE one data frame is returned.
#' @param \dots further arguments to be passed to or from other methods.
#' @return A data frame.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{splitBy}}, \code{\link{split_by}}
#' @keywords utilities
#' @examples
#' 
#' data(dietox)
#' subsetBy(~Evit, Weight < mean(Weight), data=dietox)


#' @export
#' @rdname by-subset
subset_by <- function(data, formula, subset, select, drop=FALSE, join=TRUE, ...){
    cl <- match.call(expand.dots = TRUE)
    cl[[2]] <- formula
    cl[[3]] <- cl[[4]] # writing 'subset' fails because evaluation is tried
    cl[[4]] <- data

    names(cl)[2:4] <- c("formula", "subset", "data")
    cl[[1]] <- as.name("subsetBy")
    eval(cl)
}


#' @export
#' @rdname by-subset
subsetBy <- function(formula, subset, data=parent.frame(), select, drop=FALSE, join=TRUE, ...){

    out <- splitBy(formula, data=data)
    subsetMissing <- missing(subset)
    selectMissing <- missing(select)  
    e <- substitute(subset)
    out <- lapply(out, 
                  function(x){
                      if (subsetMissing) 
                          r <- TRUE
                      else {
                          r <- eval(e, x, parent.frame())
                          if (!is.logical(r)) 
                              stop("'subset' must evaluate to logical")
                          r <- r & !is.na(r)
                      }
                      if (selectMissing) 
                          vars <- TRUE
                      else {
                          nl <- as.list(1:ncol(x))
                          names(nl) <- names(x)
                          vars <- eval(substitute(select), nl, parent.frame())
                      }
                      x[r, vars, drop = drop]
                  }
                  )
    if (join) do.call("rbind", out) else out
}

#' @title Fast summary of microbenchmark object
#'
#' @description Fast summary of microbenchmark object. The default
#'     summary method from the microbenchmark package is fairly slow
#'     in producing a summary (due to a call to a function from the
#'     multcomp package.)
#' 
#' @name mb_summary
#'
#' @param object A microbenchmark object
#' @param unit The time unit to be used
#' @param add.unit Should time unit be added as column to resulting
#'     dataframe.
#' @param ... Additional arguments; currently not used.
NULL

#' @rdname mb_summary
#' @export
mb_summary <- function (object, unit, add.unit=TRUE, ...) 
{
    if (missing(unit)) {
        unit <- if (!is.null(attr(object, "unit"))) 
            attr(object, "unit")
        else getOption("microbenchmark.unit", "t")
    }
    if (unit != "relative") 
        object$time <- .convert_to_unit(object$time, unit)
    res <- aggregate(time ~ expr, object, function(z) {
        tmp <- c(stats::fivenum(z), mean(z), length(z))
        tmp[c(1, 2, 6, 3, 4, 5, 7)]
    })
    res <- cbind(res$expr, as.data.frame(res$time))
    colnames(res) <- c("expr", "min", "lq", "mean", "median", 
        "uq", "max", "neval")
    if (unit == "relative") {
        min <- res[which.min(res$median), , drop = FALSE]
        min$neval <- 1
        res[-1] <- res[-1]/as.list(min[-1])
        attr(res, "unit") <- "relative"
    }
    else {
        attr(res, "unit") <- attr(object$time, "unit")
    }
    ## if (requireNamespace("multcomp", quietly = TRUE) && nrow(res) > 
    ##     1 && all(res["neval"] > 1)) {
    ##     tryCatch({
    ##         ops <- options(warn = -1)
    ##         mdl <- lm(time ~ expr, object)
    ##         comp <- multcomp::glht(mdl, multcomp::mcp(expr = "Tukey"))
    ##         res$cld <- multcomp::cld(comp)$mcletters$monospacedLetters
    ##     }, error = function(e) FALSE, finally = options(ops))
    ## }

    if (add.unit)
        res$unit <- attr(res, "unit")
    res
}


.convert_to_unit <- 
function (x, unit = c("ns", "us", "ms", "s", "t", "hz", "khz", 
    "mhz", "eps", "f")) 
{
    unit <- match.arg(unit)
    switch(unit, t = unit <- sprintf("%ss", .find_prefix(x * 1e-09, 
        minexp = -9, maxexp = 0, mu = FALSE)), f = unit <- sprintf("%shz", 
        .find_prefix(1e+09/x, minexp = 0, maxexp = 6, mu = FALSE)))
    unit <- tolower(unit)
    switch(unit, ns = {
        attr(x, "unit") <- "nanoseconds"
        unclass(x)
    }, us = {
        attr(x, "unit") <- "microseconds"
        unclass(x/1000)
    }, ms = {
        attr(x, "unit") <- "milliseconds"
        unclass(x/1e+06)
    }, s = {
        attr(x, "unit") <- "seconds"
        unclass(x/1e+09)
    }, eps = {
        attr(x, "unit") <- "evaluations per second"
        unclass(1e+09/x)
    }, hz = {
        attr(x, "unit") <- "hertz"
        unclass(1e+09/x)
    }, khz = {
        attr(x, "unit") <- "kilohertz"
        unclass(1e+06/x)
    }, mhz = {
        attr(x, "unit") <- "megahertz"
        unclass(1000/x)
    }, stop("Unknown unit '", unit, "'."))
}

.find_prefix <- 
function (x, f = min, minexp = -Inf, maxexp = Inf, mu = TRUE) 
{
    prefixes <- c("y", "z", "a", "f", "p", "n", "u", "m", "", 
        "k", "M", "G", "T", "P", "E", "Z", "Y")
    if (mu) 
        prefixes[7] <- "mu"
    if (is.numeric(minexp)) 
        minexp <- floor(minexp/3)
    if (is.numeric(minexp)) 
        maxexp <- floor(maxexp/3)
    e3 <- floor(log10(f(x))/3)
    e3 <- max(e3, minexp, -8)
    e3 <- min(e3, maxexp, 8)
    prefixes[e3 + 9]
}


#' @rdname mb_summary
#' @export
summary_mb <- mb_summary

#' Variable utilities: parse, select, check, map, rename
#' 
#' These functions provide flexible tools for parsing and managing variable names
#' from formulas, unquoted names, or character vectors. Demonstrated using `CO2` dataset.
#' @param ... Variable input (unquoted, character vector, or formula)
#' @importFrom rlang enquos eval_tidy as_name abort is_formula f_rhs quo_get_expr
#' @export
vparse <- function(...) {
  dots <- enquos(...)

  if (length(dots) > 1) {
    # Case: multiple unquoted inputs
    return(unname(sapply(dots, as_name)))
  }

  # Single input
  q <- dots[[1]]
  expr <- quo_get_expr(q)

  # Case 1: formula (~Treatment + Type)
  if (is_formula(expr)) {
    return(all.vars(f_rhs(expr)))
  }

  # Case 2: single unquoted variable (symbol)
  if (is.symbol(expr)) {
    return(as_name(expr))
  }

  # Case 3: character vector
  val <- eval_tidy(q)
  if (is.character(val)) {
    return(val)
  }

  abort("Invalid input to vparse: expected unquoted names, a character vector, or a formula.")
}


#' Shorthand for vparse()
#'
#' A short and convenient alias for `vparse()`. Accepts unquoted names, character vectors, or a formula.
#'
#' @param ... Variable input in any accepted `vparse()` form
#'
#' @return A character vector of variable names
#' @export
v <- function(...) {
  vparse(...)
}



#' Select columns from a data frame using flexible input
#'
#' @param df A data frame
#' @param ... Variable input (unquoted, character vector, or formula)
#' @return A data frame with selected columns
#' @export
vselect <- function(df, ...) {
  vars <- vparse(...)
  df[, vars, drop = FALSE]
}

#' Check if variables exist in a data frame
#'
#' @param df A data frame
#' @param ... Variables to check
#' @return TRUE if all variables exist, otherwise error
#' @export
vcheck <- function(df, ...) {
  vars <- vparse(...)
  missing <- vars[!vars %in% names(df)]
  if (length(missing) > 0) {
    abort(paste("Missing variables in data frame:", paste(missing, collapse = ", ")))
  }
  TRUE
}

#' Apply a function to each parsed variable name
#'
#' @param .vars Variables to parse
#' @param .f Function to apply to each name
#' @return A list of results

#' @export
vmap <- function(.vars, .f) {
  q <- enquo(.vars)
  vars <- vparse(!!q)
  lapply(vars, .f)
}


#' Rename columns in a data frame
#'
#' @param df A data frame
#' @param rename_map A named character vector (old_name = new_name)
#' @return A data frame with renamed columns
#' @export
vrename <- function(df, rename_map) {
  stopifnot(is.character(rename_map), !is.null(names(rename_map)))
  old_names <- names(rename_map)
  new_names <- unname(rename_map)

  matched <- match(old_names, names(df))
  if (any(is.na(matched))) {
    abort("Some names in rename_map do not exist in the data frame.")
  }

  names(df)[matched] <- new_names
  df
}

# Example usage with CO2 dataset
#' @examples
#' data(CO2)
#' vparse(Treatment, Type)
#' vparse(c("Treatment", "Type"))
#' vparse(~Treatment + Type)
#' vselect(CO2, Treatment, Type)
#' vcheck(CO2, ~Treatment + Type)
#' vmap(~Treatment + Type, toupper)
#' vrename(CO2, c(Treatment = "Trt", Type = "Group"))

test_that("vparse handles different input formats correctly", {
  expect_equal(vparse(Treatment, Type), c("Treatment", "Type"))
  expect_equal(vparse(c("Treatment", "Type")), c("Treatment", "Type"))
  expect_equal(vparse(~Treatment + Type), c("Treatment", "Type"))
})

test_that("vselect selects correct columns from CO2", {
  data(CO2)
  selected <- vselect(CO2, Treatment, Type)
  expect_true(all(c("Treatment", "Type") %in% names(selected)))
  expect_equal(ncol(selected), 2)
})

test_that("vcheck confirms variables exist", {
  data(CO2)
  expect_true(vcheck(CO2, Treatment, Type))
  expect_true(vcheck(CO2, ~Treatment + Type))
})

test_that("vcheck throws error for missing variables", {
  data(CO2)
  expect_error(vcheck(CO2, Nonexistent), "Missing variables")
})

test_that("vmap applies function to variable names", {
  result <- vmap(~Treatment + Type, toupper)
  expect_equal(result, list("TREATMENT", "TYPE"))
})

test_that("vrename renames columns correctly", {
  data(CO2)
  renamed <- vrename(CO2, c(Treatment = "Trt", Type = "Group"))
  expect_true(all(c("Trt", "Group") %in% names(renamed)))
  expect_false("Treatment" %in% names(renamed))
})

##########################################################################
#'
#' @title List of lm objects with a common model
#' @description The data is split into strata according to the levels
#'     of the grouping factors and individual lm fits are obtained for
#'     each stratum.
#' @name by-lmby
#' 
##########################################################################
#'
#' @aliases lmBy coef.lmBy coef.summary_lmBy summary.lmBy fitted.lmBy
#'     residuals.lmBy getBy
#' @param formula. A linear model formula object of the form `y ~ x1 +
#'     ... + xn | g1 + ... + gm`.  In the formula object, `y`
#'     represents the response, `x1, ..., xn` the covariates, and the
#'     grouping factors specifying the partitioning of the data
#'     according to which different lm fits should be performed.
#' @param data. A dataframe
#' @param id A formula describing variables from data which are to be
#'     available also in the output.
#' @param \dots Additional arguments passed on to \code{lm()}.
#' @return A list of lm fits.
#' @concept byfunction
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#'
#' @keywords models
#' @examples
#' 
#' lm_lst <- lmBy(1 / uptake ~ log(conc) | Treatment, data=CO2)
#' coef(lm_lst)
#' 
#' fitted(lm_lst)
#' residuals(lm_lst)
#' 
#' summary(lm_lst)
#' coef(summary(lm_lst))
#' coef(summary(lm_lst), simplify=TRUE)
#' 
#' @export
#' @rdname by-lmby
lm_by <- function (data., formula., id=NULL, ...) {
    cl <- match.call(expand.dots = TRUE)
    cl[[2]] <- formula.
    cl[[3]] <- data.
    names(cl)[2:3] <- c("formula.", "data.")
    cl[[1]] <- as.name("lmBy")
    eval(cl)
}

#' @export
#' @rdname by-lmby
lmBy <- function(formula., data., id=NULL, ...){
  cl   <- match.call()
  mff  <- parseGroupFormula(formula.)
  group_data  <- splitBy(mff$groupFormula, data=data., omit=FALSE)
    
  form <- mff$model
  
  model_list  <- lapply(group_data,
                function(wd) {
                    zzz <- lm(form, data=wd, ...)
                    zzz$call[[2]] <- form
                    zzz
                })

  if (!is.null(id)){
      id.vars <- unique(c(all.vars(id),
                          all.vars(mff$groupFormula)))
  } else {
    id.vars <- all.vars(mff$groupFormula)
  }


  id.data <- do.call(rbind, lapply(group_data,
                                   function(wd) {
                                       wd[1, id.vars, drop=FALSE]
                                   }))
  

  attr(model_list,  "call")     <- cl
  attr(model_list,  "dataList") <- group_data
  attr(model_list,  "idData")   <- id.data	
  
  class(model_list) <- "lmBy"
  model_list
}


#' @export
print.lmBy <- function(x, ...){
  ##lapply(c(x), print)
  print(c(x))
  return(invisible(x))
}

#' @export
summary.lmBy <- function(object, ...){
  res <- lapply(object, summary)
  class(res) <- "summary_lmBy"
  res
}

#' @export
print.summary_lmBy <- function(x, ...){
  lapply(x, print)
  invisible(x)
}

#' @export
coef.summary_lmBy <- function(object, simplify=FALSE, ...){
  ans <- lapply(object, coef)
  if (simplify){
    cc <- do.call(rbind, ans)
    cn <- colnames(cc)
    rn <- rownames(cc)

    nn <- names(ans)
    rn <- rownames(ans[[1]])
    ff <- factor(rep(nn, each=length(rn)))
    
    rownames(cc) <- NULL
    ans <- data.frame(ff, rn, as.data.frame(cc))
    colnames(ans) <- c("stratum", "parameter", cn)
  }
  ans
}





#' @export
getBy <- function(object, name=c()){
  if (missing(name)) 
    stop("'name' must not be missing")
  if (inherits(object, "lmBy"))
  {
      ii <- match(name, c("dataList", "idData"))
      if (is.na(ii))
          stop(sprintf("%s not available", name))
      attr(object,name)	
  }
  
}

#' @export
coef.lmBy <- function(object, augment=FALSE, ...){
  ans <- do.call(rbind, lapply(object, coef))
  if (augment){
    ans <- cbind(ans, getBy(object, "idData"))
  }
  ans
}

#' @export
sigma.lmBy <- function(object, ...){
    out <- sapply(object, sigma, ...)
    return(out)
}



#' @export
fitted.lmBy <- function(object, augment=FALSE, ...){
  ans <- lapply(object, fitted)
  if (augment) {
      ans <- mapply(function(a,b){
          data.frame(.fit=a,b)
      }, ans, getBy(object, "dataList"), SIMPLIFY=FALSE)
  }
  ans
}

#' @export
residuals.lmBy <- function(object, augment=FALSE, ...){
  ans <- lapply(object, residuals)
  if (augment) {
      ans <- mapply(function(a,b){
          data.frame(.fit=a,b)
      }, ans, getBy(object, "dataList"), SIMPLIFY=FALSE)
  }
  ans
}

#' @title Pipe-friendly arithmetic helpers
#' @description
#' A set of simple, vectorized, pipe-friendly arithmetic functions for
#' transforming numeric data in pipelines. These helpers make common
#' operations like multiplication, division, addition, subtraction,
#' exponentiation, and reciprocals clearer when using the native pipe `|>`.
#'
#' @details
#' All functions are vectorized and support numeric vectors, scalars, or compatible objects.
#' They are designed to improve the readability of transformation pipelines.
#'
#' @param x A numeric vector or scalar.
#' @param k A numeric scalar for addition, subtraction, multiplication, or division.
#' @param p A numeric scalar exponent (for \code{pow}).
#'
#' @return A numeric vector or scalar resulting from the transformation.
#'
#' @examples
#' x <- c(1, 2, 3)
#'
#' # Multiplication and division
#' x |> mult(10)
#' x |> divide(2)
#'
#' # Addition and subtraction
#' x |> add(5)
#' x |> subtract(1)
#'
#' # Reciprocal
#' x |> reciprocal()
#'
#' # Power
#' x |> pow(2)
#'
#' # Combined use in pipelines
#' x |>
#'   mult(2) |>
#'   add(3) |>
#'   reciprocal()
#'
#' @name pipe_arithmetic
#' @concept pipe_arithmetic
NULL

#' @rdname pipe_arithmetic
#' @concept pipe_arithmetic
#' @export
reciprocal <- function(x) {
  1 / x
}

#' @concept pipe_arithmetic
#' @rdname pipe_arithmetic
#' @export
pow <- function(x, p) {
  x ^ p
}

#' @concept pipe_arithmetic
#' @rdname pipe_arithmetic
#' @export
add <- function(x, k) {
  x + k
}

#' @concept pipe_arithmetic
#' @rdname pipe_arithmetic
#' @export
subtract <- function(x, k) {
  x - k
}

#' @concept pipe_arithmetic
#' @rdname pipe_arithmetic
#' @export
mult <- function(x, k) {
  x * k
}

#' @concept pipe_arithmetic
#' @rdname pipe_arithmetic
#' @export
divide <- function(x, k) {
  x / k
}

## #############################################################################
#'
#' @title Linear estimates matrix
#' @description Generate matrix specifying linear estimate.
#' @name linest-matrix
#'
## #############################################################################
#'
#' @param object Model object
#' 
#' @param effect A vector of variables. For each configuration of
#'     these the estimate will be calculated.
#'
#' @param at Either NULL, a list or a dataframe. 1) If a list, then the list
#'     must consist of covariates (including levels of some factors)
#'     to be used in the calculations. 2) If a dataframe, the
#'     dataframe is split rowwise and the function is invoked on each
#'     row.
#'
#' @param linest_list Linear estimate list (as generated by \code{get_linest_list}).
#' 
#' @details Check this
#'
#' @seealso \code{\link{LSmeans}}, \code{\link{linest}}
#' @keywords utilities
#' 
#' @examples
#' 
#' ## Two way anova:
#' 
#' data(warpbreaks)
#'
#' ## An additive model
#' m0 <- lm(breaks ~ wool + tension, data=warpbreaks)
#'
#' ## Estimate mean for each wool type, for tension="M":
#' K <- LE_matrix(m0, at=list(wool=c("A", "B"), tension="M"))
#' K
#' 
#' ## Vanilla computation:
#' K %*% coef(m0)
#'
#' ## Alternative; also providing standard errors etc:
#' linest(m0, K)
#' esticon(m0, K)
#'
#' ## Estimate mean for each wool type when averaging over tension;
#' # two ways of doing this
#' K <- LE_matrix(m0, at=list(wool=c("A", "B")))
#' K
#' K <- LE_matrix(m0, effect="wool")
#' K
#' linest(m0, K)
#'
#' ## The linear estimate is sometimes called to "least squares mean"
#' # (LSmeans) or popupulation means.
#' # Same as
#' LSmeans(m0, effect="wool")
#'
#' ## Without mentioning 'effect' or 'at' an average across all
#' #predictors are calculated:
#' K <- LE_matrix(m0)
#' K
#' linest(m0, K)
#' 
#' ## Because the design is balanced (9 observations per combination
#' #of wool and tension) this is the same as computing the average. If
#' #the design is not balanced, the two quantities are in general not
#' #the same.
#' mean(warpbreaks$breaks)
#'
#' ## Same as 
#' LSmeans(m0)
#'
#' ## An interaction model 
#' m1 <- lm(breaks ~ wool * tension, data=warpbreaks)
#' 
#' K <- LE_matrix(m1, at=list(wool=c("A", "B"), tension="M"))
#' K
#' linest(m1, K)
#' K <- LE_matrix(m1, at=list(wool=c("A", "B")))
#' K
#' linest(m1, K)
#' K <- LE_matrix(m1, effect="wool")
#' K
#' linest(m1, K)
#' LSmeans(m1, effect="wool")
#' 
#' K <- LE_matrix(m1)
#' K
#' linest(m1, K)
#' LSmeans(m1)

#' @export
#' @rdname linest-matrix
LE_matrix <- function(object, effect=NULL, at=NULL){
  UseMethod("LE_matrix")
}

## FIXME: LE_matrix.default: Should be a check of what 'object' is
#' @export
#' @rdname linest-matrix
LE_matrix.default <- function(object, effect=NULL, at=NULL){

    effect <- .parse.effect(effect)

    ## print("HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH")
    ## str(list(effect=effect, at=at))
    
    if (!is.null(at))
        if (!inherits(at, c("list", "data.frame"))) stop("'at' must be list or data.frame\n")

    if (!inherits(at, "data.frame")){
        out <- get_linest_list(object, effect, at)
        out <- aggregate_linest_list (out)
    } else {
        ## call function for each row in at
        at.list <- split(at, 1:nrow(at))

        
        at.list <- lapply(at.list, as.list)
        out  <- lapply(at.list, function(at.arg)
            LE_matrix(object, effect, at.arg))
        out <- do.call(rbind, out)
    }
    class(out) <- c("linest_matrix_class", "matrix")
    out    
}


#' @export
#' @rdname linest-matrix
aggregate_linest_list <- function (linest_list){
    out               <- lapply( linest_list, function( mm ) apply( mm, 2, mean ) )
    out               <- do.call(rbind, out)
    attr(out, "at")   <- attr(linest_list, "at")
    attr(out, "grid") <- attr(linest_list, "grid")
    attr(out, "offset") <- attr(linest_list, "offset")
    out
}

#' @export
print.linest_matrix_class <- function(x, ...){
  prmatrix(x)
  invisible(x)
}

#' @export
summary.linest_matrix_class <- function(object, ...){
    print(object)
    cat("at: \n"); print(attr(object, "at"))
    cat("grid: \n"); print(attr(object, "grid"))
    invisible(object)      
}


.parse.effect <- function(effect=NULL){
    ## All returns c("A", "B")
    ## parse.effect(~A+B)
    ## parse.effect(y~A+B)
    ## parse.effect(c("A", "B"))

    if (is.null(effect)) return(NULL)
    if (!inherits(effect, c("character", "formula")))
        stop("'effect' must be a character vector or a formula")
    
    if (inherits(effect, "character")){
        if (identical(effect, character(0)) | identical(effect, ""))
            NULL
        else
            effect
    } else {
        all.vars(effect[[length(effect)]])
    }
}

## This is the workhorse for generating the "contrast matrix"
#' @export
#' @rdname linest-matrix
get_linest_list <- function(object, effect=NULL, at=NULL) {

    pr <- FALSE
    ##cat(".get_linest_list\n")

    effect <- .parse.effect(effect)
    
    trms     <- delete.response( terms(object) )
    fact.lev <- get_xlevels( object )            ## factor levels
    ## cat("+++ fact.lev:\n"); print(fact.lev)
    cov.ave  <- .get_covariate_ave(object, at)  ## average of covariates (except those mentioned in 'at')
    ## cat("+++ cov.ave:\n"); print(cov.ave)
    vartype  <- get_vartypes( object )           ## which are factors and which are numerics
    ## cat("+++ vartype:\n"); print(vartype)
    at.factor.name <- intersect(vartype$factor, names(at))
    cov.ave.name   <- names( cov.ave )
    effect         <- setdiff( effect, at.factor.name )

    if (is.null(effect))
    {
        new.fact.lev <- if (length(at.factor.name) > 0) at[ at.factor.name ]
                        else NULL        
    } else {
        zz  <- set_xlevels(fact.lev, at=at)
        ## print(fact.lev)
        ## print(zz)
        new.fact.lev  <- zz[c(effect, at.factor.name)]
    }

    ## str(list(effect=effect, at=at, fact.lev=fact.lev, new.fact.lev=new.fact.lev, 
    ##          vartype=vartype, cov.ave=cov.ave, at.factor.name=at.factor.name,
    ##          cov.ave.name=cov.ave.name))

    if (is.null(new.fact.lev)){
        ##cat("case: No 'effect' and no 'at'-factors; hence just a global average. \n")
        if (length(fact.lev) > 0)
        {
            ##cat("there are factors in the model\n")
            newdata <- expand.grid(fact.lev)
            if (length( cov.ave.name ) > 0){
                ##cat("there are covariates in the model\n")
                newdata[, cov.ave.name] <- cov.ave ## Augment with covariates
            }
        } else {
            ## cat("No factors in the model\n")
            if (length( cov.ave.name ) > 0){
                ##cat("yes there are covariates\n")
                newdata <- data.frame(matrix(unlist(cov.ave), nrow=1L))                
                names(newdata) <- cov.ave.name
            } else {
                ##cat("there are no factors or covariates\n")
                newdata <- data.frame(1)
            }
        }

        out_list <- list(get_X(object, newdata=newdata, at=NULL))
        ## cat("out_list:\n"); print(out_list)
        attr(out_list, "at")   <- at[intersect(vartype$numeric, names(at))]
        attr(out_list, "grid") <- NULL
    }
    else
    {
        if(pr)cat("The general case; there are 'effect' factors or 'at' factors.\n")
        grid.data <- expand.grid(new.fact.lev)
        ## print(grid.data)
        
        grid.data <- as.data.frame(lapply(grid.data, as.character), stringsAsFactors=FALSE)

        ## if (length( cov.ave.name ) > 0){
        ##     ##cat("there are covariates in the model\n")
        ##     grid.data[, cov.ave.name] <- cov.ave ## Augment with covariates
        ## }
        
        ## nfl <<- new.fact.lev
        ## gd <<- grid.data
        str(list(new.fact.lev=new.fact.lev, grid.data=grid.data))
        
        out_list    <- list()
        for (ii in 1:nrow(grid.data)){
            config    <- grid.data[ii, ,drop=FALSE ]
            ## print(config)
            fact.lev2 <- set_xlevels(fact.lev, at=config)
            newdata   <- expand.grid(fact.lev2)
            newdata[, cov.ave.name]  <- cov.ave
            XX             <- get_X(object, newdata=newdata, at=at)
            ## str(list(XX=XX))
            ## print(XX)
            out_list[[ ii ]] <- XX
        }

        grid.data[, names(cov.ave) ] <- cov.ave
        attr(out_list, "at") <- at
        attr(out_list, "grid") <- grid.data
        attr(out_list, "offset") <- attr(XX, "offset")  ## FIXME: reference to XX; what is offset?
    }
    class(out_list) <- "linest_list_class"
    out_list
}



## --------------------------------------------------------------------

setOldClass("linest_matrix_class")

setAs("linest_matrix_class", "matrix",
      function(from){
          attr(from, "at") <- attr(from, "grid") <- NULL
          class(from) <- "matrix"
          from
      })

setAs("linest_matrix_class", "Matrix",
      function(from){
          attr(from, "at") <- attr(from, "grid") <- NULL
          class(from) <- "matrix"
          as(from, "Matrix")
      })

tochar <- function(...){
    dots <- match.call(expand.dots = FALSE)$...
    dots2char(dots)
}

dots2char <- function(dots){
    ## dots <- match.call(expand.dots = FALSE)$...
    dots1 <- dots[[1]]

    if (inherits(dots1, "call")){
        dots1 <- eval(dots1)
    }
    if (inherits(dots1, c("formula", "call"))){
        varRHS <- all.vars(dots1[[2]])
        return(varRHS)
    }

    if (is.character(dots1)){
        varRHS <- dots1
        return(varRHS)        
    }
       
    varRHS <- lapply(seq_along(dots),
                     function(i) {
                         y <- if (is.symbol(dots[[i]])) {
                                  deparse(dots[[i]])
                              }
                              else {
                                  dots[[i]]
                              }
                         return(y)
                     })
    varRHS <- unlist(varRHS)
    return(varRHS)
}

#' @title Group-wise scaling of data
#' @description Splits a data frame or matrix by grouping variables and scales numeric variables within each group.
#' @concept byfunction
#' @param formula Grouping structure: a formula, character vector, or variables as `as.quoted`.
#' @param data A data frame or matrix.
#' @param center Logical; if TRUE, center the variables.
#' @param scale Logical; if TRUE, scale the variables.
#'
#' @return A list of data frames or matrices (same class as input), one per group.
#'
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{summaryBy}}, \code{\link{transformBy}}, \code{\link{orderBy}}
#' @keywords utilities
#'
#' @examples
#' scale_by(iris, ~Species)
#' scale_by(iris, ~1)
#'
#' ## Combine result into one data frame:
#' a <- scale_by(iris, ~Species)
#' d <- do.call(rbind, a)
#'
#' ## Old interface
#' scaleBy(~Species, data = iris, center = TRUE, scale = FALSE)
#' scaleBy(~1, data = iris)
#'
#' @rdname by_scale
#' @export
scaleBy <- function(formula, data = parent.frame(), center = TRUE, scale = TRUE) {
  lapplyBy(formula, data = data, scale_df, center = center, scale = scale)
}

#' @rdname by_scale
#' @export
scale_by <- function(data, formula, center = TRUE, scale = TRUE) {
  do.call(scaleBy, list(formula = formula, data = data, center = center, scale = scale))
}



## #' @title Scale a dataframe or matrix
## #' @description Split a dataframe into a list according to the levels
## #'     of variables in the dataframe and scale the numeric variables
## #'     in each dataframe in the list.
## #' @name by_scale
## #'
## #' @param formula Variables to split data frame by, as `as.quoted`
## #'     variables, a formula or character vector.
## #' @param data A dataframe or matrix
## #' @param center Logical, should data be centered.
## #' @param scale Logical, should data be scaled.
## #' 
## #' @return A list of objects of same class as `x`
## #' 
## #' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
## #' @seealso \code{\link{orderBy}}, \code{\link{order_by}},
## #'     \code{\link{summaryBy}}, \code{\link{summary_by}},
## #'     \code{\link{transformBy}}, \code{\link{transform_by}} 
## #' @keywords utilities
## #' 
## #' @examples
## #'
## #' scaleBy(~Species, data=iris, center=TRUE, scale=FALSE)
## #' scaleBy(~1, data=iris, center=TRUE, scale=FALSE)
## #'
## #' scale_by(iris, ~Species)
## #' scale_by(iris, ~1)
## #' 
## #' ## Not combine list of dataframes to one dataframe e.g. as:
## #' a <- scale_by(iris, ~Species)
## #' d <- do.call(rbind, a)

## #' @rdname by_scale
## #' @export
## scaleBy <- function(formula, data=parent.frame(), center=TRUE, scale=TRUE){
##     lapplyBy(formula, data=data, scale_df, center=center, scale=scale)        
## }

## #' @rdname by_scale
## #' @export
## scale_by <- function(data, formula, center=TRUE, scale=TRUE){
##     arg <- list(formula=formula, data=data, center=center, scale=scale)
##     do.call(scaleBy, arg)
## }   

#' @title Compute linear estimates
#' 
#' @description Compute linear estimates, i.e. `L %*% beta` for a range of models. One example of
#'     linear estimates is population means (also known as LSMEANS).
#'
#' @name linest
#' 
#' @aliases linest linest.lm linest.glm linest.geeglm linest.lmerMod
#'     linest.merMod linest.default
#' 
#' @param object Model object
#' @param L Either \code{NULL} or a matrix with p columns where p is
#'     the number of parameters in the systematic effects in the
#'     model. If \code{NULL} then \code{L} is taken to be the p times
#'     p identity matrix
#' @param level The level of the (asymptotic) confidence interval.
#' @param confint Should confidence interval appear in output.
#' @param \dots  Additional arguments; currently not used.
#' @return A dataframe with results from computing the contrasts.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{LSmeans}}, \code{\link{LE_matrix}}
#' @keywords utilities
#'
#' @examples
#' 
#' ## Make balanced dataset
#' dat.bal <- expand.grid(list(AA=factor(1:2), BB=factor(1:3), CC=factor(1:3)))
#' dat.bal$y <- rnorm(nrow(dat.bal))
#' 
#' ## Make unbalanced dataset
#' #   'BB' is nested within 'CC' so BB=1 is only found when CC=1
#' #   and BB=2,3 are found in each CC=2,3,4
#' dat.nst <- dat.bal
#' dat.nst$CC <-factor(c(1,1,2,2,2,2,1,1,3,3,3,3,1,1,4,4,4,4))
#' 
#' mod.bal  <- lm(y ~ AA + BB * CC, data=dat.bal)
#' mod.nst  <- lm(y ~ AA + BB : CC, data=dat.nst)
#' 
#' L <- LE_matrix(mod.nst, effect=c("BB", "CC"))
#' linest( mod.nst, L )
#' 
#' @export linest

#' @export
linest <- function(object, L=NULL, level=0.95, ...){
    UseMethod("linest")
}

#' @export
linest.lm <- function(object, L=NULL, level=0.95, ...){
    bhat   <- coef(object)
    L      <- .constructL(L, bhat)    
    is.est <- is_estimable(L, null_basis(model.matrix(object)))
    VV0    <- vcov(object, complete=FALSE)
    ddf.vec <- rep(object$df.residual, nrow( L ))
    res     <- .getLb2( L, bhat, VV0, ddf.vec, is.est)
    
    p.value <- 2 * pt(abs(res[,"statistic"]), df=res[,"df"], lower.tail=FALSE)
    res <- as.data.frame(cbind(res, p.value))
    
    .finalize_linest(res, L, level)
}


#' @export
linest.glm <- function(object, L=NULL, level=0.95, ...){

    bhat    <- coef(object)
    L       <- .constructL(L, bhat)        
    is.est  <- is_estimable(L, null_basis(model.matrix(object)))
    VV0     <- vcov(object, complete=FALSE)
    ddf.vec <- rep(object$df.residual, nrow( L ))
    res     <- .getLb2(L, bhat, VV0, ddf.vec, is.est)

    if( family(object)[1] %in% c("poisson", "binomial", "Gamma", "quasipoisson", "quasibinomial")){
        p.value <- 2*pnorm(abs(res[,"statistic"]), lower.tail=FALSE)
        res <- res[, -4] # NO df's
        ## qq <- qnorm(1-(1-level)/2)
    } else {
        p.value <- 2*pt(abs(res[,"statistic"]), df=res[,"df"], lower.tail=FALSE)
        ## qq <- qt(1-(1-level)/2, df=res[,"df"])
    }
    res <- as.data.frame(cbind(res, p.value))
    .finalize_linest(res, L, level)
}

#' @export
linest.geeglm <- function(object, L=NULL, level=0.95, ...){

    bhat <- coef(object)
    L    <- .constructL(L, bhat)    
    is.est  <- is_estimable(L, null_basis(model.matrix(object)))
    VV0     <- summary(object)$cov.scaled
    ddf.vec <- rep(1, nrow(L))
    res     <- .getLb2( L, bhat, VV0, ddf.vec, is.est)

    res <- res[, 1:3]
    p.value <- 2 * pnorm(abs(res[,"statistic"]), lower.tail=FALSE)
    res <- as.data.frame(cbind(res, p.value))
    .finalize_linest(res, L, level)
}

#' @export
linest.lmerMod <- function(object, L=NULL, level=0.95, adjust.df=TRUE, ...){

    bhat <- lme4::fixef(object)
    L  <- .constructL(L, bhat)    
    is.est <- is_estimable(L, null_basis(model.matrix(object)))

    if (adjust.df){
        if (requireNamespace("pbkrtest", quietly=TRUE)){
            VVu  <- vcov(object, complete=FALSE)
            VV   <- pbkrtest::vcovAdj(object)
            ddf.vec <- unlist(lapply(1:nrow(L),
                                     function(ii) pbkrtest::ddf_Lb(VV , L[ii,], VVu)))
        } else {
            stop("adjustment of degrees of freedom requires that 'pbkrtest' is installed")
        }
    } else {
        a   <- logLik(object)
        ddf <- attributes(a)$nall - attributes(a)$df
        ddf.vec <- rep(ddf, length(bhat))
        VV  <- vcov(object, complete=FALSE)
    }

    res     <- .getLb2( L, bhat, VV, ddf.vec, is.est)
    p.value <- 2 * pt(abs(res[,"statistic"]), df=res[,"df"], lower.tail=FALSE)

    res <- as.data.frame(cbind(res, p.value))
    .finalize_linest(res, L, level)
}

#' @export
linest.merMod <- function(object, L=NULL, level=0.95, ...){
    cl <- match.call()
    cl[[1]] <- as.name("linest.lmerMod")
    cl$adjust.df <- FALSE
    eval(cl)
}

.finalize_linest <- function(.coef, L, level){

     ## cat(".finalize_linest\n")
     ## print(.coef)
    rownames(.coef) <- NULL


    
    df <- .coef$df

    qq <- 1-(1-level)/2
    
    if (!is.null(df)) {
        crit <- qt(qq, df=df)
    } else {
        crit <- qnorm(qq)
    }
    
    .coef <- cbind(.coef,
      cbind(lwr=.coef$estimate -crit*.coef$std.error,
            upr=.coef$estimate +crit*.coef$std.error))

    gr <- attr(L, "grid")
    ## gr <- lapply(gr, as.character)

    ## print(gr)

    if (!is.null(gr)){
        .coef <- cbind(gr, .coef)
    }
    ## print(.coef)

    ## .coef$estimate <- format(.coef$estimate, digits = 3, nsmall = 2)
    
    if (!is.null(rownames(L)))
        rownames(.coef) <- rownames(L)

    ## .coef <- as.data.frame(.coef)

    attr(.coef, "L") <- L
    #res  <- list(coef=.coef, grid=attr(L, "grid"), L=L)
    class(.coef) <- c("linest_class", "data.frame")
    .coef
}


.getLb2 <- function(L, bhat, VV, ddf.vec, is.est, level=0.95){
    off <- attr(L, "offset")

    ## print(L)
    ## print(VV)
    notna <- !is.na(bhat)
    used       <- which(!is.na(bhat))
    bhat.used  <- bhat[used]
    ## L   <- L[, used, drop=FALSE]
    res <- matrix(NA, nrow=nrow(L), ncol=3)

    ## print("ddddddddddddddddddddddd")
    ## print(bhat)
    ## cat("-------------\n")
    for (ii in 1:nrow(res)){
        kk <- L[ii,]
        ## print(kk)
        ## kk <<- kk
        ## bhat <<- bhat
        zz <- kk[is.na(bhat)] ## values in kk for which beta is NA
        ## print(zz)
        if (length(zz) == 0)
            is.zero <- TRUE
        else
            is.zero <- max(abs(zz)) < 1e-3

        
        ## print(is.zero)
        if (is.est[ii] && is.zero){
            kk   <- kk[used]
            est  <- sum(kk * bhat.used)
            se   <- sqrt(sum(kk * (VV %*% kk)))
            df2  <- ddf.vec[ii]
            res[ii,] <- c(est, se, df2)
        }
    }
    
    if (!is.null(off))
        res[,1] <- res[,1] + off[[1]]
    
    colnames(res) <- c("estimate", "std.error", "df") 
    statistic     <- res[,"estimate"] / res[,"std.error"]
    res <- cbind(res[,1:2, drop=FALSE], statistic, df=res[,3])

    rownames(res) <- NULL
    ##print(res)
    res
}




## .getLb2 <- function(L, bhat, VV, ddf.vec, is.est, level=0.95){
##     off <- attr(L, "offset")

##     used       <- which(!is.na(bhat))
##     bhat.used  <- bhat[used]
##     L   <- L[, used, drop=FALSE]
##     res <- matrix(NA, nrow=nrow(L), ncol=3)
    
##     for (ii in 1:nrow(res)){
##         kk <- L[ii,]
##         if (is.est[ii]){
##             est  <- sum(kk * bhat.used)
##             se   <- sqrt(sum(kk * (VV %*% kk)))
##             df2  <- ddf.vec[ii]
##             res[ii,] <- c(est, se, df2)
##         }
##     }

##     if (!is.null(off))
##         res[,1] <- res[,1] + off[[1]]
    
##     colnames(res) <- c("estimate", "std.error", "df") 
##     statistic        <- res[,"estimate"] / res[,"std.error"]
##     res <- cbind(res[,1:2, drop=FALSE], statistic, df=res[,3])

##     rownames(res) <- NULL
##     ##print(res)
##     res
## }




















.ci_fun <- function (object, parm, level = 0.95, ...) 
{
    ##object <- coef(object)
    cf <- object$estimate #cf <- coef(object)
    pnames <- 1:nrow(object) ##names(cf)
    if (missing(parm)) 
        parm <- pnames
    else if (is.numeric(parm)) 
        parm <- pnames[parm]
    a <- (1 - level)/2
    a <- c(a, 1 - a)
    pct <- a 
    DF <- object$df
    if (!is.null(DF))
        fac <- t(sapply(DF, function(d) qt(a, d)))
    else
        fac <- matrix(rep(qnorm(a), length(parm)), nrow=2)
       
    ci <- array(NA, dim = c(length(parm), 2L), dimnames = list(parm, pct))
    ses  <- object$std.error[parm]
    #print(list(a=a, DF=DF, ses=ses, fac=fac, cf=cf[parm], ci=ci))
    #ci[] <- cf[parm] + ses %o% fac
    ci[] <- cf[parm] + fac * ses
    ci
}


### UTILITIES ###

#' @export
#' @rdname linest
#' @param parm Specification of the parameters estimates for which
#'     confidence intervals are to be calculated.
confint.linest_class <- function (object, parm, level = 0.95, ...) 
{
    co <- coef(object) 
    .ci_fun(co, parm, level, ...)
}

.constructL <- function(L, bhat){
    .defineL <- function( bhat ){
        L <- diag(1, length(bhat))
        rownames(L) <- names(bhat)
        L
    }

    if (is.null(L))
        L <- .defineL(bhat)
    if (!is.matrix(L))
        L <- matrix(L, nrow=1)
    L
}

#' @export
#' @rdname linest
coef.linest_class <- function (object, ...) {
    object$coef
}

#' @export
print.linest_class <- function (x, ...){
    ## cat("Coefficients:\n")
    ##printCoefmat(x$coef)
    old <- options("digits")$digits    
    options("digits"=3)
    print.data.frame(x)
    options("digits"=old)
    return(invisible(x))
}

#' @export
#' @rdname linest
summary.linest_class <- function (object, ...) 
{
    print(object)
##     cat("Coefficients:\n")
##     printCoefmat(object$coef)
##     cat("\n")

## #    if (!is.null(object$grid)){
##         cat("Grid:\n")
##         print(object$grid)
##         cat("\n")
## #    }

##     cat("L:\n")
##     print(object$L)
##     cat("\n")

    invisible(object)
}


setOldClass("linest_class")
setAs("linest_class", "data.frame",   function(from)
{
    out <- as.data.frame(coef(from))
    if (!is.null(from$grid))
        out <- cbind(out, from$grid)
    attr(out, "K") <- from$K
    out
})

#' @title Near infra red light (NIR) measurements in milk
#' 
#' @description Near infra red light (NIR) measurements are made at
#'     152 wavelengths on 17 milk samples.  While milk runs through a
#'     glass tube, infra red light is sent through the tube and the
#'     amount of light passing though the tube is measured at
#'     different wavelengths.  Each milk sample was additionally
#'     analysed for fat, lactose, protein and dry matter.
#' @concept dataset
#' @name nir_milk
#' @docType data
#' 
#' @format
#' Data comes in two formats:
#' 
#' nir_milk: A list with two components
#' * x Datafrane with infra red light
#'  amount at different wavelengths (column names are the wavelengths;
#'  just remove the leading X).
#' * y Datafrane with response variables
#'  fat, protein, lactose and dm (drymatter)
#'
#' NIRmilk: This data frame contains 17 rows and 158 columns.
#'
#' * The first column is the sample number.
#' * The columns named in the form `Xw` contains
#'     the transmittance (fraction of electromagnetic power)
#'     transmittance through the sample at wavelength `w`.
#' * The response variables are fat, protein, lactose and dm (dry
#'     matter).
#'
#' @keywords datasets
#' @examples
#' 	
#' data(nir_milk)
#' data(NIRmilk)
#' 
"nir_milk"

#' @rdname nir_milk
"NIRmilk"


#' @title Height of math teachers
#' @description Height of a sample of math teachers in Danish high
#'     schools collected at a continued education day at Mariager Fjord Gymnasium in 2019.
#' @concept dataset
#' @format:
#' \describe{
#'   \item{height}{Height in centimeters}
#'   \item{sex}{Male or female}
#' }
#' 
#' 
#'
#' @examples
#'
#' aggregate(height ~  sex, data=math_teachers, FUN=mean)
#' aggregate(height ~  sex, data=math_teachers, FUN=function(x) {c(mean=mean(x), sd=sd(x))})
#' 
"math_teachers"


##' @title Berkeley Growth Study data
##' @description dataframe with heights of 39 boys and 54 girls from
##'     age 1 to 18 and the ages at which they were collected.
##' @concept dataset
##' @format
##' \describe{
##'   \item{gender}{Gender of child}
##'   \item{age}{Age at time of data recording}
##'   \item{subject}{Identification for each child}
##'   \item{height}{Height of child}
##' }
##'
##'
##' @details Notice that the ages are not equally spaced. Data are
##'     taken from the fda package (growth) but put in long format
##'     here.
##'
##' @references
##'     Ramsay, James O., Hooker, Giles, and Graves, Spencer (2009),
##'     _Functional data analysis with R and Matlab_, Springer, New York.
##'
##'     Ramsay, James O., and Silverman, Bernard W. (2005), _Functional
##'     Data Analysis, 2nd ed._, Springer, New York.
##'
##'     Ramsay, James O., and Silverman, Bernard W. (2002), _Applied
##'     Functional Data Analysis_, Springer, New York.
##'
##'     Tuddenham, R. D., and Snyder, M. M. (1954) "Physical growth of
##'     California boys and girls from birth to age 18", _University of
##'     California Publications in Child Development_, 1, 183-364.
##'
##'
"child_growth"     



#' Chemical composition of wine
#' 
#' Using chemical analysis determine the origin of wines
#' 
#' Data comes from the UCI Machine Learning Repository. The grape variety
#' \code{Cult} is the class identifier.  
#' @concept dataset
#' @name data-wine
#' @docType data
#' @format A data frame with 178 observations on the following 14 variables.
#'   \describe{
#'   \item{\code{Cult}}{a factor with levels \code{v1} \code{v2}
#'     \code{v3}: 3 different graph varieties}
#'   \item{\code{Alch}}{Alcohol}
#'   \item{\code{Mlca}}{Malic acid}
#'   \item{\code{Ash}}{Ash}
#'   \item{\code{Aloa}}{Alcalinity of ash}
#'   \item{\code{Mgns}}{Magnesium}
#'   \item{\code{Ttlp}}{Total phenols}
#'   \item{\code{Flvn}}{Flavanoids}
#'   \item{\code{Nnfp}}{Nonflavanoid phenols}
#'   \item{\code{Prnt}}{Proanthocyanins}
#'   \item{\code{Clri}}{Color intensity}
#'   \item{\code{Hue}}{Hue}
#'   \item{\code{Oodw}}{OD280/OD315 of diluted wines}
#'   \item{\code{Prln}}{Proline}
#' }
#' 
#' @references See references at
#'   \url{https://archive.ics.uci.edu/ml/datasets/Wine/}
#'
#' @source Frank, A. & Asuncion, A. (2010). UCI Machine Learning
#'     Repository \url{https://archive.ics.uci.edu/ml/}. Irvine, CA:
#'     University of California, School of Information and Computer
#'     Science.
#'
#' @keywords datasets
#' @usage data(wine)
#' @examples
#' 
#' data(wine)
#' ## maybe str(wine) ; plot(wine) ...
#' 
"wine"


#' @title Income data, years of educations and ethnicity
#' @description Data on income, years of educations and ethnicity for a samle of adult Americans aged over 25. The year of sampling is not avalable in the source.
#' @name income
#' @concept dataset
#' @docType data
#' @format This data frame contains:
#' \describe{
#' \item{inc:}{Income: Yearly income (thousands of dollars).}
#' \item{educ:}{Education: Number of years of education (12=high school graduate, 16=college graduate).}
#' \item{race:}{Racial-Ethnic group: "b" (black), "h" (hispanic) and "w" (white).}
#' }
#' @keywords datasets
#'
#' @details Variable names are as in the reference.
#' 
#' @references Agresti, A. (2024) Statistical Methods for the Social
#'     Sciences, Global Edition (6th edition).  ISBN-13: 9781292449197. Table 13.1
#' 
"income"



#' @title Wear of shoes
#' @description Wear of soles of shoes of materials A and B for one foot each for of ten boys.
#' @concept dataset
#' @details The shoes data are measurements of the amount wear of the
#'     soles of shoes worn by 10 boys. The soles were made to two
#'     different synthetic materials, a standard material A and a
#'     cheaper material B.
#' 
#' @name shoes
#' @docType data
#' @format This data frame contains:
#' \describe{
#' \item{A:}{Wear, material A}
#' \item{B:}{Wear, material B}
#' \item{boy:}{Id of boy}
#' \item{footA:}{The foot with material A}
#' }
#' @keywords datasets
#'
#' @references Box, Hunter, Hunter (2005) Statistics for Experimenters, 2nd edition Wiley, p. 81.
#' 
NULL

#' @rdname shoes
"shoes"

#' @rdname shoes
"shoes_long"


#' @title crickets data
#' @description Mating songs of male tree crickets.
#' @name crickets
#' @concept dataset
#' @docType data
#' @format This data frame contains:
#' \describe{
#' \item{species:}{Species, (exis, nius), see details}
#' \item{temp:}{temperature}
#' \item{pps:}{pulse per second}
#' }
#' @keywords datasets
#'
#' @details
#'
#'  Walker (1962) studied the mating songs of male tree crickets. Each
#'  wingstroke by a cricket produces a pulse of song, and females may
#'  use the number of pulses per second to identify males of the
#'  correct species. Walker (1962) wanted to know whether the chirps
#'  of the crickets Oecanthus exclamationis (abbreviated exis) and
#'  Oecanthus niveus (abbreviated nius) had different pulse rates. See
#'  the biostathandbook for details. (The
#'  abbreviations are made from the the first two and last two letters
#'  of the species.) Walker measured the pulse rate of the crickets
#'  (variable `pps`) at a variety of temperatures (`temp`):
#'
#' @examples
#' 	
#' data(crickets)
#' coplot(pps ~ temp | species, data=crickets) 
"crickets"


##
## beets
##
#' @title beets data
#'
#' @description Yield and sugar percentage in sugar beets from a split plot
#'     experiment.  Data is obtained from a split plot experiment. There are 3
#'     blocks and in each of these the harvest time defines the "whole plot" and
#'     the sowing time defines the "split plot". Each plot was 25 square meters and
#'     the yield is recorded in kg. See 'details' for the experimental layout.
#'
#' @name beets
#' @concept dataset
#' @docType data
#' @format The format is: chr "beets"
#'
#' @details
#' \preformatted{  
#' Experimental plan
#' Sowing times            1        4. april
#'                         2       12. april
#'                         3       21. april
#'                         4       29. april
#'                         5       18. may
#' Harvest times           1        2. october
#'                         2       21. october
#' Plot allocation:
#'                Block 1     Block 2     Block 3
#'             +-----------|-----------|-----------+
#'       Plot  | 1 1 1 1 1 | 2 2 2 2 2 | 1 1 1 1 1 | Harvest time
#'        1-15 | 3 4 5 2 1 | 3 2 4 5 1 | 5 2 3 4 1 | Sowing time
#'             |-----------|-----------|-----------|
#'       Plot  | 2 2 2 2 2 | 1 1 1 1 1 | 2 2 2 2 2 | Harvest time
#'       16-30 | 2 1 5 4 3 | 4 1 3 2 5 | 1 4 3 2 5 | Sowing time
#'             +-----------|-----------|-----------+  
#' }
#'
#' @references Ulrich Halekoh, Søren Højsgaard (2014)., A Kenward-Roger
#'     Approximation and Parametric Bootstrap Methods for Tests in Linear Mixed
#'     Models - The R Package pbkrtest., Journal of Statistical Software,
#'     58(10), 1-30., \url{https://www.jstatsoft.org/v59/i09/}
#'
#' @keywords datasets
#'
#' @examples
#' data(beets)
#' 
#' beets$bh <- with(beets, interaction(block, harvest))
#' summary(aov(yield ~ block + sow + harvest + Error(bh), beets))
#' summary(aov(sugpct ~ block + sow + harvest + Error(bh), beets))
#' 
"beets"


#' Gene expression signatures for p53 mutation status in 250 breast cancer
#' samples
#' 
#' Perturbations of the p53 pathway are associated with more aggressive and
#' therapeutically refractory tumours. We preprocessed the data using Robust
#' Multichip Analysis (RMA). Dataset has been truncated to the 1000 most
#' informative genes (as selected by Wilcoxon test statistics) to simplify
#' computation. The genes have been standardized to have zero mean and unit
#' variance (i.e. z-scored).
#' 
#' The factor \code{code} defines whether there was a mutation in the p53
#' sequence (code=case) or not (code=control).
#' 
#' @name data_breastcancer
#' @concept dataset
#' @docType data
#'
#' @format A data frame with 250 observations on 1001 variables. The
#'     first 1000 columns are numerical variables; the last column
#'     (named \code{code}) is a factor with levels \code{case} and
#'     \code{control}.
#'
#' @references Miller et al (2005, PubMed
#'     ID:16141321)
#'
#' @source Chris Holmes, \email{c.holmes@@stats.ox.ac.uk}
#'
#' @keywords datasets
#'
#' @examples
#' 
#' data(breastcancer)
#' bc <- breastcancer
#' pairs(bc[,1:5], col=bc$code)
#'
#' train <- sample(1:nrow(bc), 50)
#' table(bc$code[train])
#' \dontrun{
#' library(MASS)
#' z <- lda(code ~ ., data=bc, prior = c(1, 1) / 2, subset = train)
#' pc <- predict(z, bc[-train, ])$class
#' pc
#' bc[-train, "code"]
#' table(pc, bc[-train, "code"])
#' }
#' 
"breastcancer"


##
## carcass
##

#' Lean meat contents of 344 pig carcasses
#' 
#' Measurement of lean meat percentage of 344 pig carcasses together with
#' auxiliary information collected at three Danish slaughter houses
#' 
#' @name carcass
#' @concept dataset
#' @aliases carcass carcassall
#' @format carcassall: A data frame with 344 observations on the following 17
#' variables.
#'  \describe{
#'   \item{\code{weight}}{Weight of carcass}
#'   \item{\code{lengthc}}{Length of carcass from back toe to head (when
#'     the carcass hangs in the back legs)}
#'   \item{\code{lengthf}}{Length of carcass from back toe to front leg
#'     (that is, to the shoulder)}
#'   \item{\code{lengthp}}{Length of carcass from back toe to the pelvic bone}
#'   \item{\code{Fat02, Fat03, Fat11, Fat12, Fat13, Fat14, Fat16}}{Thickness of fat
#'     layer at different locations on the back of the carcass (FatXX
#'     refers to thickness at (or rather next to) rib no. XX. Notice that
#'     02 is closest to the head}
#'   \item{\code{Meat11, Meat12, Meat13}}{Thickness of meat layer at different
#'     locations on the back of the carcass, see description above}
#'   \item{\code{LeanMeat}}{Lean meat percentage determined by dissection}
#'   \item{\code{slhouse}}{Slaughter house; a factor with levels \code{slh1} and  \code{slh2}.}
#'   \item{\code{sex}}{Sex of the pig; a factor with levels \code{castrate} and \code{female}.}
#'   \item{\code{size}}{Size of the carcass; a factor with levels \code{normal} and \code{large}.
#'       Here, `normal` refers to carcass weight under 80 kg; `large` refers to carcass weights between 80 and 110 kg.}
#'   
#' }
#'
#' @details: Notice that there were slaughtered large pigs only at one slaughter house.
#' @note carcass: Contains only the variables Fat11, Fat12, Fat13,
#'     Meat11, Meat12, Meat13, LeanMeat
#' @source Busk, H., Olsen, E. V., Brøndum, J. (1999) Determination of
#'     lean meat in pig carcasses with the Autofom classification
#'     system, Meat Science, 52, 307-314
#' @keywords datasets
#' @examples
#' data(carcass)
#' head(carcass)
#' 
"carcass"
"carcassall"


#' Diet of Atlantic cod in the Gulf of St. Lawrence (Canada)
#' 
#' Stomach content data for Atlantic cod (\emph{Gadus morhua}) in the Gulf of
#' St.Lawrence, Eastern Canada.  Note: many prey items were of no interest for
#' this analysis and were regrouped into the "Other" category.
#' 
#' Cod are collected either by contracted commerical fishing vessels
#' (\code{ship.type} 90 or 99) or by research vessels.  Commercial vessels are
#' identified by a unique \code{ship.id}.
#' 
#' Either one research vessel or several commercial vessels conduct a survey
#' (\code{trip}), during which a trawl, gillnets or hooked lines are set
#' several times. Most trips are random stratified surveys (depth-based
#' stratification).
#' 
#' Each trip takes place within one of the \code{region}s.  The \code{trip}
#' label is only guaranteed to be unique within a region and the \code{set}
#' label is only guaranteed to be unique within a \code{trip}.
#' 
#' For each fish caught, the \code{fish.length} is recorded and the fish is
#' allocated a \code{fish.id}, but the \code{fish.id} is only guaranteed to be
#' unique within a \code{set}. A subset of the fish caught are selected for
#' stomach analysis (stratified random selection according to fish length; unit
#' of stratification is the set for research surveys, the combination ship.id
#' and stratum for surveys conducted by commercial vessels, although strata are
#' not shown in codstom).
#' 
#' The basic experimental unit in this data set is a cod stomach (one stomach
#' per fish).  Each stomach is uniquely identified by a combination of
#' \code{region}, \code{ship.type}, \code{ship.id}, \code{trip}, \code{set},
#' and \code{fish.id}.
#' 
#' For each prey item found in a stomach, the species and mass of the prey item
#' are recorded, so there can be multiple observations per stomach.  There may
#' also be several prey items with the same \code{prey.type} in the one stomach
#' (for example many \code{prey.types} have been recoded \code{Other}, which
#' produced many instances of \code{Other} in the same stomach).
#' 
#' If a stomach is empty, a single observation is recorded with
#' \code{prey.type} \code{Empty} and a \code{prey.mass} of zero.
#' 
#' @name codstom
#' @docType data
#'
#' @format A data frame with 10000 observations on the following 10 variables.
#' \describe{
#'   \item{\code{region}}{a factor with levels \code{SGSL} \code{NGSL} 
#'         representing the southern and northern Gulf of St. Lawrence, respectively}
#'   \item{\code{ship.type}}{a factor with levels \code{2} \code{3} \code{31} 
#'   \code{34} \code{90} \code{99}}
#'   \item{\code{ship.id}}{a factor with levels \code{11558} \code{11712}
#'     \code{136148} \code{136885}
#'     \code{136902} \code{137325} \code{151225} \code{151935} \code{99433}}
#'   \item{\code{trip}}{a factor with levels \code{10} \code{11}
#'     \code{12} \code{179} \code{1999}
#'     \code{2} \code{2001} \code{20020808} \code{3} \code{4} \code{5}
#'     \code{6} \code{7} \code{8}
#'     \code{88} \code{9} \code{95}}     
#'   \item{\code{set}}{a numeric vector}
#'   \item{\code{fish.id}}{a numeric vector}
#'   \item{\code{fish.length}}{a numeric vector, length in mm}
#'   \item{\code{prey.mass}}{a numeric vector, mass of item in stomach, in g}
#'   \item{\code{prey.type}}{a factor with levels \code{Ammodytes_sp}
#'     \code{Argis_dent}
#'     \code{Chion_opil} \code{Detritus} \code{Empty} \code{Eualus_fab}
#'     \code{Eualus_mac} \code{Gadus_mor} \code{Hyas_aran}
#'     \code{Hyas_coar}
#'     \code{Lebbeus_gro} \code{Lebbeus_pol} \code{Leptocl_mac}
#'     \code{Mallot_vil}
#'     \code{Megan_norv} \code{Ophiuroidea} \code{Other} \code{Paguridae}
#'     \code{Pandal_bor} \code{Pandal_mon} \code{Pasiph_mult}
#'     \code{Sabin_sept}
#'     \code{Sebastes_sp} \code{Them_abys} \code{Them_comp} \code{Them_lib}}    
#' }
#' @source Small subset from a larger dataset (more stomachs, more variables,
#'     more \code{prey.types}) collected by D. Chabot and M. Hanson, Fisheries &
#'     Oceans Canada \email{chabotd@@dfo-mpo.gc.ca}.
#' @concept dataset
#' @keywords datasets
#' @examples
#' 
#' data(codstom)
#' str(codstom)
#' # removes multiple occurences of same prey.type in stomachs
#' codstom1 <- summaryBy(prey.mass ~ 
#'                       region + ship.type + ship.id + trip + set + fish.id + prey.type,
#'                       data = codstom, 
#'                       FUN = sum) 
#' 
#' # keeps a single line per stomach with the total mass of stomach content
#' codstom2 <- summaryBy(prey.mass ~ region + ship.type + ship.id + trip + set + fish.id,
#'                       data = codstom, 
#'                       FUN = sum) 
#' 
#' # mean prey mass per stomach for each trip
#' codstom3 <- summaryBy(prey.mass.sum ~ region + ship.type + ship.id + trip,
#'                       data = codstom2, FUN = mean) 
#' 
#' \dontrun{          
#' # wide version, one line per stomach, one column per prey type
#' library(reshape)
#' codstom4 <- melt(codstom, id = c(1:7, 9))
#' codstom5 <- cast(codstom4, 
#'                  region + ship.type + ship.id + trip + set + fish.id + fish.length ~ 
#'                  prey.type, sum)
#' k <- length(names(codstom5))
#' prey_col <- 8:k
#' out <- codstom5[,prey_col]
#' out[is.na(out)] <- 0
#' codstom5[,prey_col] <- out
#' codstom5$total.content <- rowSums(codstom5[, prey_col])
#' }
#' 
"codstom"


#' crimeRate
#' 
#' Crime rates per 100,000 inhabitants in states of the USA for different crime
#' types in 1977.
#' 
#' @name crimeRate
#' @concept dataset
#' @docType data
#' @format This data frame contains:
#' \describe{
#' \item{state:}{State of the USA}
#' \item{murder:}{crime of murder}
#' \item{rape:}{}
#' \item{robbery:}{}
#' \item{assault:}{}
#' \item{burglary:}{residential theft}
#' \item{larceny:}{unlawful taking of personal property (pocket picking)}
#' \item{autotheft:}{}
#' }
#' @keywords datasets
#' @examples
#' 	
#' data(crimeRate)
#' 
"crimeRate"

#' crimeRate
#' 
#' Crime rates per 100,000 inhabitants in states of the USA for different crime
#' types in 1977.
#' 
#' @name crime_rate
#' @concept dataset
#' @docType data
#' @format This data frame contains:
#' \describe{
## #' \item{State:}{State of the USA}
#' \item{murder:}{crime of murder}
#' \item{rape:}{}
#' \item{robbery:}{}
#' \item{assault:}{}
#' \item{burglary:}{residential theft}
#' \item{larceny:}{unlawful taking of personal property (pocket picking)}
#' \item{autotheft:}{}
#' }
#' @keywords datasets
#' @examples
#' 	
#' data(crime_rate)
#' 
"crime_rate"






##
## dietox
##


#' Growth curves of pigs in a 3x3 factorial experiment
#' 
#' The \code{dietox} data frame has 861 rows and 7 columns.
#'
#' @details Data contains weight of slaughter pigs measured weekly for 12
#'     weeks. Data also contains the start weight (i.e. the weight at week
#'     1). The treatments are 3 different levels of Evit = vitamin E (dose: 0,
#'     100, 200 mg dl-alpha-tocopheryl acetat /kg feed) in combination with 3
#'     different levels of Cu=copper (dose: 0, 35, 175 mg/kg feed) in the feed.
#'     The cumulated feed intake is also recorded. The pigs are litter mates.
#' 
#' @concept dataset
#' @format This data frame contains the following columns:
#' 
#' \describe{
#' \item{Weight}{Weight in Kg}
#' \item{Feed}{Cumulated feed intake in Kg}
#' \item{Time}{Time (in weeks) in the experiment}
#' \item{Pig}{Factor; id of each pig}
#' \item{Evit}{Factor; vitamin E dose; see 'details'.}
#' \item{Cu}{Factor, copper dose; see 'details'}
#' \item{Start}{Start weight in experiment, i.e. weight at week 1.}
#' \item{Litter}{Factor, id of litter of each pig}
#' }
#' 
#' @source Lauridsen, C., Højsgaard, S.,Sørensen, M.T. C. (1999) Influence of
#'     Dietary Rapeseed Oli, Vitamin E, and Copper on Performance and
#'     Antioxidant and Oxidative Status of Pigs. J. Anim. Sci.77:906-916
#' @keywords datasets
#' @examples
#' 
#' data(dietox)
#' head(dietox)
#' coplot(Weight ~ Time | Evit * Cu, data=dietox)
#' 
"dietox"






##
## fatacid
##

#' @title Fish oil in pig food
#'
#' @description Fish oil in pig food
#'
#' @name fatacid
#' @docType data
#' @concept dataset
#' @format A dataframe.
#'
#' @details A fish oil fatty acid \code{X14} has been added in
#'     different concentrations to the food for pigs in a
#'     study. Interest is in studying how much of the fatty acid can
#'     be found in the tissue. The concentrations of \code{x14} in the
#'     food are \code{verb+dose+=\{0.0, 4.4, 6.2, 9.3\}}.
#'
#' The pigs are fed with this food until their weight is 60 kg.  From
#' thereof and until they are slaughtered at 100kg, their food does
#' not contain the fish oil.
#' At 60kg (sample=1) and 100kg (sample=2) muscle
#' biopsies are made and the concentration of x14 is
#' determined.
#' Measurements on the same pig are correlated, and pigs are additionally
#' related through litters.
#'
#' @references Data courtesy of Charlotte Lauridsen, Department of
#'     Animal Science, Aarhus University, Denmark.
"fatacid"


#' @title Forced expiratory volume in children
#'
#' @description Dataset to examine if respiratory function in children
#'   was influenced by smoking.
#'
#' @name fev
#' @docType data
#'
#' @format A data frame with 654 observations on the following 5 variables.
#' \describe{
#'   \item{\code{Age}}{Age in years.}
#'   \item{\code{FEV}}{Forced expiratory volume in liters per second.}
#'   \item{\code{Ht}}{Height in centimeters.}
#'   \item{\code{Gender}}{Gender.}
#'   \item{\code{Smoke}}{Smoking status.}
#' }
#'
#'
#' @references I. Tager and S. Weiss and B. Rosner and F. Speizer (1979). Effect
#' of Parental Cigarette Smoking on the Pulmonary Function of
#' Children.  American Journal of Epidemiology. 110:15-26
#'
#' @examples
#'
#' data(fev)
#' summary(fev)
#' 
"fev"


#' Heat development in cement under hardening.
#' 
#' Heat development in cement under hardening related to the chemical
#' composition.
#' 
#' @name haldCement
#' @docType data
#' @concept dataset
#' @format A data frame with 13 observations on the following 5 variables.
#'   \describe{
#'     \item{\code{x1}}{Percentage (weight) of `[3Ca0][Al2O3]`}
#'     \item{\code{x2}}{Percentage (weight) of `[3Cao][SiO2]`}
#'     \item{\code{x3}}{Percentage (weight) of `[4Ca0][Al2O3][Fe03]`}
#'     \item{\code{x4}}{Percentage (weight) of `[2Cao][SiO2]`}
#'     \item{\code{y}}{Heat development measured in calories per
#'       gram cement after 180 days}
#'   }
#'
#' @references Anders Hald (1949); Statistiske Metoder; Akademisk Forlag (in
#' Danish), page 509.
#' @keywords datasets
#' @examples
#' 
#' data(haldCement)
#' 
#' if( interactive() ){
#' pairs( haldCement )
#' }
#' m <- lm(y ~ x1 + x2 + x3 + x4, data=haldCement)
#' summary(m)
#' 
#' # Notice: The model explains practically all variation in data;
#' # yet none of the explanatory variables appear to be statistically
#' # significant.
#'
"haldCement"


#' Yield from Danish agricultural production of grain and root crop. 
#'
#' @name cropyield
#' @docType data
#' @format A dataframe with 97 rows and 7 columns. 
#'   \describe{
#'     \item{\code{year}}{From 1901 to 1997.}
#'     \item{\code{precip}}{Milimeter precipitation.}
#'     \item{\code{yield}}{Million feed units (see details).}
#'     \item{\code{area}}{Area in 1000 ha for grains and root crop.}
#'     \item{\code{fertil}}{1000 tons fertilizer.}
#'     \item{\code{avgtmp1}}{Average temperature April-June (3 months).}
#'     \item{\code{avgtmp2}}{Average temperature July-Octobre (4 months). }
#'   }
#'
#' @details A feed unit is the amount of energy in a kg of barley.
#' @references Danmarks statistik (Statistics Denmark).
#' @keywords datasets
"cropyield"



#' Milk yield data for manually milked cows.
#' 
#' Milk yield data for cows milked manually twice a day (morning and evening).
#' 
#' There are data for 222 cows. Some cows appear more than once in the dataset
#' (in different lactations) and there are 288 different lactations.
#'
#' @aliases milkman_rdm1
#' @concept dataset
#' @name milkman
#' @docType data
#'
#' @format
#' A data frame with 161836 observations on the following 12 variables.
#' \describe{
#'   \item{\code{cowno}}{a numeric vector; cow identification}
#'   \item{\code{lactno}}{a numeric vector; lactation number}
#'   \item{\code{ampm}}{a numeric vector; milking time: 1: morning; 2: evening}
#'   \item{\code{dfc}}{a numeric vector; days from calving}
#'   \item{\code{my}}{a numeric vector; milk yield (kg)}
#'   \item{\code{fatpct}}{a numeric vector; fat percentage}
#'   \item{\code{protpct}}{a numeric vector; protein percentage}
#'   \item{\code{lactpct}}{a numeric vector; lactose percentage}
#'   \item{\code{scc}}{a numeric vector; somatic cell counts}
#'   \item{\code{race}}{a factor with levels \code{RDM} \code{Holstein} \code{Jersey}}
#'   \item{\code{ecmy}}{a numeric vector; energy corrected milk}
#'   \item{\code{cowlact}}{Combination of cowno and lactno; necessary
#'     because the same cow may appear more than once in the dataset (in
#'     different lactations)}
#' }
#'
#' @keywords datasets
#'
#' @references Friggens, N. C.; Ridder, C. and Løvendahl, P. (2007). 
#' On the Use of Milk Composition Measures to Predict the Energy Balance of Dairy Cows.
#' J. Dairy Sci. 90:5453–5467 doi:10.3168/jds.2006-821.
#'
#' This study was part of the Biosens project used data from the
#' “Malkekoens energibalance og mobilisering” project; both were
#' funded by the Danish Ministry of Food, Agriculture and Fisheries
#' and the Danish Cattle Association.
#' 
#' @examples
#' 
#' data(milkman)
#' 
"milkman"
"milkman_rdm1"







#' Weight and size of 20 potatoes
#' 
#' Weight and size of 20 potatoes. Weight in grams; size in millimeter. There
#' are two sizes: \code{length} is the longest length and \code{width} is the
#' shortest length across a potato.  
#' 
#' @name potatoes
#' @docType data
#' @concept dataset
#' @format A data frame with 20 observations on the following 3 variables.
#'   \describe{
#'     \item{\code{weight}}{a numeric vector}
#'     \item{\code{length}}{a numeric vector}
#'     \item{\code{width}}{a numeric vector}
#'   }
#' 
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @source My own garden; autumn 2015.
#' @keywords datasets
#' @examples
#' 
#' data(potatoes)
#' plot(potatoes) 
#' 
"potatoes"


#' Mathematics marks for students
#' 
#' The \code{mathmark} data frame has 88 rows and 5 columns.
#'
#' @name data_mathmark
#' @concept dataset
#' @aliases mathmark math
#' @format This data frame contains the following columns: mechanics, vectors,
#' algebra, analysis, statistics.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @references David Edwards, An Introduction to Graphical Modelling, Second
#' Edition, Springer Verlag, 2000
#' @keywords datasets
#' @usage data(mathmark)
#' 
#' @examples
#' 
#' data(mathmark)
#' 
"mathmark"
"math"


#' @title Budworm data
#' 
#' @description Experiment on the toxicity to the tobacco budworm
#'     Heliothis virescens of doses of the pyrethroid
#'     trans-cypermethrin to which the moths were beginning to show
#'     resistance. Batches of 20 moths of each sex were exposed for
#'     three days to the pyrethroid and the number in each batch that
#'     were dead or knocked down was recorded. Data is reported in
#'     Collett (1991, p. 75).
#'
#' @concept data
#' @name data_budworm
#' @docType data
#' @concept dataset
#' @format This data frame contains 12 rows and 4 columns:
#'
#' \describe{
#' \item{sex:}{sex of the budworm.}
#' \item{dose:}{dose of the insecticide trans-cypermethrin (in micro grams)}.
#' \item{ndead:}{budworms killed in a trial.}
#' \item{ntotal:}{total number of budworms exposed per trial.}
#' }
#'
#' @references Venables, W.N; Ripley, B.D.(1999) Modern Applied Statistics with
#' S-Plus, Heidelberg, Springer, 3rd edition, chapter 7.2
#'
#' @source Collett, D. (1991) Modelling Binary Data, Chapman & Hall, London,
#' Example 3.7
#'
#' 
#' @keywords datasets
#' @examples
#' 
#' data(budworm)
#' 
#' ## function to caclulate the empirical logits
#' empirical.logit<- function(nevent,ntotal) {
#'    y <- log((nevent + 0.5) / (ntotal - nevent + 0.5))
#'    y
#' }
#' 
#' 
#' # plot the empirical logits against log-dose
#' 
#' log.dose <- log(budworm$dose)
#' emp.logit <- empirical.logit(budworm$ndead, budworm$ntotal)
#' plot(log.dose, emp.logit, type='n', xlab='log-dose',ylab='emprirical logit')
#' title('budworm: emprirical logits of probability to die ')
#' male <- budworm$sex=='male'
#' female <- budworm$sex=='female'
#' lines(log.dose[male], emp.logit[male], type='b', lty=1, col=1)
#' lines(log.dose[female], emp.logit[female], type='b', lty=2, col=2)
#' legend(0.5, 2, legend=c('male', 'female'), lty=c(1,2), col=c(1,2))
#' 
#' \dontrun{
#' * SAS example;
#' data budworm;
#' infile 'budworm.txt' firstobs=2;
#' input sex dose ndead ntotal;
#' run;
#' }
#' 
#' 
"budworm"


#' Coronary artery disease data
#' 
#' A cross classified table with observational data from a Danish
#' heart clinic.  The response variable is CAD (coronary artery
#' disease, some times called heart attack).
#'
#' @details
#'
#' Notice that data are collected at a heart clinic, so data do not
#' represent the population, but are conditional on patients having
#' ended up at the clinic.
#' 
#' * cad1: Complete dataset, 236 cases.
#'
#' * cad2: Incomplete dataset, 67 cases. Information on (some of) the
#'     variables 'Hyperchol', 'Smoker' and 'Inherit' is missing.
#'
#' @concept dataset
#' @name data_cad
#' @aliases cad1 cad2
#' @docType data
#' @format A data frame with 236 observations on the following 14 variables.
#'
#' \describe{
#'   \item{\code{Sex}}{Sex; a factor with levels \code{Female} \code{Male}}
#'
#'   \item{\code{AngPec}}{Angina pectoris (chest pain attacks); a
#'   factor with levels \code{Atypical} \code{None} \code{Typical}}
#'
#'   \item{\code{AMI}}{Acute myocardic infarct; a factor with
#'   levels \code{Definite} \code{NotCertain}}
#'
#'   \item{\code{QWave}}{A reading from an electrocardiogram; a
#'   factor with levels \code{No} \code{Yes}; Yes means pathological and is a sign of previous myocardial infarction. }
#'
#'   \item{\code{QWavecode}}{a factor with levels \code{Nonusable}
#'   \code{Usable}. An assesment of whether QWave is reliable.}
#'
#'   \item{\code{STcode}}{a factor with levels
#'   \code{Nonusable} \code{Usable}. An assesment of whether STchange is reliable.}
#'
#'   \item{\code{STchange}}{A reading from an electrocardiogram; a factor
#'   with levels \code{No} \code{Yes}. An STchange indicates a blockage of the coronary artery.}
#'
#'   \item{\code{SuffHeartF}}{Sufficient heart frequency; a factor with levels \code{No}, \code{Yes}}
#' 
#'   \item{\code{Hypertrophi}}{a factor with levels \code{No}, \code{Yes}. Hypertrophy refers to an
#'   increased size of the heart muscle due to exercise. }
#'
#'   \item{\code{Hyperchol}}{a factor with levels \code{No} \code{Yes}. Hypercholesterolemia, also called high cholesterol,
#'    is the presence of high levels of cholesterol in the blood.}
#'
#'   \item{\code{Smoker}}{Is the patient a smoker; a factor with levels \code{No}, \code{Yes}.}
#'
#'   \item{\code{Inherit}}{Hereditary predispositions for CAD; a factor with levels  \code{No}, \code{Yes}.}
#'
#'   \item{\code{Heartfail}}{Previous heart failures; a factor with  levels \code{No} \code{Yes}}
#'
#'   \item{\code{CAD}}{Coronary Artery Disease; a factor with levels
#'    \code{No} \code{Yes}}.  CAD refers to a reduction of blood flow
#'    to the heart muscle (commonly known as a heart attack). The
#'    diagnosis made from biopsies.
#'
#' }
#'
#' 
#' @references Hansen, J. F. (1980). The clinical diagnosis of
#'     ischaemic heart disease due to coronary artery disease. Danish
#'     Medical Bulletin
#'
#' Højsgaard, Søren and Thiesson, Bo (1995). BIFROST - Block recursive
#' models Induced From Relevant knowledge, Observations and
#' Statistical Techniques. Computational Statistics and Data Analysis,
#' vol. 19, p. 155-175
#' #'
#' @keywords datasets
#' @usage data(cad1)
#' 
#' @examples
#' 
#' data(cad1)
#' ## maybe str(cad1) ; plot(cad1) ...
#' 
"cad1"
"cad2"


#' Personality traits
#'
#' The `peronality` dataframe has 240 rows and 32 columns
#'
#' @name data_personality
#' @concept dataset
#' @format This dataframe has recordings on the following 32
#'     variables: distant, talkatv, carelss, hardwrk, anxious,
#'     agreebl, tense, kind, opposng, relaxed, disorgn, outgoin,
#'     approvn, shy, discipl, harsh, persevr, friendl, worryin,
#'     respnsi, contrar, sociabl, lazy, coopera, quiet, organiz,
#'     criticl, lax, laidbck, withdrw, givinup, easygon
#'
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @references Origin unclear
#' @keywords datasets
#' @usage data(personality)
#' @examples
#' data(personality)
#' str(personality)
"personality"

#' @title Split matrix or dataframe into list
#'
#' @description Split matrix or dataframe into list by columns or by rows
#'
#' @name split_byrow_bycol
#' @param x Matrix or dataframe.
#' @param idx Index to split by. If NULL, split by columns or rows.
#' @param as.list If TRUE, return list of dataframes. If FALSE, return list of matrices.
#'
#' @examples
#' x <- mtcars[1:3, 1:6]
#' x  |> split_bycol()
#' x  |> split_bycol(as.list=TRUE)
#' x  |> split_bycol(as.list=FALSE)
#' x  |> split_bycol(idx=c(1,1,1,2,2,3,3,3))
#' ## x  |> split_bycol(idx=c(1,1,7,2,2,3,3,3)) ## Gives error
#' 
#' x <- mtcars[1:6, 1:6]

#' x  |> split_byrow()
#' x  |> split_byrow(idx=c(1,1,2,2))
#' 


#' @export
#' @rdname split_byrow_bycol
split_bycol <- function(x, idx=NULL, as.list=FALSE) {
  if (!inherits(x, c("matrix", "data.frame")))
    stop("'x' must be matrix or dataframe\n")

   if (ncol(x) == 0) {
       return(list()) 
   } 

  isdf <- inherits(x, "data.frame")

  if (as.list){
      out <- as.list(as.data.frame(x))            
  } else {
      if (is.null(idx))
          idx <- 1:ncol(x)
      if (!(is.numeric(idx))) {
          stop("'idx' must be numeric\n")
      }
      if (!(length(idx) == ncol(x))){
          idx <- rep(idx, length.out = ncol(x))
      }
      if (max(idx)>ncol(x))
          stop("max idx exceeds number of columns\n")

      out <- lapply(seq_along(unique(idx)),
                    function(i) {
                        zz <- x[, idx == i, drop=FALSE]
                        names(zz) <- names(x)[idx == i]
                        zz
                    })
  }

  if (!isdf){
      out <- lapply(out, as.matrix)
  }
  return(out)
}    

#' @examples
#' m <- as.matrix(x)
#' u <- x |> split_byrow(idx=c(1,1,2,2))
#' y <- m |> split_byrow(idx=c(1,1,2,2))
#' 
#' @export
#' @rdname split_byrow_bycol
split_byrow <- function (x, idx=NULL)  {
    if (!inherits(x, c("matrix", "data.frame"))) 
        stop("'x' must be matr ix or dataframe\n")

  isdf <- inherits(x, "data.frame")
  
   if (nrow(x) == 0) { 
        return(list())
    }

    if (is.null(idx)) {
        idx <- 1:nrow(x)
    }

    if (!(length(idx) == nrow(x))){
      idx <- rep(idx, length.out = nrow(x))
    }
  
    out <- split(x, idx)

    if (!isdf){
        out <- lapply(out, function(z){
          matrix(z, ncol=ncol(x))
    })
    }
    

    if (!is.null(colnames(x))) {
        out <- lapply(out, function(z){
          colnames(z) <- colnames(x)
          z
        })
    }
    return(out)
}

.check_forecast_input <- function(fc_object, trans_fun, y0, level) {

  ## Check forecast object
  if (!inherits(fc_object, "forecast"))
    stop("`fc_object` must be a 'forecast' object from the forecast package.", call. = FALSE)

  ## Check model supports simulate()
  if (is.null(fc_object$model))
    stop("`fc_object$model` is NULL - cannot simulate future paths.", call. = FALSE)

  if (!is.function(stats::simulate))
    stop("Internal error: stats::simulate() not found.", call. = FALSE)

  ## Check that simulate() works on the model
  test_sim <- try(stats::simulate(fc_object$model, nsim = 1, future = TRUE), silent = TRUE)
  if (inherits(test_sim, "try-error"))
    stop("`fc_object$model` does not support simulate(..., future = TRUE).",
         "\nModel class: ", paste(class(fc_object$model), collapse = ", "),
         call. = FALSE)

  ## Check level
  if (!is.numeric(level) || length(level) != 1 || level <= 0 || level >= 100)
    stop("`level` must be a single number between 0 and 100.", call. = FALSE)

  ## Check y0
  if (!is.numeric(y0) || length(y0) != 1)
    stop("`y0` must be a single numeric value.", call. = FALSE)

  ## Check trans_fun
  if (!is.function(trans_fun))
    stop("`trans_fun` must be a function of the form function(x, y0).", call. = FALSE)

  ## Try calling trans_fun to detect structural errors
  z <- try(trans_fun(c(1,2,3), y0 = y0), silent = TRUE)
  if (inherits(z, "try-error"))
    stop("`trans_fun` failed with test input: trans_fun(c(1,2,3), y0).\n",
         "Please ensure it has the form function(x, y0).", call. = FALSE)

  ## trans_fun must return numeric vector of same length
  if (!is.numeric(z) || length(z) != 3)
    stop("`trans_fun(x, y0)` must return a numeric vector of the same length as `x`.",
         call. = FALSE)

  ## Check fc_object$x exists and is numeric ts
  if (is.null(fc_object$x))
    stop("`fc_object$x` is NULL; cannot reconstruct history.", call. = FALSE)
  if (!is.numeric(fc_object$x))
    stop("`fc_object$x` must be numeric.", call. = FALSE)
  if (is.null(tsp(fc_object$x)))
    stop("`fc_object$x` must be a time series (ts) object.", call. = FALSE)

  ## Check forecast horizon
  if (!is.numeric(fc_object$mean) || is.null(tsp(fc_object$mean)))
    stop("`fc_object$mean` must be a numeric time series (ts).", call. = FALSE)

  invisible(TRUE)
}


#' Transform forecasts from model scale to data scale by simulation
#'
#' @description
#' `transform_forecast()` takes a univariate time series forecast fitted on a
#' transformed (model) scale and produces a new forecast object on the
#' original data scale. This is done by simulating future paths on the
#' model scale, transforming each path with a user-supplied function, and
#' then computing pointwise means and prediction intervals on the
#' transformed scale.
#'
#' @details
#' The function assumes that `fc_object` is a `"forecast"` object as
#' produced by the \pkg{forecast} package, and that `fc_object$model`
#' supports `simulate()` with arguments `nsim` and `future`.
#'
#' The transformation function `trans_fun` must have the form
#' `trans_fun(x, y0)`, where `x` is a numeric vector representing a path on
#' the model scale (for example log-values or percentage changes), and
#' `y0` is a scalar "starting value" on the output scale. The function must
#' return a numeric vector of the same length as `x` giving the
#' corresponding path on the output scale.
#'
#' Internally, `transform_forecast()` first reconstructs a historical series on
#' the output scale by applying `trans_fun()` to `fc_object$x` with the
#' supplied `y0`. It then simulates `nsim` future paths from
#' `fc_object$model` on the model scale, transforms each path to the
#' output scale using `trans_fun()` with `y0` equal to the last value of
#' the reconstructed historical series, and finally computes the pointwise
#' mean and prediction intervals (of nominal coverage `level`) across
#' the simulated paths. The result is returned as a new `"forecast"` object
#' with `x`, `mean`, `lower`, and `upper` on the output scale.
#'
#' @param fc A `"forecast"` object (from the \pkg{forecast} package)
#'   for a univariate time series, typically obtained via
#'   \code{forecast::forecast()}.
#' @param trans_fun A function of the form \code{function(x, y0) ...}
#'   that takes a numeric vector `x` on the model scale and a scalar
#'   starting value `y0` on the output scale and returns a numeric vector
#'   of the same length on the output scale.
#' @param xreg_future Optional matrix or vector of future values for regressors in ARMAX model. 
#' @param nsim Integer; number of simulated future paths to use. Larger
#'   values give smoother prediction intervals but take longer to compute.
#' @param level Numeric; prediction interval coverage in percent.
#' @param y0 Numeric; starting value on the output scale used to
#'   reconstruct the historical series from `fc_object$x`. For percentage
#'   change models it is often natural to set \code{y0 = 1} and interpret
#'   the resulting series as an index.
#'
#' @return
#' A `"forecast"` object similar to `fc_object`, but with the components
#' `x`, `mean`, `lower`, and `upper` defined on the output (data) scale.
#'
#' @seealso
#' \code{\link[forecast]{forecast}}, \code{\link[forecast]{auto.arima}},
#' \code{\link[stats]{simulate}}, \code{\link[stats]{ts}}.
#'
#' @examples
#' ## Example 1: Log-transform of the Canadian lynx data
#' if (requireNamespace("forecast", quietly = TRUE)) {
#'
#'   llynx   <- log(lynx)
#'   fit_log <- forecast::auto.arima(llynx)
#'   fc_log  <- forecast::forecast(fit_log, h = 20)
#'   forecast::autoplot(fc_log)
#'   ## transformation: log -> original scale
#'   trans_log <- function(z, y0) {
#'     exp(z)
#'   }
#'   fc_lynx <- transform_forecast(fc_log, trans_fun = trans_log,
#'                             nsim = 20)
#'   forecast::autoplot(fc_lynx) + ggplot2::theme_minimal()
#' }
#'
#'
#' \dontrun{
#' if (requireNamespace("forecast", quietly = TRUE)) {
#' ## Example 2 (variation): CO2 series, log-transform
#'   lco2    <- log(co2)
#'   fit_co2 <- forecast::auto.arima(lco2)
#'   fc_log_co2 <- forecast::forecast(fit_co2, h = 24)
#'   forecast::autoplot(fc_log_co2)
#'   trans_log <- function(z, y0) exp(z)
#'
#'   fc_co2 <- transform_forecast(fc_log_co2, trans_fun = trans_log,
#'                          nsim = 30)
#'   forecast::autoplot(fc_co2) + ggplot2::theme_minimal()
#'}
#'
#' # ## Example 3: Percentage change in income and consumption
#if (requireNamespace("forecast", quietly = TRUE) &&
#'     requireNamespace("fpp2", quietly = TRUE)) {
#'   income <- uschange[, "Income"]  # quarterly percentage changes (%)
#'   consumption <- uschange[, "Consumption"]  # quarterly percentage changes (%)
#' 
#'  forecast::checkresiduals(income)
#'  forecast::checkresiduals(consumption)
#'  
#'   trans_pct <- function(r, y0) {
#'   y0 * cumprod(1 + r / 100)
#' }
#' 
#' ## ARIMA model for income
#' fit_income_pct <- forecast::auto.arima(income)
#' fit_income_pct
#'
#' fc_income_pct  <- forecast::forecast(fit_income_pct, h = 48)
#' forecast::autoplot(fc_income_pct)
#' 
#'  fc_income  <- transform_forecast(fc_income_pct, trans_fun = trans_pct,
#'                            nsim = 200, level = 95, y0 = 1)
#'  forecast::autoplot(fc_income) + ggplot2::theme_minimal()
#'  
#'  ## ARIMA model for consumption
#' 
#'  fit_cons_pct0 <- forecast::auto.arima(consumption, seasonal = F)
#'  fit_cons_pct0
#'  fc_cons_pct0  <- forecast::forecast(fit_cons_pct0, h = 48)
#'  
#'  fc_cons0  <- transform_forecast(fc_cons_pct0, trans_fun = trans_pct, 
#'                                 nsim = 200, level = 95, y0 = 1)
#'  forecast::autoplot(fc_cons0)
#'  
#'  ## ARIMAX model for consumption with income as xreg
#'   
#'  fit_cons_pct <- forecast::auto.arima(consumption, xreg = income)
#'  fit_cons_pct
#'  
#'  fc_income_pct$mean |> head()
#'  
#'  fc_cons_pct  <- forecast::forecast(fit_cons_pct, h = 48, 
#'                  xreg = fc_income_pct$mean)
#'  forecast::autoplot(fc_cons_pct)
#'  
#'  fc_cons  <- transform_forecast(fc_cons_pct, trans_fun = trans_pct, 
#'                                 xreg_future = fc_income$mean, 
#'                                 nsim = 200, level = 95, y0 = 1)
#'  forecast::autoplot(fc_cons)
#' 
#' #'  
#'  }
#'  }
#'  
#' @export
transform_forecast <- function(fc,
                               trans_fun,          # e.g. trans_pct
                               xreg_future = NULL,
                               nsim        = 1000,
                               level       = 95,
                               y0          = 1    # baseline index level
                               ) {

  if (!inherits(fc, "forecast"))
    stop("'fc' must be an object of class 'forecast'")

  fit <- fc$model
  if (!inherits(fit, "Arima"))
    stop("'fc$model' must be an 'Arima' object from forecast::Arima()")

  h <- length(fc$mean)

  ## --- Construct index for the observed data -----------------------------

  # fc$x is the historical percentage-change series r_t
  r_hist <- as.numeric(fc$x)

  # Index for history based at y0 (e.g. 1):
  # y_t = y0 * cumprod(1 + r_t/100)
  y_hist <- trans_fun(r_hist, y0 = y0)

  # Last historical index level
  y_last <- tail(y_hist, 1)
##  cat("Last historical index level (y_last): ", y_last, "\n")

  ## --- xreg handling -----------------------------------------------------
  model_has_xreg <- !is.null(fit$xreg)

  if (model_has_xreg && is.null(xreg_future)) {
    stop("Model was fitted with xreg; you must supply 'xreg_future'.")
  }

  if (!model_has_xreg && !is.null(xreg_future)) {
    warning("Model was fitted without xreg; 'xreg_future' will be ignored.")
    xreg_future <- NULL
  }

  if (!is.null(xreg_future)) {
    xreg_future <- as.matrix(xreg_future)
    if (nrow(xreg_future) != h) {
      stop("nrow(xreg_future) must equal forecast horizon h = ", h,
           ", but is ", nrow(xreg_future), ".")
    }
  }

  ## --- Levels for prediction intervals ----------------------------------
  level <- sort(as.numeric(level))
  if (any(level <= 0 | level >= 100))
    stop("'level' must be between 0 and 100")

  ## --- Simulate future percentage changes and backtransform -------------
  sim_mat <- replicate(nsim, {
    r_sim <- if (is.null(xreg_future)) {
      simulate(fit, nsim = h, future = TRUE)
    } else {
      simulate(fit, nsim = h, future = TRUE, xreg = xreg_future)
    }

    r_sim <- as.numeric(r_sim)

    # Backtransform: future index starting from *last observed* index level
    trans_fun(r_sim, y0 = y_last)
  })

  # h x nsim matrix of index values
  mean_fc <- rowMeans(sim_mat)

  alpha <- (100 - level) / 200
  probs <- rbind(alpha, 1 - alpha)
  lower <- upper <- matrix(NA_real_, nrow = h, ncol = length(level))
  for (j in seq_along(level)) {
    q <- apply(sim_mat, 1, quantile, probs = probs[, j])
    lower[, j] <- q[1, ]
    upper[, j] <- q[2, ]
  }
  colnames(lower) <- colnames(upper) <- paste0(level, "%")

  ## --- Time structure ----------------------------------------------------
  x_old   <- fc$x
  mean_ts <- fc$mean

  # ts for history index
  y_hist_ts <- ts(y_hist,
                  start     = start(x_old),
                  frequency = frequency(x_old))

  # ts for forecast mean
  mean_bt <- ts(mean_fc,
                start     = start(mean_ts),
                frequency = frequency(mean_ts))

  ## --- Build new forecast object ----------------------------------------
  fc_new        <- fc
  fc_new$x      <- y_hist_ts     # history on index/“absolute” scale
  fc_new$mean   <- mean_bt       # back-transformed forecasts
  fc_new$lower  <- lower
  fc_new$upper  <- upper
  fc_new$level  <- level
  fc_new$method <- paste0(fc$method,
                          " (back-transformed from % changes via simulation)")

  fc_new
}

























# transform_forecast <- function(fc,
#                                trans_fun,
#                                nsim        = 1000,
#                                level       = 95,
#                                y0          = 1,
#                                xreg_future = NULL,
#                                backtransform_x = TRUE) {
#   # fc: forecast object from forecast::forecast(Arima(...))
#   # trans_fun: inverse transformation, applied ELEMENTWISE
#   #            (vector in, same-length vector out), e.g. exp, or function(z,y0) y0*exp(z)
#   # nsim: number of simulated paths
#   # level: numeric vector of confidence levels (e.g. 95 or c(80,95))
#   # y0: passed to trans_fun
#   # xreg_future: future xreg values (matrix or vector) for horizon h
#   # backtransform_x: if TRUE, also backtransform fc$x (and fc$fitted if present)
# 
#   if (!inherits(fc, "forecast"))
#     stop("'fc' must be an object of class 'forecast'")
# 
#   fit <- fc$model
#   if (!inherits(fit, "Arima"))
#     stop("'fc$model' must be an 'Arima' object from the forecast package")
# 
#   # Forecast horizon
#   h <- length(fc$mean)
# 
#   # Did the model use xreg when fitted?
#   model_has_xreg <- !is.null(fit$xreg)
# 
#   # --- xreg_future checks ---------------------------------------------------
#   if (model_has_xreg && is.null(xreg_future)) {
#     stop("This ARIMA model was fitted with xreg, so you must supply 'xreg_future'.")
#   }
# 
#   if (!model_has_xreg && !is.null(xreg_future)) {
#     warning("Model was fitted without xreg; 'xreg_future' will be ignored.")
#     xreg_future <- NULL
#   }
# 
#   if (!is.null(xreg_future)) {
#     xreg_future <- as.matrix(xreg_future)
#     if (nrow(xreg_future) != h) {
#       stop("nrow(xreg_future) must equal forecast horizon h = ", h,
#            ", but is ", nrow(xreg_future), ".")
#     }
#   }
# 
#   # --- level checks ---------------------------------------------------------
#   level <- sort(as.numeric(level))
#   if (any(level <= 0 | level >= 100))
#     stop("'level' must be between 0 and 100")
# 
#   y_hist  <- trans_fun(as.numeric(fc$x), y0 = y0)
#   y_last  <- tail(y_hist, 1)
#   
#   
#     # --- Simulate on model (transformed) scale and backtransform -------------
#   sim_mat <- replicate(nsim, {
#     z_sim <- if (is.null(xreg_future)) {
#       simulate(fit, nsim = h, future = TRUE)
#     } else {
#       simulate(fit, nsim = h, future = TRUE, xreg = xreg_future)
#     }
# 
#     # Backtransform simulated path to original scale
# ##    trans_fun(as.numeric(z_sim), y0 = y0)
#         trans_fun(as.numeric(z_sim), y0 = y0) + y_last
#   })
# 
#   # sim_mat: h x nsim on ORIGINAL scale
#   mean_fc <- rowMeans(sim_mat)
# 
#   # lower/upper for each level
#   alpha <- (100 - level) / 200
#   probs <- rbind(alpha, 1 - alpha)
# 
#   lower <- upper <- matrix(NA_real_, nrow = h, ncol = length(level))
#   for (j in seq_along(level)) {
#     q <- apply(sim_mat, 1, quantile, probs = probs[, j])
#     lower[, j] <- q[1, ]
#     upper[, j] <- q[2, ]
#   }
#   colnames(lower) <- colnames(upper) <- paste0(level, "%")
# 
#   # Preserve time index from fc$mean
#   mean_ts <- fc$mean
#   mean_bt <- ts(mean_fc,
#                 start     = start(mean_ts),
#                 frequency = frequency(mean_ts))
# 
#   # --- Build new forecast object -------------------------------------------
#   fc_new <- fc
#   fc_new$mean  <- mean_bt
#   fc_new$lower <- lower
#   fc_new$upper <- upper
#   fc_new$level <- level
# 
#   # Optionally backtransform history and fitted values as well
#   if (backtransform_x && !is.null(fc$x)) {
#     x_old <- fc$x
#     x_bt  <- trans_fun(as.numeric(x_old), y0 = y0)
#     fc_new$x <- ts(x_bt,
#                    start     = start(x_old),
#                    frequency = frequency(x_old))
#   }
# 
#   if (backtransform_x && !is.null(fc$fitted)) {
#     f_old <- fc$fitted
#     f_bt  <- trans_fun(as.numeric(f_old), y0 = y0)
#     fc_new$fitted <- ts(f_bt,
#                         start     = start(f_old),
#                         frequency = frequency(f_old))
#   }
# 
#   fc_new$method <- paste0(fc$method, " (back-transformed via simulation)")
#   fc_new
# }





# transform_forecast <- function(fc,
#                                trans_fun,
#                                nsim        = 1000,
#                                level       = 95,
#                                y0          = 1,
#                                xreg_future = NULL) {
#   # fc: forecast object from forecast::forecast(Arima(...))
#   # trans_fun: function(z, y0) returning back-transformed vector
#   # nsim: number of simulated paths
#   # level: numeric vector of confidence levels, e.g. 95 or c(80, 95)
#   # y0: parameter passed to trans_fun
#   # xreg_future: future xreg (matrix / vector) for the forecast horizon
# 
#   if (!inherits(fc, "forecast"))
#     stop("'fc' must be an object of class 'forecast'")
# 
#   fit <- fc$model
#   if (!inherits(fit, "Arima"))
#     stop("'fc$model' must be an 'Arima' object from the forecast package")
# 
#   # Forecast horizon
#   h <- length(fc$mean)
# 
#   # Did the model use xreg when fitted?
#   model_has_xreg <- !is.null(fit$xreg)
# 
#   # --- Check xreg_future consistency ---------------------------------------
#   if (model_has_xreg && is.null(xreg_future)) {
#     stop("This ARIMA model was fitted with xreg, ",
#          "so you must supply 'xreg_future'.")
#   }
# 
#   if (!model_has_xreg && !is.null(xreg_future)) {
#     warning("Model was fitted without xreg; 'xreg_future' will be ignored.")
#     xreg_future <- NULL
#   }
# 
#   if (!is.null(xreg_future)) {
#     xreg_future <- as.matrix(xreg_future)
#     if (nrow(xreg_future) != h) {
#       stop("nrow(xreg_future) must equal forecast horizon h = ", h,
#            ", but is ", nrow(xreg_future), ".")
#     }
#   }
# 
#   # Ensure 'level' is a numeric vector
#   level <- sort(as.numeric(level))
#   if (any(level <= 0 | level >= 100))
#     stop("'level' must be between 0 and 100")
# 
#   # --- Simulation on model scale -------------------------------------------
#   # Each column of sim_mat is one simulated path on ORIGINAL (model) scale
#   sim_mat <- replicate(nsim, {
#     if (is.null(xreg_future)) {
#       z <- simulate(fit, nsim = h, future = TRUE)
#     } else {
#       z <- simulate(fit, nsim = h, future = TRUE, xreg = xreg_future)
#     }
#     # Back-transform this simulated path
#     trans_fun(z, y0 = y0)
#   })
# 
#   # sim_mat: h x nsim matrix on transformed-back scale
#   # --- Compute summary stats -----------------------------------------------
#   mean_fc <- rowMeans(sim_mat)
# 
#   # Build lower/upper matrices for each level
#   alpha  <- (100 - level) / 200
#   probs  <- rbind(alpha, 1 - alpha)  # 2 x L
# 
#   lower <- upper <- matrix(NA_real_, nrow = h, ncol = length(level))
#   for (j in seq_along(level)) {
#     q <- apply(sim_mat, 1, quantile, probs = probs[, j])
#     lower[, j] <- q[1, ]
#     upper[, j] <- q[2, ]
#   }
# 
#   # Preserve ts attributes for mean
#   mean_ts <- fc$mean
#   mean_bt <- ts(mean_fc,
#                 start     = stats::start(mean_ts),
#                 frequency = stats::frequency(mean_ts))
# 
#   # lower/upper as matrices with ts attributes if you like
#   colnames(lower) <- colnames(upper) <- paste0(level, "%")
# 
#   # --- Construct new forecast object --------------------------------------
#   fc_new <- fc
#   fc_new$mean  <- mean_bt
#   fc_new$lower <- lower
#   fc_new$upper <- upper
#   fc_new$level <- level
# 
#   # Optionally back-transform the historical series as well:
#   # (comment out if you prefer to leave fc$x as is)
# #  if (!is.null(fc$x)) {
# #    fc_new$x <- trans_fun(fc$x, y0 = y0)
# #  }
# 
# #   if (!is.null(fc$x)) {
# #   x_old <- fc$x
# #   fc_new$x <- ts(
# #     trans_fun(as.numeric(x_old), y0 = y0),
# #     start     = start(x_old),
# #     frequency = frequency(x_old)
# #   )
# # }
# 
#   fc_new$method <- paste0(fc$method, " (back-transformed via simulation)")
# 
#   fc_new
# }






#' #' @export
#' transform_forecast <- function(fc_object, trans_fun,
#'                            nsim = 2000L, level = 95, y0 = 1, xreg_future=NULL) {
#' 
#'   ## Input validation (same as before, but allow y_hist = NULL)
#'   .check_forecast_input(fc_object, trans_fun, y0, level)
#' 
#'   d <- fc_object$x
#'   h <- length(fc_object$mean)
#' 
#'   # ## 0) Historical series on output scale
#'   # if (is.null(y_hist)) {
#'   #   # reconstruct from model-scale data and y0
#'   #   y_hist <- trans_fun(as.numeric(d), y0)
#'   #   attributes(y_hist) <- attributes(d)
#'   # } else {
#'   #   # user-supplied; basic sanity check
#'   #   if (!is.numeric(y_hist) || is.null(tsp(y_hist)))
#'   #     stop("`y_hist` must be a numeric 'ts' object.", call. = FALSE)
#'   # }
#' 
#'   
#'   ## 0) Historical series on output scale
#'   y_hist <- trans_fun(as.numeric(d), y0)
#'   attributes(y_hist) <- attributes(d)
#' 
#'   
#'   ## last historical value on output scale
#'   Y_last <- as.numeric(utils::tail(y_hist, 1))
#' 
#'   ## 1) Simulated future on model scale
#'   sim_mat <- replicate(
#'     nsim,
#'     stats::simulate(fc_object$model, nsim = h, future = TRUE)
#'   )
#' 
#' 
#'   
#'   ## 2) Transform each path to output scale, anchored at Y_last
#'   level_mat <- apply(sim_mat, 2, trans_fun, y0 = Y_last)
#'   level_mat
#'   
#'   ## 3) Mean and prediction interval
#'   mean_level  <- rowMeans(level_mat)
#'   level
#'   alpha       <- (100 - level) / 200
#'   alpha
#' 
#'     lower_level <- apply(level_mat, 1, stats::quantile, probs = alpha)
#'   upper_level <- apply(level_mat, 1, stats::quantile, probs = 1 - alpha)
#' 
#'   upper_level
#'   lower_level
#'   
#'   
#'   
#'   ## 4) ts objects on same time scale as original forecast
#'   freq     <- stats::frequency(fc_object$mean)
#'   start_fc <- stats::start(fc_object$mean)
#' 
#'   mean_ts  <- stats::ts(mean_level,  start = start_fc, frequency = freq)
#'   lower_ts <- stats::ts(lower_level, start = start_fc, frequency = freq)
#'   upper_ts <- stats::ts(upper_level, start = start_fc, frequency = freq)
#' 
#'   ## 5) New forecast object
#'   fc_new        <- fc_object
#'   fc_new$x      <- y_hist
#'   fc_new$mean   <- mean_ts
#'   fc_new$level  <- level
#'   fc_new$lower  <- stats::setNames(cbind(lower_ts), paste0(level, "%"))
#'   fc_new$upper  <- stats::setNames(cbind(upper_ts), paste0(level, "%"))
#' 
#'   fc_new
#' }
#' 
#' 
#' 
#' 




















## EKSEMPEL

## library(forecast)
## library(ggplot2)

## ## Change in income
## library(fpp2)
## d <- uschange[,"Income"]      

## trans1 <- function(r, y0) {
##   y0 * cumprod(1 + r/100)
## }

## fit    <- auto.arima(d)
## fc_pct <- forecast(fit, h = 24)

## fc_lev <- transform_forecast(fc_pct, trans1, nsim = 5000, level = 95,y0=1)
## autoplot(fc_lev) + theme_minimal()

## ## Lynx
## llynx <- log(lynx)
## fit    <- auto.arima(llynx)
## fc_pct <- forecast(fit, h = 24)


## trans2 <- function(z, y0) exp(z)

## fit_log <- auto.arima(log(lynx))
## fc_log  <- forecast(fit_log, h = 20)

## fc_lynx <- transform_forecast(fc_log, trans_fun = trans2, y0 = 1)
## autoplot(fc_lynx) + theme_minimal()

## #############################################################################
#'
#' @title Compute  LS-means (aka population means or
#'     marginal means)
#' @description LS-means (least squares means, also known as
#'     population means and as marginal means) for a range of model types.
#' @name ls-means
#' 
## #############################################################################
#' 
#' @details There are restrictions on the formulas allowed in the model object.
#'     For example having \code{y ~ log(x)} will cause an error. Instead one
#'     must define the variable \code{logx = log(x)} and do \code{y ~ logx}.
#' 
#' @aliases LSmeans LSmeans.default LSmeans.lmerMod popMeans
#'     popMeans.default popMeans.lmerMod
#' 
#' @param object Model object
#' @param effect A vector of variables. For each configuration of
#'     these the estimate will be calculated.
#' @param at A list of values of covariates (including levels of some
#'     factors) to be used in the calculations
#' @param level The level of the (asymptotic) confidence interval.
#' @param adjust.df Should denominator degrees of freedom be adjusted?
#' @param \dots  Additional arguments; currently not used.
#' 
#' @return A dataframe with results from computing the contrasts.
#'
#' @note \code{LSmeans} and \code{popMeans} are synonymous. 
#' 
#' @section Warning: Notice that \code{LSmeans} and \code{LE_matrix}
#'     fails if the model formula contains an offset (as one would
#'     have in connection with e.g. Poisson regression. 
#' 
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{LE_matrix}}, \code{\link{linest}}
#' @keywords utilities
#' @examples
#' 
#' ## Two way anova:
#' 
#' data(warpbreaks)
#' 
#' m0 <- lm(breaks ~ wool + tension, data=warpbreaks)
#' m1 <- lm(breaks ~ wool * tension, data=warpbreaks)
#' LSmeans(m0)
#' LSmeans(m1)
#' 
#' ## same as:
#' K <- LE_matrix(m0);K
#' linest(m0, K)
#' K <- LE_matrix(m1);K
#' linest(m1, K)
#' 
#' LE_matrix(m0, effect="wool")
#' LSmeans(m0, effect="wool")
#' 
#' LE_matrix(m1, effect="wool")
#' LSmeans(m1, effect="wool")
#' 
#' LE_matrix(m0, effect=c("wool", "tension"))
#' LSmeans(m0, effect=c("wool", "tension"))
#' 
#' LE_matrix(m1, effect=c("wool", "tension"))
#' LSmeans(m1, effect=c("wool", "tension"))
#' 
#' 
#' ## Regression; two parallel regression lines:
#' 
#' data(Puromycin)
#' 
#' m0 <- lm(rate ~ state + log(conc), data=Puromycin)
#' ## Can not use LSmeans / LE_matrix here because of
#' ## the log-transformation. Instead we must do:
#' Puromycin$lconc <- log( Puromycin$conc )
#' m1 <- lm(rate ~ state + lconc, data=Puromycin)
#' 
#' LE_matrix(m1)
#' LSmeans(m1)
#' 
#' LE_matrix(m1, effect="state")
#' LSmeans(m1, effect="state")
#' 
#' LE_matrix(m1, effect="state", at=list(lconc=3))
#' LSmeans(m1, effect="state", at=list(lconc=3))
#' 
#' ## Non estimable contrasts
#' 
#' ## ## Make balanced dataset
#' dat.bal <- expand.grid(list(AA=factor(1:2), BB=factor(1:3),
#'                             CC=factor(1:3)))
#' dat.bal$y <- rnorm(nrow(dat.bal))
#' 
#' ## ## Make unbalanced dataset
#' #      'BB' is nested within 'CC' so BB=1 is only found when CC=1
#' #       and BB=2,3 are found in each CC=2,3,4
#' dat.nst <- dat.bal
#' dat.nst$CC <-factor(c(1, 1, 2, 2, 2, 2, 1, 1, 3, 3,
#'                       3, 3, 1, 1, 4, 4, 4, 4))
#' 
#' mod.bal  <- lm(y ~ AA + BB * CC,    data=dat.bal)
#' mod.nst  <- lm(y ~ AA + BB : CC, data=dat.nst)
#' 
#' LSmeans(mod.bal, effect=c("BB", "CC"))
#' LSmeans(mod.nst, effect=c("BB", "CC"))
#' LSmeans(mod.nst, at=list(BB=1, CC=1))
#' 
#' LSmeans(mod.nst, at=list(BB=1, CC=2))
#' ## Above: NA's are correct; not an estimable function
#' 
#' if( require( lme4 )){
#'  warp.mm <- lmer(breaks ~ -1 + tension + (1|wool), data=warpbreaks)
#'  LSmeans(warp.mm, effect="tension")
#'  class(warp.mm)
#'  fixef(warp.mm)
#'  coef(summary(warp.mm))
#'  vcov(warp.mm)
#'  if (require(pbkrtest))
#'    vcovAdj(warp.mm)
#' }
#' 
#' LSmeans(warp.mm, effect="tension")
#' 
#' @export LSmeans
#'


#' @export
#' @rdname ls-means
LSmeans <- function(object, effect=NULL, at=NULL, level=0.95, ...){
    UseMethod("LSmeans")
}

#' @export
#' @rdname ls-means
LSmeans.default <- function(object, effect=NULL, at=NULL, level=0.95, ...){
    K   <- LE_matrix(object, effect=effect, at=at)
    out <- linest(object, K, level=level, ...)
    out
}

#' @export
#' @rdname ls-means
LSmeans.lmerMod <- function(object, effect=NULL, at=NULL, level=0.95, adjust.df=TRUE, ...){
    K   <- LE_matrix(object, effect=effect, at=at)
    out <- linest(object, K, level=level, adjust.df=adjust.df, ...)
    out
}

#' @export
#' @rdname ls-means
popMeans         <- LSmeans

#' @export
#' @rdname ls-means
popMeans.default <- LSmeans.default

#' @export
#' @rdname ls-means
popMeans.lmerMod <- LSmeans.lmerMod

###############################################################################
##
#' @title Contrasts for lm, glm, lme, and geeglm objects
#' @description Computes linear functions (i.e. weighted sums) of the
#'     estimated regression parameters. Can also test the hypothesis,
#'     that such a function is equal to a specific value.
#' @name esticon
##
###############################################################################
#'
## #' @aliases esticon.lm esticon.gls
#' 
#' @details Let the estimated parameters of the model be
#' \deqn{\beta_1, \beta_2, \dots, \beta_p}
#'
#' A linear function of the estimates is of the form \deqn{l=\lambda_1
#'     \beta_1+\lambda_2 \beta_2+ \dots+\lambda_p \beta_p} where
#'     \eqn{\lambda_1, \lambda_2, \dots,\lambda_p} is specified by the
#'     user.
#' 
#' The esticon function calculates l, its standard error and by default also a
#' 95 pct confidence interval.  It is sometimes of interest to test the
#' hypothesis \eqn{H_0: l=\beta_0} for some value \eqn{\beta_0}
#' given by the user. A test is provided for the hypothesis \eqn{H_0:
#' l=0} but other values of \eqn{\beta_0} can be specified.
#' 
#' In general, one can specify r such linear functions at one time by
#' specifying L to be an \eqn{r\times p} matrix where each row consists
#' of p numbers \eqn{\lambda_1,\lambda_2,\dots, \lambda_p}. Default is
#' then that \eqn{\beta_0} is a p vector of 0s but other values can be
#' given.
#' 
#' It is possible to test simultaneously that all specified linear functions
#' are equal to the corresponding values in \eqn{\beta_0}.
#' 
#' For computing contrasts among levels of a single factor, 'contrast.lm' may
#' be more convenient.
#' 
#' @param obj Regression object (of type lm, glm, lme, geeglm).
## #' @param x A linear contrast object (as returned by \code{esticon()}. 
#' @param L Matrix (or vector) specifying linear functions of the regression
#'     parameters (one
#'     linear function per row).  The number of columns must match the number of
#'     fitted regression parameters in the model. See 'details' below.
#' @param parm a specification of which parameters are to be given
#'          confidence intervals, either a vector of numbers or a vector
#'         of names.  If missing, all parameters are considered.
#' @param beta0 A vector of numbers
#' @param conf.int TRUE
## #' @param conf.level The desired confidence level.
#' @param level The confidence level
#' @param joint.test Logical value. If TRUE a 'joint' Wald test for the
#'     hypothesis L beta = beta0 is made. Default is that the 'row-wise' tests are
#'     made, i.e. (L beta)i=beta0i.  If joint.test is TRUE, then no confidence
#'     interval etc. is calculated.
#' @param \dots Additional arguments; currently not used.
#' 
#' @return Returns a matrix with one row per linear function.  Columns contain
#'     estimated coefficients, standard errors, t values, degrees of freedom,
#'     two-sided p-values, and the lower and upper endpoints of the 1-alpha
#'     confidence intervals.
#' 
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @keywords utilities
#' @examples
#' 
#' data(iris)
#' lm1  <- lm(Sepal.Length ~ Sepal.Width + Species + Sepal.Width : Species, data=iris)
#' ## Note that the setosa parameters are set to zero
#' coef(lm1)
#' 
#' ## Estimate the intercept for versicolor
#' lambda1 <- c(1, 0, 1, 0, 0, 0)
#' esticon(lm1, L=lambda1)
#' 
#' ## Estimate the difference between versicolor and virgica intercept
#' ## and test if the difference is 1
#' lambda2 <- c(0, 1, -1, 0, 0, 0)
#' esticon(lm1, L=lambda2, beta0=1)
#' 
#' ## Do both estimates at one time
#' esticon(lm1, L=rbind(lambda1, lambda2), beta0=c(0, 1))
#' 
#' ## Make a combined test for that the difference between versicolor and virgica intercept
#' ## and difference between versicolor and virginica slope is zero:
#' lambda3 <- c(0, 0, 0, 0, 1, -1)
#' esticon(lm1, L=rbind(lambda2, lambda3), joint.test=TRUE)
#' 
#' # Example using esticon on coxph objects (thanks to Alessandro A. Leidi).
#' # Using dataset 'veteran' in the survival package
#' # from the Veterans' Administration Lung Cancer study
#' 
#' if (require(survival)){
#' data(veteran)
#' sapply(veteran, class)
#' levels(veteran$celltype)
#' attach(veteran)
#' veteran.s <- Surv(time, status)
#' coxmod <- coxph(veteran.s ~ age + celltype + trt, method='breslow')
#' summary(coxmod)
#' 
#' # compare a subject 50 years old with celltype 1
#' # to a subject 70 years old with celltype 2
#' # both subjects on the same treatment
#' AvB <- c(-20, -1, 0, 0, 0)
#' 
#' # compare a subject 40 years old with celltype 2 on treat=0
#' # to a subject 35 years old with celltype 3 on treat=1
#' CvB <- c(5, 1, -1, 0, -1)
#' 
#' est <- esticon(coxmod, L=rbind(AvB, CvB))
#' est
#' ##exp(est[, c(2, 7, 8)])
#' }


#' @export 
esticon <- function(obj, L, beta0, conf.int = TRUE, level=0.95, joint.test=FALSE,...)
  UseMethod("esticon")

#' @export 
esticon.gls <- function (obj, L, beta0, conf.int = TRUE, level=0.95, joint.test=FALSE,...){
    if (joint.test) .wald(obj, L, beta0)
    else {
        stat.name <- "X2.stat"
        ##vcv <- vcov(obj)
        vcv <- vcov(obj, complete=FALSE)
        coef.mat  <- matrix(coef(obj))
        df  <- 1
        .esticonCore(obj, L, beta0, conf.int=conf.int,level,coef.mat,vcv,df,stat.name)
    }
}

#' @export 
esticon.geeglm <- function (obj, L, beta0, conf.int = TRUE, level=0.95, joint.test=FALSE,...){
    if (joint.test) .wald(obj, L, beta0)
    else {
        stat.name <- "X2.stat"
        coef.mat  <- summary(obj)$coef
        vcv <- summary(obj)$cov.scaled
        df  <- 1
        .esticonCore(obj, L, beta0, conf.int=conf.int,level,coef.mat,vcv,df,stat.name)
    }
}

#' @export 
esticon.lm <- function (obj, L, beta0, conf.int = TRUE, level=0.95, joint.test=FALSE,...){
    if (joint.test) .wald(obj, L, beta0)
    else {
        stat.name <- "t.stat"
        coef.mat  <- summary.lm(obj)$coefficients
        coef.vec  <- coef(obj)
        vcv <- summary.lm(obj)$cov.unscaled * summary.lm(obj)$sigma^2
        df  <- obj$df.residual
        .esticonCore(obj, L, beta0, conf.int=conf.int, level, coef.mat, vcv, df, stat.name, coef.vec=coef.vec)
  }
}

#' @export 
esticon.glm <- function (obj, L, beta0, conf.int = TRUE, level=0.95, joint.test=FALSE,...){
    if (joint.test) .wald(obj, L, beta0)
    else {
        coef.mat <- summary.lm(obj)$coefficients
        vcv <- summary(obj)$cov.scaled
        if(family(obj)[1] %in% c("poisson", "binomial")){
            stat.name <- "X2.stat"
            df <- 1
        } else {
            stat.name <- "t.stat"
            df <- obj$df.residual
        }
        .esticonCore(obj, L, beta0, conf.int=conf.int,level=level,coef.mat,vcv,df,stat.name)
    }
}

#' @export 
esticon.merMod <- function (obj, L, beta0, conf.int = TRUE, level=0.95, joint.test=FALSE, ...){ 
    if (joint.test) .wald(obj, L, beta0)
    else {
        stat.name <- "X2.stat"
        coef.mat  <- matrix(lme4::fixef(obj))
        ##vcv <- as.matrix(vcov(obj))
        vcv <- as.matrix(vcov(obj, complete=FALSE))

        
        df  <- 1
        .esticonCore(obj, L, beta0, conf.int=conf.int, level, coef.mat, vcv, df, stat.name)
    }
}

#' @export 
esticon.coxph <-
    function (obj, L, beta0, conf.int = TRUE, level = 0.95, joint.test = FALSE, ...){
        if (joint.test == TRUE) .wald(obj, L, beta0)
        else {
            cf <- summary(obj)$coefficients
            vcv <- obj$var
            stat.name <- "X2.stat"
            df <- 1
            .esticonCore(obj, L, beta0, conf.int = conf.int, level = level,
                         cf, vcv, df, stat.name)
        }
    }


### ######################################################
###
### esticon.lme needs better testing at some point
###
### ######################################################

#' @export 
esticon.lme <- function (obj, L, beta0, conf.int = NULL, level=0.95, joint.test=FALSE,...){
  warning("The esticon function has not been thoroughly teste on 'lme' objects")
  if (joint.test) .wald(obj, L, beta0)
  else {
      stat.name <- "t.stat"
      coef.mat <- summary(obj)$tTable
      rho <- summary(obj)$cor
      vcv <- rho * outer(coef.mat[, 2], coef.mat[, 2])
      tmp <- L
      tmp[tmp == 0] <- NA
      df.all <- t(abs(t(tmp) * obj$fixDF$X))
      df <- apply(df.all, 1, min, na.rm = TRUE)
      problem <- apply(df.all != df, 1, any, na.rm = TRUE)
      if (any(problem))
          warning(paste("Degrees of freedom vary among parameters used to ",
                        "construct linear contrast(s): ",
                        paste((1:nrow(tmp))[problem],
                              collapse = ","),
                        ". Using the smallest df among the set of parameters.",
                        sep = ""))
      df <- min(df)
      .esticonCore(obj, L, beta0, conf.int=conf.int,level,coef.mat,vcv,df,stat.name)
  }
}


### .functions below here ###

.wald <- function (obj, L, beta0)
{
    if (!is.matrix(L) && !is.data.frame(L))
        L <- matrix(L, nrow = 1)

    if (missing(beta0))
      beta0 <- rep(0, nrow(L))

    df <- nrow(L)

    if (inherits(obj, "geese")){
        coef.mat  <- obj$beta
        vcv <- obj$vbeta
    } else if (inherits(obj, "geeglm")){
        coef.mat  <- obj$coef
        vcv <- summary(obj)$cov.scaled
    } else if (inherits(obj, "gls")) {        
        ##vcv <- vcov(obj)
        vcv <- vcov(obj, complete=FALSE)
        coef.mat  <- matrix(coef(obj))
    } else if (inherits(obj, "gee")) {
        coef.mat  <- obj$coef
        vcv <- obj$robust.variance
    }
    else if (inherits(obj, "lm")) {
        coef.mat  <- summary.lm(obj)$coefficients[, 1]
        vcv <- summary.lm(obj)$cov.unscaled * summary.lm(obj)$sigma^2
        if (inherits(obj, "glm")) {
            vcv <- summary(obj)$cov.scaled
        }
    }
    else if (inherits(obj, "coxph")) {
        coef.mat <- obj$coef
        vcv <- obj$var
    }
    else
        stop("obj must be of class 'lm', 'glm', 'aov', 'gls', 'gee', 'geese', 'coxph'")
    
    u      <- (L %*% coef.mat)-beta0
    vcv.u  <- L %*% vcv %*% t(L)
    W      <- t(u) %*% solve(vcv.u) %*% u
    prob   <- 1 - pchisq(W, df = df)
    out <- as.data.frame(cbind(W, df, prob))
    names(out) <- c("X2.stat", "DF", "Pr(>|X^2|)")
    as.data.frame(out)
}


.esticonCore <- function (obj, L, beta0, conf.int = NULL, level, coef.mat,
                          vcv, df, stat.name, coef.vec=coef.mat[,1]) {

    ## Notice
    ## coef.mat: summary(obj)$coefficients
    ## coef.vec: coef(obj)
    ## cl <- match.call(); print(cl);  print(coef.mat)
    
    if (missing(L)) stop("Contrast matrix 'L' is missing")
    if (!is.matrix(L) && !is.data.frame(L))
        L <- matrix(L, nrow = 1)
    if (missing(beta0))
        beta0 <- rep(0, nrow(L))

    ##print(list(L=L, beta0=beta0))
    
    idx <- !is.na(coef.vec) ## Only want columns of L for identifiable parameters
    L   <- L[ , idx, drop=FALSE]
    
    if (!dim(L)[2] == dim(coef.mat)[1])
        stop(paste("\n Dimension of ",
                   deparse(substitute(L)),
                   ": ", paste(dim(L), collapse = "x"),
                   ", not compatible with no of parameters in ",
                   deparse(substitute(obj)), ": ", dim(coef.mat)[1], sep = ""))

    ct      <- L %*% coef.mat[, 1]
    ct.diff <- L %*% coef.mat[, 1] - beta0
    vcv.L   <- L %*% vcv %*% t(L)

    if (is.null(vn <- rownames(L)))
        vn <- 1:nrow(L)
    rownames(vcv.L)  <- colnames(vcv.L) <- vn

    vc      <- sqrt(diag(vcv.L))

    
    switch(stat.name,
           t.stat = {
               prob <- 2 * (1 - pt(abs(ct.diff/vc), df))
           },
           X2.stat = {
               prob <- 1 - pchisq((ct.diff/vc)^2, df = 1)
           })

                      
    if (stat.name == "X2.stat") {
        out <- cbind(hyp=beta0, est = ct, stderr = vc, t.value = (ct.diff/vc)^2,
                     df = df, prob = prob )
    }
    else if (stat.name == "t.stat") {
        out <- cbind(hyp=beta0, est = ct, stderr = vc, t.value = ct.diff/vc,
                     df = df, prob = prob)
    }

    dim.names <- list(NULL, c("beta0","estimate","std.error",
                              "statistic","df","p.value"))    
    dimnames(out) <- dim.names
    out <- out[, c(2,3,4,6,1,5), drop=FALSE]
    
    conf.int <- "wald"
    if (!is.null(conf.int)) {
        if (level <= 0 || level >= 1)
            stop("level should be betweeon 0 and 1. Usual values are 0.95, 0.90")
        
        alpha <- 1 - level
        switch(stat.name,
               t.stat  = { quant <- qt(1 - alpha/2, df)  },
               X2.stat = { quant <- qnorm(1 - alpha/2) })
        
        cint <- cbind(ct.diff - vc * quant, ct.diff + vc * quant)
        colnames(cint) <- c("lwr", "upr")

        out <- cbind(out, cint)
    }
    out <- as.data.frame(out)
    ## FIXME esticon_class added; not sure if this is good idea?
    class(out) <- c("esticon_class", "data.frame")
    attr(out, "L") <- L
    attr(out, "vcv") <- vcv.L
    out
}


#' @rdname esticon
#' @param object An \code{esticon_class} object. 

#' @export
coef.esticon_class <- function (object, ...) {
    out <- object$estimate
    names(out) <- rownames(object)
    out
}

#' @export
#' @rdname esticon
summary.esticon_class <- function (object, ...) 
{
    cat("Coefficients:\n")
    ##printCoefmat(object$coef)
    printCoefmat(object)
    cat("\n")

    cat("L:\n")
    ##print(object$L)
    print(attr(object, "L"))
    cat("\n")

    invisible(object)
}


#' @export
print.esticon_class <- function(x, ...){
    printCoefmat(x[,1:6])
}




#' @export
#' @rdname esticon
confint.esticon_class <- function (object, parm, level = 0.95, ...) 
{
    cf <- coef(object)
    pnames <- names(cf)
    if (missing(parm)) 
        parm <- pnames
    else if (is.numeric(parm)) 
        parm <- pnames[parm]
    a <- (1 - level)/2
    a <- c(a, 1 - a)

    ## stats:::confint.default code
    ## pct <- stats:::format.perc(a, 3)
    ## fac <- qnorm(a)

    ## replaced with
    pct <- a 
    DF <- object$df
    fac  <- if (!is.null(DF)) qt(a, DF[1])
            else fac <- qnorm(a)
    ## to here
    
    ci <- array(NA, dim = c(length(parm), 2L), dimnames = list(parm, 
                                                               pct))
    ses <- sqrt(diag(vcov(object)))[parm]
    ##str(list(cf=cf, parm=parm, fac=fac, ses=ses))
    ci[] <- cf[parm] + ses %o% fac
    ci
}


#' @export
#' @rdname esticon
vcov.esticon_class <- function (object, ...){
    attr(object, "vcv")
}

#######################################################################
#'
#' @title Formula based version of lapply and sapply
#' @description This function is a wrapper for calling lapply on the
#'     list resulting from first calling splitBy.
#' @name by-lapply
#' 
#######################################################################
#' @param formula A formula describing how data should be split.
#' @param data A dataframe.
#' @param FUN A function to be applied to each element in the split
#'    list, see 'Examples' below.
#' @param \dots optional arguments to FUN.
#' @param USE.NAMES Same as for `sapply`
#' @param simplify Same as for `sapply`
#' @return A list.
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link{splitBy}}, \code{\link{split_by}}
#' @keywords utilities
#' @concept byfunction
#' @examples
#' fun <- function(x) range(x$uptake)
#' lapplyBy(~Treatment + Type, data=CO2, FUN=fun)
#' sapplyBy(~Treatment + Type, data=CO2, FUN=fun)
#'
#' # Same as
#' lapply(splitBy(~Treatment + Type, data=CO2), FUN=fun)

#' @export
#' @rdname by-lapply
lapply_by <- function(data, formula, FUN, ...){
    cl   <- match.call(expand.dots = TRUE)
    cl[[2]] <- formula
    cl[[3]] <- data
    names(cl)[2:3] <- c("formula", "data")
    cl[[1]] <- as.name("lapplyBy")
    eval(cl)
}

#' @export
#' @rdname by-lapply
lapplyBy <- function (formula, data = parent.frame(), FUN, ...) 
{
    out <- splitBy(formula, data = data)
    gr  <- unique(attr(out, "grps"))
    out <- lapply(out, FUN, ...)
    out <- out[gr]
    out
}

#' @export
#' @rdname by-lapply
sapply_by <- function(data, formula, FUN, ..., simplify = TRUE, USE.NAMES = TRUE){
    cl   <- match.call(expand.dots = TRUE)
    cl[[2]] <- formula
    cl[[3]] <- data
    names(cl)[2:3] <- c("formula", "data")
    cl[[1]] <- as.name("sapplyBy")
    eval(cl)
}

#' @export
#' @rdname by-lapply
sapplyBy <- function (formula, data = parent.frame(), FUN, ..., simplify = TRUE, USE.NAMES = TRUE) 
{
    out <- splitBy(formula, data = data)
    gr  <- unique(attr(out, "grps"))
    out <- sapply(out, FUN, ..., simplify=simplify, USE.NAMES=USE.NAMES)
    if (!simplify)
        out <- out[gr] 
    out
}

#' @title Plot linear model object
#'
#' @param lm_fit An object of class 'lm'
#' @param format The format of the plot (or a list of plots if format is "list")
#' @param global_aes Currently no effect.
#'
#' @concept plotting
#' @examples
#' data(income)
#' m1 <- lm(inc ~ race + educ, data=income)
#' plot_lm(m1)
#' plot_lm(m1, "2x2")
#' plot_lm(m1, "1x4")
#' plot_lm(m1, "4x1")
#' plot_lm(m1, "list")
#' 
#' @export
plot_lm <- function(lm_fit, format="2x2", global_aes=NULL) {
    
    if(!inherits(lm_fit, "lm"))
        stop("'lm_fit' must inherit form 'lm'\n")

    format <- match.arg(format, c("2x2", "1x4", "4x1", "list"))
    
    dd <- data.frame(
        fitted_values          = predict(lm_fit),
        resp        = predict(lm_fit)+resid(lm_fit),
        std_res = rstandard(lm_fit))


    pl1 <- ggplot(dd,
                  aes(x = .data$fitted_values,
                      y = .data$std_res)) +
        geom_point() + geom_hline(yintercept=c(0, -1.96, 1.95))
    
    pl2 <- ggplot(dd,
                  aes(x=.data$fitted_values,
                      y=.data$resp)) +
        geom_point() +  geom_smooth(method="lm", formula=y ~ x, se=FALSE)
    
    pl3 <- ggplot(dd,
                  aes(x=.data$fitted_values,
                      y=sqrt(abs(.data$std_res)))) +
        geom_point()

    pl4 <- ggplot(dd,
                  aes(sample = .data$std_res)) +
        stat_qq() + stat_qq_line() +
        labs(y="std_res", x = "theoretical quantiles")

    switch(format,
           "2x2"={ cowplot::plot_grid(pl1, pl2, pl3, pl4, nrow=2) },
           "1x4"={ cowplot::plot_grid(pl1, pl2, pl3, pl4, nrow=1) },
           "4x1"={ cowplot::plot_grid(pl1, pl2, pl3, pl4, nrow=4) },
           "list"={ list(pl1=pl1, pl2=pl2, pl3=pl3, pl4=pl4) }
           )

}


#' @title Two-way interaction plot 
#'
#' @description Plots the mean of the response for
#'     two-way combinations of factors, thereby illustrating possible
#'     interactions.
#'
#' @name interaction-plot
#'
#' @note This is a recent addition to the package and is subject to change.
#'
#' @param .data A data frame
#' @param .formula A formula of the form `y ~ x1 + x2`
#' @param interval Either `conf.int`, `boxplot` or `none`
#' @concept plotting
#' @examples
#'
#' income$educf <- cut(income$educ, breaks=3)
#' income |> interaction_plot(inc ~ race + educf)
#' income |> interaction_plot(inc ~ race + educf, interval="conf.int")
#' income |> interaction_plot(inc ~ race + educf, interval="boxplot")
#' income |> interaction_plot(inc ~ race + educf, interval="none")
#' 
#' @import dplyr
#' @import ggplot2

#' @export
interaction_plot <- function(.data, .formula, interval="conf.int"){

    interval = match.arg(tolower(interval), c("conf.int", "boxplot", "none"))

    if (!inherits(.formula, "formula")) stop("'.formula' is not formula")
    if (length(.formula) != 3)          stop("'.formula' is not two sided")
    
    lhs <- all.vars(.formula[[2]])
    rhs <- all.vars(.formula[[3]])
    if (length(rhs) < 2) stop("rhs must have at least two elements")

    rr <- sym(lhs)
    s1 <- sym(rhs[1])
    s2 <- sym(rhs[2])
    ## s2 <- sym(rhs[-1])

    # lapply(rhs, sym)
    
    ## str(list(rr, s1, s2))
    ## If lhs is transformed
    resp <- eval(.formula[[2]], .data)
    .data$RESP <- resp  ## KLUDGY
    rr2 <- sym("RESP")  ## KLUDGY

    dd1 <- .data |> group_by(!!sym(s1), !!sym(s2)) 
    tmp <- dd1 |> summarise(val = mean({{ rr2 }}),
                             sd  = sd({{ rr2 }}) / sqrt(n()),
                             lwr = .data$val - 1.96 * sd, upr=.data$val + 1.96 * sd,
                             .groups="keep")
    ##print(.data); print(dd1); print(tmp)

    switch(interval,
           "boxplot" = {
               ## BOXPLOT
               pp <- ggplot(.data, aes(x = factor(!!sym(s1)), y = !!sym(rr2),
                                       colour = !!sym(s2))) 
               
               pp + labs(y = deparse(.formula[[2]]), x=rhs[1]) +
                   geom_boxplot() + ## FIXME want conf. interval bars
                   geom_point(data = tmp, aes(y = .data$val)) +
                   geom_line(data = tmp, aes(y = .data$val, group = !!sym(s2))) + 
                   theme_bw()
           },
           "conf.int" = {
               ## mean +/- 2 sd
               pp <- ggplot(tmp, aes(x = factor(!!sym(s1)), y = .data$val,
                                     colour = !!sym(s2)))
               
               pp + labs(y = deparse(.formula[[2]]), x=rhs[1]) +
                   geom_point(data = tmp, aes(y = .data$val)) +
                   geom_line(data = tmp, aes(y = .data$val, group = !!sym(s2))) + 
                   geom_errorbar(aes(ymin=.data$lwr, ymax=.data$upr), data=tmp,
                                 width=.4, position=position_dodge(0.1)) +
                   geom_point(aes(x = factor(!!sym(s1)), y = !!sym(rr2),
                                       colour = !!sym(s2)), data=.data)
               

               
           },
           "none" = {
               pp <- ggplot(tmp, aes(x = factor(!!sym(s1)), y = .data$val,
                                     colour = !!sym(s2)))               
               pp + labs(y = deparse(.formula[[2]]), x=rhs[1]) + 
                   geom_point(data = tmp, aes(y = .data$val)) +
                   geom_line(data = tmp, aes(y = .data$val, group = !!sym(s2))) 
           })
}



#' Plot the response variable against the predictor variables.
#'
#' @param formula. A formula of the form y ~ x1 + x2 + ... + xn, where
#'     y is the response variable and x1, x2, ..., xn are the
#'     predictor variables. A dot as right hand side is allowed.
#' 
#' @param data. A data frame containing the variables in the formula.
#' @param geoms A list of ggplot2 geoms to be added to the plot.
#' @param global_aes A list of global aesthetics to be added to the plot.
#' @param plot A logical value indicating whether the plot should be displayed.
#' @param nrow,ncol Number of rows / columns in plot. 
#'
#' @concept plotting
#' @return A list of ggplot2 plots.
#' @export
#'
#' @examples
#' library(ggplot2)
#' response_plot(iris, Sepal.Width ~ ., geoms=geom_point())
#' response_plot(iris, Sepal.Width ~ ., geoms=geom_point(), global_aes=list(color="Species"))
#' personality |> response_plot(easygon~., geoms=geom_point(), global_aes=NULL)
#' 
response_plot <- function(data., formula., geoms=NULL, global_aes=NULL, plot=TRUE, nrow=NULL, ncol=NULL) {  
    trms <- terms(formula., data=data.)
    ## trms
    yy <- as.character(formula.[[2]])
    xx <- setdiff(attr(trms, "term.labels"), yy)
    ##    list(x=xx, y=yy) |> str()

    aes_template <- lapply(xx, function(x_){
        c(list(x=x_, y=yy), global_aes) 
    })
    ## aes_template

    aes_list <- aes_template |> lapply(ggpubr_create_aes)
    ## aes_list
    
    plot_basic <- lapply(aes_list, function(z_){
        data.  |> ggplot(mapping = z_)    
    })
    ## plot_basic
    
    plot_list <-lapply(plot_basic, function(pl_) {
        pl_ + geoms
    } )
    if (plot){
        s <- cowplot::plot_grid(plotlist = plot_list, nrow=nrow, ncol=ncol)        
        print(s)
    }
    
    return(invisible(plot_list))
}


### Taken from ggpubr::create_aes to avoid circular dependencies

ggpubr_create_aes <- function (.list, parse = TRUE) 
{
  # if (missing(parse)) {
  #   parse <- base::getOption("ggpubr.parse_aes", default = TRUE)
  # }
#  if (parse) {
    return(create_aes.parse(.list))
#  }
#  else {
#    return(create_aes.name(.list))
#  }
}

create_aes.parse <- function (.list) 
{
#  .list <- .list %>% purrr::map(function(x) parse_expression(x))
  .list <- .list |> lapply(function(x) parse_expression(x))
  do.call(ggplot2::aes, .list)
}

parse_expression <- function (x) 
{
  if (is_parsable_aes(x)) {
    if (contains_space(x)) {
      if (!is_math_string(x)) 
        return(as.name(x))
    }
    x <- parse(text = x)[[1]]
  }
  x
}

is_numeric_char <- function (x) 
{
  if (is.character(x)) 
    res <- grepl("^[[:digit:]]+$", x)
  else res <- FALSE
  res
}

is_parsable_aes <- function (x) 
{
  is.character(x) & (!is_numeric_char(x)) & (length(x) == 1)
}

contains_space <- function (x) 
{
  grepl("\\s", x)
}

is_math_string <- function (x) 
{
  operators <- c("+", "-", "*", "^", "%%", "%/%", "/", "==", 
                 ">", "<", "!=", "<=", ">=")
  contains_math_operators <- unlist(lapply(operators, grepl, 
                                           x, fixed = TRUE))
  contains_parentheses <- grepl(pattern = "\\(.*\\)", x)
  any(c(contains_math_operators, contains_parentheses))
}




# parse_as_expression <- function (text) 
# {
#   stopifnot(is.character(text))
#   out <- vector("expression", length(text))
#   for (i in seq_along(text)) {
#     expr <- parse(text = text[[i]])
#     out[[i]] <- if (length(expr) == 0) 
#       NA
#     else expr[[1]]
#   }
#   out
# }

#' Align and combine fixed-effect coefficients from multiple models
#'
#' @description
#' Extracts and aligns the fixed-effect estimates from a list of fitted model objects,
#' returning them in a single tidy data frame with consistent columns for easy comparison.
#' Works with a mix of model types such as `lm`, `glm`, `gls`, `lmer`, etc.
#' 
#' For models without p-values (e.g., `lmer`), the function computes approximate
#' Wald statistics and two-sided normal p-values.
#'
#' @param models A named list of fitted model objects. Each element should be a
#' model that can be passed to `broom::tidy()`.
#'
#' @return A tibble with columns:
#' \describe{
#'   \item{model}{The name of the model (from the list).}
#'   \item{term}{The term name (coefficient).}
#'   \item{estimate}{The estimated coefficient.}
#'   \item{std.error}{The standard error.}
#'   \item{statistic}{The Wald statistic (estimate / std.error).}
#'   \item{p.value}{Two-sided normal p-value.}
#' }
#'
#'
#' @examples
#' # Example using the built-in CO2 dataset
#' data(CO2)
#'
#' # Fit models
#' lm_fit  <- lm(uptake ~ conc + Type + Treatment, data = CO2)
#' glm_fit <- glm(uptake ~ conc + Type + Treatment, family = Gamma(identity), data = CO2)
#'
#' # Combine estimates
#' models_list <- list(lm = lm_fit, glm = glm_fit)
#' result <- align_coefs(models_list)
#' print(result)
#'
#' @importFrom purrr imap_dfr
#' @export
align_coefs <- function(models) {
  purrr::imap_dfr(models, function(mod, label) {
    out <- broom::tidy(mod)
    
    # Filter to fixed effects only if 'effect' column exists
    if ("effect" %in% names(out)) {
      out <- out |> dplyr::filter(.data$effect == "fixed")
    }
    
    # Add missing columns with NA
    if (!"std.error" %in% names(out)) out$std.error <- NA_real_
    if (!"statistic" %in% names(out)) out$statistic <- NA_real_
    if (!"p.value" %in% names(out)) out$p.value <- NA_real_

    # Compute missing statistic and p.value if possible
    out |> 
      dplyr::mutate(
        statistic = ifelse(is.na(.data$statistic) & !is.na(.data$estimate) & !is.na(.data$std.error),
                           .data$estimate / .data$std.error, .data$statistic),
        p.value = ifelse(is.na(.data$p.value) & !is.na(.data$statistic),
                         2 * (1 - pnorm(abs(.data$statistic))),
                         .data$p.value),
        model = label
      ) |> 
      dplyr::select(dplyr::all_of(c("model", "term", "estimate", "std.error", "statistic", "p.value")))
  })
}



##' @title Add predicted values of different types to dataframe
##' 
##' @param data dataframe or tibble
##' @param model model object
##' @param var name of new variable in dataframe / tibble
##' @param type type of predicted value
##' @param transformation A possible transformation of predicted variable, e.g. reciprocal(), log() etc
##' @return dataframe / tibble
##' @author Søren Højsgaard
##' @examples
##' data(cars)
##' lm1 <- lm(dist ~ speed + I(speed^2), data=cars)
##' lm1 |> response() |> head()
##' cars <- cars |> add_pred(lm1)
##' cars |> head()
##' cars <- cars |> add_resid(lm1)
##' cars
##' 
##' @export
add_pred <- function (data, model, var = "pred", type = NULL, transformation=NULL) 
{
    pred2 <- function (model, data, type = NULL) 
    {
        if (is.null(type)) {
            stats::predict(model, data)
        }
        else {
            stats::predict(model, data, type = type)
        }
    }

    pp <- pred2(model, data, type = type)
    if (!is.null(transformation)){
        pp <- transformation(pp)
    }
    data[[var]] <- pp
    data
}

## ##' @title Reciprocal function
## ##' @description  A function returning the reciprocal of its argument
## ##' @param x An R object for which 1/x makes sense
## ##' @author Søren Højsgaard
## ##' @export
## reciprocal <- function(x){
##   1/x    
## }

## ##' @title Power function
## ##' @description  A function returning x raised to the power p.
## ##' @param x An object for which x^p makes sense
## ##' @param p A power
## ##' @author Søren Højsgaard
## ##' @export
## pow <- function(x, p){
##   x^p    
## }


##' @title Add residuals of different types to dataframe
##' 
##' @param data dataframe or tibble
##' @param model model object
##' @param var name of new variable in dataframe / tibble
##' @param type type of residual value 
##' @return dataframe / tibble
##' @author Søren Højsgaard
##' @examples
##' data(cars)
##' lm1 <- lm(dist ~ speed + I(speed^2), data=cars)
##' lm1 |> response() |> head()
##' cars <- cars |> add_pred(lm1)
##' cars |> head()
##' cars <- cars |> add_resid(lm1)
##' cars 
##'
##' @export
add_resid <- function (data, model, var = "resid", type) {

    resid2 <- function(model, type){
        UseMethod("resid2")
    }
    resid2.lm <- function(model,
                          type=c("working", "response", "deviance", 
                                 "pearson", "partial", "rstandard", "rstudent")){
        type <- match.arg(type)

        if (identical(type, "rstandard")){
            return(stats::rstandard(model))                        
        }
        
        if (identical(type, "rstudent")){
            return(stats::rstudent(model))                       
        }

        return(stats::residuals(model, type=type))
    }

    resid2.merMod <- function(model,
                              type=c("deviance", "response")){
        return(residuals(model, type=type))
    }
    
    
    if (missing(type))
        type="working"


    data[[var]] <- resid2(model, type)
    data
}


##' @title Get response variable from model
##' @param object lm or glm object 
## ' @param data dataframe or tibble
## ' @param model model object
## ' @param var name of new variable in dataframe / tibble
## ' @param type type of residual value
##' @examples
##' data(cars)
##' lm1 <- lm(dist ~ speed + I(speed^2), data=cars)
##' lm1 |> response() |> head()
##' cars <- cars |> add_pred(lm1)
##' cars |> head()
##' cars <- cars |> add_resid(lm1)
##' cars 
##' @export
response <- function(object){

    is_lm <- function(object) {
        identical(class(object), "lm")
    }
    
    is_glm <- function(object) {
        cls <- class(object)
        (all(c("lm", "glm") %in% cls)) && (length(cls) == 2)
    }
    
    obs <- function(object){
        UseMethod("obs")
    }
    
    obs.lm <- function(object) {
        obs <- model.response(model.frame(object))        
        
        if (is_glm(object)){
            wgt <- unname(model.weights(model.frame(object)))
            if (!is.null(wgt)) {
                obs <- obs * wgt
            }
        }
        return(obs)
    }
    return(obs(object))
}



##' @title Add interaction columns to data frame
##' @param .data dataframe
##' @param .formula right hand sided formula
##' @return dataframe
##' @author Søren Højsgaard
##' @export
add_int <- function(.data, .formula) {

    ff <- rhsf2list(.formula)
    lapply(ff, function(g){
        if (length(g)>1){
            var <- paste(g, collapse="_")
            ia <- apply(.data[,g], 1, function(r) paste(r, collapse="_"))
            .data[[var]] <<- ia
        }})
    .data
}


rhsf2list <- function (.formula) {
    if (is.character(.formula)) 
        return(list(.formula))
    if (is.numeric(.formula)) 
        return(lapply(list(.formula), "as.character"))
    if (is.list(.formula)) 
        return(lapply(.formula, "as.character"))
    .formula0 <- .formula[[length(.formula)]]
    .formula1 <- unlist(strsplit(paste(deparse(.formula0), collapse = ""), 
                             " *\\+ *"))
    .formula2 <- unlist(lapply(.formula1, strsplit, " *\\* *| *: *| *\\| *"), 
                    recursive = FALSE)
    .formula2
}

## Move to doBy
##
## https://stackoverflow.com/questions/70735907/how-to-evaluate-in-a-formula-in-r




##' @title Formula operations and coercion.
##'
##' @description Formula operations and coercion as a supplement to `update.formula()`
##'
##' @name formula_ops
##' 
##' @param chr1 Character vector to be coerced to formulas.
##' @param frm1,frm2 Formulas to be coerced to character vectors.
##' @param y Response
##' @param terms Character string.
##' @param op Either "+" (default) or "-".
##' @param object Character vector or formula.
##' @param noint Boolean.
##' @param string Boolean.
##' @param n Positive integer.
##' 
##' @examples
##'
##' formula_poly("z", 2)
##' formula_poly("z", 2, noint=TRUE)
##'
##' as_rhs_chr(c("a", "b", "z"))
##' as_rhs_chr(c("a*b", "z"))
##'
##' as_rhs_chr(y~a+b+z)
##' as_rhs_chr(y~a+b+z, string=TRUE)
##' as_rhs_chr(y~a+b+z)
##' as_rhs_chr(y~a*b+z)
##' as_rhs_chr(y~a*b+z, string=TRUE)
##'
##' as_lhs_chr(y~a*b+z)
##' as_lhs_chr(log(y)~a*b+z)      ## Not what one might expect
##' as_lhs_chr(cbind(y, u)~a*b+z) ## Not what one might expect
##'
##' formula_chr_to_form(c("a*b", "z"))
##' formula_chr_to_form(c("a*b", "z"), "y")
##' formula_chr_to_form(c("a*b", "z"), "log(y)")
##'
##' formula_add(y~a*b+z, ~-1)
##' formula_add(y~a*b+z, ~a:b)
##'
##' formula_add_str(y~x1 + x2, "x3")
##' formula_add_str(y~x1 + x2, "x1")
##' formula_add_str(y~x1 + x2, "x1", op="-")
##'

##' @rdname formula_ops
#' @export
formula_add_str <- function(frm1, terms, op="+"){
    ## FIXME: Need to handle case where frm1 is a rhs-formula
    ch <- as.character(frm1)
    ch[3] <- paste0(ch[3], op, terms)
    
    frm12 <- as.formula(paste(ch[2], ch[1], ch[3]))
    frm1 <- update(frm1, frm12)
    return(frm1)
}


##' @rdname formula_ops
##' @export
formula_add <- function(frm1, frm2){

    stopifnot_formula(frm1)
    stopifnot_formula(frm2)

    ## Right-hand-side    
    frm1_rhs <- as_rhs_chr(frm1)
    frm2_rhs <- as_rhs_chr(frm2)
    o_rhs <- c(frm1_rhs, frm2_rhs)

    o_rhs <- o_rhs|> paste0(collapse="+") 
    ## str(o_rhs)
    ## Remove redunancies    
    o_rhs <- simplify_rhs(o_rhs)
    
    ## Left-hand-side
    frm1_lhs <- as_lhs_chr(frm1)
    frm2_lhs <- as_lhs_chr(frm2)
    
    if ((length(frm1_lhs) > 0) && (length(frm2_lhs) > 0))
        stop("Can not handle two left-sides\n")

    o_lhs <- c(frm1_lhs, frm2_lhs)
    formula_chr_to_form(o_rhs, o_lhs)
}


##' @rdname formula_ops
##' @export
formula_poly <- function(chr1, n, noint=FALSE, y=NULL){
    if (n > 1){
        b <- paste0(chr1, "^", 2:n)
        o <- paste0("I(", b, ")", collapse = "+")
        o <- paste0(chr1, "+", o)
    } else {
        o <- chr1
    }

    if (noint){
        o <- paste0(y, "-1 +", o)
    }
    formula(paste(y, "~", o))
}

##' @rdname formula_ops
##' @export
formula_nth <- function(frm1, n){
    fs <- as_rhs_chr(frm1, string=TRUE)
    fs2 <- paste0("(", fs, ")^", n)
    o <- as_rhs_frm(fs2)
    o
}

##' @rdname formula_ops
##' @export
formula_to_interaction_matrix <- function(frm1){
    ## aa <- as_rhs_chr(frm1)  |> strsplit(":")
    aa <- terms_labels(frm1)
    aa <- aa |> strsplit(":")

    nms <- unique(unlist(aa))
    mm <- matrix(0, nrow=length(nms), ncol=length(nms), dimnames=list(nms, nms))
    for(i in 1:length(aa)){
        g <- aa[[i]]
        mm[g, g] <- 1
    }
    mm
}


##' @rdname formula_ops
##' @param rhs,lhs right-hand-side and left-hand-side for formula (as characters)
##' @export
formula_chr_to_form <- function(rhs, lhs=character(0)){
    rhs <- to_str(rhs)
    o <- paste0(lhs, "~", rhs)    
    as.formula(o)
}

## UTILITIES

##' @rdname formula_ops
##' @param collapse Character to use as separator.
##' @export
to_str <- function(chr1, collapse="+"){
    paste0(chr1, collapse=collapse)
}

##' @rdname formula_ops
##' @export
terms_labels <- function(frm1){
    terms(frm1)  |> attr("term.labels") |> sort()
}

##' @rdname formula_ops
##' @export
simplify_rhs <- function(object){
    UseMethod("simplify_rhs")
}

##' @rdname formula_ops
##' @export
simplify_rhs.formula <- function(object){
    l <- terms_labels(object)
    to_str(l) |> as_rhs_frm()
}

##' @rdname formula_ops
##' @export
simplify_rhs.character <- function(object){
    o <- as_rhs_frm(object)  |>
        terms_labels() |>
        to_str()
    o
}

## Bad name
formula_rhs_to_chr <- function(frm1, string=TRUE){
    frm1 <- terms_labels(frm1)
    if (length(frm1) == 0)
        frm1 <- "-1"
    if (string)
        frm1 <- frm1 |> to_str()
    frm1
}


is_rhsf <- function(object){
    is(object, "formula") && ((object |> terms() |> attr("response")) == 0)
}

stopifnot_rhsf <- function(object){
    if (!is_rhsf(object))
        stop("argument is not a right-hand-sided formula\n")
}

stopifnot_formula <- function(a){
    if (!is(a, "formula"))
        stop("argument is not a formula")
}

stopifnot_chr <- function(a){
    if (!is(a, "character"))
        stop("argument is not a character")
}



## RETURNING FORMULAS

##' @export
as_rhs_frm.character <- function(object){
    object <- object |> to_str()
    paste0("~", object)  |> as.formula()
}

##' @export
as_rhs_frm.formula <- function(frm1){
    formula(delete.response(terms(frm1)))    
}

##' @export
as_lhs_frm.character <- function(object){
    stopifnot_chr(object)
    paste0(object, "~ 1")  |> as.formula()
}

formula_lhs_to_chr <- function(frm1){
    if (is(frm1, "formula")){
        r <- terms(frm1) |> attr("response")
        if (r > 0)
            frm1 <- (terms(frm1) |> attr("variables"))[[r+1]] |> as.character()
        else
            frm1 <- character(0)
    }
    frm1
}

##' @export
as_lhs_frm.formula <- function(frm1){
    o <- frm1 |> formula_lhs_to_chr()
    as.formula(paste0(o, "~1"))
}


## RETURNING CHARACTERS

##' @export
as_rhs_chr.character <- function(object, string=TRUE){
    ob <- object |> to_str(collapse = "")
    rev(strsplit(ob,"\\s~\\s")[[1]])[1]
}

##' @export
as_rhs_chr.formula <- function(object, string=TRUE){
    formula_rhs_to_chr(object, string=string)    
}

##' @export
as_lhs_chr.character <- function(object, string=TRUE){
    object2 <- strsplit(object,"\\s~\\s")[[1]]
    if (length(object2) == 2)
        object2[1]
    else
        character(0)
}

##' @export
as_lhs_chr.formula <- function(object, string=FALSE){
    r <- terms(object) |> attr("response")
    if (r > 0)
        object <- (terms(object) |> attr("variables"))[[r+1]] |> as.character()
    else
        object <- character(0)
    object
}


## EXPORTED

##' @rdname formula_ops
##' @export
as_rhs_frm <- function(object){
    UseMethod("as_rhs_frm")    
}

##' @rdname formula_ops
##' @export
as_lhs_frm <- function(object){
    UseMethod("as_lhs_frm")
}

##' @rdname formula_ops
##' @export
as_rhs_chr <- function(object, string=FALSE){
    UseMethod("as_rhs_chr")
}

##' @rdname formula_ops
##' @export
as_lhs_chr <- function(object, string=FALSE){
    UseMethod("as_lhs_chr")
}


##' @rdname formula_ops
##' @param list_of_formulas list of formulas
##' @export
unique_formula <- function(list_of_formulas){

    l2 <- lapply(list_of_formulas, function(x){
        environment(x) <- NULL
        x
    }
    )

    o <- list_of_formulas[!duplicated(l2)]
    return(o)
}

## ##' @export
## formula_to_rhs <- function(frm1){
##     terms(frm1) |> delete.response() |> formula()
## }

## FIXME: REPLACE
## formula_chr_to_rhs <- function(chr1){
##     o <- to_str(chr1)
##     as.formula(paste("~", o))
## }

#' Set default values in a functions arguments
#'
#' `set_default()` takes a function and returns a new version with
#' updated default values for specified arguments.
#'
#' This is useful when you want to programmatically create specialized
#' versions of a function with certain arguments preset to default
#' values.
#'
#' @param fun A function whose arguments will get new default values.
#' @param nms Character vector of argument names, or something coercible to that
#'   (e.g. a call to `v()`).
#' @param vls Optional vector or list of values to use as defaults.
#'   If `nms` is a named list, `vls` can be `NULL`.
#' @concept functional
#' @return A new function with updated default values in its formals.
#' 
#' @details
#' - The specified arguments will be moved to the end of the formal argument list in the returned function.
#' - You can supply arguments as a named list or as separate names and values.
#'
#' @examples
#' ## Simple example
#' f1 <- function(x, y, z) { x + y + z }
#'
#' ## Add defaults for x and y
#' set_default(f1, list(x = 1, y = 2))
#' # function (z, x = 1, y = 2) { x + y + z }
#'
#' ## Same using separate vectors of names and values
#' set_default(f1, c("x", "y"), c(1, 2))
#'
#' ## Works with v() style if supported
#' # set_default(f1, v(x, y), c(1, 2))
#'
#' ## Another example with more arguments
#' f2 <- function(a, b, c, d) { a + b + c + d }
#' set_default(f2, list(b = 10, d = 5))
#'
#' @export
set_default <- function(fun, nms, vls = NULL) {
    args <- nms_vls_to_list(nms, vls)

    nms <- names(args)
    vls <- args

    fmls <- formals(fun)

    i <- match(nms, names(fmls))
    j <- c(setdiff(seq_along(fmls), i), i) # new order

    fmls <- fmls[j]
    fmls[nms] <- vls
    formals(fun) <- fmls
    fun
}





## #' @export
## set_default <- function(fun, nms, vls=NULL){

##     args <- nms_vls_to_list(nms, vls)

##     nms <- names(args)
##     vls <- args
    
##     fmls <- formals(fun)
    
##     i <- match(nms, names(fmls))
##     j <- c(setdiff(seq_along(fmls), i), i) # new order
    
##     fmls <- fmls[j]
##     fmls[nms] <- vls
##     formals(fun) <- fmls
##     fun    
## }











#' @title Section a function and set default values in function
#' 
#' @description Section a functions domain by fixing certain
#'     arguments of a function call.
#'
#' @name section_fun
#' @concept functional 
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk} based on code
#'   adapted from the curry package.
#' 
#' @param fun Function to be sectioned
#' @param nms Either a named list of the form name=value where each
#'     name is the name of an argument of the function (in which case
#'     `vls` is ignored) or a character vector of names of arguments.
#' @param vls A vector or list of values of the arguments
#' 
#' @param method One of the following: 1) "args" (default); based on
#'     substituting fixed values into the function argument list as
#'     default values). For backward compatibility can also be "def".
#'     2) "body" for substituting fixed values into the function
#'     body. For backward compatibility can also be "sub". 3) "env":
#'     (for environment); using an auxillary argument for storing
#'     sectioned values. 
#'     
#'
#' @param envir Environment
#' @param object An object from section_fun (a scaffold object).
#'
#' @seealso [doBy::bquote_fun_list()]
#' @details Let E be a subset of the cartesian product X x Y where X
#'     and Y are some sets. Consider a function f(x,y) defined on
#'     E. Then for any x in X, the section of E defined by x (denoted
#'     Ex) is the set of $y$s in Y such that (x, y) is in
#'     E. Correspondingly, the section of f(x,y) defined by x is the
#'     function $f_x$ defined on Ex given by $f_x(y)=f(x,y)$.
#'
#' `section_fun` is a wrapper for calling `set_default` (default
#'     method), `section_fun_env` or `section_fun_sub`. Notice that
#'     creating a sectioned function with `section_fun_sub` can be
#'     time consuming.
#'
#' @return A new function: The input function `fun` but with certain
#'     arguments fixed at specific values.
#' 
#' @examples
#'
#' f  <- function(x, y){x + y}
#' 
#' f_ <- section_fun(f, list(y = 10),    method="args") ## "def"" is default
#' f_ <- section_fun(f, nms="y", vls=10, method="args") ## SAME AS ABOVE
#' f_
#' f_(x=1)
#'
#' f_ <- section_fun(f, list(y = 10),    method="body") ## 
#' f_ <- section_fun(f, nms="y", vls=10, method="body") ## SAME AS ABOVE
#' f_
#' f_(x=1)
#'
#' f_ <- section_fun(f, list(y = 10),    method="env")
#' f_ <- section_fun(f, nms="y", vls=10, method="env") ## SAME AS ABOVE
#' f_
#' f_(x=1)
#' get_section(f_)
#' get_fun(f_)
#'
#'  
#' ## With more complicated values:
#' g <- function(A, B) {
#'   A + B
#' }
#' g_ <- section_fun(g, list(A = matrix(1:4, nrow=2)))
#' g_ <- section_fun(g, "A", list(matrix(1:4, nrow=2)))
#' g_(diag(1, 2))
#'
#' g_ <- section_fun(g, list(A = matrix(1:4, nrow=2)))
#'
#' ## Using built in function
#' set.seed(123)
#' rnorm5 <- section_fun(rnorm, list(n=5)) 
#' rnorm5(0, 1)
#'
#' set.seed(123)
#' rnorm(5)
#'
#' 
NULL

## FIXME: If nms is a list it should be reported if not all elements
## of nms have names. Guess the same applies to caracas.



#' @rdname section_fun
#' @export
section_fun <- function(fun, nms, vls=NULL, method="args") {
    method_ <- match.arg(method, c("args", "env", "body", "def", "sub"))

    args <- nms_vls_to_list(nms, vls)

    switch(method,
           "args"=,
           "def"={set_default(fun, args)},
           "body"=,
           "sub"={section_fun_sub_worker(fun, args)},
           "env"={section_fun_env_worker(fun, args)}
           )        
}



#' @rdname section_fun
#' @export
section_fun_sub <- function(fun, nms, vls=NULL, envir=parent.frame()){

    args <- nms_vls_to_list(nms, vls)
    section_fun_sub_worker(fun, args)
}


section_fun_sub_worker <- function(fun, args, envir=parent.frame()){

    fmls  <- formals(fun)
    idx <- match(names(args), names(fmls))
    idx <- idx[!is.na(idx)]
    if (length(idx) > 0){
        fmls  <- fmls[-idx]
    }


    nn <- names(fmls)
    vv <- paste0(fmls)
    nc <- nchar(vv)
    fmls2 <- ""
    
    if (length(fmls)>0){
        fmls2 <-
            sapply(1:length(fmls), function(i){
                if (nc[i] > 0){
                    paste0(nn[i], "=", vv[i])
                } else {
                    nn[i]
                }    
            })
    }

    hd <- paste0("function(", paste0(fmls2, collapse = ", "), ")")

    aux <- sapply(1:length(args),
                  function(i){
                      nm <- names(args)[i]
                      ss <- deparse(args[[i]])
                      ss <- paste0(ss, collapse = " ")
                      paste0(nm, " = ", ss)
                  })
    
    bd1 <- paste0("\n ## section\n ", paste0(aux, collapse=";\n "), "\n ## section (end)")
    bd1
    
    bd2 <- deparse(body(fun))

    bd2 <- bd2[2:(length(bd2) - 1)]
    bd2 <- gsub("^ *", "", bd2) ## Remove leading whites
    bd2 <- paste0(paste0(bd2, collapse="\n "))
    
    bd <- paste0("\n{ ", paste0(c(bd1, bd2), collapse="\n "), "\n}")
    
    ff  <- paste0(c(hd, bd), collapse="")
    out <- eval(parse(text=ff))
    environment(out) <- environment(fun)
    out

}


nms_vls_to_list <- function(nms, vls){
    ## cat("nms: "); print(nms); cat(" vls: "); print(vls)

    all_has_names <- function(x){
        (!is.null(names(x))) &&
            all(nchar(names(x)) > 0)            
    }
    if (!(is.atomic(nms) || is.list(nms))){
        stop("'nms' must be list or atomic")
    }
    if (inherits(nms, "list")){
        if (!is.null(vls)) {
            warning("vls ignored")
        }
        out <- nms
    } else {
        if (all_has_names(nms)){
            vls <- as.list(nms)
            out <- vls
        } else {                   
            if (length(nms) != length(vls)){
                stop("'nms' and 'vls' must have same length\n")
            }
            vls <- as.list(vls)
            names(vls) <- nms
            out <- vls
        }        
    }
    return(out)
}




#' @rdname section_fun
#' @export
section_fun_env <- function(fun, nms, vls=NULL) {

    args <- nms_vls_to_list(nms, vls)
    section_fun_env_worker(fun, args)
    ##.partial(fun, args)
}


section_fun_env_worker <- function(fun, args){
    fun <- as.section_function(fun)
    apply_args(fun, args)
}



as.section_function <- function(fun) {
  if (!inherits(fun, c("function", "section_function")))
    stop("Can not create section_function\n")

  if (inherits(fun, 'section_function')) {
    section_function_update(fun)
  } else {
    from <- parent.frame()
    section_function_create(fun, from)
  }
}


section_function_create <- function(fun, from = parent.frame()) {

  arg_env <- new.env(parent = emptyenv())
  assign('args',     list(), envir = arg_env)
  assign('args_end', list(), envir = arg_env)
  
  fmls <- get_formals(fun)
  arg_getter <- get_args(arg_env)
  do_section_function(fun, arg_env, fmls, from)  
}


section_function_update <- function(fun, from = parent.frame()) {
  #cat("section_function_update\n")
  arg_env <- clone_environment(attr(fun, "arg_env"))
  fmls <- get_formals(fun)
  #cat("formals:\n"); print(fmls)

  fmls[names(get_section(fun))] <- NULL
  
  fun_orig <- environment(fun)$fun
  arg_getter <- get_args(arg_env)
  do_section_function(fun_orig, arg_env, fmls, from)
}

do_section_function <- function(fun, arg_env, fmls, from){
    ## fmls <- get_formals(fun)
    arg_getter <- get_args(arg_env)
    
    new_fun <- function() {}    
    formals(new_fun) <- fmls
    body(new_fun) <- bquote(
    {
        . <- "use get_section(function_name) to see section"
        . <- "use get_fun(function_name) to see original function"
        args <- arg_getter()
        do.call(.(fun), args)
    },
    list(fun = substitute(fun, from))
    )
  
  structure(new_fun, class = c('section_function', 'function'), arg_env = arg_env)
}




get_formals <- function(fun){
  if (is.primitive(fun)) {
    fmls <- formals(args(fun))
  } else {
    fmls <- formals(fun)
  }
  fmls
}

## SOME UTILITIES

#' @rdname section_fun
#' @export
get_section <- function(object){
    if (!inherits(object, "section_function"))
        stop("'object' must be section_function object\n")
    attr(object, "arg_env")$args
}

#' @rdname section_fun
#' @export
get_fun <- function(object){
    if (!inherits(object, "section_function"))
        stop("'object' must be section_function object\n")
    environment(object)$fun
}

#' @export
print.section_function <- function(x, ...){
  x2 <- x
  attributes(x2) <- NULL
  print.default(x2)
  return(invisible(x))
}


#' @export
summary.section_function <- function(object, ...){

    cat("Original function: \n")
    print(get_fun(object))

    cat("Section: \n")
    str(get_section(object))

    return(invisible(object))
}


get_args <- function(added_env) {
    function() {
        ## cat("in get_args:\n")
        env <- parent.frame()
        args <- names(formals(sys.function(sys.parent(1))))
        ## cat("args:\n ");print(args) ## To be specified in call
        
        if (length(args) > 0) {
            vals <- mget(args, envir = env)
            ## cat("vals:\n"); str(vals)  ## To be specified in call
            
            ellipsis <- names(vals) == '...'
            if (any(ellipsis)) {
                vals <- append(vals, eval(quote(list(...)), env), which(ellipsis))
                vals[ellipsis] <- NULL
            }
            vals <- vals[!vapply(vals, is_missing_arg, logical(1))]
        } else {
            vals <- NULL
        }
        
        ## cat("when done: \n")
        ## str(list(args=added_env$args, vals=vals, args_end=added_env$args_end))   
        out <- c(added_env$args, vals, added_env$args_end)
        ## str(out)
        out
  }
}


apply_args <- function(fun, args, last = FALSE) {
  fmls <- formals(fun)
  #cat("+++ formals: \n"); print(fmls)
  arg_env <- attr(fun, 'arg_env')

  fmls[names(args)] <- NULL
  formals(fun) <- fmls[!names(fmls) %in% names(args)]
  #cat("+++ +++ formals: \n"); print(formals(fun))
  if (last) {
    assign('args_end', append(args, arg_env$args_end), envir = arg_env)
  } else {
    ##arg_env <- clone_environment(arg_env)
    common <- intersect(names(arg_env$args), names(args))

    if (length(common) > 0)
      arg_env$args[common] <- NULL    
    assign('args', append(arg_env$args, args), envir = arg_env)
  }
  structure(fun, class = c('section_function', 'function'), arg_env = arg_env)
}



is_missing_arg <- function(x) {
    identical(x, quote(expr = ))
}

clone_environment <- function(e1){
  as.environment(as.list(e1, all.names=TRUE))
}

test_that("section_fun", {

  g0 = function(x,y,z) {
    x + y + z + u
  }

  g1 <- section_fun_sub(g0, list(z=100))  
  g2 <- section_fun_sub(g1, list(y=10))
  g3 <- section_fun_sub(g2, list(x=1))

  u <- 1000
  expect_equal(g0(x=1, y=10, z=100), 1111)
  expect_equal(g1(x=1, y=10),        1111)
  expect_equal(g2(x=1),              1111)
  expect_equal(g3(),                 1111)

  f0 = function(x,y,z) {
    x + y + z + u
  }

  ## f1 <- section_fun(f0, list(z=200))  
  ## f2 <- section_fun(f1, list(y=20))
  ## f3 <- section_fun(f2, list(x=2))

  
  ## u <- 2000
  ## expect_equal(f0(x=2, y=20, z=200), 2222)
  ## expect_equal(f1(x=2, y=20),        2222)
  ## expect_equal(f2(x=2),              2222)
  ## expect_equal(f3(),                 2222)
  
})

#' @title Groupwise summary statistics
#'
#' @description Computes summary statistics by groups, similar to the `summary` procedure in SAS.
#' A more flexible alternative to base R's \code{\link[stats]{aggregate}}.
#'
#' @name by-summary
#' @concept byfunction
#' @param formula A formula specifying response and grouping variables.
#' @param data A data frame.
#' @param FUN A function or list of functions to apply to the response variables.
#' @param id A formula indicating variables to retain (not grouped by).
#' @param keep.names Logical; keep original variable names if only one function is applied.
#' @param p2d Replace parentheses in output names with dots?
#' @param order Logical; should result be ordered by grouping variables?
#' @param full.dimension Logical; if TRUE, repeat rows so output matches input size.
#' @param var.names Optional custom names for response variables.
#' @param fun.names Optional custom names for functions applied.
#' @param ... Additional arguments passed to functions in `FUN`.
#'
#' @return A data frame of grouped summary statistics.
#'
#' @details
#' Extra arguments in `...` are passed to all functions in `FUN`. If needed, wrap functions to handle these consistently (e.g., for `na.rm = TRUE`).
#'
#' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
#' @seealso \code{\link[stats]{aggregate}}, \code{\link{orderBy}}, \code{\link{transformBy}}, \code{\link{splitBy}}
#' @keywords summary grouping
#'




#' @examples
#' data(CO2)
#'
#' # Simple groupwise mean
#' summaryBy(uptake ~ Type + Treatment, data = CO2, FUN = mean)
#' summaryBy(cbind(uptake, conc) ~ Type + Treatment, data = CO2, FUN = mean)
#'
#' # Compare with
#' aggregate(cbind(uptake, conc) ~ Type + Treatment, data = CO2, FUN = mean)
#' 
#' ## Using '.' on the right hand side of a formula means to stratify by
#' ## all variables not used elsewhere:
#' summaryBy(uptake ~ ., data = CO2, FUN = mean)
#' 
#' # Multiple functions using a custom summary function
#' myfun <- function(x, ...)
#'   c(m = mean(x, na.rm = TRUE), v = var(x, na.rm = TRUE), n = length(x))
#' summaryBy(uptake ~ Type + Treatment, data = CO2, FUN = myfun)
#'
#' # Summary on transformed variables
#' # works:
#' summaryBy(cbind(lu=log(uptake), conc) ~ Type, data = CO2, FUN = mean)
#' # fails:
#' #summaryBy(cbind(log(uptake), conc) ~ Type, data = CO2, FUN = mean)

#' @export
#' @rdname by-summary
summary_by <- function(data, formula, id=NULL, FUN=mean,
                       keep.names=FALSE,
                       p2d=FALSE, order=TRUE, full.dimension=FALSE,
                       var.names=NULL, fun.names=NULL,
                       ...){
    cl   <- match.call(expand.dots = TRUE)
    cl[[2]] <- formula
    cl[[3]] <- data
    names(cl)[2:3] <- c("formula", "data")
    cl[[1]] <- as.name("summaryBy")
    eval(cl)
}


#' @export
#' @rdname by-summary
summaryBy <- function (formula, data=parent.frame(), id=NULL, FUN=mean,
                       keep.names=FALSE,
                       p2d=FALSE, order=TRUE, full.dimension=FALSE,
                       var.names=NULL, fun.names=NULL,
                       ...){

    if (!inherits(data, "tbl_df")) is.tib = FALSE
    else {is.tib = TRUE; data = as.data.frame(data)}
      
    debug.info <- 0

    zzz <- .get_variables(formula, data, id, debug.info) ## ; str(zzz)
    lhs.num <- zzz$lhs.num
    rhs.grp <- zzz$rhs.grp
    ids.var <- zzz$form.ids.var

##    str(list(lhs.num=lhs.num))
    
    rh.trivial <- length( rhs.grp ) == 0 #; cat(sprintf("rh.trivial=%d\n", rh.trivial))

    rh.string <- .get_rhs_string( data, rhs.grp )
    rh.unique <- unique(rh.string)
    rh.idx    <- match(rh.unique, rh.string)
    rh.string.factor <- factor(rh.string, levels=rh.unique) ## This is important

### Get data for id.vars; use ids.var, data, rh.idx
    if (length(ids.var)>0){
      id.data <-  data[ rh.idx, ids.var, drop=FALSE ] ##; print(id.data)
    }

### Get lhs data; use lhs.num, data

    ## Below, we need lhs in the form cbind(y1, y2, y3) for further computations.
    ##
    ## It is OK to have computations in this form a la cbind(y1, y2,
    ## y3, y4=y1+y2, y5=log(y2)).
    ##
    ## If lhs is y1 this is translated into cbind(y1).
    ##
    ## If lhs is ., this is translated into c(y1, y2, y3).
    ##
    ## It is also so (alas) that one may write y1 + y2 + y3,
    ## which is also translated into c(y1, y2, y3)

    
    if (length(lhs.num) > 1) {
        ## If lhs.num is vector of length > 1, wrap with cbind:
        lhs.num <- paste0("cbind( ", toString(lhs.num), " )")
    } else {
        ## strip cbind( ... ) if it is there; then put cbind around
        ## (if lhs is simply y1, then there is no cbind around).
        ff2 <- gsub("^cbind\\((.*)\\)$", "\\1", lhs.num)
        lhs.num <- paste0("cbind( ", ff2, " )")
    }
        
    ## aa <- lapply(paste(lhs.num), function(x)eval(parse(text=x), data))
    ## lh.data <- do.call(cbind, aa)
    ## replace two lines above with
    lh.data <- eval(parse(text=lhs.num), data)

    ## Hack: redefine lhs.num : name of variables in data frame
    lhs.num <- colnames(lh.data)
    
    ##colnames(lh.data) <- lhs.num


### Function names; use FUN
    funNames <- .get_fun_names( FUN )

    ##print(rh.string.factor)
### Calculate groupwise statistics
    if (!is.list(FUN))
        FUN <- list(FUN)

      out <- NULL
      for (ff in 1:length(FUN)) {  ## loop over functions
          ##currFUN <- FUN[[ff]]
          currFUN <- match.fun( FUN[[ff]] )
          for (vv in 1:length(lhs.num)) {  ## loop over variables
              
              currVAR <- lh.data[,lhs.num[vv]]
              
              zzz <- tapply(currVAR, rh.string.factor,
                            function(x){ currFUN(x, ...) }, simplify=FALSE)
              zzz  <- do.call(rbind, zzz)
              out  <- cbind(out, zzz)
          }
      }
      
      
      
      if (!is.null(var.names) && length(var.names)==length(lhs.num))
          lhs.names <- var.names
      else
          lhs.names <- lhs.num
      
### Set names for columns
      
      ##print(funNames)
      if (!is.null(fun.names) ) ##&& length(fun.names)==length(funNames))
          funNames <- fun.names
      
      newnames <- .get_col_names(ncol(out), colnames(out), funNames,
                                 lhs.names, keep.names)
      ##cat(sprintf("newnames    = %s\n", toString( newnames )))
      colnames(out) <- newnames
      out <- as.data.frame(out)
      
      
### Pad the rhs data to the result
      if (!rh.trivial){
          out <- cbind(data[rh.idx, rhs.grp, drop=FALSE], out)
      }
      
### Pad id.data to result
      ##print(id.data)
      if (length(ids.var)>0){
          out <- cbind(out, id.data)
      }
      
### Must the result have full dimension?
      if (full.dimension){
          rrr <- as.numeric(rh.string.factor)
          out <- out[rrr,, drop=FALSE]
      }
      
### Order the result by the rhs
      if (order && !rh.trivial){
          rhs.string <- paste (rhs.grp, collapse='+')
          out <- orderBy(as.formula(paste("~", rhs.string)), data=out)
      }
      
### Replace '('s and ')'s with '.'s
      if (p2d)
          names(out) <-  gsub("\\)","\\.", gsub("\\(","\\.",names(out)))
      
### Finalize
      rownames(out) <- 1:nrow(out)
      if (length(unique(names(out))) != length(names(out)))
          warning("dataframe contains replicate names \n", call.=FALSE)
      
      if (is.tib) as_tibble(out) else out      
      ## out      
}





.get_rhs_string <- function(data, rhs.var, sep.string="@"){

  if (length(rhs.var)==0){
    rep.int("1", nrow(data))
  } else {
    rh.string <- paste(data[,rhs.var[1]])
    if (length( rhs.var ) > 1){
      for (ii in 2:length( rhs.var )){
        rh.string <- paste(rh.string, sep.string, data[, rhs.var[ii]], sep='')
      }
    }
    rh.string
  }
}


.get_fun_names <- function( FUN ){

    if (!is.list(FUN))
      funNames <- paste(deparse(substitute(FUN, env=parent.frame())), collapse = " ")
    else
      funNames <- unlist(lapply(substitute(FUN, env=parent.frame())[-1], function(a) paste(a)))

    ##cat(sprintf("funNames   = %s\n", toString(funNames)))
    funNames
}

.get_col_names <- function(ncol.ans, colNames, funNames, lhs.num, keep.names){
### Names for new variables

  ## Does the columns of ans have names??
  oldnames <- colNames #colnames(ans)
  if (is.null(oldnames))
    hasNames <- 0
  else {
    hasNames <- 1*(prod(nchar(oldnames))>0)
  }
  ##cat(sprintf("hasNames=%i\n", hasNames))

  ## Dim of response (per variable on LHS)
  dimr     <- (ncol.ans)/length(lhs.num)

  only.one.response <- (ncol.ans==length(lhs.num))
  if ( keep.names && only.one.response ){
    newnames <- lhs.num
  } else {
    if (hasNames>0){ ## newnames = var.fun
      funNames <- colNames[1:dimr]
      newnames <- unlist(lapply(lhs.num, function(v){paste(v, funNames, sep='.')}))
    } else {
      if (length(funNames) != dimr){
        funNames <- paste("FUN", 1:dimr, sep='')
        newnames <- unlist(lapply(lhs.num, function(v){paste(v, funNames, sep='.')}))
      } else {
        newnames <- unlist(lapply(funNames, function(x) paste(lhs.num, x, sep='.')))
      }
      if (length(newnames)!=ncol.ans){
        funNames <- paste(funNames, 1:dimr, sep=".")
        newnames  <- unlist(lapply(funNames, function(x) paste(lhs.num, x, sep='.')))
      }
    }
  }
  newnames
}


.get_variables <- function(formula, data, id, debug.info){

    data.var  <- names(data)

    if (!inherits(formula, c("formula", "list")))
        stop("'formula' must be a formula or a list")
    
    if (inherits(formula, "formula")){
        if (length(formula) != 3) stop("Formula must have a left hand side")
        rhs       <- formula[[3]]
        form.rhs.var   <- all.vars(rhs) ## May contain "." and "1"
        
        lhs       <- formula[[2]]
        form.lhs.var   <- all.vars(lhs) ## May contain "."
                                        #print(form.lhs.var)
        
        zz <- .lhsParse(lhs)
        form.lhs.var <-
            if (length(zz)==1)
                paste(zz)
            else
                paste(unlist(.lhsParse(lhs)))
        
        ##print(form.lhs.var)
        
    } else {
        if (length(formula)>=2){
            lhs <- formula[[1]]
            rhs <- formula[[2]]
            form.lhs.var <- lhs
            form.rhs.var <- rhs
        } else {
            stop("Invalid specification of formula")
        }
    }
    
    if (is.null(id)){
        form.ids.var <- character(0)
    } else {
        if (!inherits(id, c("formula", "character"))){
            stop("'id' must be a formula or a character vector")
        }

        if (inherits(id, "formula")){
            form.ids.var   <- all.vars(id)
        } else {
            form.ids.var   <- id
        }
    }
    
    data.cls  <- lapply(data, class)
    data.num.idx   <- data.cls %in% c("numeric","integer")
    data.num.var   <- data.var[ data.num.idx  ]
    data.fac.var   <- data.var[ !data.num.idx ]
    
    ##   print(form.lhs.var)
    ##   print(data.num.var)
    lhs.num   <- intersect( form.lhs.var, data.num.var )
    rhs.num   <- intersect( form.rhs.var, data.num.var )
    ids.num   <- intersect( form.ids.var, data.num.var )
    lhs.fac   <- intersect( form.lhs.var, data.fac.var )
    rhs.fac   <- intersect( form.rhs.var, data.fac.var )
    ids.fac   <- intersect( form.ids.var, data.fac.var )
    
    lll <- list(data.var=data.var,
                form.lhs.var=form.lhs.var, form.rhs.var=form.rhs.var, form.ids.var=form.ids.var,
                lhs.num=lhs.num, rhs.num=rhs.num, ids.num=ids.num,
                lhs.fac=lhs.fac, rhs.fac=rhs.fac, ids.fac=ids.fac )
    
                                        #if (debug.info>=1)
    ##{ cat("status:\n"); str(lll, vec.len=20) }
    
    if ( "." %in% form.lhs.var ){ ## need all numeric variables not metioned elswhere on lhs
        form.lhs.var <- setdiff(form.lhs.var, ".")
        lhs.num <- union( form.lhs.var, setdiff(data.num.var, c(rhs.num, ids.num)))
        if ( length( lhs.fac ) > 0 ){
            isSpecial <- rep(NA, length( lhs.fac ))
            for (j in 1:length(lhs.fac)){
                isSpecial[j]<- (class(data[,lhs.fac[j]])[1] %in% c("POSIXt", "Date"))
            }
            lhs.num <- union( lhs.num, lhs.fac[ isSpecial ] )
        }
    } else {
        lhs.num <- form.lhs.var
    }
    
    ## The grouping variable
    if ("." %in% form.rhs.var){ ## need all factors not mentioned elsewhere as grouping factors
        free.fac <- setdiff(data.fac.var, c(lhs.fac, ids.fac))
        rhs.grp  <- c(setdiff(form.rhs.var, "."), free.fac)
    } else {
        rhs.grp <- form.rhs.var
    }
    rhs.grp <- intersect( rhs.grp, data.var )
    
    rrr <- list(lhs.num=lhs.num, rhs.fac=rhs.fac, form.ids.var=form.ids.var,
                form.rhs.var=form.rhs.var, rhs.grp=rhs.grp)
    ##str(rrr)
    rrr
}



.lhsParse <- function(x){
    ##cat(".lhsParse:"); print(x); print(class(x))
    if (inherits(x, 'name')){
        value <- x
    } else {
        s <- paste(x[[1]])
        value <- switch(s,
                        '+'={  c(.lhsParse(x[[2]]),.lhsParse(x[[3]]))},
                        'I'={  x[[2]]},
                        {  deparse(x)})
    }
    value
}






## ' 
## ' data(dietox)
## ' dietox12    <- subset(dietox,Time==12)
## '
## ' fun <- function(x){
## '   c(m=mean(x), v=var(x), n=length(x))
## ' }
## ' 
## ' summaryBy(cbind(Weight, Feed) ~ Evit + Cu, data=dietox12,
## '           FUN=fun)
## ' 
## ' summaryBy(list(c("Weight", "Feed"), c("Evit", "Cu")), data=dietox12,
## '           FUN=fun)
## '
## ' ## Computations on several variables is done using cbind( )
## ' summaryBy(cbind(Weight, Feed) ~ Evit + Cu, data=subset(dietox, Time > 1),
## '    FUN=fun)
## ' 
## ' ## Calculations on transformed data is possible using cbind( ), but
## ' # the transformed variables must be named
## ' 
## ' summaryBy(cbind(lw=log(Weight), Feed) ~ Evit + Cu, data=dietox12, FUN=mean)
## '  
## ' ## There are missing values in the 'airquality' data, so we remove these
## ' ## before calculating mean and variance with 'na.rm=TRUE'. However the
## ' ## length function does not accept any such argument. Hence we get
## ' ## around this by defining our own summary function in which length is
## ' ## not supplied with this argument while mean and var are:
## ' 
## ' sumfun <- function(x, ...){
## '   c(m=mean(x, na.rm=TRUE, ...), v=var(x, na.rm=TRUE, ...), l=length(x))
## ' }
## ' summaryBy(cbind(Ozone, Solar.R) ~ Month, data=airquality, FUN=sumfun)
## ' ## Compare with
## ' aggregate(cbind(Ozone, Solar.R) ~ Month, data=airquality, FUN=sumfun)
## ' 
## ' ## Using '.' on the right hand side of a formula means to stratify by
## ' ## all variables not used elsewhere:
## ' 
## ' data(warpbreaks)
## ' summaryBy(breaks ~ wool + tension, warpbreaks, FUN=mean)
## ' summaryBy(breaks ~ ., warpbreaks, FUN=mean)
## ' summaryBy(. ~ wool + tension, warpbreaks, FUN=mean)
## ' 
## ' summaryBy(. ~ wool + tension, warpbreaks, FUN=mean)
## '
## '
## '



## ' @title Function to calculate groupwise summary statistics
## ' @description Function to calculate groupwise summary statistics,
## '     much like the summary procedure of SAS
## ' @name by-summary


## '
## ' @details Extra arguments (...) are passed onto the functions in
## '     FUN. Hence care must be taken that all functions in FUN accept
## '     these arguments - OR one can explicitly write a functions which
## '     get around this.  This can particularly be an issue in
## '     connection with handling NAs. See examples below.  Some code
## '     for this function has been suggested by Jim
## '     Robison-Cox. Thanks.
## ' 
## ' @param formula A formula object, see examples below.
## ' @param data A data frame.
## ' @param FUN A list of functions to be applied, see examples below.
## ' @param id A formula specifying variables which data are not grouped by but
## '     which should appear in the output. See examples below.
## ' @param keep.names If TRUE and if there is only ONE function in FUN, then the
## '     variables in the output will have the same name as the variables in the
## '     input, see 'examples'.
## ' @param p2d Should parentheses in output variable names be replaced by dots?
## ' @param order Should the resulting dataframe be ordered according to the
## '     variables on the right hand side of the formula? (using \link{orderBy}
## ' @param full.dimension If TRUE then rows of summary statistics are repeated
## '     such that the result will have the same number of rows as the input
## '     dataset.
## ' @param var.names Option for user to specify the names of the variables on the
## '     left hand side.
## ' @param fun.names Option for user to specify function names to apply to the
## '     variables on the left hand side.
## ' @param \dots Additional arguments to FUN. This could for example be NA actions.

## ' @return A dataframe.
## ' @author Søren Højsgaard, \email{sorenh@@math.aau.dk}
## ' @seealso \code{\link{ave}}, \code{\link{descStat}}, \code{\link{orderBy}}, \code{\link{order_by}},
## '     \code{\link{splitBy}}, \code{\link{split_by}}, \code{\link{transformBy}}, \code{\link{transform_by}}
## ' @keywords univar

%\VignetteEngine{knitr::knitr} 
%\VignettePackage{doBy}
%\VignetteIndexEntry{doby: Groupwise computations and miscellaneous utilities}






\documentclass[11pt]{article}

\usepackage{hyperref,url,color,Sweave,a4wide}
\usepackage[utf8]{inputenc}


\usepackage[authoryear,round]{natbib}
\bibliographystyle{plainnat}

\RequirePackage{color,fancyvrb,amsmath,amsfonts}

\def\textasciigrave{'}
\def\proglang#1{{#1}}
\def\R{\proglang{R}}
\def\pkg#1{{\bf #1}}
\def\doby{\pkg{doBy}}
\def\code#1{\texttt{#1}}
\def\summaryby{\code{summaryBy}}


% reduce whitespace between R code and R output
\let\oldknitrout\knitrout
\renewenvironment{knitrout}{
  \begin{oldknitrout}
    \footnotesize
    \topsep=0pt
}{
  \end{oldknitrout}
}

<<echo=FALSE>>=
require( doBy )
prettyVersion <- packageDescription("doBy")$Version
prettyDate <- format(Sys.Date())
@

\title{Groupwise computations and other utilities in the \texttt{doBy} package}
\author{S{\o}ren H{\o}jsgaard}
\date{\pkg{doBy} version \Sexpr{prettyVersion} as of \Sexpr{prettyDate}}


<<include=FALSE>>=
library(knitr)
@

% show R> prompt before R commands
<<r setup, echo=FALSE>>=
knitr::opts_chunk$set(prompt=TRUE) 
library(doBy)
if (!dir.exists("figures")) dir.create("figures")
opts_chunk$set(
               tidy=FALSE,fig.path='figures/doBy'
           )

oopt <- options()
options("digits"=4, "width"=90, "prompt"="> ", "continue"="  ")
options(useFancyQuotes="UTF-8")
@ %def


\definecolor{darkred}{rgb}{.7,0,0}
\definecolor{midnightblue}{rgb}{0.098,0.098,0.439}





\begin{document}

\maketitle
\tableofcontents
\parindent0pt
\parskip5pt

\section{Introduction}
\label{sec:introduction}


The \doby{} package contains a variety of utility functions. This
working document describes some of these functions. The package
originally grew out of a need to calculate groupwise summary
statistics (much in the spirit of \code{PROC SUMMARY} of the
\proglang{SAS} system), but today the package contains many different
utilities.

<<echo=F>>=
library(doBy)
@ %def

\section{Data used for illustration}
\label{sec:co2data}

The description of the \code{doBy} package is based on the \code{mtcars}
dataset.

<<>>=
head(mtcars)
tail(mtcars)
@ 

A description of the variable names can be found here: \url{https://rstudio-pubs-static.s3.amazonaws.com/61800_faea93548c6b49cc91cd0c5ef5059894.html}; see also \url{https://www.jstor.org/stable/2529336?seq=12}.




\section{Groupwise computations}

\subsection{The \texttt{summaryBy}  and \texttt{summary\_by} functions}

\label{sec:summaryBy}

The \summaryby{} function is used for calculating quantities like
``the mean and variance of numerical variables $x$ and $y$ for
each combination of two factors $A$ and $B$''. 
Notice: A functionality similar to \summaryby\ is provided by 
\code{aggregate()} from base \R.
%% 

<<>>=
myfun1 <- function(x){
    c(m=mean(x), s=sd(x))
}
summaryBy(cbind(mpg, cyl, lh=log(hp)) ~ vs, 
          data=mtcars, FUN=myfun1)
@ %def

A simpler call is
<<>>=
summaryBy(mpg ~ vs, data=mtcars, FUN=mean)
@

Instead of formula we may specify a list containing the left hand side
and the right hand side of a formula\footnote{This is a feature of
  \summaryby\ and it does not work with \code{aggregate}.} but that is
possible only for variables already in the dataframe:
<<>>=
summaryBy(list(c("mpg", "cyl"), "vs"), 
          data=mtcars, FUN=myfun1)
@

Inspired by the \pkg{dplyr} package, there is a \verb|summary_by| function which
does the samme as \summaryby{} but with the data argument being the first so
that one may write
<<>>= 
mtcars |> summary_by(cbind(mpg, cyl, lh=log(hp)) ~ vs,
                      FUN=myfun1)
@



\subsection{The \code{orderBy} function and \texttt{order\_by} functions}
\label{orderBy}

Ordering (or sorting) a data frame is possible with the \code{orderBy}
function. For example, we order the rows according to \code{gear} and \code{carb} (within \code{gear}):
<<>>=
x1 <- orderBy(~ gear + carb, data=mtcars)
head(x1, 4)
tail(x1, 4)
@ %def

If we want the ordering to be by decreasing values of one of the
variables, we can do
<<>>=
x2 <- orderBy(~ -gear + carb, data=mtcars)
@ %def

Alternative forms are:
<< >>=
x3 <- orderBy(c("gear", "carb"), data=mtcars)
x4 <- orderBy(c("-gear", "carb"), data=mtcars)
x5 <- mtcars |> order_by(c("gear", "carb"))
x6 <- mtcars |> order_by(~ -gear + carb)
@

 
\subsection{The \code{splitBy} and \texttt{split\_by} functions}
\label{splitBy}

Suppose we want to split the \code{airquality} data into a list of dataframes, e.g.\ one
dataframe for each month. This can be achieved by:
<<>>=
x <- splitBy(~ Month, data=airquality)
x <- splitBy(~ vs, data=mtcars)
lapply(x, head, 4)
attributes(x)
@ %def

Alternative forms are:
<< >>=
splitBy("vs", data=mtcars)
mtcars |> split_by(~ vs)
@
%% 
 


\subsection{The \code{subsetBy} and \code{subset\_by}  functions}
\label{subsetBy}

Suppose we want to select those rows within each month for which the the
wind speed is larger than the mean wind speed (within the month). This
is achieved by:
<<>>=
x <- subsetBy(~am, subset=mpg > mean(mpg), data=mtcars)
head(x)
@ %def
Note that the statement \code{Wind > mean(Wind)} is evaluated within
each month. 

Alternative forms are
<< >>=
x <- subsetBy("am", subset=mpg > mean(mpg), data=mtcars)
x <- mtcars  |> subset_by("vs", subset=mpg > mean(mpg))
x <- mtcars  |> subset_by(~vs, subset=mpg > mean(mpg))
@



\subsection{The \code{transformBy} and \code{transform\_by} functions}
\label{sec:transformby}

The \code{transformBy} function is analogous to the \code{transform}
function except that it works within groups. For example:
<<>>=
head(x)
x <- transformBy(~vs, data=mtcars, 
                 min.mpg=min(mpg), max.mpg=max(mpg))
head(x)
@ %def

Alternative forms:
<< >>=
x <- transformBy("vs", data=mtcars, 
                 min.mpg=min(mpg), max.mpg=max(mpg))
x <- mtcars |> transform_by("vs",
                             min.mpg=min(mpg), max.mpg=max(mpg))
@


\subsection{The \code{lapplyBy} and \code{lapply\_by} function}
\label{sec:lapplyby}

This \code{lapplyBy} function is a wrapper for first splitting data
into a list according to the formula (using splitBy) and then applying
a function to each element of the list (using lapply).

<<>>=
lapplyBy(~vs, data=mtcars,
         FUN=function(d) lm(mpg~cyl, data=d)  |> summary()  |> coef())
@ 





\section{Miscellaneous utilities}
% \label{sec:xxx}


\subsection{The \code{firstobs()} / \code{lastobs()} functions}
\label{firstlast}

To obtain the indices of the first/last occurences of an item in a
vector do:
<<>>=
x <- c(1, 1, 1, 2, 2, 2, 1, 1, 1, 3)
firstobs(x)
lastobs(x)
@ %def

The same can be done on variables in a data frame, e.g.
<<>>=
firstobs(~vs, data=mtcars)
lastobs(~vs, data=mtcars)
@ %def

\subsection{The \code{which.maxn()} and \code{which.minn()} functions}
\label{sec:whichmaxn}

The location of the $n$ largest / smallest entries in a numeric vector
can be obtained with
<<>>=
x <- c(1:4, 0:5, 11, NA, NA)
which.maxn(x, 3)
which.minn(x, 5)
@ %def

\subsection{Subsequences - \code{subSeq()}}
% \label{sec:xxx}

Find (sub) sequences in a vector:

<<>>=
x <- c(1, 1, 2, 2, 2, 1, 1, 3, 3, 3, 3, 1, 1, 1)
subSeq(x)
subSeq(x, item=1)
subSeq(letters[x])
subSeq(letters[x], item="a")
@ %def


\subsection{Recoding values of a vector - \code{recodeVar()}}
% \label{sec:xxx}

<<>>=
x <- c("dec", "jan", "feb", "mar", "apr", "may")
src1 <- list(c("dec", "jan", "feb"), c("mar", "apr", "may"))
tgt1 <- list("winter", "spring")
recodeVar(x, src=src1, tgt=tgt1)
@ %def

\subsection{Renaming columns of a dataframe or matrix --  \code{renameCol()}}
% \label{sec:xxx}

<<>>=
head(renameCol(mtcars, c("vs", "mpg"), c("vs_", "mpg_")))
@ %def

\subsection{Time since an event - \code{timeSinceEvent()}}
% \label{sec:xxx}

Consider the vector
<<>>=
yvar <- c(0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0)
@ %def

Imagine that "1" indicates an event of some kind which takes place
at a certain time point. By default time points are assumed
equidistant but for illustration we define time time variable
<<>>=
tvar <- seq_along(yvar) + c(0.1, 0.2)
@ %def

Now we find time since event as
<<>>=
tse <- timeSinceEvent(yvar, tvar)
tse
@ %def

The output reads as follows:
\begin{itemize}
\item \verb"abs.tse": Absolute time since (nearest) event.
\item \verb"sign.tse": Signed time since (nearest) event.
\item \verb"ewin": Event window: Gives a symmetric window around each event.
\item \verb"run": The value of \verb"run" is set to $1$ when the first
  event occurs and is increased by $1$ at each subsequent event.
\item \verb"tae": Time after event.
\item \verb"tbe": Time before event.
\end{itemize}

<<fig.height=4>>=
plot(sign.tse ~ tvar, data=tse, type="b")
grid()
rug(tse$tvar[tse$yvar == 1], col="blue",lwd=4)
points(scale(tse$run), col=tse$run, lwd=2)
lines(abs.tse + .2 ~ tvar, data=tse, type="b",col=3)
@ %def

<<fig.height=4>>=
plot(tae ~ tvar, data=tse, ylim=c(-6,6), type="b")
grid()
lines(tbe ~ tvar, data=tse, type="b", col="red")
rug(tse$tvar[tse$yvar==1], col="blue", lwd=4)
lines(run ~ tvar, data=tse, col="cyan", lwd=2)
@ %def

<<fig.height=4>>=
plot(ewin ~ tvar, data=tse, ylim=c(1, 4))
rug(tse$tvar[tse$yvar==1], col="blue", lwd=4)
grid()
lines(run ~ tvar, data=tse, col="red")
@ %def


We may now find times for which time since an event is at most 1 as
<<>>=
tse$tvar[tse$abs <= 1]
@ %def

\subsection{Example: Using \code{subSeq()} and \code{timeSinceEvent()}}

Consider the \verb|lynx| data:
<<fig.height=4>>=
lynx <- as.numeric(lynx)
tvar <- 1821:1934
plot(tvar, lynx, type="l")
@ %def

Suppose we want to estimate the cycle lengths. One way of doing this
is as follows:
<<>>=
yyy <- lynx > mean(lynx)
head(yyy)
sss <- subSeq(yyy, TRUE)
sss
@ %def

<<fig.height=4>>=
plot(tvar, lynx, type="l")
rug(tvar[sss$midpoint], col="blue", lwd=4)
@ %def

Create the "event vector"
<<>>=
yvar <- rep(0, length(lynx))
yvar[sss$midpoint] <- 1
str(yvar)
@ %def

<<>>=
tse <- timeSinceEvent(yvar,tvar)
head(tse, 20)
@ %def

We get two different (not that different) estimates of period
lengths:
<<>>=
len1 <- tapply(tse$ewin, tse$ewin, length)
len2 <- tapply(tse$run, tse$run, length)
c(median(len1), median(len2), mean(len1), mean(len2))
@ %def

We can overlay the cycles as:
<<fig.height=4>>=
tse$lynx <- lynx
tse2 <- na.omit(tse)
plot(lynx ~ tae, data=tse2)
@ %def

<<fig.height=4>>=
plot(tvar, lynx, type="l", lty=2)
mm <- lm(lynx ~ tae + I(tae^2) + I(tae^3), data=tse2)
lines(fitted(mm) ~ tvar, data=tse2, col="red")
@ %def


\section{Acknowledgements}
\label{discussion}

Credit is due to
Dennis Chabot, Gabor Grothendieck, Paul Murrell and Jim Robison-Cox for
reporting various bugs and making various suggestions to the
functionality in the \doby{} package.

<<echo=F>>=
options(oopt)
@ %def


\end{document}



% \appendix

% \section{The data}
% \label{sec:appdata}

% The reduced \code{C02} are:
% <<>>=
% CO2
% @ %def

% The reduced \code{airquality} data are:
% <<>>=
% head(airquality, n=20)
% @ %def











% On the to-do-list is to allow to write as follows (but this feature is not
% implemented yet):
% <<eval=F>>= 
% summaryBy(list(c("conc", "uptake", lu="log(uptake)"), "Plant"), 
%           data=CO2, FUN=c(mean, sd))
% @

% Inspired by the \pkg{dplyr} package, there is a \verb|summary_by| function which
% does the samme as \summaryby{} but with the data argument being the first so
% that one may write
% <<results=hide>>= 
% CO2 |> summary_by(cbind(conc, uptake, lu=log(uptake)) ~ Plant, 
%                    FUN=myfun1)
% @
% which is the same as writing:
% <<results=hide>>= 
% summary_by(CO2, cbind(conc, uptake, lu=log(uptake)) ~ Plant, 
%            FUN=myfun1)
% @
 




%% Same as
%% <<results=hide>>= 
%% aggregate(cbind(conc, uptake, log(uptake)) ~ Plant, data=CO2, FUN=myfun1)
%% aggregate(conc ~ Plant, data=CO2, FUN=mean)
%% @
%% 


%%<< >>= 
%%library(magrittr)
%%CO2 |> summary_by(cbind(conc, uptake) ~ Plant, FUN=myfun1)
%%@
%%
%%
%%<< >>= 
%%summaryBy2( list(c("conc","uptake"), "Plant"), data=CO2, FUN=myfun1)
%%@
%%
%%
%%Above \code{myfun1()} is a function that returns a vector of named
%%values. Note that the values returned by the function has been named as
%%\code{m} and \code{v}. An alternative specification is:
%%
%%@
%%<<>>=
%%summaryBy( list(c("conc","uptake"), "Plant"), data=CO2, FUN=myfun1)
%%@ %def
%%
%%If the result of the function(s) are not named, then the names in the
%%output data in general become less intuitive:
%%@
%%<<>>=
%%myfun2 <- function(x){c(mean(x), var(x))}
%%summaryBy( conc + uptake ~ Plant, data=CO2, FUN=myfun2)
%%@ %def
%%
%%
%%Another usage is to specify a list of functions each of which returns
%%a single value:
%%
%%@
%%<<>>=
%%summaryByOLD( conc + uptake ~ Plant, data=CO2, FUN=list( mean, var ) )
%%@ %def
%%


%%Notice that if we specify a list of functions of which some returns a
%%vector with more than one element, then the proper names are not
%%retrieved:
%%%%@
%%<<>>=
%%summaryBy(uptake~Plant, data=CO2, FUN=list( mean, var, myfun1 ))
%%@ %def
%%
%%One can ``hard code'' the function names into the output as
%%@
%%<<>>=
%%summaryByOLD(uptake~Plant, data=CO2, FUN=list( mean, var, myfun1 ),
%%          fun.names=c("mean","var","mm","vv"))
%%@ %def
%%

% \subsubsection{Statistics on functions of data}
% \label{sec:xxx}
% We may want to calculate the mean and variance for the logarithm of
% \code{uptake}, for \code{uptake}+\code{conc} (not likely to be a
% useful statistic) as well as for \code{uptake} and
% \code{conc}. This can be achieved as:
% @
% <<>>=
% summaryByOLD(log(uptake) + I(conc+uptake) + conc+uptake ~ Plant, data=CO2,
%           FUN=myfun1)
% @ %def
%%
%%@
%%<<>>=
%%summaryBy(cbind(lu=log(uptake), conc + uptake, conc, uptake) ~ Plant, data=CO2,
%%          FUN=myfun1)
%%@ %def
%%



%%
%%
%%The names of the variables become involved with this. The user may
%%control the names of the variables directly:
%%@
%%<<>>=
%%summaryBy(log(uptake) + I(conc+uptake) + conc + uptake ~ Plant, data=CO2,
%%          FUN=myfun1, var.names=c("log.upt", "conc+upt", "conc", "upt"))
%%@ %def
%%



%%If one does not want output variables to contain parentheses then
%%setting \code{p2d=TRUE} causes the parentheses to be replaced by dots
%%(``.'').
%%@
%%<<>>=
%%summaryBy(log(uptake)+I(conc+uptake)~Plant, data=CO2, p2d=TRUE,
%%FUN=myfun1)
%%@ %def
%%
%%
%%
%%
%%
%%\subsubsection{Copying variables out with the \code{id} argument}
%%\label{sec:xxx}
%%
%%To get the value of the \code{Type} and \code{Treat} in the first row of the
%%groups (defined by the values of \code{Plant}) copied to the output
%%dataframe we use the \code{id} argument in one of the following forms:
%%
%%@
%%<<>>=
%%summaryBy(conc+uptake~Plant, data=CO2, FUN=myfun1, id=~Type+Treat)
%%summaryBy(conc+uptake~Plant, data=CO2, FUN=myfun1, id=c("Type","Treat"))
%%@ %def
%%








%%\subsubsection{Using '.' on the left hand side of a formula}
%%\label{sec:xxx}
%%
%%It is possible  to use the dot (".") on the left hand side of
%%the formula. The dot means "all numerical variables which do not
%%appear elsewhere" (i.e.\ on the right hand side of the formula and in
%%the \code{id} statement):
%%@
%%<<>>=
%%summaryBy(log(uptake)+I(conc+uptake)+. ~Plant, data=CO2, FUN=myfun1)
%%@ %def
%%
%%
%%\subsubsection{Using '.' on the right hand side of a formula}
%%\label{sec:xxx}
%%
%%The dot (".") can also be used on the right hand side of the formula
%%where it refers to "all non--numerical variables which are not
%%specified elsewhere":
%%@
%%<<>>=
%%summaryBy(log(uptake) ~Plant+., data=CO2, FUN=myfun1)
%%@ %def
%%
%%\subsubsection{Using '1' on the right hand side of the formula}
%%\label{sec:xxx}
%%
%%Using 1 on the
%%  right hand side means no grouping:
%%@
%%<<>>=
%%summaryBy(log(uptake) ~ 1, data=CO2, FUN=myfun1)
%%@ %def
%%
%%
%%
%%\subsubsection{Preserving names of variables using \code{keep.names}}
%%\label{sec:xxx}
%%If the function applied to data only returns one value, it is possible
%%to force that the summary variables retain the original names by
%%setting \code{keep.names=TRUE}. A
%%typical use of this could be
%%@
%%<<>>=
%%summaryBy(conc+uptake+log(uptake)~Plant,
%%data=CO2, FUN=mean, id=~Type+Treat, keep.names=TRUE)
%%@ %def
%%







%%\subsection{The \code{splitBy} function}
%%\label{splitBy}
%%
%%Suppose we want to split the \code{airquality} data into a list of dataframes, e.g.\ one
%%dataframe for each month. This can be achieved by:
%%@
%%<<>>=
%%x<-splitBy(~Month, data=airquality)
%%x
%%@ %def
%%Hence for month 5, the relevant entry-name in the list is '5' and this
%%part of data  can
%%be extracted as
%%@
%%%<<eval=F, results=hide>>=
%%%x[['5']]
%%%@ %def
%%
%%Information about the grouping is stored as a dataframe
%%in an attribute called \code{groupid} and can be retrieved with:
%%<<>>=
%%attr(x,"groupid")
%%@ %def
%%


%% \subsection{The \code{sampleBy} function}
%% \label{sampleBy}
%% 
%% Suppose we want a random sample of 50 \% of the observations from a
%% dataframe. This can be achieved with:
%% @
%% <<results=hide>>=
%% sampleBy(~1, frac=0.5, data=airquality)
%% @ %def
%% 
%% Suppose instead that we want a  systematic sample of  every fifth
%% observation within each month. This is achieved with:
%% @
%% <<results=hide>>=
%% sampleBy(~Month, frac=0.2, data=airquality,systematic=T)
%% @ %def
%% 
%% 


%% Inspired by the \pkg{dplyr} package, there is an \verb|order_by| function
%% << >>= 
%% x5 <- airquality |> order_by(c("Temp", "Month"))
%% x6 <- airquality |> order_by(c("-Temp", "Month"))
%% @
%% which is the same as
%% << >>= 
%% x5 <- order_by(airquality, c("Temp", "Month"))
%% x6 <- order_by(airquality, c("-Temp", "Month"))
%% @

%% <<echo=F, results=hide>>= 
%% c(all.equal(x1, x3), all.equal(x1, x5), 
%%   all.equal(x2, x4), all.equal(x2, x6))
%% @



%% Suppose we want to calculate the weekwise feed efficiency of the pigs
%% in the \code{dietox} data, i.e. weight gain divided by feed intake.
%% @
%% <<>>=
%% data(dietox)
%% dietox <- orderBy(~Pig+Time, data=dietox)
%% FEfun  <- function(d){c(NA, diff(d$Weight)/diff(d$Feed))}
%% v      <- lapplyBy(~Pig, data=dietox, FEfun)
%% dietox$FE <- unlist(v)
%% @ %def
%% 
%% Technically, the above is the same as
%% @
%% <<>>=
%% dietox <- orderBy(~Pig+Time, data=dietox)
%% wdata  <- splitBy(~Pig, data=dietox)
%% v      <- lapply(wdata, FEfun)
%% dietox$FE <- unlist(v)
%% @ %def
%% 
%%
%% \subsection{The \code{scaleBy} function}
%% 
%% Standardize the \code{iris} data within each value of \code{"Species"}:
%% @
%% <<>>=
%% x <- scaleBy(cbind(Sepal.Length, Sepal.Width, Petal.Length, Petal.Width) ~ Species,     
%%              data=iris)
%% lapply(x, head, 4)
%% head(iris)
%% @ def
%% 
%% Alternative forms:
%% << >>= 
%% x <- scaleBy( list(c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"),
%%                  "Species"),     data=iris)
%% x <- scaleBy(. ~ Species, data=iris)
%% x <- scaleBy(list(".", "Species"),  data=iris)
%% x <- iris  >% scale_by(. ~ Species)
%% @
%% 









%%\section{Create By--functions on the fly}
%%\label{sec:create-functions-fly}
%%
%%Create a function for creating groupwise t-tests
%%
%%@
%%<<>>=
%%mydata <- data.frame(y=rnorm(32), x=rnorm(32),
%%g1=factor(rep(c(1,2),each=16)), g2=factor(rep(c(1,2), each=8)),
%%g3=factor(rep(c(1,2),each=4)))
%%head(mydata)
%%@ %def
%%
%%
%%@
%%<<>>=
%%## Based on the formula interface to t.test
%%t.testBy1 <- function(formula, group, data, ...){
%%  formulaFunBy(formula, group, data, FUN=t.test, class="t.testBy1", ...)
%%}
%%## Based on the default interface to t.test
%%t.testBy2 <- function(formula, group, data, ...){
%%  xyFunBy(formula, group, data, FUN=t.test, class="t.testBy1", ...)
%%}
%%@ %def
%%
%%Notice: The optional \code{class} argument will facilitate that you
%%create your own print / summary methods etc.
%%
%%@
%%<<>>=
%%t.testBy1(y~g1, ~g2, data=mydata)
%%t.testBy2(y~x,  ~g2, data=mydata)
%%@ %def
%%

---
title: "Pipe-friendly Arithmetic Helpers in doBy"
author: "Søren Højsgaard"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Pipe-friendly Arithmetic Helpers in doBy}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

The **doBy** package includes a suite of simple, *pipe-friendly* arithmetic
helper functions that make common transformations clearer and more expressive
when using the native R pipe (`|>`).  

These helpers let you write transformations in readable pipelines
without nested or hard-to-read expressions. All functions are
vectorized and work with scalars or numeric vectors.

---

## Available functions

- `add()`: add a constant
- `subtract()`: subtract a constant
- `mult()`: multiply by a constant
- `divide()`: divide by a constant
- `reciprocal()`: compute 1/x
- `pow()`: raise to a power

---

## Load doBy

```r
library(doBy)
x <- c(1, 2, 3)

# Addition
x |> add(5)
# [1] 6 7 8

# Subtraction
x |> subtract(1)
# [1] 0 1 2

# Multiplication
x |> mult(10)
# [1] 10 20 30

# Division
x |> divide(2)
# [1] 0.5 1.0 1.5

# Reciprocal
x |> reciprocal()
# [1] 1.0000000 0.5000000 0.3333333

# Power
x |> pow(2)
# [1] 1 4 9
x |>
  mult(2) |>
  add(3) |>
  reciprocal()
```

---
title: "Stability selection"
author: "Søren Højsgaard"
output: 
  html_document:
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{stability: Stability selection}
  %\VignettePackage{doBy}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options("digits"=3)
library(doBy)
library(boot)
#devtools::load_all()
```




## Stability selection: differences in models due to different data sets

Suppose we have collection of regression models $M_1, M_2, \ldots, M_p$. By a regression model we here mean a set of predictors and a model for the response. To asses the predictive ability of the models we perform cross validation: Fit each model to one set of data and predict another set of data, do so repeatedly. Compute a prediction error for each model. The model with the smallest prediction error is the best model.

But before we get there, we need to select a model to generate the collection of candidate models. One approach is the following:

A model selection method consists of two parts:

1.  A criterion for selecting a model and
2.  A strategy for generating candidate models, that is a strategy for navigating the model space.

If we have $K$ datasets we can perform $K$ model selections, and this gives us a collection of candidate models. We can then perform cross validation on these models.
Say we want to perform $K$ different model selections. We can do this by resampling or by creating subgroups. Resampling is done by drawing $n$ observations with replacement from the data set. Subgroups are created by dividing the data set into $K$ subgroups as follows: Form $K$ subgroups by dividing the data set into $K$ equal parts. Then for each $k=1,\ldots,K$ the $k$th subgroup is left out and the model is fitted to the remaining data. Default below is the subgroup method.

## Generating data sets

Consider this subset of the mtcars dataset:

```{r}
dat0 <- mtcars[1:6,]
dat0
```

Generate a list of data sets by resampling or by creating subgroups.

```{r}
generate_data_list(dat0, K=3, method="subgroups")
generate_data_list(dat0, K=3, method="resample")

```

## Model selection


Focus on entire dataset. For evaluation of the predictive ability of the selected model we set 20% of the data aside for validation. Hence we select models based on the remaining 80% of the data.
Fit a linear model to each data set and select the best model by stepwise regression. This gives an indication of the stability of the model.

```{r}
set.seed(1411)
dat <- personality
train <- sample(1:nrow(dat), 0.6*nrow(dat))
dat.training <- dat[train, ]
dat.testing <- dat[-train, ]
```

```{r}
mod1 <- glm(agreebl ~ ., data = dat.training)
```

For each fold we can select models on the analysis part of the data set.

```{r}
set.seed(1411)
n.searches <- 12
mod_stab <- model_stability_glm(dat.training, mod1,
                           n.searches=n.searches, method="subgroups", trace=0)
mod_stab
summary(mod_stab)
```

Specifically, the matrix shows the predictors selected in each model and below is the frequency with which each model is selected.

The outcome of the steps above is a list of models and as it is the same model selection method applied to each group of data, the difference (if any) in models is due to differences in the data in each group.



We can obtain fitted models as follows

```{r}
formula(mod_stab, fit=FALSE) [1:2]
```

This gives us a set of candidate models. 


## Predictive performance

We have selected a model based on subgroups of the training data. We can now evaluate the predictive performance of the selected model on the test data.

```{r}
fit_list <- formula(mod_stab, fit=TRUE)

set.seed(1411)
cv.error <- cv_glm_fitlist(dat.training, fit_list, K=4)
cv.error


cv3 <- sapply(fit_list, 
       function(fit) {
         x <- update(fit, data=dat.training)
         modelr::rmse(x, dat.testing)
       }
)
cv3

plot(cv.error, cv3)
```

We can now evaluate the predictive performance of the selected model on the test data.

```{r}
par(mfrow=c(1,2))
plot(sqrt(cv.error), main="Subgroups")
plot(cv3, main="Subgroups")
```

The plot on the left shows the cross validation error for the models selected in the subgroups. The plot on the right shows the prediction error for the models on the test data.

## Stability selection

---
title: "Functional programming: Sectioning function to a smaller domain"
author: "Søren Højsgaard"
output: 
  rmarkdown::html_vignette
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{section_fun: Section functions to a smaller domain}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options("digits"=3)
library(doBy)
library(microbenchmark)
#devtools::load_all()
```


## Sectioning a function domain with `section_fun()`

The `section_fun` utility in **doBy** creates a new function by **fixing some arguments** of an existing function. The result is a *section* of the original function, defined only on the remaining arguments.

For example, if you have:

$$
f(x,y) = x + y
$$

then fixing $x=10$ yields:

$$
f_x(y) = 10 + y
$$

In `R` terms, `section_fun` lets you programmatically create such specialized versions.

---

## How `section_fun` works

`section_fun()` offers three ways to fix arguments:

1. **Defaults (method = "def")** – Inserts the fixed values as defaults in the argument list.
2. **Substitution (method = "sub")** – Rewrites the function body with the fixed values.
3. **Environment (method = "env")** – Stores fixed values in an auxiliary environment.

Example:

```{r}
fun  <- function(a, b, c=4, d=9) {
    a + b + c + d
}
```

```{r}
fun_def <- section_fun(fun, list(b=7, d=10))
fun_def
fun_body <- section_fun(fun, list(b=7, d=10), method="sub")
fun_body
fun_env <- section_fun(fun, list(b=7, d=10), method = "env")
fun_env
```

You can inspect the environment-based section:

```{r}
get_section(fun_env) 
## same as: attr(fun_env, "arg_env")$args 
get_fun(fun_env) 
## same as: environment(fun_env)$fun
```




Example evaluations:

```{r}
fun(a=10, b=7, c=5, d=10)
fun_def(a=10, c=5)
fun_body(a=10, c=5)
fun_env(a=10, c=5)
```





## Benchmarking example

Suppose you want to benchmark a function for different input values without writing repetitive code:

```{r}
inv_toep <- function(n) {
    solve(toeplitz(1:n))
}
```

Instead of typing the following

```{r, eval=F}
microbenchmark(
    inv_toep(4), inv_toep(8), inv_toep(16),
    times=3
)
```

you can create specialized versions programmatically:

```{r}
n.vec  <- c(4, 8, 16)
fun_list <- lapply(n.vec,
                   function(ni) {
                       section_fun(inv_toep, list(n=ni))
                   })
fun_list
```

Inspect and evaluate:

```{r}
fun_list[[1]]
fun_list[[1]]()
```

To use with microbenchmark, we need expressions:

```{r}
bquote_list <- function(fun_list) {
    lapply(fun_list, function(g){
        bquote(.(g)())
    })
}
```

We get:

```{r}
bq_fun_list <- bquote_list(fun_list)
bq_fun_list
bq_fun_list[[1]]
eval(bq_fun_list[[1]])
```


Now run:

```{r}
microbenchmark(
  list = bq_fun_list,
  times = 5
)
```

Running the code below provides a benchmark of the different ways of sectioning in terms of speed.

```{r}
n.vec  <- seq(20, 80, by=20)
fun_def <- lapply(n.vec,
                  function(n){
                      section_fun(inv_toep, list(n=n), method="def")
                  })
fun_body <- lapply(n.vec,
                  function(n){
                      section_fun(inv_toep, list(n=n), method="sub")
                  })
fun_env <- lapply(n.vec,
                  function(n){
                      section_fun(inv_toep, list(n=n), method="env")
                  })

names(fun_def)  <- paste0("def", n.vec)
names(fun_body) <- paste0("body", n.vec)
names(fun_env)  <- paste0("env", n.vec)

bq_fun_list <- bquote_list(c(fun_def, fun_body, fun_env))
bq_fun_list |> head()

mb <- microbenchmark(
  list  = bq_fun_list,
  times = 2
)
mb
```

---
title: "doby: Groupwise computations and miscellaneous utilities"
author: "Søren Højsgaard"
output: 
  rmarkdown::html_vignette:
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{doby: Groupwise computations and miscellaneous utilities}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options("digits"=3)
library(doBy)
library(boot)
#devtools::load_all()
```


The \doby{} package contains a variety of utility functions. This
working document describes some of these functions. The package
originally grew out of a need to calculate groupwise summary
statistics (much in the spirit of \code{PROC SUMMARY} of the
\proglang{SAS} system), but today the package contains many different
utilities.


```{r}
library(doBy)
```

\section{Data used for illustration}
\label{sec:co2data}

The description of the \code{doBy} package is based on the \code{mtcars}
dataset.

```{r}
head(mtcars)
tail(mtcars)
``` 







# Groupwise computations


## summaryBy()  and summary_by()

\label{sec:summaryBy}

The \summaryby{} function is used for calculating quantities like
``the mean and variance of numerical variables $x$ and $y$ for
each combination of two factors $A$ and $B$''. 
Notice: A functionality similar to \summaryby\ is provided by 
\code{aggregate()} from base \R.

```{r}
myfun1 <- function(x){
    c(m=mean(x), s=sd(x))
}
summaryBy(cbind(mpg, cyl, lh=log(hp)) ~ vs, 
          data=mtcars, FUN=myfun1)
```

A simpler call is
```{r}
summaryBy(mpg ~ vs, data=mtcars, FUN=mean)
```

Instead of formula we may specify a list containing the left hand side
and the right hand side of a formula\footnote{This is a feature of
  \summaryby\ and it does not work with \code{aggregate}.} but that is
possible only for variables already in the dataframe:

```{r}
summaryBy(list(c("mpg", "cyl"), "vs"), 
          data=mtcars, FUN=myfun1)
```

Inspired by the \pkg{dplyr} package, there is a \verb|summary_by| function which
does the samme as \summaryby{} but with the data argument being the first so
that one may write

```{r} 
mtcars |> summary_by(cbind(mpg, cyl, lh=log(hp)) ~ vs,
                      FUN=myfun1)
```


## orderBy() and order_by()

Ordering (or sorting) a data frame is possible with the \code{orderBy}
function. For example, we order the rows according to \code{gear} and \code{carb} (within \code{gear}):
```{r}
x1 <- orderBy(~ gear + carb, data=mtcars)
head(x1, 4)
tail(x1, 4)
``` 

If we want the ordering to be by decreasing values of one of the
variables, we can do
```{r}
x2 <- orderBy(~ -gear + carb, data=mtcars)
``` 

Alternative forms are:
```{r}
x3 <- orderBy(c("gear", "carb"), data=mtcars)
x4 <- orderBy(c("-gear", "carb"), data=mtcars)
x5 <- mtcars |> order_by(c("gear", "carb"))
x6 <- mtcars |> order_by(~ -gear + carb)
```


## splitBy() and split_by()

Suppose we want to split the \code{airquality} data into a list of dataframes, e.g.\ one
dataframe for each month. This can be achieved by:
```{r}
x <- splitBy(~ Month, data=airquality)
x <- splitBy(~ vs, data=mtcars)
lapply(x, head, 4)
attributes(x)
``` 

Alternative forms are:
```{r}
splitBy("vs", data=mtcars)
mtcars |> split_by(~ vs)
```
 

## subsetBy() and subset_by()

Suppose we want to select those rows within each month for which the the
wind speed is larger than the mean wind speed (within the month). This
is achieved by:
```{r}
x <- subsetBy(~am, subset=mpg > mean(mpg), data=mtcars)
head(x)
``` 
Note that the statement \code{Wind > mean(Wind)} is evaluated within
each month. 

Alternative forms are
```{r}
x <- subsetBy("am", subset=mpg > mean(mpg), data=mtcars)
x <- mtcars  |> subset_by("vs", subset=mpg > mean(mpg))
x <- mtcars  |> subset_by(~vs, subset=mpg > mean(mpg))
```


## transformBy() and transform_by()


The \code{transformBy} function is analogous to the \code{transform}
function except that it works within groups. For example:
```{r}
head(x)
x <- transformBy(~vs, data=mtcars, 
                 min.mpg=min(mpg), max.mpg=max(mpg))
head(x)
``` 

Alternative forms:
```{r}
x <- transformBy("vs", data=mtcars, 
                 min.mpg=min(mpg), max.mpg=max(mpg))
x <- mtcars |> transform_by("vs",
                             min.mpg=min(mpg), max.mpg=max(mpg))
```


## lapplyBy() and lapply_by()

This \code{lapplyBy} function is a wrapper for first splitting data
into a list according to the formula (using splitBy) and then applying
a function to each element of the list (using lapply).

```{r}
lapplyBy(~vs, data=mtcars,
         FUN=function(d) lm(mpg~cyl, data=d)  |> summary()  |> coef())
``` 



# Miscellaneous utilities

## firstobs() and lastobs()

To obtain the indices of the first/last occurences of an item in a
vector do:
```{r}
x <- c(1, 1, 1, 2, 2, 2, 1, 1, 1, 3)
firstobs(x)
lastobs(x)
``` 

The same can be done on variables in a data frame, e.g.
```{r}
firstobs(~vs, data=mtcars)
lastobs(~vs, data=mtcars)
``` 

\subsection{The \code{which.maxn()} and \code{which.minn()} functions}
\label{sec:whichmaxn}

The location of the $n$ largest / smallest entries in a numeric vector
can be obtained with
```{r}
x <- c(1:4, 0:5, 11, NA, NA)
which.maxn(x, 3)
which.minn(x, 5)
``` 

## Subsequences - subSeq()


Find (sub) sequences in a vector:

```{r}
x <- c(1, 1, 2, 2, 2, 1, 1, 3, 3, 3, 3, 1, 1, 1)
subSeq(x)
subSeq(x, item=1)
subSeq(letters[x])
subSeq(letters[x], item="a")
``` 

## Recoding values of a vector - recodeVar()


```{r}
x <- c("dec", "jan", "feb", "mar", "apr", "may")
src1 <- list(c("dec", "jan", "feb"), c("mar", "apr", "may"))
tgt1 <- list("winter", "spring")
recodeVar(x, src=src1, tgt=tgt1)
``` 


## Renaming columns of a dataframe or matrix - renameCol()

```{r}
head(renameCol(mtcars, c("vs", "mpg"), c("vs_", "mpg_")))
``` 


## Time since an event - timeSinceEvent()


Consider the vector
```{r}
yvar <- c(0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0)
``` 

Imagine that "1" indicates an event of some kind which takes place
at a certain time point. By default time points are assumed
equidistant but for illustration we define time time variable
```{r}
tvar <- seq_along(yvar) + c(0.1, 0.2)
``` 

Now we find time since event as
```{r}
tse <- timeSinceEvent(yvar, tvar)
tse
``` 

The output reads as follows:
\begin{itemize}
\item \verb"abs.tse": Absolute time since (nearest) event.
\item \verb"sign.tse": Signed time since (nearest) event.
\item \verb"ewin": Event window: Gives a symmetric window around each event.
\item \verb"run": The value of \verb"run" is set to $1$ when the first
  event occurs and is increased by $1$ at each subsequent event.
\item \verb"tae": Time after event.
\item \verb"tbe": Time before event.
\end{itemize}

```{r}
plot(sign.tse ~ tvar, data=tse, type="b")
grid()
rug(tse$tvar[tse$yvar == 1], col="blue",lwd=4)
points(scale(tse$run), col=tse$run, lwd=2)
lines(abs.tse + .2 ~ tvar, data=tse, type="b",col=3)
``` 

```{r}
plot(tae ~ tvar, data=tse, ylim=c(-6,6), type="b")
grid()
lines(tbe ~ tvar, data=tse, type="b", col="red")
rug(tse$tvar[tse$yvar==1], col="blue", lwd=4)
lines(run ~ tvar, data=tse, col="cyan", lwd=2)
``` 

```{r}
plot(ewin ~ tvar, data=tse, ylim=c(1, 4))
rug(tse$tvar[tse$yvar==1], col="blue", lwd=4)
grid()
lines(run ~ tvar, data=tse, col="red")
``` 


We may now f
ind times for which time since an event is at most 1 as
```{r}
tse$tvar[tse$abs <= 1]
``` 


## Example: Using subSeq() and timeSinceEvent()


Consider the \verb|lynx| data:

```{r}
lynx <- as.numeric(lynx)
tvar <- 1821:1934
plot(tvar, lynx, type="l")
``` 

Suppose we want to estimate the cycle lengths. One way of doing this
is as follows:

```{r}
yyy <- lynx > mean(lynx)
head(yyy)
sss <- subSeq(yyy, TRUE)
sss
``` 


```{r}
plot(tvar, lynx, type="l")
rug(tvar[sss$midpoint], col="blue", lwd=4)
``` 

Create the "event vector"
```{r}
yvar <- rep(0, length(lynx))
yvar[sss$midpoint] <- 1
str(yvar)
``` 

```{r}
tse <- timeSinceEvent(yvar,tvar)
head(tse, 20)
``` 

We get two different (not that different) estimates of period
lengths:
```{r}
len1 <- tapply(tse$ewin, tse$ewin, length)
len2 <- tapply(tse$run, tse$run, length)
c(median(len1), median(len2), mean(len1), mean(len2))
``` 

We can overlay the cycles as:
```{r}
tse$lynx <- lynx
tse2 <- na.omit(tse)
plot(lynx ~ tae, data=tse2)
```

```{r}
plot(tvar, lynx, type="l", lty=2)
mm <- lm(lynx ~ tae + I(tae^2) + I(tae^3), data=tse2)
lines(fitted(mm) ~ tvar, data=tse2, col="red")
```



\section{Acknowledgements}
\label{discussion}

Credit is due to
Dennis Chabot, Gabor Grothendieck, Paul Murrell and Jim Robison-Cox for
reporting various bugs and making various suggestions to the
functionality in the \doby{} package.

<!-- ```  -->


<!-- \end{document} -->



<!-- % \appendix -->

<!-- % \section{The data} -->
<!-- % \label{sec:appdata} -->

<!-- % The reduced \code{C02} are: -->
<!-- % ```{r} -->
<!-- % CO2 -->
<!-- % ```  -->

<!-- % The reduced \code{airquality} data are: -->
<!-- % ```{r} -->
<!-- % head(airquality, n=20) -->
<!-- % ```  -->











<!-- % On the to-do-list is to allow to write as follows (but this feature is not -->
<!-- % implemented yet): -->
<!-- % <<eval=F>>=  -->
<!-- % summaryBy(list(c("conc", "uptake", lu="log(uptake)"), "Plant"),  -->
<!-- %           data=CO2, FUN=c(mean, sd)) -->
<!-- % ``` -->

<!-- % Inspired by the \pkg{dplyr} package, there is a \verb|summary_by| function which -->
<!-- % does the samme as \summaryby{} but with the data argument being the first so -->
<!-- % that one may write -->
<!-- % <<results=hide>>=  -->
<!-- % CO2 |> summary_by(cbind(conc, uptake, lu=log(uptake)) ~ Plant,  -->
<!-- %                    FUN=myfun1) -->
<!-- % ``` -->
<!-- % which is the same as writing: -->
<!-- % <<results=hide>>=  -->
<!-- % summary_by(CO2, cbind(conc, uptake, lu=log(uptake)) ~ Plant,  -->
<!-- %            FUN=myfun1) -->
<!-- % ``` -->
 




<!-- %% Same as -->
<!-- %% <<results=hide>>=  -->
<!-- %% aggregate(cbind(conc, uptake, log(uptake)) ~ Plant, data=CO2, FUN=myfun1) -->
<!-- %% aggregate(conc ~ Plant, data=CO2, FUN=mean) -->
<!-- %% ``` -->
<!-- %%  -->


<!-- %%<< >>=  -->
<!-- %%library(magrittr) -->
<!-- %%CO2 |> summary_by(cbind(conc, uptake) ~ Plant, FUN=myfun1) -->
<!-- %%``` -->
<!-- %% -->
<!-- %% -->
<!-- %%<< >>=  -->
<!-- %%summaryBy2( list(c("conc","uptake"), "Plant"), data=CO2, FUN=myfun1) -->
<!-- %%``` -->
<!-- %% -->
<!-- %% -->
<!-- %%Above \code{myfun1()} is a function that returns a vector of named -->
<!-- %%values. Note that the values returned by the function has been named as -->
<!-- %%\code{m} and \code{v}. An alternative specification is: -->
<!-- %% -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy( list(c("conc","uptake"), "Plant"), data=CO2, FUN=myfun1) -->
<!-- %%```  -->
<!-- %% -->
<!-- %%If the result of the function(s) are not named, then the names in the -->
<!-- %%output data in general become less intuitive: -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%myfun2 <- function(x){c(mean(x), var(x))} -->
<!-- %%summaryBy( conc + uptake ~ Plant, data=CO2, FUN=myfun2) -->
<!-- %%```  -->
<!-- %% -->
<!-- %% -->
<!-- %%Another usage is to specify a list of functions each of which returns -->
<!-- %%a single value: -->
<!-- %% -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryByOLD( conc + uptake ~ Plant, data=CO2, FUN=list( mean, var ) ) -->
<!-- %%```  -->
<!-- %% -->


<!-- %%Notice that if we specify a list of functions of which some returns a -->
<!-- %%vector with more than one element, then the proper names are not -->
<!-- %%retrieved: -->
<!-- %%%%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(uptake~Plant, data=CO2, FUN=list( mean, var, myfun1 )) -->
<!-- %%```  -->
<!-- %% -->
<!-- %%One can ``hard code'' the function names into the output as -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryByOLD(uptake~Plant, data=CO2, FUN=list( mean, var, myfun1 ), -->
<!-- %%          fun.names=c("mean","var","mm","vv")) -->
<!-- %%```  -->
<!-- %% -->

<!-- % \subsubsection{Statistics on functions of data} -->
<!-- % \label{sec:xxx} -->
<!-- % We may want to calculate the mean and variance for the logarithm of -->
<!-- % \code{uptake}, for \code{uptake}+\code{conc} (not likely to be a -->
<!-- % useful statistic) as well as for \code{uptake} and -->
<!-- % \code{conc}. This can be achieved as: -->
<!-- % ``` -->
<!-- % ```{r} -->
<!-- % summaryByOLD(log(uptake) + I(conc+uptake) + conc+uptake ~ Plant, data=CO2, -->
<!-- %           FUN=myfun1) -->
<!-- % ```  -->
<!-- %% -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(cbind(lu=log(uptake), conc + uptake, conc, uptake) ~ Plant, data=CO2, -->
<!-- %%          FUN=myfun1) -->
<!-- %%```  -->
<!-- %% -->



<!-- %% -->
<!-- %% -->
<!-- %%The names of the variables become involved with this. The user may -->
<!-- %%control the names of the variables directly: -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(log(uptake) + I(conc+uptake) + conc + uptake ~ Plant, data=CO2, -->
<!-- %%          FUN=myfun1, var.names=c("log.upt", "conc+upt", "conc", "upt")) -->
<!-- %%```  -->
<!-- %% -->



<!-- %%If one does not want output variables to contain parentheses then -->
<!-- %%setting \code{p2d=TRUE} causes the parentheses to be replaced by dots -->
<!-- %%(``.''). -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(log(uptake)+I(conc+uptake)~Plant, data=CO2, p2d=TRUE, -->
<!-- %%FUN=myfun1) -->
<!-- %%```  -->
<!-- %% -->
<!-- %% -->
<!-- %% -->
<!-- %% -->
<!-- %% -->
<!-- %%\subsubsection{Copying variables out with the \code{id} argument} -->
<!-- %%\label{sec:xxx} -->
<!-- %% -->
<!-- %%To get the value of the \code{Type} and \code{Treat} in the first row of the -->
<!-- %%groups (defined by the values of \code{Plant}) copied to the output -->
<!-- %%dataframe we use the \code{id} argument in one of the following forms: -->
<!-- %% -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(conc+uptake~Plant, data=CO2, FUN=myfun1, id=~Type+Treat) -->
<!-- %%summaryBy(conc+uptake~Plant, data=CO2, FUN=myfun1, id=c("Type","Treat")) -->
<!-- %%```  -->
<!-- %% -->








<!-- %%\subsubsection{Using '.' on the left hand side of a formula} -->
<!-- %%\label{sec:xxx} -->
<!-- %% -->
<!-- %%It is possible  to use the dot (".") on the left hand side of -->
<!-- %%the formula. The dot means "all numerical variables which do not -->
<!-- %%appear elsewhere" (i.e.\ on the right hand side of the formula and in -->
<!-- %%the \code{id} statement): -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(log(uptake)+I(conc+uptake)+. ~Plant, data=CO2, FUN=myfun1) -->
<!-- %%```  -->
<!-- %% -->
<!-- %% -->
<!-- %%\subsubsection{Using '.' on the right hand side of a formula} -->
<!-- %%\label{sec:xxx} -->
<!-- %% -->
<!-- %%The dot (".") can also be used on the right hand side of the formula -->
<!-- %%where it refers to "all non--numerical variables which are not -->
<!-- %%specified elsewhere": -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(log(uptake) ~Plant+., data=CO2, FUN=myfun1) -->
<!-- %%```  -->
<!-- %% -->
<!-- %%\subsubsection{Using '1' on the right hand side of the formula} -->
<!-- %%\label{sec:xxx} -->
<!-- %% -->
<!-- %%Using 1 on the -->
<!-- %%  right hand side means no grouping: -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(log(uptake) ~ 1, data=CO2, FUN=myfun1) -->
<!-- %%```  -->
<!-- %% -->
<!-- %% -->
<!-- %% -->
<!-- %%\subsubsection{Preserving names of variables using \code{keep.names}} -->
<!-- %%\label{sec:xxx} -->
<!-- %%If the function applied to data only returns one value, it is possible -->
<!-- %%to force that the summary variables retain the original names by -->
<!-- %%setting \code{keep.names=TRUE}. A -->
<!-- %%typical use of this could be -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%summaryBy(conc+uptake+log(uptake)~Plant, -->
<!-- %%data=CO2, FUN=mean, id=~Type+Treat, keep.names=TRUE) -->
<!-- %%```  -->
<!-- %% -->







<!-- %%\subsection{The \code{splitBy} function} -->
<!-- %%\label{splitBy} -->
<!-- %% -->
<!-- %%Suppose we want to split the \code{airquality} data into a list of dataframes, e.g.\ one -->
<!-- %%dataframe for each month. This can be achieved by: -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%x<-splitBy(~Month, data=airquality) -->
<!-- %%x -->
<!-- %%```  -->
<!-- %%Hence for month 5, the relevant entry-name in the list is '5' and this -->
<!-- %%part of data  can -->
<!-- %%be extracted as -->
<!-- %%``` -->
<!-- %%%<<eval=F, results=hide>>= -->
<!-- %%%x[['5']] -->
<!-- %%%```  -->
<!-- %% -->
<!-- %%Information about the grouping is stored as a dataframe -->
<!-- %%in an attribute called \code{groupid} and can be retrieved with: -->
<!-- %%```{r} -->
<!-- %%attr(x,"groupid") -->
<!-- %%```  -->
<!-- %% -->


<!-- %% \subsection{The \code{sampleBy} function} -->
<!-- %% \label{sampleBy} -->
<!-- %%  -->
<!-- %% Suppose we want a random sample of 50 \% of the observations from a -->
<!-- %% dataframe. This can be achieved with: -->
<!-- %% ``` -->
<!-- %% <<results=hide>>= -->
<!-- %% sampleBy(~1, frac=0.5, data=airquality) -->
<!-- %% ```  -->
<!-- %%  -->
<!-- %% Suppose instead that we want a  systematic sample of  every fifth -->
<!-- %% observation within each month. This is achieved with: -->
<!-- %% ``` -->
<!-- %% <<results=hide>>= -->
<!-- %% sampleBy(~Month, frac=0.2, data=airquality,systematic=T) -->
<!-- %% ```  -->
<!-- %%  -->
<!-- %%  -->


<!-- %% Inspired by the \pkg{dplyr} package, there is an \verb|order_by| function -->
<!-- %% << >>=  -->
<!-- %% x5 <- airquality |> order_by(c("Temp", "Month")) -->
<!-- %% x6 <- airquality |> order_by(c("-Temp", "Month")) -->
<!-- %% ``` -->
<!-- %% which is the same as -->
<!-- %% << >>=  -->
<!-- %% x5 <- order_by(airquality, c("Temp", "Month")) -->
<!-- %% x6 <- order_by(airquality, c("-Temp", "Month")) -->
<!-- %% ``` -->

<!-- %% <<echo=F, results=hide>>=  -->
<!-- %% c(all.equal(x1, x3), all.equal(x1, x5),  -->
<!-- %%   all.equal(x2, x4), all.equal(x2, x6)) -->
<!-- %% ``` -->



<!-- %% Suppose we want to calculate the weekwise feed efficiency of the pigs -->
<!-- %% in the \code{dietox} data, i.e. weight gain divided by feed intake. -->
<!-- %% ``` -->
<!-- %% ```{r} -->
<!-- %% data(dietox) -->
<!-- %% dietox <- orderBy(~Pig+Time, data=dietox) -->
<!-- %% FEfun  <- function(d){c(NA, diff(d$Weight)/diff(d$Feed))} -->
<!-- %% v      <- lapplyBy(~Pig, data=dietox, FEfun) -->
<!-- %% dietox$FE <- unlist(v) -->
<!-- %% ```  -->
<!-- %%  -->
<!-- %% Technically, the above is the same as -->
<!-- %% ``` -->
<!-- %% ```{r} -->
<!-- %% dietox <- orderBy(~Pig+Time, data=dietox) -->
<!-- %% wdata  <- splitBy(~Pig, data=dietox) -->
<!-- %% v      <- lapply(wdata, FEfun) -->
<!-- %% dietox$FE <- unlist(v) -->
<!-- %% ```  -->
<!-- %%  -->
<!-- %% -->
<!-- %% \subsection{The \code{scaleBy} function} -->
<!-- %%  -->
<!-- %% Standardize the \code{iris} data within each value of \code{"Species"}: -->
<!-- %% ``` -->
<!-- %% ```{r} -->
<!-- %% x <- scaleBy(cbind(Sepal.Length, Sepal.Width, Petal.Length, Petal.Width) ~ Species,      -->
<!-- %%              data=iris) -->
<!-- %% lapply(x, head, 4) -->
<!-- %% head(iris) -->
<!-- %% ``` def -->
<!-- %%  -->
<!-- %% Alternative forms: -->
<!-- %% << >>=  -->
<!-- %% x <- scaleBy( list(c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"), -->
<!-- %%                  "Species"),     data=iris) -->
<!-- %% x <- scaleBy(. ~ Species, data=iris) -->
<!-- %% x <- scaleBy(list(".", "Species"),  data=iris) -->
<!-- %% x <- iris  >% scale_by(. ~ Species) -->
<!-- %% ``` -->
<!-- %%  -->









<!-- %%\section{Create By--functions on the fly} -->
<!-- %%\label{sec:create-functions-fly} -->
<!-- %% -->
<!-- %%Create a function for creating groupwise t-tests -->
<!-- %% -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%mydata <- data.frame(y=rnorm(32), x=rnorm(32), -->
<!-- %%g1=factor(rep(c(1,2),each=16)), g2=factor(rep(c(1,2), each=8)), -->
<!-- %%g3=factor(rep(c(1,2),each=4))) -->
<!-- %%head(mydata) -->
<!-- %%```  -->
<!-- %% -->
<!-- %% -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%## Based on the formula interface to t.test -->
<!-- %%t.testBy1 <- function(formula, group, data, ...){ -->
<!-- %%  formulaFunBy(formula, group, data, FUN=t.test, class="t.testBy1", ...) -->
<!-- %%} -->
<!-- %%## Based on the default interface to t.test -->
<!-- %%t.testBy2 <- function(formula, group, data, ...){ -->
<!-- %%  xyFunBy(formula, group, data, FUN=t.test, class="t.testBy1", ...) -->
<!-- %%} -->
<!-- %%```  -->
<!-- %% -->
<!-- %%Notice: The optional \code{class} argument will facilitate that you -->
<!-- %%create your own print / summary methods etc. -->
<!-- %% -->
<!-- %%``` -->
<!-- %%```{r} -->
<!-- %%t.testBy1(y~g1, ~g2, data=mydata) -->
<!-- %%t.testBy2(y~x,  ~g2, data=mydata) -->
<!-- %%```  -->
<!-- %% -->

---
title: "Sectioning a function"
author: "Søren Højsgaard"
output: 
  html_document:
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{section_fun: Section functions to a smaller domain}
  %\VignettePackage{doBy}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options("digits"=3)
library(doBy)
library(boot)
#devtools::load_all()
```

## Section a functions domain: with `section_fun()`

The function `section_fun` is used to create a new function that is a section of the original function. The section is defined by fixing some of the arguments of the original function. The new function is defined on the remaining arguments.

For example, let $f(x,y)=x+y$. Then $f_x(y)=f(10, y)$ is a
section of $f$ to be a function of $y$ alone.

More generally, let $E$ be a subset of the cartesian product $X \times Y$ where $X$
and $Y$ are some sets. Consider a function $f(x,y)$ defined on
$E$. Then for any $x \in X$, the section of $E$ defined by $x$
(denoted $E_x$) is the set of $y$'s in $Y$ such that $(x, y)$ is in
$E$, i.e.
$$
 E_x = \{ y \in Y | (x,y) \in E\}
$$

Correspondingly, the section of $f(x,y)$ defined by $x$ is the
function $f_x$ defined on $E_x$ given by $f_x(y)=f(x,y)$.

## The `section_fun` function in `doBy`

The function `section_fun` is used to create a new function that is a section of the original function. The section is defined by fixing some of the arguments of the original function. The new function is defined on the remaining arguments.
There are the following approaches:

1) insert the section values as default values in the function definition (default),
2) insert the section values in the function body,
3) store the section values in an auxillary environment.

Consider this function:


```{r}
fun  <- function(a, b, c=4, d=9){
    a + b + c + d
}
```

```{r}
fun_def <- section_fun(fun, list(b=7, d=10))
fun_def
fun_body <- section_fun(fun, list(b=7, d=10), method="sub")
fun_body
fun_env <- section_fun(fun, list(b=7, d=10), method = "env")
fun_env
```


In the last case, we can see the section and the original function definition as:

```{r}
get_section(fun_env) 
## same as: attr(fun_env, "arg_env")$args 
get_fun(fun_env) 
## same as: environment(fun_env)$fun
```


We get:

```{r}
fun(a=10, b=7, c=5, d=10)
fun_def(a=10, c=5)
fun_body(a=10, c=5)
fun_env(a=10, c=5)
```


## Example: benchmarking

Consider a simple task: Creating and inverting Toeplitz matrices for increasing dimensions: 

```{r}
n <- 4
toeplitz(1:n)
```

An implementation is

```{r}
inv_toep <- function(n) {
    solve(toeplitz(1:n))
}
inv_toep(4)
```


We can benchmark timing for different values of $n$ as


```{r}
library(microbenchmark)
microbenchmark(
    inv_toep(4), inv_toep(8), inv_toep(16),
    inv_toep(32), inv_toep(64),
    times=5
)
```


However, it is tedious (and hence error prone) to write these function calls. 

A programmatic approach using \code{section\_fun} is as follows: First create a list of sectioned functions:
```{r}
n.vec  <- c(3, 4, 5)
fun_list <- lapply(n.vec,
                  function(ni){
                      section_fun(inv_toep, list(n=ni))}
                  )
```

We can inspect and evaluate each / all functions as:
```{r}
fun_list[[1]]
fun_list[[1]]()
```


To use the list of functions in connection with microbenchmark we bquote all functions using


```{r}
bquote_list <- function(fnlist){
    lapply(fnlist, function(g) {
        bquote(.(g)())
    }
    )
}
```

We get:
```{r}
bq_fun_list <- bquote_list(fun_list)
bq_fun_list[[1]]
## Evaluate one:
eval(bq_fun_list[[1]])
## Evaluate all:
## lapply(bq_fun_list, eval)
```

To use microbenchmark we must name the elements of the list:

```{r}
names(bq_fun_list) <- n.vec
microbenchmark(
  list  = bq_fun_list,
  times = 5
)
```

Running the code below provides a benchmark of the different ways of sectioning in terms of speed.

```{r}
n.vec  <- seq(20, 80, by=20)
fun_def <- lapply(n.vec,
                  function(ni){
                      section_fun(inv_toep, list(n=ni), method="def")}
                  )
fun_body <- lapply(n.vec,
                  function(ni){
                      section_fun(inv_toep, list(n=ni), method="sub")}
                  )
fun_env <- lapply(n.vec,
                  function(ni){
                      section_fun(inv_toep, list(n=ni), method="env")}
                  )

bq_fun_list <- bquote_list(c(fun_def, fun_body, fun_env))
names(bq_fun_list) <- paste0(rep(c("def", "body", "env"), each=length(n.vec)), rep(n.vec, 3))

mb <- microbenchmark(
  list  = bq_fun_list,
  times = 2
)
```

% \VignetteEngine{knitr::knitr} 
%\VignettePackage{doBy}
%\VignetteIndexEntry{doBy: Linear estimates and LSmeans}
%\VignetteIndexEntry{LSmeans}
%\VignetteIndexEntry{population means}
%\VignetteIndexEntry{contrasts}
%\VignetteIndexEntry{estimable functions}

\documentclass[11pt]{article}

\usepackage{hyperref,url,color,Sweave,a4wide}
\usepackage[utf8]{inputenc}
\usepackage[authoryear,round]{natbib}
\bibliographystyle{plainnat}

\RequirePackage{color,fancyvrb,amsmath,amsfonts}

\def\textasciigrave{'}
\def\tt#1{\texttt{#1}}
\def\R{\texttt{R}}
\def\pkg#1{{\bf #1}}
\def\doby{\pkg{doBy}}

\def\code#1{\texttt{#1}}
\def\cc#1{\texttt{#1}}
\def\comi#1{{\it #1}}
\def\esticon{\code{esticon()}}
\def\lsmeans{\code{LSmeans}}
\def\linmat{\code{LE_matrix()}}
\def\linest{\code{linest()}}

% reduce whitespace between R code and R output
% \let\oldknitrout\knitrout
% \renewenvironment{knitrout}{
%   \begin{oldknitrout}
%     \footnotesize
%     \topsep=0pt
% }{
%   \end{oldknitrout}
% }


\DeclareMathOperator{\EE}{\mathbb{E}}
\DeclareMathOperator{\var}{\mathbb{V}ar}
\DeclareMathOperator{\cov}{\mathbb{C}ov}
\DeclareMathOperator{\norm}{N}
\DeclareMathOperator{\spanx}{span}
\DeclareMathOperator{\corr}{Corr}
\DeclareMathOperator{\deter}{det}
\DeclareMathOperator{\trace}{tr}
\def\inv{^{-1}}
\newcommand{\transp}{^{\top}}


<<echo=FALSE>>=
require( doBy )
prettyVersion <- packageDescription("doBy")$Version
prettyDate <- format(Sys.Date())
@

\title{Linear estimates and LS--means in the \texttt{doBy} package}
\author{S{\o}ren H{\o}jsgaard and Ulrich Halekoh}
\date{\pkg{doBy} version \Sexpr{prettyVersion} as of \Sexpr{prettyDate}}

% show R> prompt before R commands
<<r setup, echo=FALSE>>=
knitr::opts_chunk$set(prompt=TRUE) 
library(knitr)
if (!dir.exists("figures")) dir.create("figures")
opts_chunk$set(
               tidy=FALSE,fig.path='figures/LSmeans', fig.height=3.5 
           )
oopt <- options()
options("digits"=4, "width"=90, "prompt"="> ", "continue"="  ")
options(useFancyQuotes="UTF-8")
library(ggplot2)
@



\definecolor{darkred}{rgb}{.7,0,0}
\definecolor{midnightblue}{rgb}{0.098,0.098,0.439}




\begin{document}
% \SweaveOpts{concordance=TRUE}

\maketitle
\tableofcontents
\parindent0pt
\parskip5pt

\section{Introduction}
\label{sec:introduction}

\subsection{Linear functions of parameters}
\label{sec:line-funct-param}

A linear function of a $p$--dimensional parameter vector $\beta$ has
the form
\begin{displaymath}
  C=L\beta
\end{displaymath}
where $L$ is a $q\times p$ matrix which we call the \comi{Linear
  Estimate Matrix} of simply \comi{LE-matrix}.  The corresponding
linear estimate is $\hat C = L \hat \beta$.  A \comi{linear hypothesis} has
the form $H_0: L\beta=m$ for some $q$ dimensional vector $m$.


\subsection{LSmeans (population means, marginal means)}
\label{sec::lsmeans}

A special type of linear estimates is the so called \comi{least-squares
means} (or \comi{LS-means}). Other names for these estimates include
\comi{population means} and \comi{marginal means}. Consider an imaginary field
experiment analyzed with model of the form
<<eval=F>>=
lm( y ~ treat + block + year)
@ %def
where \cc{treat} is a treatment factor, \cc{block} is a blocking
factor and \cc{year} is the year (a factor) where the experiment is
repeated over several years. This model specifies the conditional mean
$\EE(Y|\cc{treat}, \cc{block},\cc{year})$. One may be interested in
predictions of the
form $\EE(Y|\cc{treat})$. This quantity can not formally be found from the
model. However, it is tempting to average the fitted values of
$\EE(Y|\cc{treat}, \cc{block},\cc{year})$ across the levels of
\cc{block} and \cc{year} and think of this average as
$\EE(Y|\cc{treat})$. This average is precisely what is called the
LS--means. If the experiment is balanced then this average is
identical to the average of the observations when stratified according
to \cc{treat}.



An alternative is to think of \cc{block} and \cc{year} as random
effects, for example:
<<eval=F>>=
library(lme4)
lmer( y ~ treat + (1|block) + (1|year))
@ %def

In this case one would directly obtain $\EE(Y|\cc{treat})$ from the
model. However, there are at least two reasons why one may be hesitant
to consider such a random effects model.
\begin{itemize}
\item Suppose there are three
blocks and the experiment is repeated over three consecutive
years. This means that the random effects are likely to be estimated
with a large uncertainty (the estimates will have only two degrees of
freedom).
\item Furthermore, treating \cc{block} and \cc{year} as random
effects means they should in principle come from a large population of
possible blocks and years. This may or may not be reasonable for the
blocks, but it is  a questionable assumption for the years.
\end{itemize}



\subsection{A simulated dataset}
\label{sec:simulated-dataset}

In the following sections we consider these data:

<<>>=
library(doBy)
set.seed(141164)
dd <- expand.grid(A=factor(1:3), B=factor(1:3), C=factor(1:2))
dd$y <- rnorm(nrow(dd))
dd$x <- rnorm(nrow(dd))^2
dd$z <- rnorm(nrow(dd))
head(dd,10)
@ %def

Consider the additive model
\begin{equation}
  \label{eq:1}
  y_i = \beta_0 + \beta^1_{A(i)}+\beta^2_{B(i)} + \beta^3_{C(i)} + \beta^4 x_i + e_i
\end{equation}
where $e_i \sim N(0,\sigma^2)$. We fit this model:
<<>>=
mm <- lm(y ~ A + B + C + x, data=dd)
coef(mm)
@ %def

Notice that the parameters corresponding to the factor levels
\code{A1}, \code{B1} and \code{C2} are set to zero to ensure
identifiability of the remaining parameters.


\subsection{LSmeans: details}
\label{sec:what-are-these}

LSmeans, population means and marginal means are used synonymously in
the literature. 
These quantities are a special kind of contrasts as defined in
Section~\ref{sec:line-funct-param}.
LSmeans seems to be the most widely used term, so we
shall adopt this terms here too.

The model (\ref{eq:1}) is a model for the conditional mean
$\EE(y|A,B,C,x)$.  Sometimes one is interested in quantities like
$\EE(y|A)$. This quantity can not formally be found because of the presences of
unless $B$, $C$ and $x$. 
However, suppose that $A$ is a treatment of main interest, $B$ is a
blocking factor, $C$ represents days on which the experiment was
carried out and $x$ denotes e.g. temperature for the specific block on the specific date. Then it is tempting to fix $x$ at its average value $\bar x$ and average $\EE(y|A,B,C, \bar x)$ over $B$
and $C$ (average over block and day) and think of this average as
$\EE(y|A)$.

The population mean for $A=1$ is
\begin{equation}
  \label{eq:2}
  \beta^0 + \beta^1_{A1} + \frac{1}{3} (\beta^2_{B1}+\beta^2_{B2}+\beta^2_{B3})
  + \frac{1}{2}(\beta^3_{C1}+\beta^3_{C2})
\end{equation}

Recall that the parameters corresponding to the factor levels
\code{A1}, \code{B1} and \code{C2} are set to zero to ensure
identifiability of the remaining parameters. Therefore we may also
write the population mean for $A=1$ as
\begin{equation}
  \label{eq:3}
  \beta^0 + \frac{1}{3} (\beta^2_{B2}+\beta^2_{B3})
  + \frac{1}{2}(\beta^3_{C2}) + \beta^4 \bar x.
\end{equation}

This quantity can be estimated as (if $\bar x = 1.242$) :

<<>>=
w <- c(1, 0, 0, 1/3, 1/3, 1/2, 1.242)
sum(coef(mm) * w)
@ %def

We may find the population mean for all three levels of $A$ as

<<>>=
L <- matrix(c(1, 0, 0, 1/3, 1/3, 1/2, 1.242,
              1, 1, 0, 1/3, 1/3, 1/2, 1.242,
              1, 0, 1, 1/3, 1/3, 1/2, 1.242), nr=3, byrow=TRUE)
L %*% coef(mm)
@ %def

Notice that the matrix W is based on that the first level of $A$ is
set as the reference level. If the reference level is changed then so
must $W$ be.

% Given that one has specified $W$, we can use the \esticon\ function in the
% \code{doBy} as illustrated below:

% <<>>=
% esticon(mm, W)
% @ %def



\def\popmatrix{LEmatrix}
\def\popmeans{LSmeans}

\subsection{Using \popmatrix\  and \popmeans} 

Writing the matrix $W$ is somewhat tedious and hence error prone. In
 addition, there is a potential risk of getting the wrong answer if the the reference level of a factor has been changed.  The \popmatrix\ function provides an automated way of generating such matrices.  The above \verb+W+ matrix is constructed by

<<>>=
L <- LE_matrix(mm, effect='A')
L
@ %def

The \verb+effect+ argument requires  to calculate the population means
for each level of
$A$ aggregating across the levels of the other variables in the data.

The \popmeans\ function is simply a wrapper around first a call
to \popmatrix\ followed by a call to (by default) \linest:

<<>>=
linest(mm, L=L)
LSM <- LSmeans(mm, effect='A')
LSM
@ %def

\subsection{Special details}
\label{sec:special-details}

\subsubsection{Summary}
\label{sec:summary}

More details about how the matrix was constructed is provided by the
\code{summary()} method; for example:

<<>>=
summary(LSM)
@ %def

\subsubsection{Specification of \code{effect}}
\label{sec:spec-code}

The \code{effect} argument can be specified in two ways:

<<>>=
## 1) a vector of characters and 
LE_matrix(mm, effect= c("A", "C"))
## 2) a right hand sided formula:
## LE_matrix(mm, effect= ~ A + C)
@

\subsubsection{Fixing values in the \code{at} list}
\label{sec:fixing-valu-code}

We can fix values of the explanatory variables using the \code{at=} argument: For example:

<<>>=
LE_matrix(mm, at=list(A=1, x=2))
@ 

\subsubsection{Combining \code{effect} and \code{at}}
\label{sec:comb-code-code}

<<>>=
LE_matrix(mm, effect= ~ A + C, at=list(A=1)) 
@

\subsubsection{Total average}
\label{sec:total-average}

Omitting \code{effect} as in

<<>>=
LE_matrix(mm)
LSmeans(mm)
@ %def
gives the ``total average''.



\subsubsection{Log transformed covariates}

Unless otherwise specified, numerical covariates are fixed at their
average value, for example for \code{x} below:

If we use \code{log(x)} instead we will get an error when
calculating LS--means. Instead one must modify data:

<<>>=
mm2 <- lm(y ~ A + B + C + log(x), data=dd)
## LSmeans(mm2, effect=~A) ## Will fail
mm2 <- lm(y ~ A + B + C + log.x,
          data=transform(dd, log.x=log(x)))
LSmeans(mm2, effect=~A)
LSmeans(mm2, effect=~A)$L
@ 

This also highlights what is computed: The average of the log of
\code{x}; not the log of the average of \code{x}.

\subsubsection{Powers of covariates}

Similar phenomenon for powers of covariates:

<<>>=
mm2 <- lm(y ~ A + B + C + x + I(x^2), data=dd)
LSmeans(mm2, effect=~A)
LSmeans(mm2, effect=~A)$L
@ %def


Above \verb|I(x^2)| is the average of the squared values of
\code{x}; not the  square of the average of
\code{x}, cfr.\ the following.

<<>>=
mm2 <- lm(y ~ A + B + C + x + x2,
          data=transform(dd, x2=x^2))
LSmeans(mm2, effect=~A)
LSmeans(mm2, effect=~A)$L
@ %def


% If we want to evaluate the LS--means at \code{conc=10} then we can do:
% <<>>=
% LSmeans(co2.lm4, effect="Treatment", at=list(conc=10, conc2=100))
% @ %def



\section{Miscellaneous}
\label{sec:miscellaneous}


\subsection{Excerpt of the CO2 data}
\label{sec:excerpt-co2-data}

For illustration we use subsets of the CO2 data:

<<>>=
data(CO2)
CO2 <- subset(CO2, conc < 300) ## OK
CO2.bal <- CO2
rownames(CO2.bal) <- NULL
@ 

% <<>>=
% xtabs(~Type + Treatment + conc, data=CO2.bal) |>
%     ftable(row.vars = "Type")
% @ 

Data is perfectly balanced.
We also use an unbalanced subset of the data
<<>>=
CO2.ubal <- CO2.bal[-c(1, 5, 12, 17, 18, 19, 20, 28),]
CO2.ubal |> head()
xtabs(~Type + Treatment + conc, data=CO2.ubal) |>
    ftable(row.vars = "Type")
@

% <<>>=
% p1 <- ggplot(CO2.bal, aes(x=conc, y=uptake, color=Treatment, group=Plant)) +
%     geom_line() + geom_point() + facet_grid(~Type)
% p2 <- ggplot(CO2.ubal, aes(x=conc, y=uptake, color=Treatment, group=Plant)) +
%     geom_line() + geom_point() + facet_grid(~Type)
% library(patchwork) ## Multiple graphs
% p1 / p2
% @ 

\subsection{Two linear models for CO2 data}
\label{sec:two-linear-models}

<<>>=
library(broom)
form.add <- uptake ~ conc + Treatment + Type
form.int <- uptake ~ conc * Treatment + Type
fm1.bal <- lm(form.add, data=CO2.bal)
fm1.ubal <- lm(form.add, data=CO2.ubal)
@ 

\subsection{Linear estimates and LSmeans}
\label{sec:line-estim-using}

\paragraph{Common task 1:}
Consider this task: Estimate the value of the response \tt uptake
for each combination of \tt Type and \tt Treatment . This
can be obtained, for example, as follows:

<<>>=
LSmeans(fm1.bal, effect=~Type+Treatment, at=list(conc=100))
LSmeans(fm1.ubal, effect=~Type+Treatment, at=list(conc=100))
@ 

\paragraph{Common task 2:}
Estimate the value of the response \tt uptake when \tt
 for each level of \tt Treatment.  Formally this question can
not be answered under the  model because the model gives the
conditional mean of \tt uptake given \tt conc, \tt Type and \tt
Treatment. There is, so to speak, no way of getting rid of \tt Type.
There are different workarounds for this situation: We can average over \tt Type and fix \t conc at its average. Alternatively, we can fit model without \tt Type effect and
  repeat the steps above.


<<>>=
LSmeans(fm1.bal, effect=~Treatment)
LSmeans(fm1.ubal, effect=~Treatment)
@

<<>>=
## Fit model without Type
fm2.bal <- update(fm1.bal, .~. - Type)
fm2.ubal <- update(fm1.ubal, .~. - Type)

LSmeans(fm2.bal, effect=~Treatment)
LSmeans(fm2.ubal, effect=~Treatment)
@ 

Notice that the predictions (estimates) are identical for the balanced dataset but not for the unbalanced dataset. Notice also that the standard errors of the predictions are somewhat larger when fitting a model without \tt Type effect. This is to be expected as the variation due to \tt Type goes into the residual.




\subsection{Generalized linear models}
\label{sec:gener-line-models}

We can calculate LS--means for e.g.\ a Poisson or a gamma model. Default is that
the calculation is calculated on the scale of the linear
predictor.

<<>>=
fm.glm <- glm(form.add, family=Gamma, data=CO2.ubal)
LSmeans(fm.glm, effect=~Treatment, type="link")
@ %def

\subsection{Generalized estimating equations}
\label{sec:gener-estim-equat}

For gee-type models we get
<<>>=
library(geepack)
fm.gee <- geeglm(uptake ~ conc + Treatment + Type,
                 id=Plant, family=Gamma, data=CO2.ubal)
LSmeans(fm.gee, effect=~Treatment)
@ %def



\subsection{Linear mixed effects model} 
\label{sec:linear-mixed-effects}


For illustration we treat \verb|Plant| as a random effect:

<< >>= 
library(lme4)
fm.mix <- lmer(uptake ~ conc + Treatment + Type + (1|Plant), data=CO2.ubal)
LSmeans(fm.mix, effect=~Treatment)
@

Notice that the degrees of freedom by default are adjusted using a
Kenward--Roger approximation. Unadjusted degrees of freedom are obtained by setting \verb|adjust.df=FALSE|. 





\subsection{Pairwise comparisons}
\label{sec:pairwise-comparisons}

We will just mention that for certain other linear estimates, the
matrix $L$ can be generated automatically using \cc{glht()} from the
\pkg{multcomp} package. For example, pairwise comparisons of all
levels of \code{dose} can be obtained with

<< >>= 
library("multcomp")
g1 <- glht(fm2.ubal, mcp(Treatment="Tukey"))
tidy(g1)
@

The L matrix is 
<< >>= 
L <- g1$linfct
L
linest(fm2.ubal, L=L)
@




% \subsection{Manual computations}
% \label{sec:manual-computations}

% Averaging over predicted values amounts to averaging over weighted
% sums of the regression coefficients. The weights can be generated
% manually as:

% <<>>=
% coef(fm1.ubal)

% L01 <- matrix(c(1, 100, 0, 0,
%                 1, 100, 1, 0,
%                 1, 100, 0, 1,
%                 1, 100, 1, 1), ncol=4, byrow=T)
% L01 %*% coef(fm1.ubal)

% L02 <- matrix(c(1, 100, 0, 0.5,
%                 1, 100, 1, 0.5), ncol=4, byrow=T)
% L02 %*% coef(fm1.ubal)

% L03 <- matrix(c(1, 179.8, 0, 0.5,
%                 1, 179.8, 1, 0.5), ncol=4, byrow=T)
% L03 %*% coef(fm1.ubal)
% @ 

% \subsection{Automatic generation of LEmatrix}
% \label{sec:autom-gener-lematr}

% Averaging over predicted values amounts to averaging over weighted
% sums of the regression coefficients. The weights can be generated
% automatically as follows:

% \paragraph{Common task 1:}

% We calulate genereate the $L$ matrix automaically and the matrix and the model object into a function that returns the predictions, standard errors etc.

% <<>>=
% L1 <- LE_matrix(fm1.ubal, effect=~Treatment + Type, at=list(conc=100))
% L1
% C1 <- linest(fm1.ubal, L=L1)
% C1
% @ 

% The steps above can be made with one function call:

% <<>>=
% LSM1 <- LSmeans(fm1.ubal, effect=~Treatment + Type, at=list(conc=100))
% LSM1
% @


% \paragraph{Common task 2: }
% Likewise, for the second situation:

% <<>>=
% L2 <- LE_matrix(fm1.ubal, effect=~Treatment, at=list(conc=100)); L2
% C2 <- linest(fm1.ubal, L=L2); C2
% LSM2 <- LSmeans(fm1.ubal, effect=~Treatment, at=list(conc=100))
% LSM2
% @ 


% \paragraph{Common task 3: }
% Finally for the third situation:

% <<>>=
% L3 <- LE_matrix(fm1.ubal, effect=~Treatment);
% L3
% C3 <- linest(fm1.ubal, L=L3);
% C3
% LSM3 <- LSmeans(fm1.ubal, effect=~Treatment)
% LSM3
% @ 



% \subsection{ssssssssssssssss}
% \label{sec:ssssssssssssssss}


% Notice this information:
% <<>>=
% summary(L3)
% summary(C3)
% summary(LSM3)
% @ 


% \subsection{Using the \code{at=} argument}
% \label{sec:at-argument}


% <<chick-fig, fig.cap="ChickWeight data.">>=
% library(ggplot2)
% ChickWeight$Diet <- factor(ChickWeight$Diet)
% qplot(Time, weight, data=ChickWeight, colour=Chick, facets=~Diet,
%       geom=c("point","line")) + theme(legend.position="none") 
% @ %def

% Consider random regression model:
% <<>>=
% library(lme4)
% chick <- lmer(weight ~ Time * Diet + (0 + Time | Chick), 
%            data=ChickWeight)
% coef(summary(chick))
% @ %def


% The LE--matrix for \cc{Diet} becomes:
% <<>>=
% L <- LE_matrix(chick, effect="Diet")
% t(L)
% @ %def

% The value of \cc{Time} is by default taken to be the average of that
% variable. Hence the \lsmeans\ is the predicted weight for each diet at
% that specific point of time. We can consider other points of time with
% <<>>=
% K1 <- LE_matrix(chick, effect="Diet", at=list(Time=1))
% t(K1)
% @ %def

% The \lsmeans\ for the intercepts is the predictions at
% \cc{Time=0}. The \lsmeans\ for the slopes becomes
% <<>>=
% K0 <- LE_matrix(chick, effect="Diet", at=list(Time=0))
% t(K1 - K0)
% linest(chick, L=K1 - K0)
% @ %def


% We may be interested in finding the population means
% at all levels of  $A$
% but only at $C=1$. This is obtained by using the \code{at} argument
% (in which case the average is only over the remaining factor $B$):

% <<>>=
% popMatrix(mm, effect='A', at=list(C='1'))
% @ %def

% Another way of
% creating the population means
% at  all levels of $(A,C)$ is therefore

% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2')))
% @ %def

% We may have several variables in the \code{at} argument:
% @
% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2'), B='1'))
% @ %def



% We can create  our own function for comparing trends:

% <<>>=
% LSmeans_trend <- function(object, effect, trend){
%     L <- LE_matrix(object, effect=effect, at=as.list(setNames(1, trend))) -
%         LE_matrix(object, effect=effect, at=as.list(setNames(0, trend)))
%     linest(object, L=L)
% }
% ## LSmeans_trend(chick, effect="Diet", trend="Time")
% @ %def


% <<>>=
% LSmeans(fm.mix, effect=~Treatment, adjust.df=FALSE)
% LSmeans(fm.mix, effect=~Treatment, adjust.df=TRUE)
% @ %def





% \subsection{Example: Non--estimable linear functions}
% \label{sec:exampl-non-estim}

% <<>>=
% ## Make balanced dataset
% dat.bal <- expand.grid(list(AA=factor(1:2), BB=factor(1:3), CC=factor(1:3)))
% dat.bal$y <- rnorm(nrow(dat.bal))

% ## Make unbalanced dataset:  'BB' is nested within 'CC' so BB=1
% ## is only found when CC=1 and BB=2,3 are found in each CC=2,3,4
% dat.nst <- dat.bal
% dat.nst$CC <-factor(c(1,1,2,2,2,2,1,1,3,3,3,3,1,1,4,4,4,4))
% @ %def

% << >>= 
% dat.nst
% @

% Consider this simulated dataset:
% <<>>=
% head(dat.nst, 4)
% ftable(xtabs( ~ AA + BB + CC, data=dat.nst), row.vars="AA")
% @ %def

% Data is highly "unbalanced":
% Whenever \verb|BB=1| then \verb|CC| is always \verb|1|; whenever
% \verb|BB| is not \verb|1| then \verb|CC| is never \verb|1|.
% We have
% <<>>=
% mod.nst  <- lm(y ~ AA + BB : CC, data=dat.nst)
% coef(mod.nst)
% coef(summary(mod.nst))
% @ %def


% In this case some of the \lsmeans\ values are not estimable; for example:
% <<>>=
% lsm.BC <- LSmeans(mod.nst, effect=c("BB", "CC"))
% lsm.BC
% lsm.BC2 <- LSmeans(mod.nst, effect="BB", at=list(CC=2))
% lsm.BC2
% @ %def

% We describe the situation in 
% Section~\ref{sec:handl-non-estim} where we focus on \verb|lsm.BC2|.





% \subsection{Handling non--estimability}
% \label{sec:handl-non-estim}

% The model matrix for the model in Section~\ref{sec:exampl-non-estim}
% does not have full column rank and therefore not all values are
% calculated by \cc{LSmeans()}.

% <<>>=
% X <- model.matrix( mod.nst ) 
% Matrix::rankMatrix(X)
% dim(X)
% as(X, "Matrix")
% @ %def

% We consider a  model, i.e.\
% an $n$ dimensional random vector $y=(y_i)$ for which
% $\EE(y)=\mu=X\beta$ and $\cov(y)=V$ where $X$ does not have
% full column rank
% We are
% interested in linear functions of $\beta$, say
% \begin{displaymath}
%   c=l\transp\beta= \sum_j l_j \beta_j .
% \end{displaymath}

% <<>>=
% L <- LE_matrix(mod.nst, effect="BB", at=list(CC=2))
% t(L)
% linest(mod.nst, L=L)
% @ %def

% A least squares estimate of $\beta$ is
% \begin{displaymath}
%   \hat \beta = G X\transp y
% \end{displaymath}

% where $G$ is a generalized inverse of $X\transp  X$. Since the
% generalized inverse is not unique then neither is the estimate
% $\hat\beta$. Hence $\hat c = l\transp\hat \beta$ is in general not
% unique.

% One least squares estimate of $\beta$ and one corresponding linear estimate $L\hat\beta$ is: 
% <<>>=
% XtXinv <- MASS::ginv(t(X)%*%X)
% bhat <- as.numeric(XtXinv %*% t(X) %*% dat.nst$y)
% zapsmall(bhat)
% L %*% bhat
% @ %def

% For some values of $l$ (i.e.\ for some rows of $L$)
% the estimate $\hat c=l\transp \beta$ is
% unique (i.e.\ it does not depend on the choice of generalized
% inverse). 
% Such linear functions are said to be estimable and can be
% described as follows:

% All we specify with $\mu=X\beta$
% is that $\mu$ is a vector in the column space $C(X)$ of $X$.
% We can only learn about $\beta$ through $X\beta$ so the only thing we can
% say something about is linear combinations $\rho\transp X\beta$. Hence
% we can only say something about $l\transp\beta$ if there exists
% $\rho$ such that 
% $$
% l\transp\beta=\rho\transp X \beta,
% $$ 
% i.e., if
% $l=X\transp\rho$ for some $\rho$, which is if $l$ is in the column space
% $C(X\transp)$ of $X\transp$. This is the same as saying that $l$ must be 
% perpendicular to
% all vectors in the null space $N(X)$ of $X$. To check
% this, we find a basis $B$ for $N(X)$. This can be done in many ways,
% for example via a singular value decomposition of $X$, i.e.\
% \begin{displaymath}
%   X = U D V\transp
% \end{displaymath}
% A basis for $N(X)$ is given by those columns of $V$ that corresponds
% to zeros on the diagonal of $D$.

% <<>>=
% S <- svd(X)
% B <- S$v[, S$d < 1e-10, drop=FALSE ]; 
% head(B) ## Basis for N(X)
% @ %def

% From 
% << >>= 
% rowSums(L %*% B)
% @
% we conclude that the first row of $L$ is not perpendicular to all vectors in thenull space $N(X)$ whereas the two last rows of $L$ are. Hence these two linear estimates are estimable; their value does not depend on the choice of generalized inverse:
% << >>= 
% lsm.BC2
% @










% \subsection{Alternatives - \texttt{esticon()}}
% \label{sec:alternatives}


% An alternative is to do:

% << >>= 
% c1 <- esticon(fm1.ubal, L2)
% c1
% @

% Notice: \verb|esticon| has been in the \doby\ package for many years;
% \verb|linest| is a newer addition; \verb|esticon| is not actively
% maintained but remains in \doby\ for historical reasons.








% \subsection{Using the \code{at} argument}

% We may be interested in finding the population means
% at all levels of  $A$
% but only at $C=1$. This is obtained by using the \code{at} argument
% (in which case the average is only over the remaining factor $B$):

% <<>>=
% popMatrix(mm, effect='A', at=list(C='1'))
% @ %def

% Another way of
% creating the population means
% at  all levels of $(A,C)$ is therefore

% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2')))
% @ %def

% We may have several variables in the \code{at} argument:
% @
% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2'), B='1'))
% @ %def

% \subsection{Ambiguous specification when using the \texttt{effect} and
%   \texttt{at} arguments}

% There is room for an ambiguous specification if a variable appears in
% both the \code{effect} and the \code{at} argument, such as
% @
% <<>>=
% popMatrix(mm, effect=c('A','C'), at=list(C='1'))
% @ %def

% This ambiguity is due to the fact that the \verb+effect+ argument asks
% for the populations means at all levels of the variables but the
% \verb+at+ chooses only specific levels.

% This ambiguity is resolved as follows: Any variable in the \code{at}
% argument is removed from the \code{effect} argument such as the
% statement above is equivalent to
% @
% <<eval=T>>=
% ## popMatrix(mm, effect='A', at=list(C='1'))
% @ %def

% \subsection{Using covariates}

% Next consider the model where a covariate is included:
% @
% <<>>=
% mm2 <- lm(y~A+B+C+C:x, data=dd)
% coef(mm2)
% @ %def

% In this case we get
% <<>>=
% popMatrix(mm2,effect='A', at=list(C='1'))
% @ %def

% Above, $x$ has been replaced by its average and that is the general
% rule for models including covariates. However we may use the \code{at}
% argument to ask for calculation of the population mean at some
% user-specified value of $x$, say 12:
% <<>>=
% popMatrix(mm2,effect='A', at=list(C='1',x=12))
% @ %def

 
% <<>>=
% mm22 <- lm(y~A+B+C+C:x+I(x^2), data=dd)
% coef(mm22)
% @ %def 

 
% <<>>=
% popMatrix(mm22,effect='A', at=list(C='1'))
% @ %def 


 
% <<>>=
% dd <- transform(dd, x.sq=x^2)
% mm23 <- lm(y~A+B+C+C:x+x.sq, data=dd)
% coef(mm23)
% popMatrix(mm23,effect='A', at=list(C='1'))
% @ %def 























% \subsection{Using transformed covariates}

% Next consider the model where a  transformation of a covariate is included:
% @
% <<>>=
% mm3 <- lm(y~A+B+C+C:I(log(x)), data=dd)
% coef(summary(mm3))
% @ %def

% In this case we can not use \popmatrix\ (and hence
% \popmeans) directly.  Instead we first have to
% generate a new variable, say \verb+log.x+, with
% \verb+log.x+$=\log(x)$, in the data and then proceed as

% <<>>=
% dd <- transform(dd, log.x = log(x))
% mm32 <- lm(y~A+B+C+C:log.x, data=dd)
% popMatrix(mm32, effect='A', at=list(C='1'))
% @ %def

% \subsection{The \code{engine} argument of \popmeans}

% The \popmatrix is a function to generate a linear tranformation matrix of the model
% parameters with emphasis on  constructing such matrices for population
% means.
% \popmeans\ invokes by default the \esticon\ function on this
% linear transformation matrix for calculating parameter estimates and
% confidecne intervals.
% A similar function to \esticon\ is the \verb+glht+ function of the \verb+multcomp+
%  package.

%  The \code{glht()} function
%  can be chosen via the \verb+engine+ argument of \popmeans:

% <<>>=
%  library(multcomp)
% g<-popMeans(mm,effect='A', at=list(C='1'),engine="glht")
% g
% @ %def

% This allows to apply the methods available on the \verb+glht+ object like

% <<>>=
% summary(g,test=univariate())
% confint(g,calpha=univariate_calpha())
% @
% which yield the same results as the \esticon\ function.

% By default the functions will adjust the tests  and confidence intervals for multiplicity
% <<>>=
% summary(g)
% confint(g)
% @






% \subsection{Ambiguous specification when using the \texttt{effect} and
%   \texttt{at} arguments}

% There is room for an ambiguous specification if a variable appears in
% both the \code{effect} and the \code{at} argument, such as

% <<>>=
% LE_matrix(fm1.ubal, effect=c('Treatment','Type'), at=list(Type='Mississippi'))
% LE_matrix(fm1.ubal, effect=~Treatment + Type, at=list(Type='Mississippi'))
% @ %def

% This ambiguity is due to the fact that the \verb+effect+ argument asks
% for the populations means at all levels of the variables but the
% \verb+at+ chooses only specific levels.

% This ambiguity is resolved as follows: Any variable in the \code{at}
% argument is removed from the \code{effect} argument such as the
% statement above is equivalent to\footnote{FIXME}

% <<eval=T>>=
% LE_matrix(fm1.ubal, effect=c('Treatement'), at=list(Type='Mississippi'))
% LE_matrix(fm1.ubal, effect=~Treatement, at=list(Type='Mississippi'))
% @ %def















%%\bibliography{doBy}


\end{document}





% \section{LS--means for linear models}
% \label{sec:linear-model}


% \subsection{LS--means -- a first example}
% \label{sec:ls-means-population}

% <<echo=F>>=
% simdat<-structure(list(treat = structure(c(1L, 1L, 1L, 2L, 1L, 2L, 2L, 2L
% ), .Label = c("t1", "t2"), class = "factor"), year = structure(c(1L,
% 1L, 1L, 1L, 2L, 2L, 2L, 2L), .Label = c("1", "2"), class = "factor"),
%     y = c(0.5, 1, 1.5, 3, 3, 4.5, 5, 5.5)), .Names = c("treat", "year",
% "y"), row.names = c(NA, -8L), class = "data.frame")
% @ %def

% Consider these simulated data
% <<>>=
% simdat
% @ %def
% shown in the figure below.
% <<simdat-fig,include=F,fig.height=3, fig.width=6>>=
% library(ggplot2)
% qplot(treat, y, data=simdat, color=year, size=I(3))
% @ %def
% \includegraphics[height=6cm,width=12cm]{figures/LSmeans-simdat-fig}

% The LS--means under an additive model for the factor \cc{treat} is
% <<>>=
% msim <- lm(y ~ treat + year, data=simdat)
% LSmeans( msim, effect="treat")
% @ %def
% whereas the population means are
% <<>>=
% summaryBy(y~treat, data=simdat, FUN=mean)
% @ %def
% Had data been balanced (same number of observations for each
% combination of \cc{treat} and \cc{year}) the results would have been the
% same. An argument in favor of the LS--means is that these figures
% better represent what one would expect on in an ``average year''.


% \subsection{Example: Warpbreaks}
% \label{sec:example:-warpbreaks}

% <<>>=
% summary( warpbreaks )
% head( warpbreaks, 4 )
% ftable(xtabs( ~ wool + tension, data=warpbreaks))
% @ %def

% <<echo=F>>=
%  opar <- par(mfrow = c(1, 2), oma = c(0, 0, 1.1, 0))
%      plot(breaks ~ tension, data = warpbreaks, col = "lightgray",
%           varwidth = TRUE, subset = wool == "A", main = "Wool A")
%      plot(breaks ~ tension, data = warpbreaks, col = "lightgray",
%           varwidth = TRUE, subset = wool == "B", main = "Wool B")
%      mtext("warpbreaks data", side = 3, outer = TRUE)
%      par(opar)
% @ %def

% <<>>=
% (warp.lm <- lm(breaks ~ wool + tension, data=warpbreaks))
% @ %def

% The fitted values are:
% <<>>=
% uni <- unique(warpbreaks[,2:3])
% prd <- cbind(breaks=predict(warp.lm, newdata=uni), uni); prd
% @ %def

% \subsection{The LS--means}
% \label{sec:lsmeans}

% We may be interested in making predictions of the number of breaks for
% each level of \cc{tension} for \emph{any} type or an \emph{average}
% type of \code{wool}.  The idea behind LS--means is
% to average the predictions above over the two
% wool types. These quantities are the \lsmeans\ for the effect
% \cc{tension}.

% This is done with:
% <<>>=
% LSmeans(warp.lm, effect="tension")
% @ %def

% The term \lsmeans\ comes from that these quantities are the same as
% the least squares main effects of \cc{tension} when data is balanced:
% <<>>=
% doBy::summaryBy(breaks ~ tension, data=warpbreaks, FUN=mean)
% @ %def
% When data is not balanced these quantities are in general not the same.




% \subsection{LS--means for models with interactions}
% \label{sec:models-with-inter}

% Consider a model with interaction:
% <<>>=
% warp.lm2 <- update(warp.lm, .~.+wool:tension)
% coef( summary( warp.lm2 ))
% @ %def

% In this case the contrast matrix becomes:
% <<>>=
% K2 <- LE_matrix(warp.lm2, effect="tension"); K2
% linest(warp.lm2, K=K2)
% @ %def


% For the sake of illustration we treat \cc{wool} as a random effect:

% <<>>=
% library(lme4)
% warp.mm <- lmer(breaks ~ tension + (1|wool), data=warpbreaks)
% LSmeans(warp.mm, effect="tension")
% @ %def

% Notice here that the estimates themselves are very similar to those
% above but the standard errors are much larger. This comes from that
% there that \code{wool} is treated as a random effect.

% <<>>=
% VarCorr(warp.mm)
% @ %def

%\usepackage{framed}
%\usepackage{comment}
%%\definecolor{shadecolor}{gray}{0.91}

%%\definecolor{darkred}{rgb}{.7,0,0}
%%\definecolor{midnightblue}{rgb}{0.098,0.098,0.439}

%% \DefineVerbatimEnvironment{Sinput}{Verbatim}{
%%   fontfamily=tt,
%%   %%fontseries=b,
%%   %% xleftmargin=2em,
%%   formatcom={\color{midnightblue}}
%% }
%% \DefineVerbatimEnvironment{Soutput}{Verbatim}{
%%   fontfamily=tt,
%%   %%fontseries=b,
%%   %% xleftmargin=2em,
%%   formatcom={\color{blue}}
%% }
%% \DefineVerbatimEnvironment{Scode}{Verbatim}{
%%   fontfamily=tt,
%%   %%fontseries=b,
%%   %% xleftmargin=2em,
%%   formatcom={\color{blue}}
%% }
%% 
%%\fvset{listparameters={\setlength{\topsep}{-2pt}}}
%\renewenvironment{Schunk}{\linespread{.90}}{}

%%\renewenvironment{Schunk}{\begin{shaded}\small}{\end{shaded}}









%% A special type of linear estimates is the so called least--squares
%% means (or LS--means). Other names for these estimates include
%% population means and marginal means. Notice that the \pkg{lsmeans} package
%% \cite{Lenth:2013} also provides computations of LS--means, see
%% \url{http://cran.r-project.org/web/packages/lsmeans/}.

%% <<echo=F>>=
%% simdat<-structure(list(treat = structure(c(1L, 1L, 1L, 2L, 1L, 2L, 2L, 2L
%% ), .Label = c("t1", "t2"), class = "factor"), year = structure(c(1L,
%% 1L, 1L, 1L, 2L, 2L, 2L, 2L), .Label = c("1", "2"), class = "factor"),
%%     y = c(0.5, 1, 1.5, 3, 3, 4.5, 5, 5.5)), .Names = c("treat", "year",
%% "y"), row.names = c(NA, -8L), class = "data.frame")
%% @ %def

%% \subsection{LS--means on a simple example}
%% \label{sec:ls-means-simple}

%% Consider these simulated data, also shown in Fig.~\ref{fig:simdat2-fig}:
%% <<>>=
%% simdat
%% @ %def

%% <<simdat2-fig, fig.cap="LSmeans - simulated data.">>=
%% library(ggplot2)
%% qplot(treat, y, data=simdat, color=year)
%% @ %def


%% The LS--means under an additive model for the factor \cc{treat} is the predicted outcome for each level of \cc{treat} for an ``average year'':
%% <<>>=
%% msim <- lm(y ~ treat + year, data=simdat)
%% lsm <- LSmeans(msim, effect="treat")
%% lsm
%% @ %def


%% Notice that the population means are
%% <<>>=
%% summaryBy(y ~ treat, data=simdat, FUN=function(x) c(m=mean(x), s=sd(x)))
%% ## or aggregate(y ~ treat, data=simdat, FUN=function(x) c(m=mean(x), s=sd(x)))
%% @ %def

%% Had data been balanced (same number of observations for each
%% combination of \cc{treat} and \cc{year}) the results would have been the
%% same. An argument in favor of the LS--means is that these figures
%% better represent what one would expect on in an ``average year''.

%% An alternative is to think of  \cc{year} as a random
%% effect, for example:
%% <<eval=F>>=
%% library(lme4)
%% lmer( y ~ treat  + (1|year), data=simdat)
%% @ %def

%% In this case one would directly obtain $\EE(Y|\cc{treat})$ from the
%% model. However, there are at least two reasons why one may be hesitant
%% to consider such a random effects model.
%% \begin{itemize}
%% \item It is a ``never ending'' discussion whether \verb|year| should
%%   be treated as fixed or random. For \verb|year| to be a random
%%   effect, it should in principle come from a large population of
%%   possible years. This is certainly a dubious assumption for the
%%   years.
%% \item If we accept to think of year as a random effect, then the
%%   variance describing this random effect will be poorly determined
%%   (because there are only two years in the study).
%% \end{itemize}
 

% <<>>=
% warp.poi <- glm(breaks ~ wool + tension, family=poisson, data=warpbreaks)
% LSmeans(warp.poi, effect="tension", type="link")
% LSmeans(warp.poi, effect="tension", type="response")
% @ %def





%% SANITY CHECK
%% @
%% <<>>=
%% K <- LE_matrix(warp.poi, effect="tension")
%% doBy::esticon(warp.poi, K)
%% @ %def

% <<>>=
% warp.qpoi <- glm(breaks ~ wool + tension, family=quasipoisson, data=warpbreaks)
% LSmeans(warp.qpoi, effect="tension", type="link")
% LSmeans(warp.qpoi, effect="tension", type="response")
% @ %def


% For comparison with the linear model, we use identity link
% <<echo=F,results='hide'>>=
% warp.poi2 <- glm(breaks ~ wool + tension, family=poisson(link=identity),
%                  data=warpbreaks)
% LSmeans(warp.poi2, effect="tension", type="link")
% @ %def



%% SANITY CHECK
%% @
%% <<>>=
%% K <- LE_matrix(warp.poi2, effect="tension")
%% doBy::esticon(warp.poi2, K)
%% @ %def


% <<>>=
% warp.gam <- glm(breaks ~ wool + tension, family=Gamma(link=identity),
%                  data=warpbreaks)
% LSmeans(warp.gam, effect="tension", type="link")
% @ %def

%% SANITY CHECK
%% @
%% <<>>=
%% K <- LE_matrix(warp.gam, effect="tension")
%% doBy::esticon(warp.gam, K)
%% @ %def


% Notice that the linear estimates are practically the same as for the
% linear model, but the standard errors are smaller and hence the
% confidence intervals are narrower.

% An alternative is to fit a quasi Poisson ``model''

% <<>>=
% warp.poi3 <- glm(breaks ~ wool + tension, family=quasipoisson(link=identity),
%                  data=warpbreaks)
% LSmeans(warp.poi3, effect="tension")
% @ %def


% \subsubsection{For balanced data}
% \label{sec:balanced-case}


% When the dataset is balanced the results will be the same

% <<>>=
% aggregate(pred ~ Treatment, data=bb1, FUN=mean)

% fm2.bal <- update(fm1.bal, .~. - Type)
% grid2 <- expand.grid(
%     Treatment = c("nonchilled", "chilled"),
%     conc      = 100
% )
% bb2 <- cbind(pred=predict(fm2.bal, newdata=grid2), grid2)
% bb2
% @ 

% \subsubsection{For unbalanced data}
% \label{sec:unbalanced-case}

% In general, however, the results will not be identical




% <<>>=
% L <- matrix(c(1, 100, 0, 0,
%               1, 100, 0, 1,
%               1, 100, 1, 0,
%               1, 100, 1, 1), ncol=4, byrow=T)
% @ 




% \subsection{Tooth growth}
% \label{sec:tooth-growth}

% % The response is the length of odontoblasts cells (cells responsible for
% % tooth growth) in 60 guinea pigs.  Each animal received one of
% % three dose levels of vitamin C (0.5, 1, and 2 mg/day) by one of
% % two delivery methods, (orange juice (coded as OJ) or ascorbic acid 
% % (a form of vitamin C and (coded as VC)).

% % <<>>=
% % head(ToothGrowth, 4)
% % ftable(xtabs(~ dose + supp, data=ToothGrowth))
% % @ %def

% % <<echo=F, fig.cap="Plot of length against dose for difference sources of vitamin C.">>=
% % opar <- par(mfrow = c(1, 2), oma = c(0, 0, 1.1, 0))
% % plot(len  ~ dose, data = ToothGrowth, col = "lightgray",
% %      subset = supp == "OJ", main = "supp=OJ")
% % plot(len ~ dose, data = ToothGrowth, col = "lightgray",
% %      subset = supp == "VC", main = "supp=VC")
% % mtext("ToothGrowth data", side = 3, outer = TRUE)
% % par(opar)
% % @ %def

% % <<>>=
% % ToothGrowth |> interaction_plot(len ~ dose + supp)
% % @ 

% % The interaction plot suggests a mild interaction which is supported by a formal comparison:

% << >>= 
% ToothGrowth$dose <- factor(ToothGrowth$dose)
% head(ToothGrowth)
% tooth1 <- lm(len ~ dose + supp, data=ToothGrowth)
% tooth2 <- lm(len ~ dose * supp, data=ToothGrowth)
% anova(tooth1, tooth2)
% @


%% <<linear:interaction, fig.cap="Interaction plot for the ToothGrowth data. The average length for each group is a dot. Boxplot outliers are crosses." >>= 
%% tooth_avglen <- ToothGrowth |>
%%   group_by(dose, supp) |>
%%   summarise(val = mean(len))
%% tooth_avglen
%% ggplot(ToothGrowth, aes(x = factor(dose), y = len, colour = supp)) +
%%     geom_boxplot(outlier.shape = 4) +
%%     geom_point(data = tooth_avglen, aes(y = val)) +
%%     geom_line(data = tooth_avglen, aes(y = val, group = supp))
%%@


% \section{Computing linear estimates}
% \label{sec:comp-line-estim}


% For now, we focus on the additive model:
% << >>= 
% tooth1
% @

% Consider computing the estimated length for each dose of orange juice (OJ):
% One option: Construct the LE--matrix $L$ directly:

% << >>= 
% L <- matrix(c(1, 0, 0, 0, 
%               1, 1, 0, 0,
%               1, 0, 1, 0), nrow=3, byrow=T)
% @

% Then do:

% << >>= 
% c1 <- linest(tooth1, L)
% c1
% @

% We can do:

% << >>= 
% summary(c1)
% coef(c1)
% confint(c1)
% @

% \subsection{Automatic generation of $L$}
% \label{sec:autom-gener-l}


% The matrix $L$ can be generated as follows:
% << >>= 
% L <- LE_matrix(tooth1, effect="dose", at=list(supp="OJ"))
% L
% @


% \subsection{OLD: Least-squares means (LS--means)}
% \label{sec:least-squares-means}


% A related question could be: What is the estimated length for each
% dose if we ignore the source of vitamin C (i.e.\ whether it is OJ or
% VC). One approach would be to fit a model in which source does not appear:

% % << >>= 
% % tooth0 <- update(tooth1, . ~ . - supp)
% % L0 <- LE_matrix(tooth0, effect="dose")
% % L0
% % linest(tooth0, L=L0)
% % @

% An alternative would be to stick to the original model but compute the
% estimate for an ``average vitamin C source''. That would correspond to
% giving weight $1/2$ to each of the two vitamin C source
% parameters. However, as one of the parameters is already set to zero
% to obtain identifiability, we obtain the LE--matrix $L$ as

% % << >>= 
% % L1 <- matrix(c(1, 0, 0, 0.5, 
% %                1, 1, 0, 0.5,
% %                1, 0, 1, 0.5), nrow=3, byrow=T)
% % linest(tooth1, L=L1)
% % @

% Such a particular linear estimate is sometimes called a least-squares
% mean or an LSmean or a marginal mean. Notice that the parameter
% estimates under the two approaches are identical. This is is because
% data is balanced: There are $10$ observations per supplementation
% type. Had data not been balanced, the estimates would in general have been different. 

% % Notice: One may generate $L$ automatically with
% % << >>= 
% % L1 <- LE_matrix(tooth1, effect="dose")
% % L1
% % @

% Notice: One may obtain the LSmean directly as:

% % << >>= 
% % LSmeans(tooth1, effect="dose")
% % @

%% \subsection{Additive model}
%% \label{sec:additive-model}

%% Returning to the \verb|ToothGrowth| data, orange juice and ascorbic
%% acid are just two of many ways of supplying vitamin C (citrus and lime
%% juice would be two alternatives). Here one can therefore argue, that
%% it would make sense to estimate the the effect for each dose for an
%% ``average vitamin C source'':

%% << >>= 
%% LSmeans(tooth1, effect="dose")
%% @

% which is the same as

% <<results='hide'>>= 
% L <- LE_matrix(tooth1, effect="dose")
% le <- linest(tooth1, L=L)
% coef(le)
% @

% %% \subsection{Interaction model}
% %% \label{sec:interaction-model}


% For a model with interactions, the LSmeans are
% << >>= 
% LSmeans(tooth2, effect="dose")
% @

% In this case, the LE--matrix is
% << >>= 
% L <- LE_matrix(tooth2, effect="dose")
% t(L)
% @



%% \usepackage[inline,nomargin,draft]{fixme}
%% \newcommand\FXInline[2]{\textbf{\color{blue}#1}:
%%   \emph{\color{blue} - #2}}
%% \usepackage[cm]{fullpage}

---
title: "vtools: Flexible Variable Selection"
author: "Søren Højsgaard"
output:
  rmarkdown::html_vignette
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Working with vtools: Flexible Variable Selection}
  %\VignettePackage{doBy}
  %\VignetteEngine{knitr::knitr}
  %\VignetteEncoding{UTF-8}
---

```r
library(doBy)
library(rlang)
data(CO2)
```

## Overview

The `v*` functions in the `doBy` package allow for flexible and consistent handling of variable input.
These tools support unquoted names, character vectors, and formulas.

The main functions are:

- `vparse()` – extract variable names from flexible input
- `v()` – shorthand for `vparse()`
- `vselect()` – select columns from a data frame using flexible input
- `vcheck()` – verify that variables exist in a data frame
- `vmap()` – apply a function to each parsed variable name
- `vrename()` – rename variables in a data frame

---

## Examples

### `vparse()`
```r
vparse(Treatment, Type)
vparse(c("Treatment", "Type"))
vparse(~Treatment + Type)
```

### `v()` – shorthand for `vparse()`
```r
v(Treatment, Type)
v(~Treatment + Type)
v(c("Treatment", "Type"))
```

### `vselect()`
```r
vselect(CO2, Treatment, Type)
vselect(CO2, ~Treatment + Type)
```

### `vcheck()`
```r
vcheck(CO2, Treatment)
vcheck(CO2, ~Treatment + Type)
```

### `vmap()`
```r
vmap(~Treatment + Type, toupper)
```

### `vrename()`
```r
vrename(CO2, c(Treatment = "Trt", Type = "Group"))
```

## Summary

The `v*` tools simplify working with variable names and selection in data frames,
making it easier to write functions that accept flexible input formats.

% \VignetteEngine{knitr::knitr} 
%\VignettePackage{doBy}
%\VignetteIndexEntry{doBy: Linear estimates and LSmeans}
%\VignetteIndexEntry{LSmeans}
%\VignetteIndexEntry{population means}
%\VignetteIndexEntry{contrasts}
%\VignetteIndexEntry{estimable functions}

\documentclass[11pt]{article}

\usepackage{hyperref,url,color,Sweave,a4wide}
\usepackage[utf8]{inputenc}
\usepackage[authoryear,round]{natbib}
\bibliographystyle{plainnat}

\RequirePackage{color,fancyvrb,amsmath,amsfonts}

\def\textasciigrave{'}
\def\tt#1{\texttt{#1}}
\def\R{\texttt{R}}
\def\pkg#1{{\bf #1}}
\def\doby{\pkg{doBy}}

\def\code#1{\texttt{#1}}
\def\cc#1{\texttt{#1}}
\def\comi#1{{\it #1}}
\def\esticon{\code{esticon()}}
\def\lsmeans{\code{LSmeans}}
\def\linmat{\code{LE_matrix()}}
\def\linest{\code{linest()}}

% reduce whitespace between R code and R output
% \let\oldknitrout\knitrout
% \renewenvironment{knitrout}{
%   \begin{oldknitrout}
%     \footnotesize
%     \topsep=0pt
% }{
%   \end{oldknitrout}
% }


\DeclareMathOperator{\EE}{\mathbb{E}}
\DeclareMathOperator{\var}{\mathbb{V}ar}
\DeclareMathOperator{\cov}{\mathbb{C}ov}
\DeclareMathOperator{\norm}{N}
\DeclareMathOperator{\spanx}{span}
\DeclareMathOperator{\corr}{Corr}
\DeclareMathOperator{\deter}{det}
\DeclareMathOperator{\trace}{tr}
\def\inv{^{-1}}
\newcommand{\transp}{^{\top}}


<<echo=FALSE>>=
require( doBy )
prettyVersion <- packageDescription("doBy")$Version
prettyDate <- format(Sys.Date())
@

\title{Linear estimates and LS--means in the \texttt{doBy} package}
\author{S{\o}ren H{\o}jsgaard and Ulrich Halekoh}
\date{\pkg{doBy} version \Sexpr{prettyVersion} as of \Sexpr{prettyDate}}

% show R> prompt before R commands
<<r setup, echo=FALSE>>=
knitr::opts_chunk$set(prompt=TRUE) 
library(knitr)
if (!dir.exists("figures")) dir.create("figures")
opts_chunk$set(
               tidy=FALSE,fig.path='figures/LSmeans', fig.height=3.5 
           )
oopt <- options()
options("digits"=4, "width"=90, "prompt"="> ", "continue"="  ")
options(useFancyQuotes="UTF-8")
library(ggplot2)
@



\definecolor{darkred}{rgb}{.7,0,0}
\definecolor{midnightblue}{rgb}{0.098,0.098,0.439}




\begin{document}

<<include=FALSE>>=
library(knitr)
opts_chunk$set(
concordance=TRUE
)
@

% \SweaveOpts{concordance=TRUE}

\maketitle
\tableofcontents
\parindent0pt
\parskip5pt

\section{Introduction}
\label{sec:introduction}

\subsection{Linear functions of parameters}
\label{sec:line-funct-param}

A linear function of a $p$--dimensional parameter vector $\beta$ has
the form
\begin{displaymath}
  C=L\beta
\end{displaymath}
where $L$ is a $q\times p$ matrix which we call the \comi{Linear
  Estimate Matrix} of simply \comi{LE-matrix}.  The corresponding
linear estimate is $\hat C = L \hat \beta$.  A \comi{linear hypothesis} has
the form $H_0: L\beta=m$ for some $q$ dimensional vector $m$.


\subsection{LSmeans (population means, marginal means)}
\label{sec::lsmeans}

A special type of linear estimates is the so called \comi{least-squares
means} (or \comi{LS-means}). Other names for these estimates include
\comi{population means} and \comi{marginal means}. Consider an imaginary field
experiment analyzed with model of the form
<<eval=F>>=
lm( y ~ treat + block + year)
@ %def
where \cc{treat} is a treatment factor, \cc{block} is a blocking
factor and \cc{year} is the year (a factor) where the experiment is
repeated over several years. This model specifies the conditional mean
$\EE(Y|\cc{treat}, \cc{block},\cc{year})$. One may be interested in
predictions of the
form $\EE(Y|\cc{treat})$. This quantity can not formally be found from the
model. However, it is tempting to average the fitted values of
$\EE(Y|\cc{treat}, \cc{block},\cc{year})$ across the levels of
\cc{block} and \cc{year} and think of this average as
$\EE(Y|\cc{treat})$. This average is precisely what is called the
LS--means. If the experiment is balanced then this average is
identical to the average of the observations when stratified according
to \cc{treat}.



An alternative is to think of \cc{block} and \cc{year} as random
effects, for example:
<<eval=F>>=
library(lme4)
lmer( y ~ treat + (1|block) + (1|year))
@ %def

In this case one would directly obtain $\EE(Y|\cc{treat})$ from the
model. However, there are at least two reasons why one may be hesitant
to consider such a random effects model.
\begin{itemize}
\item Suppose there are three
blocks and the experiment is repeated over three consecutive
years. This means that the random effects are likely to be estimated
with a large uncertainty (the estimates will have only two degrees of
freedom).
\item Furthermore, treating \cc{block} and \cc{year} as random
effects means they should in principle come from a large population of
possible blocks and years. This may or may not be reasonable for the
blocks, but it is  a questionable assumption for the years.
\end{itemize}



\subsection{A simulated dataset}
\label{sec:simulated-dataset}

In the following sections we consider these data:

<<>>=
library(doBy)
set.seed(141164)
dd <- expand.grid(A=factor(1:3), B=factor(1:3), C=factor(1:2))
dd$y <- rnorm(nrow(dd))
dd$x <- rnorm(nrow(dd))^2
dd$z <- rnorm(nrow(dd))
head(dd,10)
@ %def

Consider the additive model
\begin{equation}
  \label{eq:1}
  y_i = \beta_0 + \beta^1_{A(i)}+\beta^2_{B(i)} + \beta^3_{C(i)} + \beta^4 x_i + e_i
\end{equation}
where $e_i \sim N(0,\sigma^2)$. We fit this model:
<<>>=
mm <- lm(y ~ A + B + C + x, data=dd)
coef(mm)
@ %def

Notice that the parameters corresponding to the factor levels
\code{A1}, \code{B1} and \code{C2} are set to zero to ensure
identifiability of the remaining parameters.


\subsection{LSmeans: details}
\label{sec:what-are-these}

LSmeans, population means and marginal means are used synonymously in
the literature. 
These quantities are a special kind of contrasts as defined in
Section~\ref{sec:line-funct-param}.
LSmeans seems to be the most widely used term, so we
shall adopt this terms here too.

The model (\ref{eq:1}) is a model for the conditional mean
$\EE(y|A,B,C,x)$.  Sometimes one is interested in quantities like
$\EE(y|A)$. This quantity can not formally be found because of the presences of
unless $B$, $C$ and $x$. 
However, suppose that $A$ is a treatment of main interest, $B$ is a
blocking factor, $C$ represents days on which the experiment was
carried out and $x$ denotes e.g. temperature for the specific block on the specific date. Then it is tempting to fix $x$ at its average value $\bar x$ and average $\EE(y|A,B,C, \bar x)$ over $B$
and $C$ (average over block and day) and think of this average as
$\EE(y|A)$.

The population mean for $A=1$ is
\begin{equation}
  \label{eq:2}
  \beta^0 + \beta^1_{A1} + \frac{1}{3} (\beta^2_{B1}+\beta^2_{B2}+\beta^2_{B3})
  + \frac{1}{2}(\beta^3_{C1}+\beta^3_{C2})
\end{equation}

Recall that the parameters corresponding to the factor levels
\code{A1}, \code{B1} and \code{C2} are set to zero to ensure
identifiability of the remaining parameters. Therefore we may also
write the population mean for $A=1$ as
\begin{equation}
  \label{eq:3}
  \beta^0 + \frac{1}{3} (\beta^2_{B2}+\beta^2_{B3})
  + \frac{1}{2}(\beta^3_{C2}) + \beta^4 \bar x.
\end{equation}

This quantity can be estimated as (if $\bar x = 1.242$) :

<<>>=
w <- c(1, 0, 0, 1/3, 1/3, 1/2, 1.242)
sum(coef(mm) * w)
@ %def

We may find the population mean for all three levels of $A$ as

<<>>=
L <- matrix(c(1, 0, 0, 1/3, 1/3, 1/2, 1.242,
              1, 1, 0, 1/3, 1/3, 1/2, 1.242,
              1, 0, 1, 1/3, 1/3, 1/2, 1.242), nr=3, byrow=TRUE)
L %*% coef(mm)
@ %def

Notice that the matrix W is based on that the first level of $A$ is
set as the reference level. If the reference level is changed then so
must $W$ be.

% Given that one has specified $W$, we can use the \esticon\ function in the
% \code{doBy} as illustrated below:

% <<>>=
% esticon(mm, W)
% @ %def



\def\popmatrix{LEmatrix}
\def\popmeans{LSmeans}

\subsection{Using \popmatrix\  and \popmeans} 

Writing the matrix $W$ is somewhat tedious and hence error prone. In
 addition, there is a potential risk of getting the wrong answer if the the reference level of a factor has been changed.  The \popmatrix\ function provides an automated way of generating such matrices.  The above \verb+W+ matrix is constructed by

<<>>=
L <- LE_matrix(mm, effect='A')
L
@ %def

The \verb+effect+ argument requires  to calculate the population means
for each level of
$A$ aggregating across the levels of the other variables in the data.

The \popmeans\ function is simply a wrapper around first a call
to \popmatrix\ followed by a call to (by default) \linest:

<<>>=
linest(mm, L=L)
LSM <- LSmeans(mm, effect='A')
LSM
@ %def

\subsection{Special details}
\label{sec:special-details}

\subsubsection{Summary}
\label{sec:summary}

More details about how the matrix was constructed is provided by the
\code{summary()} method; for example:

<<>>=
summary(LSM)
@ %def

\subsubsection{Specification of \code{effect}}
\label{sec:spec-code}

The \code{effect} argument can be specified in two ways:

<<>>=
## 1) a vector of characters and 
LE_matrix(mm, effect= c("A", "C"))
## 2) a right hand sided formula:
## LE_matrix(mm, effect= ~ A + C)
@

\subsubsection{Fixing values in the \code{at} list}
\label{sec:fixing-valu-code}

We can fix values of the explanatory variables using the \code{at=} argument: For example:

<<>>=
LE_matrix(mm, at=list(A=1, x=2))
@ 

\subsubsection{Combining \code{effect} and \code{at}}
\label{sec:comb-code-code}

<<>>=
LE_matrix(mm, effect= ~ A + C, at=list(A=1)) 
@

\subsubsection{Total average}
\label{sec:total-average}

Omitting \code{effect} as in

<<>>=
LE_matrix(mm)
LSmeans(mm)
@ %def
gives the ``total average''.



\subsubsection{Log transformed covariates}

Unless otherwise specified, numerical covariates are fixed at their
average value, for example for \code{x} below:

If we use \code{log(x)} instead we will get an error when
calculating LS--means. Instead one must modify data:

<<>>=
mm2 <- lm(y ~ A + B + C + log(x), data=dd)
## LSmeans(mm2, effect=~A) ## Will fail
mm2 <- lm(y ~ A + B + C + log.x,
          data=transform(dd, log.x=log(x)))
LSmeans(mm2, effect=~A)
LSmeans(mm2, effect=~A)$L
@ 

This also highlights what is computed: The average of the log of
\code{x}; not the log of the average of \code{x}.

\subsubsection{Powers of covariates}

Similar phenomenon for powers of covariates:

<<>>=
mm2 <- lm(y ~ A + B + C + x + I(x^2), data=dd)
LSmeans(mm2, effect=~A)
LSmeans(mm2, effect=~A)$L
@ %def


Above \verb|I(x^2)| is the average of the squared values of
\code{x}; not the  square of the average of
\code{x}, cfr.\ the following.

<<>>=
mm2 <- lm(y ~ A + B + C + x + x2,
          data=transform(dd, x2=x^2))
LSmeans(mm2, effect=~A)
LSmeans(mm2, effect=~A)$L
@ %def


% If we want to evaluate the LS--means at \code{conc=10} then we can do:
% <<>>=
% LSmeans(co2.lm4, effect="Treatment", at=list(conc=10, conc2=100))
% @ %def



\section{Miscellaneous}
\label{sec:miscellaneous}


\subsection{Excerpt of the CO2 data}
\label{sec:excerpt-co2-data}

For illustration we use subsets of the CO2 data:

<<>>=
data(CO2)
CO2 <- subset(CO2, conc < 300) ## OK
CO2.bal <- CO2
rownames(CO2.bal) <- NULL
@ 

% <<>>=
% xtabs(~Type + Treatment + conc, data=CO2.bal) |>
%     ftable(row.vars = "Type")
% @ 

Data is perfectly balanced.
We also use an unbalanced subset of the data
<<>>=
CO2.ubal <- CO2.bal[-c(1, 5, 12, 17, 18, 19, 20, 28),]
CO2.ubal |> head()
xtabs(~Type + Treatment + conc, data=CO2.ubal) |>
    ftable(row.vars = "Type")
@

% <<>>=
% p1 <- ggplot(CO2.bal, aes(x=conc, y=uptake, color=Treatment, group=Plant)) +
%     geom_line() + geom_point() + facet_grid(~Type)
% p2 <- ggplot(CO2.ubal, aes(x=conc, y=uptake, color=Treatment, group=Plant)) +
%     geom_line() + geom_point() + facet_grid(~Type)
% library(patchwork) ## Multiple graphs
% p1 / p2
% @ 

\subsection{Two linear models for CO2 data}
\label{sec:two-linear-models}

<<>>=
library(broom)
form.add <- uptake ~ conc + Treatment + Type
form.int <- uptake ~ conc * Treatment + Type
fm1.bal <- lm(form.add, data=CO2.bal)
fm1.ubal <- lm(form.add, data=CO2.ubal)
@ 

\subsection{Linear estimates and LSmeans}
\label{sec:line-estim-using}

\paragraph{Common task 1:}
Consider this task: Estimate the value of the response \tt uptake
for each combination of \tt Type and \tt Treatment . This
can be obtained, for example, as follows:

<<>>=
LSmeans(fm1.bal, effect=~Type+Treatment, at=list(conc=100))
LSmeans(fm1.ubal, effect=~Type+Treatment, at=list(conc=100))
@ 

\paragraph{Common task 2:}
Estimate the value of the response \tt uptake when \tt
 for each level of \tt Treatment.  Formally this question can
not be answered under the  model because the model gives the
conditional mean of \tt uptake given \tt conc, \tt Type and \tt
Treatment. There is, so to speak, no way of getting rid of \tt Type.
There are different workarounds for this situation: We can average over \tt Type and fix \t conc at its average. Alternatively, we can fit model without \tt Type effect and
  repeat the steps above.


<<>>=
LSmeans(fm1.bal, effect=~Treatment)
LSmeans(fm1.ubal, effect=~Treatment)
@

<<>>=
## Fit model without Type
fm2.bal <- update(fm1.bal, .~. - Type)
fm2.ubal <- update(fm1.ubal, .~. - Type)

LSmeans(fm2.bal, effect=~Treatment)
LSmeans(fm2.ubal, effect=~Treatment)
@ 

Notice that the predictions (estimates) are identical for the balanced dataset but not for the unbalanced dataset. Notice also that the standard errors of the predictions are somewhat larger when fitting a model without \tt Type effect. This is to be expected as the variation due to \tt Type goes into the residual.




\subsection{Generalized linear models}
\label{sec:gener-line-models}

We can calculate LS--means for e.g.\ a Poisson or a gamma model. Default is that
the calculation is calculated on the scale of the linear
predictor.

<<>>=
fm.glm <- glm(form.add, family=Gamma, data=CO2.ubal)
LSmeans(fm.glm, effect=~Treatment, type="link")
@ %def

\subsection{Generalized estimating equations}
\label{sec:gener-estim-equat}

For gee-type models we get
<<>>=
library(geepack)
fm.gee <- geeglm(uptake ~ conc + Treatment + Type,
                 id=Plant, family=Gamma, data=CO2.ubal)
LSmeans(fm.gee, effect=~Treatment)
@ %def



\subsection{Linear mixed effects model} 
\label{sec:linear-mixed-effects}


For illustration we treat \verb|Plant| as a random effect:

<< >>=
library(lme4)
fm.mix <- lmer(uptake ~ conc + Treatment + Type + (1|Plant), data=CO2.ubal)
LSmeans(fm.mix, effect=~Treatment)
@

Notice that the degrees of freedom by default are adjusted using a
Kenward--Roger approximation. Unadjusted degrees of freedom are obtained by setting \verb|adjust.df=FALSE|. 





\subsection{Pairwise comparisons}
\label{sec:pairwise-comparisons}

We will just mention that for certain other linear estimates, the
matrix $L$ can be generated automatically using \cc{glht()} from the
\pkg{multcomp} package. For example, pairwise comparisons of all
levels of \code{dose} can be obtained with

<< >>=
library("multcomp")
g1 <- glht(fm2.ubal, mcp(Treatment="Tukey"))
tidy(g1)
@

The L matrix is 
<< >>=
L <- g1$linfct
L
linest(fm2.ubal, L=L)
@




% \subsection{Manual computations}
% \label{sec:manual-computations}

% Averaging over predicted values amounts to averaging over weighted
% sums of the regression coefficients. The weights can be generated
% manually as:

% <<>>=
% coef(fm1.ubal)

% L01 <- matrix(c(1, 100, 0, 0,
%                 1, 100, 1, 0,
%                 1, 100, 0, 1,
%                 1, 100, 1, 1), ncol=4, byrow=T)
% L01 %*% coef(fm1.ubal)

% L02 <- matrix(c(1, 100, 0, 0.5,
%                 1, 100, 1, 0.5), ncol=4, byrow=T)
% L02 %*% coef(fm1.ubal)

% L03 <- matrix(c(1, 179.8, 0, 0.5,
%                 1, 179.8, 1, 0.5), ncol=4, byrow=T)
% L03 %*% coef(fm1.ubal)
% @ 

% \subsection{Automatic generation of LEmatrix}
% \label{sec:autom-gener-lematr}

% Averaging over predicted values amounts to averaging over weighted
% sums of the regression coefficients. The weights can be generated
% automatically as follows:

% \paragraph{Common task 1:}

% We calulate genereate the $L$ matrix automaically and the matrix and the model object into a function that returns the predictions, standard errors etc.

% <<>>=
% L1 <- LE_matrix(fm1.ubal, effect=~Treatment + Type, at=list(conc=100))
% L1
% C1 <- linest(fm1.ubal, L=L1)
% C1
% @ 

% The steps above can be made with one function call:

% <<>>=
% LSM1 <- LSmeans(fm1.ubal, effect=~Treatment + Type, at=list(conc=100))
% LSM1
% @


% \paragraph{Common task 2: }
% Likewise, for the second situation:

% <<>>=
% L2 <- LE_matrix(fm1.ubal, effect=~Treatment, at=list(conc=100)); L2
% C2 <- linest(fm1.ubal, L=L2); C2
% LSM2 <- LSmeans(fm1.ubal, effect=~Treatment, at=list(conc=100))
% LSM2
% @ 


% \paragraph{Common task 3: }
% Finally for the third situation:

% <<>>=
% L3 <- LE_matrix(fm1.ubal, effect=~Treatment);
% L3
% C3 <- linest(fm1.ubal, L=L3);
% C3
% LSM3 <- LSmeans(fm1.ubal, effect=~Treatment)
% LSM3
% @ 



% \subsection{ssssssssssssssss}
% \label{sec:ssssssssssssssss}


% Notice this information:
% <<>>=
% summary(L3)
% summary(C3)
% summary(LSM3)
% @ 


% \subsection{Using the \code{at=} argument}
% \label{sec:at-argument}


% <<chick-fig, fig.cap="ChickWeight data.">>=
% library(ggplot2)
% ChickWeight$Diet <- factor(ChickWeight$Diet)
% qplot(Time, weight, data=ChickWeight, colour=Chick, facets=~Diet,
%       geom=c("point","line")) + theme(legend.position="none") 
% @ %def

% Consider random regression model:
% <<>>=
% library(lme4)
% chick <- lmer(weight ~ Time * Diet + (0 + Time | Chick), 
%            data=ChickWeight)
% coef(summary(chick))
% @ %def


% The LE--matrix for \cc{Diet} becomes:
% <<>>=
% L <- LE_matrix(chick, effect="Diet")
% t(L)
% @ %def

% The value of \cc{Time} is by default taken to be the average of that
% variable. Hence the \lsmeans\ is the predicted weight for each diet at
% that specific point of time. We can consider other points of time with
% <<>>=
% K1 <- LE_matrix(chick, effect="Diet", at=list(Time=1))
% t(K1)
% @ %def

% The \lsmeans\ for the intercepts is the predictions at
% \cc{Time=0}. The \lsmeans\ for the slopes becomes
% <<>>=
% K0 <- LE_matrix(chick, effect="Diet", at=list(Time=0))
% t(K1 - K0)
% linest(chick, L=K1 - K0)
% @ %def


% We may be interested in finding the population means
% at all levels of  $A$
% but only at $C=1$. This is obtained by using the \code{at} argument
% (in which case the average is only over the remaining factor $B$):

% <<>>=
% popMatrix(mm, effect='A', at=list(C='1'))
% @ %def

% Another way of
% creating the population means
% at  all levels of $(A,C)$ is therefore

% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2')))
% @ %def

% We may have several variables in the \code{at} argument:
% @
% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2'), B='1'))
% @ %def



% We can create  our own function for comparing trends:

% <<>>=
% LSmeans_trend <- function(object, effect, trend){
%     L <- LE_matrix(object, effect=effect, at=as.list(setNames(1, trend))) -
%         LE_matrix(object, effect=effect, at=as.list(setNames(0, trend)))
%     linest(object, L=L)
% }
% ## LSmeans_trend(chick, effect="Diet", trend="Time")
% @ %def


% <<>>=
% LSmeans(fm.mix, effect=~Treatment, adjust.df=FALSE)
% LSmeans(fm.mix, effect=~Treatment, adjust.df=TRUE)
% @ %def





% \subsection{Example: Non--estimable linear functions}
% \label{sec:exampl-non-estim}

% <<>>=
% ## Make balanced dataset
% dat.bal <- expand.grid(list(AA=factor(1:2), BB=factor(1:3), CC=factor(1:3)))
% dat.bal$y <- rnorm(nrow(dat.bal))

% ## Make unbalanced dataset:  'BB' is nested within 'CC' so BB=1
% ## is only found when CC=1 and BB=2,3 are found in each CC=2,3,4
% dat.nst <- dat.bal
% dat.nst$CC <-factor(c(1,1,2,2,2,2,1,1,3,3,3,3,1,1,4,4,4,4))
% @ %def

% << >>= 
% dat.nst
% @

% Consider this simulated dataset:
% <<>>=
% head(dat.nst, 4)
% ftable(xtabs( ~ AA + BB + CC, data=dat.nst), row.vars="AA")
% @ %def

% Data is highly "unbalanced":
% Whenever \verb|BB=1| then \verb|CC| is always \verb|1|; whenever
% \verb|BB| is not \verb|1| then \verb|CC| is never \verb|1|.
% We have
% <<>>=
% mod.nst  <- lm(y ~ AA + BB : CC, data=dat.nst)
% coef(mod.nst)
% coef(summary(mod.nst))
% @ %def


% In this case some of the \lsmeans\ values are not estimable; for example:
% <<>>=
% lsm.BC <- LSmeans(mod.nst, effect=c("BB", "CC"))
% lsm.BC
% lsm.BC2 <- LSmeans(mod.nst, effect="BB", at=list(CC=2))
% lsm.BC2
% @ %def

% We describe the situation in 
% Section~\ref{sec:handl-non-estim} where we focus on \verb|lsm.BC2|.





% \subsection{Handling non--estimability}
% \label{sec:handl-non-estim}

% The model matrix for the model in Section~\ref{sec:exampl-non-estim}
% does not have full column rank and therefore not all values are
% calculated by \cc{LSmeans()}.

% <<>>=
% X <- model.matrix( mod.nst ) 
% Matrix::rankMatrix(X)
% dim(X)
% as(X, "Matrix")
% @ %def

% We consider a  model, i.e.\
% an $n$ dimensional random vector $y=(y_i)$ for which
% $\EE(y)=\mu=X\beta$ and $\cov(y)=V$ where $X$ does not have
% full column rank
% We are
% interested in linear functions of $\beta$, say
% \begin{displaymath}
%   c=l\transp\beta= \sum_j l_j \beta_j .
% \end{displaymath}

% <<>>=
% L <- LE_matrix(mod.nst, effect="BB", at=list(CC=2))
% t(L)
% linest(mod.nst, L=L)
% @ %def

% A least squares estimate of $\beta$ is
% \begin{displaymath}
%   \hat \beta = G X\transp y
% \end{displaymath}

% where $G$ is a generalized inverse of $X\transp  X$. Since the
% generalized inverse is not unique then neither is the estimate
% $\hat\beta$. Hence $\hat c = l\transp\hat \beta$ is in general not
% unique.

% One least squares estimate of $\beta$ and one corresponding linear estimate $L\hat\beta$ is: 
% <<>>=
% XtXinv <- MASS::ginv(t(X)%*%X)
% bhat <- as.numeric(XtXinv %*% t(X) %*% dat.nst$y)
% zapsmall(bhat)
% L %*% bhat
% @ %def

% For some values of $l$ (i.e.\ for some rows of $L$)
% the estimate $\hat c=l\transp \beta$ is
% unique (i.e.\ it does not depend on the choice of generalized
% inverse). 
% Such linear functions are said to be estimable and can be
% described as follows:

% All we specify with $\mu=X\beta$
% is that $\mu$ is a vector in the column space $C(X)$ of $X$.
% We can only learn about $\beta$ through $X\beta$ so the only thing we can
% say something about is linear combinations $\rho\transp X\beta$. Hence
% we can only say something about $l\transp\beta$ if there exists
% $\rho$ such that 
% $$
% l\transp\beta=\rho\transp X \beta,
% $$ 
% i.e., if
% $l=X\transp\rho$ for some $\rho$, which is if $l$ is in the column space
% $C(X\transp)$ of $X\transp$. This is the same as saying that $l$ must be 
% perpendicular to
% all vectors in the null space $N(X)$ of $X$. To check
% this, we find a basis $B$ for $N(X)$. This can be done in many ways,
% for example via a singular value decomposition of $X$, i.e.\
% \begin{displaymath}
%   X = U D V\transp
% \end{displaymath}
% A basis for $N(X)$ is given by those columns of $V$ that corresponds
% to zeros on the diagonal of $D$.

% <<>>=
% S <- svd(X)
% B <- S$v[, S$d < 1e-10, drop=FALSE ]; 
% head(B) ## Basis for N(X)
% @ %def

% From 
% << >>= 
% rowSums(L %*% B)
% @
% we conclude that the first row of $L$ is not perpendicular to all vectors in thenull space $N(X)$ whereas the two last rows of $L$ are. Hence these two linear estimates are estimable; their value does not depend on the choice of generalized inverse:
% << >>= 
% lsm.BC2
% @










% \subsection{Alternatives - \texttt{esticon()}}
% \label{sec:alternatives}


% An alternative is to do:

% << >>= 
% c1 <- esticon(fm1.ubal, L2)
% c1
% @

% Notice: \verb|esticon| has been in the \doby\ package for many years;
% \verb|linest| is a newer addition; \verb|esticon| is not actively
% maintained but remains in \doby\ for historical reasons.








% \subsection{Using the \code{at} argument}

% We may be interested in finding the population means
% at all levels of  $A$
% but only at $C=1$. This is obtained by using the \code{at} argument
% (in which case the average is only over the remaining factor $B$):

% <<>>=
% popMatrix(mm, effect='A', at=list(C='1'))
% @ %def

% Another way of
% creating the population means
% at  all levels of $(A,C)$ is therefore

% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2')))
% @ %def

% We may have several variables in the \code{at} argument:
% @
% <<>>=
% popMatrix(mm, effect='A', at=list(C=c('1','2'), B='1'))
% @ %def

% \subsection{Ambiguous specification when using the \texttt{effect} and
%   \texttt{at} arguments}

% There is room for an ambiguous specification if a variable appears in
% both the \code{effect} and the \code{at} argument, such as
% @
% <<>>=
% popMatrix(mm, effect=c('A','C'), at=list(C='1'))
% @ %def

% This ambiguity is due to the fact that the \verb+effect+ argument asks
% for the populations means at all levels of the variables but the
% \verb+at+ chooses only specific levels.

% This ambiguity is resolved as follows: Any variable in the \code{at}
% argument is removed from the \code{effect} argument such as the
% statement above is equivalent to
% @
% <<eval=T>>=
% ## popMatrix(mm, effect='A', at=list(C='1'))
% @ %def

% \subsection{Using covariates}

% Next consider the model where a covariate is included:
% @
% <<>>=
% mm2 <- lm(y~A+B+C+C:x, data=dd)
% coef(mm2)
% @ %def

% In this case we get
% <<>>=
% popMatrix(mm2,effect='A', at=list(C='1'))
% @ %def

% Above, $x$ has been replaced by its average and that is the general
% rule for models including covariates. However we may use the \code{at}
% argument to ask for calculation of the population mean at some
% user-specified value of $x$, say 12:
% <<>>=
% popMatrix(mm2,effect='A', at=list(C='1',x=12))
% @ %def

 
% <<>>=
% mm22 <- lm(y~A+B+C+C:x+I(x^2), data=dd)
% coef(mm22)
% @ %def 

 
% <<>>=
% popMatrix(mm22,effect='A', at=list(C='1'))
% @ %def 


 
% <<>>=
% dd <- transform(dd, x.sq=x^2)
% mm23 <- lm(y~A+B+C+C:x+x.sq, data=dd)
% coef(mm23)
% popMatrix(mm23,effect='A', at=list(C='1'))
% @ %def 























% \subsection{Using transformed covariates}

% Next consider the model where a  transformation of a covariate is included:
% @
% <<>>=
% mm3 <- lm(y~A+B+C+C:I(log(x)), data=dd)
% coef(summary(mm3))
% @ %def

% In this case we can not use \popmatrix\ (and hence
% \popmeans) directly.  Instead we first have to
% generate a new variable, say \verb+log.x+, with
% \verb+log.x+$=\log(x)$, in the data and then proceed as

% <<>>=
% dd <- transform(dd, log.x = log(x))
% mm32 <- lm(y~A+B+C+C:log.x, data=dd)
% popMatrix(mm32, effect='A', at=list(C='1'))
% @ %def

% \subsection{The \code{engine} argument of \popmeans}

% The \popmatrix is a function to generate a linear tranformation matrix of the model
% parameters with emphasis on  constructing such matrices for population
% means.
% \popmeans\ invokes by default the \esticon\ function on this
% linear transformation matrix for calculating parameter estimates and
% confidecne intervals.
% A similar function to \esticon\ is the \verb+glht+ function of the \verb+multcomp+
%  package.

%  The \code{glht()} function
%  can be chosen via the \verb+engine+ argument of \popmeans:

% <<>>=
%  library(multcomp)
% g<-popMeans(mm,effect='A', at=list(C='1'),engine="glht")
% g
% @ %def

% This allows to apply the methods available on the \verb+glht+ object like

% <<>>=
% summary(g,test=univariate())
% confint(g,calpha=univariate_calpha())
% @
% which yield the same results as the \esticon\ function.

% By default the functions will adjust the tests  and confidence intervals for multiplicity
% <<>>=
% summary(g)
% confint(g)
% @






% \subsection{Ambiguous specification when using the \texttt{effect} and
%   \texttt{at} arguments}

% There is room for an ambiguous specification if a variable appears in
% both the \code{effect} and the \code{at} argument, such as

% <<>>=
% LE_matrix(fm1.ubal, effect=c('Treatment','Type'), at=list(Type='Mississippi'))
% LE_matrix(fm1.ubal, effect=~Treatment + Type, at=list(Type='Mississippi'))
% @ %def

% This ambiguity is due to the fact that the \verb+effect+ argument asks
% for the populations means at all levels of the variables but the
% \verb+at+ chooses only specific levels.

% This ambiguity is resolved as follows: Any variable in the \code{at}
% argument is removed from the \code{effect} argument such as the
% statement above is equivalent to\footnote{FIXME}

% <<eval=T>>=
% LE_matrix(fm1.ubal, effect=c('Treatement'), at=list(Type='Mississippi'))
% LE_matrix(fm1.ubal, effect=~Treatement, at=list(Type='Mississippi'))
% @ %def















%%\bibliography{doBy}


\end{document}





% \section{LS--means for linear models}
% \label{sec:linear-model}


% \subsection{LS--means -- a first example}
% \label{sec:ls-means-population}

% <<echo=F>>=
% simdat<-structure(list(treat = structure(c(1L, 1L, 1L, 2L, 1L, 2L, 2L, 2L
% ), .Label = c("t1", "t2"), class = "factor"), year = structure(c(1L,
% 1L, 1L, 1L, 2L, 2L, 2L, 2L), .Label = c("1", "2"), class = "factor"),
%     y = c(0.5, 1, 1.5, 3, 3, 4.5, 5, 5.5)), .Names = c("treat", "year",
% "y"), row.names = c(NA, -8L), class = "data.frame")
% @ %def

% Consider these simulated data
% <<>>=
% simdat
% @ %def
% shown in the figure below.
% <<simdat-fig,include=F,fig.height=3, fig.width=6>>=
% library(ggplot2)
% qplot(treat, y, data=simdat, color=year, size=I(3))
% @ %def
% \includegraphics[height=6cm,width=12cm]{figures/LSmeans-simdat-fig}

% The LS--means under an additive model for the factor \cc{treat} is
% <<>>=
% msim <- lm(y ~ treat + year, data=simdat)
% LSmeans( msim, effect="treat")
% @ %def
% whereas the population means are
% <<>>=
% summaryBy(y~treat, data=simdat, FUN=mean)
% @ %def
% Had data been balanced (same number of observations for each
% combination of \cc{treat} and \cc{year}) the results would have been the
% same. An argument in favor of the LS--means is that these figures
% better represent what one would expect on in an ``average year''.


% \subsection{Example: Warpbreaks}
% \label{sec:example:-warpbreaks}

% <<>>=
% summary( warpbreaks )
% head( warpbreaks, 4 )
% ftable(xtabs( ~ wool + tension, data=warpbreaks))
% @ %def

% <<echo=F>>=
%  opar <- par(mfrow = c(1, 2), oma = c(0, 0, 1.1, 0))
%      plot(breaks ~ tension, data = warpbreaks, col = "lightgray",
%           varwidth = TRUE, subset = wool == "A", main = "Wool A")
%      plot(breaks ~ tension, data = warpbreaks, col = "lightgray",
%           varwidth = TRUE, subset = wool == "B", main = "Wool B")
%      mtext("warpbreaks data", side = 3, outer = TRUE)
%      par(opar)
% @ %def

% <<>>=
% (warp.lm <- lm(breaks ~ wool + tension, data=warpbreaks))
% @ %def

% The fitted values are:
% <<>>=
% uni <- unique(warpbreaks[,2:3])
% prd <- cbind(breaks=predict(warp.lm, newdata=uni), uni); prd
% @ %def

% \subsection{The LS--means}
% \label{sec:lsmeans}

% We may be interested in making predictions of the number of breaks for
% each level of \cc{tension} for \emph{any} type or an \emph{average}
% type of \code{wool}.  The idea behind LS--means is
% to average the predictions above over the two
% wool types. These quantities are the \lsmeans\ for the effect
% \cc{tension}.

% This is done with:
% <<>>=
% LSmeans(warp.lm, effect="tension")
% @ %def

% The term \lsmeans\ comes from that these quantities are the same as
% the least squares main effects of \cc{tension} when data is balanced:
% <<>>=
% doBy::summaryBy(breaks ~ tension, data=warpbreaks, FUN=mean)
% @ %def
% When data is not balanced these quantities are in general not the same.




% \subsection{LS--means for models with interactions}
% \label{sec:models-with-inter}

% Consider a model with interaction:
% <<>>=
% warp.lm2 <- update(warp.lm, .~.+wool:tension)
% coef( summary( warp.lm2 ))
% @ %def

% In this case the contrast matrix becomes:
% <<>>=
% K2 <- LE_matrix(warp.lm2, effect="tension"); K2
% linest(warp.lm2, K=K2)
% @ %def


% For the sake of illustration we treat \cc{wool} as a random effect:

% <<>>=
% library(lme4)
% warp.mm <- lmer(breaks ~ tension + (1|wool), data=warpbreaks)
% LSmeans(warp.mm, effect="tension")
% @ %def

% Notice here that the estimates themselves are very similar to those
% above but the standard errors are much larger. This comes from that
% there that \code{wool} is treated as a random effect.

% <<>>=
% VarCorr(warp.mm)
% @ %def

%\usepackage{framed}
%\usepackage{comment}
%%\definecolor{shadecolor}{gray}{0.91}

%%\definecolor{darkred}{rgb}{.7,0,0}
%%\definecolor{midnightblue}{rgb}{0.098,0.098,0.439}

%% \DefineVerbatimEnvironment{Sinput}{Verbatim}{
%%   fontfamily=tt,
%%   %%fontseries=b,
%%   %% xleftmargin=2em,
%%   formatcom={\color{midnightblue}}
%% }
%% \DefineVerbatimEnvironment{Soutput}{Verbatim}{
%%   fontfamily=tt,
%%   %%fontseries=b,
%%   %% xleftmargin=2em,
%%   formatcom={\color{blue}}
%% }
%% \DefineVerbatimEnvironment{Scode}{Verbatim}{
%%   fontfamily=tt,
%%   %%fontseries=b,
%%   %% xleftmargin=2em,
%%   formatcom={\color{blue}}
%% }
%% 
%%\fvset{listparameters={\setlength{\topsep}{-2pt}}}
%\renewenvironment{Schunk}{\linespread{.90}}{}

%%\renewenvironment{Schunk}{\begin{shaded}\small}{\end{shaded}}









%% A special type of linear estimates is the so called least--squares
%% means (or LS--means). Other names for these estimates include
%% population means and marginal means. Notice that the \pkg{lsmeans} package
%% \cite{Lenth:2013} also provides computations of LS--means, see
%% \url{http://cran.r-project.org/web/packages/lsmeans/}.

%% <<echo=F>>=
%% simdat<-structure(list(treat = structure(c(1L, 1L, 1L, 2L, 1L, 2L, 2L, 2L
%% ), .Label = c("t1", "t2"), class = "factor"), year = structure(c(1L,
%% 1L, 1L, 1L, 2L, 2L, 2L, 2L), .Label = c("1", "2"), class = "factor"),
%%     y = c(0.5, 1, 1.5, 3, 3, 4.5, 5, 5.5)), .Names = c("treat", "year",
%% "y"), row.names = c(NA, -8L), class = "data.frame")
%% @ %def

%% \subsection{LS--means on a simple example}
%% \label{sec:ls-means-simple}

%% Consider these simulated data, also shown in Fig.~\ref{fig:simdat2-fig}:
%% <<>>=
%% simdat
%% @ %def

%% <<simdat2-fig, fig.cap="LSmeans - simulated data.">>=
%% library(ggplot2)
%% qplot(treat, y, data=simdat, color=year)
%% @ %def


%% The LS--means under an additive model for the factor \cc{treat} is the predicted outcome for each level of \cc{treat} for an ``average year'':
%% <<>>=
%% msim <- lm(y ~ treat + year, data=simdat)
%% lsm <- LSmeans(msim, effect="treat")
%% lsm
%% @ %def


%% Notice that the population means are
%% <<>>=
%% summaryBy(y ~ treat, data=simdat, FUN=function(x) c(m=mean(x), s=sd(x)))
%% ## or aggregate(y ~ treat, data=simdat, FUN=function(x) c(m=mean(x), s=sd(x)))
%% @ %def

%% Had data been balanced (same number of observations for each
%% combination of \cc{treat} and \cc{year}) the results would have been the
%% same. An argument in favor of the LS--means is that these figures
%% better represent what one would expect on in an ``average year''.

%% An alternative is to think of  \cc{year} as a random
%% effect, for example:
%% <<eval=F>>=
%% library(lme4)
%% lmer( y ~ treat  + (1|year), data=simdat)
%% @ %def

%% In this case one would directly obtain $\EE(Y|\cc{treat})$ from the
%% model. However, there are at least two reasons why one may be hesitant
%% to consider such a random effects model.
%% \begin{itemize}
%% \item It is a ``never ending'' discussion whether \verb|year| should
%%   be treated as fixed or random. For \verb|year| to be a random
%%   effect, it should in principle come from a large population of
%%   possible years. This is certainly a dubious assumption for the
%%   years.
%% \item If we accept to think of year as a random effect, then the
%%   variance describing this random effect will be poorly determined
%%   (because there are only two years in the study).
%% \end{itemize}
 

% <<>>=
% warp.poi <- glm(breaks ~ wool + tension, family=poisson, data=warpbreaks)
% LSmeans(warp.poi, effect="tension", type="link")
% LSmeans(warp.poi, effect="tension", type="response")
% @ %def





%% SANITY CHECK
%% @
%% <<>>=
%% K <- LE_matrix(warp.poi, effect="tension")
%% doBy::esticon(warp.poi, K)
%% @ %def

% <<>>=
% warp.qpoi <- glm(breaks ~ wool + tension, family=quasipoisson, data=warpbreaks)
% LSmeans(warp.qpoi, effect="tension", type="link")
% LSmeans(warp.qpoi, effect="tension", type="response")
% @ %def


% For comparison with the linear model, we use identity link
% <<echo=F,results='hide'>>=
% warp.poi2 <- glm(breaks ~ wool + tension, family=poisson(link=identity),
%                  data=warpbreaks)
% LSmeans(warp.poi2, effect="tension", type="link")
% @ %def



%% SANITY CHECK
%% @
%% <<>>=
%% K <- LE_matrix(warp.poi2, effect="tension")
%% doBy::esticon(warp.poi2, K)
%% @ %def


% <<>>=
% warp.gam <- glm(breaks ~ wool + tension, family=Gamma(link=identity),
%                  data=warpbreaks)
% LSmeans(warp.gam, effect="tension", type="link")
% @ %def

%% SANITY CHECK
%% @
%% <<>>=
%% K <- LE_matrix(warp.gam, effect="tension")
%% doBy::esticon(warp.gam, K)
%% @ %def


% Notice that the linear estimates are practically the same as for the
% linear model, but the standard errors are smaller and hence the
% confidence intervals are narrower.

% An alternative is to fit a quasi Poisson ``model''

% <<>>=
% warp.poi3 <- glm(breaks ~ wool + tension, family=quasipoisson(link=identity),
%                  data=warpbreaks)
% LSmeans(warp.poi3, effect="tension")
% @ %def


% \subsubsection{For balanced data}
% \label{sec:balanced-case}


% When the dataset is balanced the results will be the same

% <<>>=
% aggregate(pred ~ Treatment, data=bb1, FUN=mean)

% fm2.bal <- update(fm1.bal, .~. - Type)
% grid2 <- expand.grid(
%     Treatment = c("nonchilled", "chilled"),
%     conc      = 100
% )
% bb2 <- cbind(pred=predict(fm2.bal, newdata=grid2), grid2)
% bb2
% @ 

% \subsubsection{For unbalanced data}
% \label{sec:unbalanced-case}

% In general, however, the results will not be identical




% <<>>=
% L <- matrix(c(1, 100, 0, 0,
%               1, 100, 0, 1,
%               1, 100, 1, 0,
%               1, 100, 1, 1), ncol=4, byrow=T)
% @ 




% \subsection{Tooth growth}
% \label{sec:tooth-growth}

% % The response is the length of odontoblasts cells (cells responsible for
% % tooth growth) in 60 guinea pigs.  Each animal received one of
% % three dose levels of vitamin C (0.5, 1, and 2 mg/day) by one of
% % two delivery methods, (orange juice (coded as OJ) or ascorbic acid 
% % (a form of vitamin C and (coded as VC)).

% % <<>>=
% % head(ToothGrowth, 4)
% % ftable(xtabs(~ dose + supp, data=ToothGrowth))
% % @ %def

% % <<echo=F, fig.cap="Plot of length against dose for difference sources of vitamin C.">>=
% % opar <- par(mfrow = c(1, 2), oma = c(0, 0, 1.1, 0))
% % plot(len  ~ dose, data = ToothGrowth, col = "lightgray",
% %      subset = supp == "OJ", main = "supp=OJ")
% % plot(len ~ dose, data = ToothGrowth, col = "lightgray",
% %      subset = supp == "VC", main = "supp=VC")
% % mtext("ToothGrowth data", side = 3, outer = TRUE)
% % par(opar)
% % @ %def

% % <<>>=
% % ToothGrowth |> interaction_plot(len ~ dose + supp)
% % @ 

% % The interaction plot suggests a mild interaction which is supported by a formal comparison:

% << >>= 
% ToothGrowth$dose <- factor(ToothGrowth$dose)
% head(ToothGrowth)
% tooth1 <- lm(len ~ dose + supp, data=ToothGrowth)
% tooth2 <- lm(len ~ dose * supp, data=ToothGrowth)
% anova(tooth1, tooth2)
% @


%% <<linear:interaction, fig.cap="Interaction plot for the ToothGrowth data. The average length for each group is a dot. Boxplot outliers are crosses." >>= 
%% tooth_avglen <- ToothGrowth |>
%%   group_by(dose, supp) |>
%%   summarise(val = mean(len))
%% tooth_avglen
%% ggplot(ToothGrowth, aes(x = factor(dose), y = len, colour = supp)) +
%%     geom_boxplot(outlier.shape = 4) +
%%     geom_point(data = tooth_avglen, aes(y = val)) +
%%     geom_line(data = tooth_avglen, aes(y = val, group = supp))
%%@


% \section{Computing linear estimates}
% \label{sec:comp-line-estim}


% For now, we focus on the additive model:
% << >>= 
% tooth1
% @

% Consider computing the estimated length for each dose of orange juice (OJ):
% One option: Construct the LE--matrix $L$ directly:

% << >>= 
% L <- matrix(c(1, 0, 0, 0, 
%               1, 1, 0, 0,
%               1, 0, 1, 0), nrow=3, byrow=T)
% @

% Then do:

% << >>= 
% c1 <- linest(tooth1, L)
% c1
% @

% We can do:

% << >>= 
% summary(c1)
% coef(c1)
% confint(c1)
% @

% \subsection{Automatic generation of $L$}
% \label{sec:autom-gener-l}


% The matrix $L$ can be generated as follows:
% << >>= 
% L <- LE_matrix(tooth1, effect="dose", at=list(supp="OJ"))
% L
% @


% \subsection{OLD: Least-squares means (LS--means)}
% \label{sec:least-squares-means}


% A related question could be: What is the estimated length for each
% dose if we ignore the source of vitamin C (i.e.\ whether it is OJ or
% VC). One approach would be to fit a model in which source does not appear:

% % << >>= 
% % tooth0 <- update(tooth1, . ~ . - supp)
% % L0 <- LE_matrix(tooth0, effect="dose")
% % L0
% % linest(tooth0, L=L0)
% % @

% An alternative would be to stick to the original model but compute the
% estimate for an ``average vitamin C source''. That would correspond to
% giving weight $1/2$ to each of the two vitamin C source
% parameters. However, as one of the parameters is already set to zero
% to obtain identifiability, we obtain the LE--matrix $L$ as

% % << >>= 
% % L1 <- matrix(c(1, 0, 0, 0.5, 
% %                1, 1, 0, 0.5,
% %                1, 0, 1, 0.5), nrow=3, byrow=T)
% % linest(tooth1, L=L1)
% % @

% Such a particular linear estimate is sometimes called a least-squares
% mean or an LSmean or a marginal mean. Notice that the parameter
% estimates under the two approaches are identical. This is is because
% data is balanced: There are $10$ observations per supplementation
% type. Had data not been balanced, the estimates would in general have been different. 

% % Notice: One may generate $L$ automatically with
% % << >>= 
% % L1 <- LE_matrix(tooth1, effect="dose")
% % L1
% % @

% Notice: One may obtain the LSmean directly as:

% % << >>= 
% % LSmeans(tooth1, effect="dose")
% % @

%% \subsection{Additive model}
%% \label{sec:additive-model}

%% Returning to the \verb|ToothGrowth| data, orange juice and ascorbic
%% acid are just two of many ways of supplying vitamin C (citrus and lime
%% juice would be two alternatives). Here one can therefore argue, that
%% it would make sense to estimate the the effect for each dose for an
%% ``average vitamin C source'':

%% << >>= 
%% LSmeans(tooth1, effect="dose")
%% @

% which is the same as

% <<results='hide'>>= 
% L <- LE_matrix(tooth1, effect="dose")
% le <- linest(tooth1, L=L)
% coef(le)
% @

% %% \subsection{Interaction model}
% %% \label{sec:interaction-model}


% For a model with interactions, the LSmeans are
% << >>= 
% LSmeans(tooth2, effect="dose")
% @

% In this case, the LE--matrix is
% << >>= 
% L <- LE_matrix(tooth2, effect="dose")
% t(L)
% @



%% \usepackage[inline,nomargin,draft]{fixme}
%% \newcommand\FXInline[2]{\textbf{\color{blue}#1}:
%%   \emph{\color{blue} - #2}}
%% \usepackage[cm]{fullpage}

\name{prostate}
\docType{data}
\alias{prostate}
\title{Prostate Tumor Gene Expression Dataset}
\description{
  This is the Prostate Tumor Gene Expression dataset used in Chung and Keles (2010).
}
\usage{ data(prostate) }
\format{
  A list with two components:
  \describe{
  \item{x}{ Gene expression data. A matrix with 102 rows and 6033 columns.}
  \item{y}{ Class index. A vector with 102 elements.}
  }
}
\details{
  The prostate dataset consists of 52 prostate tumor and 50 normal samples.
  Normal and tumor classes are coded in 0 and 1, respectively, in \code{y} vector.
  Matrix \code{x} is gene expression data and
  arrays were normalized, log transformed, and standardized 
  to zero mean and unit variance across genes as described
  in Dettling (2004) and Dettling and Beuhlmann (2002).
  See Chung and Keles (2010) for more details. 
}
\source{
Singh D, Febbo P, Ross K, Jackson D, Manola J, Ladd C, Tamayo P, Renshaw A,
DAmico A, Richie J, Lander E, Loda M, Kantoff P, Golub T, and Sellers W (2002),
"Gene expression correlates of clinical prostate cancer behavior", \emph{Cancer Cell}, 
Vol. 1, pp. 203--209.
}
\references{
Chung D and Keles S (2010), 
"Sparse partial least squares classification for high dimensional data",
\emph{Statistical Applications in Genetics and Molecular Biology}, Vol. 9, Article 17.
  
Dettling M (2004), "BagBoosting for tumor classification with gene expression data",
\emph{Bioinformatics}, Vol. 20, pp. 3583--3593.

Dettling M and Beuhlmann P (2002), "Supervised clustering of genes", \emph{Genome Biology},
Vol. 3, pp. research0069.1--0069.15.
}
\examples{
data(prostate)
prostate$x[1:5,1:5]
prostate$y
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{data_budworm}
\alias{data_budworm}
\alias{budworm}
\title{Budworm data}
\format{
This data frame contains 12 rows and 4 columns:

\describe{
\item{sex:}{sex of the budworm.}
\item{dose:}{dose of the insecticide trans-cypermethrin (in micro grams)}.
\item{ndead:}{budworms killed in a trial.}
\item{ntotal:}{total number of budworms exposed per trial.}
}
}
\source{
Collett, D. (1991) Modelling Binary Data, Chapman & Hall, London,
Example 3.7
}
\usage{
budworm
}
\description{
Experiment on the toxicity to the tobacco budworm
Heliothis virescens of doses of the pyrethroid
trans-cypermethrin to which the moths were beginning to show
resistance. Batches of 20 moths of each sex were exposed for
three days to the pyrethroid and the number in each batch that
were dead or knocked down was recorded. Data is reported in
Collett (1991, p. 75).
}
\examples{

data(budworm)

## function to caclulate the empirical logits
empirical.logit<- function(nevent,ntotal) {
   y <- log((nevent + 0.5) / (ntotal - nevent + 0.5))
   y
}


# plot the empirical logits against log-dose

log.dose <- log(budworm$dose)
emp.logit <- empirical.logit(budworm$ndead, budworm$ntotal)
plot(log.dose, emp.logit, type='n', xlab='log-dose',ylab='emprirical logit')
title('budworm: emprirical logits of probability to die ')
male <- budworm$sex=='male'
female <- budworm$sex=='female'
lines(log.dose[male], emp.logit[male], type='b', lty=1, col=1)
lines(log.dose[female], emp.logit[female], type='b', lty=2, col=2)
legend(0.5, 2, legend=c('male', 'female'), lty=c(1,2), col=c(1,2))

\dontrun{
* SAS example;
data budworm;
infile 'budworm.txt' firstobs=2;
input sex dose ndead ntotal;
run;
}


}
\references{
Venables, W.N; Ripley, B.D.(1999) Modern Applied Statistics with
S-Plus, Heidelberg, Springer, 3rd edition, chapter 7.2
}
\concept{data}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recover_pca_data.R
\name{recover_pca_data}
\alias{recover_pca_data}
\title{Recover data from principal component analysis}
\usage{
recover_pca_data(object, comp = 1)
}
\arguments{
\item{object}{An object of class \code{prcomp}.}

\item{comp}{The number of components to be used. Must be smaller
than the number of variables.}
}
\value{
A dataframe
}
\description{
Recover data from principal component analysis based on the first (typically few) components.
}
\examples{

crime <- doBy::crimeRate
rownames(crime) <- crime$state
crime$state <- NULL

o <- order(apply(scale(crime), 1, sum))
dat <- crime[o,]
head(dat)
tail(dat)
matplot(scale(dat), type="l")

pc1 <- prcomp(dat, scale. = TRUE)
summary(pc1)
rec2 <- recover_pca_data(pc1, 2)

pairs(rec2)

par(mfrow=c(1,2))
matplot(scale(dat), type="l")
matplot(scale(rec2), type="l")

j <- merge(dat, rec2, by=0)
pairs(j[,-1])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vtools.R
\name{vparse}
\alias{vparse}
\title{Variable utilities: parse, select, check, map, rename}
\usage{
vparse(...)
}
\arguments{
\item{...}{Variable input (unquoted, character vector, or formula)}
}
\description{
These functions provide flexible tools for parsing and managing variable names
from formulas, unquoted names, or character vectors. Demonstrated using \code{CO2} dataset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{carcass}
\alias{carcass}
\alias{carcassall}
\title{Lean meat contents of 344 pig carcasses}
\format{
carcassall: A data frame with 344 observations on the following 17
variables.
\describe{
\item{\code{weight}}{Weight of carcass}
\item{\code{lengthc}}{Length of carcass from back toe to head (when
the carcass hangs in the back legs)}
\item{\code{lengthf}}{Length of carcass from back toe to front leg
(that is, to the shoulder)}
\item{\code{lengthp}}{Length of carcass from back toe to the pelvic bone}
\item{\code{Fat02, Fat03, Fat11, Fat12, Fat13, Fat14, Fat16}}{Thickness of fat
layer at different locations on the back of the carcass (FatXX
refers to thickness at (or rather next to) rib no. XX. Notice that
02 is closest to the head}
\item{\code{Meat11, Meat12, Meat13}}{Thickness of meat layer at different
locations on the back of the carcass, see description above}
\item{\code{LeanMeat}}{Lean meat percentage determined by dissection}
\item{\code{slhouse}}{Slaughter house; a factor with levels \code{slh1} and  \code{slh2}.}
\item{\code{sex}}{Sex of the pig; a factor with levels \code{castrate} and \code{female}.}
\item{\code{size}}{Size of the carcass; a factor with levels \code{normal} and \code{large}.
Here, \code{normal} refers to carcass weight under 80 kg; \code{large} refers to carcass weights between 80 and 110 kg.}

}
}
\source{
Busk, H., Olsen, E. V., Brøndum, J. (1999) Determination of
lean meat in pig carcasses with the Autofom classification
system, Meat Science, 52, 307-314
}
\usage{
carcass
}
\description{
Measurement of lean meat percentage of 344 pig carcasses together with
auxiliary information collected at three Danish slaughter houses
}
\details{
: Notice that there were slaughtered large pigs only at one slaughter house.
}
\note{
carcass: Contains only the variables Fat11, Fat12, Fat13,
Meat11, Meat12, Meat13, LeanMeat
}
\examples{
data(carcass)
head(carcass)

}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_stability.r
\name{set_list_set_matrix}
\alias{set_list_set_matrix}
\alias{set_list2matrix}
\alias{matrix2set_list}
\title{Matrix representatation of list of vectors and vice versa}
\usage{
set_list2matrix(set_list, aggregate = FALSE)

matrix2set_list(set_matrix)
}
\arguments{
\item{set_list}{list of vectors}

\item{aggregate}{should the vectors be aggregated}

\item{set_matrix}{matrix representatation}
}
\description{
Matrix representatation of list of vectors and vice versa
}
\examples{
l <- list(c(1,2,3), c(3,2,4), c(3,2,4))
m1 <- set_list2matrix(l)
m1
matrix2set_list(m1)

m2 <- set_list2matrix(l, aggregate=TRUE)
m2
matrix2set_list(m2)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelling.R
\name{align_coefs}
\alias{align_coefs}
\title{Align and combine fixed-effect coefficients from multiple models}
\usage{
align_coefs(models)
}
\arguments{
\item{models}{A named list of fitted model objects. Each element
should be a model that can be passed to \code{broom::tidy()}.}
}
\value{
A tibble with columns:
\itemize{
\item \code{model}: Model name.
\item \code{term}: Coefficient name.
\item \code{estimate}: Estimated coefficient.
\item \code{std.error}: Standard error.
\item \code{statistic}: Wald statistic (\code{estimate / std.error}).
\item \code{p.value}: Two-sided normal p-value.
}
}
\description{
Extracts and aligns the fixed-effect estimates from a
list of fitted model objects, returning them in a single tidy
data frame with consistent columns for easy comparison.  Works
with a mix of model types such as \code{lm}, \code{glm}, \code{gls}, \code{lmer},
etc.

For models without p-values (e.g., \code{lmer}), the function computes
approximate Wald statistics and two-sided normal p-values.
}
\examples{
# Example using the built-in CO2 dataset
data(CO2)

# Fit models
lm_fit  <- lm(uptake ~ conc + Type + Treatment, data = CO2)
glm_fit <- glm(uptake ~ conc + Type + Treatment,
  family = Gamma(identity), data = CO2)

# Combine estimates
models_list <- list(lm = lm_fit, glm = glm_fit)
result <- align_coefs(models_list)
print(result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.r
\name{response_plot}
\alias{response_plot}
\title{Plot the response variable against the predictor variables.}
\usage{
response_plot(
  data.,
  formula.,
  geoms = NULL,
  global_aes = NULL,
  plot = TRUE,
  nrow = NULL,
  ncol = NULL
)
}
\arguments{
\item{data.}{A data frame containing the variables in the formula.}

\item{formula.}{A formula of the form y ~ x1 + x2 + ... + xn, where
y is the response variable and x1, x2, ..., xn are the
predictor variables. A dot as right hand side is allowed.}

\item{geoms}{A list of ggplot2 geoms to be added to the plot.}

\item{global_aes}{A list of global aesthetics to be added to the plot.}

\item{plot}{A logical value indicating whether the plot should be displayed.}

\item{nrow, ncol}{Number of rows / columns in plot.}
}
\value{
A list of ggplot2 plots.
}
\description{
Plot the response variable against the predictor variables.
}
\examples{
library(ggplot2)
response_plot(iris, Sepal.Width ~ ., geoms=geom_point())
response_plot(iris, Sepal.Width ~ ., geoms=geom_point(), global_aes=list(color="Species"))
personality |> response_plot(easygon~., geoms=geom_point(), global_aes=NULL)

}
\concept{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linear_modelling.R
\name{reciprocal}
\alias{reciprocal}
\title{Reciprocal function}
\usage{
reciprocal(x)
}
\arguments{
\item{x}{An R object for whih 1/x makes sense}
}
\description{
A simple function returning the reciprocal of its argument
}
\author{
Søren Højsgaard
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_rbind_list.R
\name{rbind_list}
\alias{rbind_list}
\title{Bind list of data frames and add list names as a column}
\usage{
rbind_list(lst, name = "name")
}
\arguments{
\item{lst}{A named list of data frames or tibbles.}

\item{name}{A character scalar: name of the column to hold the list names (default "name").}
}
\value{
A data frame or tibble, depending on the class of the input.
}
\description{
Binds a named list of data frames (or tibbles) into a single data frame.
Adds the list name as a new column (first column).
}
\examples{
lst <- list(a = data.frame(x = 1:2), b = data.frame(x = 3:4))
rbind_list(lst)

lst <- split(iris, iris$Species)
rbind_list(lst)


}
\concept{data_handling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pick.R
\name{pick_elements}
\alias{pick_elements}
\alias{pick1}
\alias{pick2}
\title{Extract (pick) elements without using brackets}
\usage{
pick1(x, which)

pick2(x, which)
}
\arguments{
\item{x}{A list, data frame, or vector.}

\item{which}{The index or name of the element(s) to extract.}
}
\value{
\itemize{
\item \code{pick1()} returns a subset of \code{x}.
\item \code{pick2()} returns a single element from \code{x}.
}
}
\description{
Extract (pick) elements without using brackets so that elements can be picked out as part of a pipe workflow.
}
\details{
These two helper functions extract elements from lists, data frames, or vectors.
They are simple wrappers for the standard bracket operators in R:
\itemize{
\item \code{pick1()} uses single brackets (\code{[}) and returns a \strong{subset}.
\item \code{pick2()} uses double brackets (\code{[[}) and returns the \strong{element itself}.
}

These are safer and more flexible than \code{$}, especially when used with the base R pipe (\verb{|>})
or in functional programming.
}
\examples{
lst <- list(a = 1:3, b = 4:6)

# Without pipe
pick1(lst, "a")      # List with one element
pick2(lst, "a")      # Just the vector 1:3

# With base R pipe
lst |> pick1("a")
lst |> pick2("a")

df <- data.frame(x = 1:5, y = letters[1:5])

df |> pick1("y")     # Returns a data frame with column 'y'
df |> pick2("y")     # Returns column 'y' as a character vector

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelling.R
\name{add_pred}
\alias{add_pred}
\title{Add predicted values of different types to dataframe / tibble}
\usage{
add_pred(data, model, var = "pred", type = NULL, transformation = NULL)
}
\arguments{
\item{data}{dataframe or tibble}

\item{model}{model object}

\item{var}{name of new variable in dataframe / tibble}

\item{type}{type of predicted value}

\item{transformation}{A possible transformation of predicted
variable, e.g. reciprocal(), log() etc}
}
\value{
dataframe / tibble
}
\description{
Add predicted values of different types to dataframe / tibble
}
\examples{
data(cars)
lm1 <- lm(dist ~ speed + I(speed^2), data=cars)
cars <- cars |> add_pred(lm1)
cars <- cars |> add_pred(lm1, var="pred_log",
  transformation=log)
cars <- cars |> add_pred(lm1, var="pred_reicp",
  transformation=reciprocal)
cars |> head()
}
\seealso{
\code{\link[=add_resid]{add_resid()}}, \code{\link[=response]{response()}}
}
\author{
Søren Højsgaard
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{taylor}
\alias{taylor}
\title{Taylor expansion (one dimension)}
\usage{
taylor(fn, x0, ord = 1)
}
\arguments{
\item{fn}{A function of one variable and that variable must be named 'x'.}

\item{x0}{The point in which to to the Taylor expansion.}

\item{ord}{The order of the Taylor expansion.}
}
\value{
function.
}
\description{
Returns Taylor polynomial approximating a function fn(x)
}
\examples{
fn <- function(x) log(x)
ord <- 2
x0 <- 2

xv  <- seq(.2, 5, .1)
plot(xv, fn(xv), type="l")
lines(xv, taylor(fn, x0=x0, ord=ord)(xv), lty=2)
abline(v=x0)

fn <- function(x)sin(x)
ord <- 4
x0 <- 0
xv <- seq(-2*pi, 2*pi, 0.1)
plot(xv, fn(xv), type="l")
lines(xv, taylor(fn, x0=x0, ord=ord)(xv), lty=2)
abline(v=x0)



}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{milkman}
\alias{milkman}
\alias{milkman_rdm1}
\title{Milk yield data for manually milked cows.}
\format{
A data frame with 161836 observations on the following 12 variables.
\describe{
\item{\code{cowno}}{a numeric vector; cow identification}
\item{\code{lactno}}{a numeric vector; lactation number}
\item{\code{ampm}}{a numeric vector; milking time: 1: morning; 2: evening}
\item{\code{dfc}}{a numeric vector; days from calving}
\item{\code{my}}{a numeric vector; milk yield (kg)}
\item{\code{fatpct}}{a numeric vector; fat percentage}
\item{\code{protpct}}{a numeric vector; protein percentage}
\item{\code{lactpct}}{a numeric vector; lactose percentage}
\item{\code{scc}}{a numeric vector; somatic cell counts}
\item{\code{race}}{a factor with levels \code{RDM} \code{Holstein} \code{Jersey}}
\item{\code{ecmy}}{a numeric vector; energy corrected milk}
\item{\code{cowlact}}{Combination of cowno and lactno; necessary
because the same cow may appear more than once in the dataset (in
different lactations)}
}
}
\usage{
milkman
}
\description{
Milk yield data for cows milked manually twice a day (morning and evening).
}
\details{
There are data for 222 cows. Some cows appear more than once in the dataset
(in different lactations) and there are 288 different lactations.
}
\examples{

data(milkman)

}
\references{
Friggens, N. C.; Ridder, C. and Løvendahl, P. (2007).
On the Use of Milk Composition Measures to Predict the Energy Balance of Dairy Cows.
J. Dairy Sci. 90:5453–5467 doi:10.3168/jds.2006-821.

This study was part of the Biosens project used data from the
“Malkekoens energibalance og mobilisering” project; both were
funded by the Danish Ministry of Food, Agriculture and Fisheries
and the Danish Cattle Association.
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{haldCement}
\alias{haldCement}
\title{Heat development in cement under hardening.}
\format{
A data frame with 13 observations on the following 5 variables.
\describe{
\item{\code{x1}}{Percentage (weight) of \verb{[3Ca0][Al2O3]}}
\item{\code{x2}}{Percentage (weight) of \verb{[3Cao][SiO2]}}
\item{\code{x3}}{Percentage (weight) of \verb{[4Ca0][Al2O3][Fe03]}}
\item{\code{x4}}{Percentage (weight) of \verb{[2Cao][SiO2]}}
\item{\code{y}}{Heat development measured in calories per
gram cement after 180 days}
}
}
\usage{
haldCement
}
\description{
Heat development in cement under hardening related to the chemical
composition.
}
\examples{

data(haldCement)

if( interactive() ){
pairs( haldCement )
}
m <- lm(y ~ x1 + x2 + x3 + x4, data=haldCement)
summary(m)

# Notice: The model explains practically all variation in data;
# yet none of the explanatory variables appear to be statistically
# significant.

}
\references{
Anders Hald (1949); Statistiske Metoder; Akademisk Forlag (in
Danish), page 509.
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{data-wine}
\alias{data-wine}
\alias{wine}
\title{Chemical composition of wine}
\format{
A data frame with 178 observations on the following 14 variables.
\describe{
\item{\code{Cult}}{a factor with levels \code{v1} \code{v2}
\code{v3}: 3 different graph varieties}
\item{\code{Alch}}{Alcohol}
\item{\code{Mlca}}{Malic acid}
\item{\code{Ash}}{Ash}
\item{\code{Aloa}}{Alcalinity of ash}
\item{\code{Mgns}}{Magnesium}
\item{\code{Ttlp}}{Total phenols}
\item{\code{Flvn}}{Flavanoids}
\item{\code{Nnfp}}{Nonflavanoid phenols}
\item{\code{Prnt}}{Proanthocyanins}
\item{\code{Clri}}{Color intensity}
\item{\code{Hue}}{Hue}
\item{\code{Oodw}}{OD280/OD315 of diluted wines}
\item{\code{Prln}}{Proline}
}
}
\source{
Frank, A. & Asuncion, A. (2010). UCI Machine Learning
Repository \url{https://archive.ics.uci.edu/ml/}. Irvine, CA:
University of California, School of Information and Computer
Science.
}
\usage{
data(wine)
}
\description{
Using chemical analysis determine the origin of wines
}
\details{
Data comes from the UCI Machine Learning Repository. The grape variety
\code{Cult} is the class identifier.
}
\examples{

data(wine)
## maybe str(wine) ; plot(wine) ...

}
\references{
See references at
\url{https://archive.ics.uci.edu/ml/datasets/Wine/}
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{data_mathmark}
\alias{data_mathmark}
\alias{mathmark}
\alias{math}
\title{Mathematics marks for students}
\format{
This data frame contains the following columns: mechanics, vectors,
algebra, analysis, statistics.
}
\usage{
data(mathmark)
}
\description{
The \code{mathmark} data frame has 88 rows and 5 columns.
}
\examples{

data(mathmark)

}
\references{
David Edwards, An Introduction to Graphical Modelling, Second
Edition, Springer Verlag, 2000
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_binomial_to_bernoulli_data.r
\name{binomial_to_bernoulli_data}
\alias{binomial_to_bernoulli_data}
\title{Convert binomial data to bernoulli data}
\usage{
binomial_to_bernoulli_data(
  data.,
  y,
  size,
  type = c("rest", "total"),
  response_name = "response",
  rest_name = NULL
)
}
\arguments{
\item{data.}{A dataframe}

\item{y}{Column with 'successes' in binomial distribution \code{y~bin(size, p)}}

\item{size}{Column with 'failures', i.e. size-y or 'total', i.e. size.}

\item{type}{Whether \code{size} is rest (i.e. 'failures') or 'total'}

\item{response_name}{Name of response variable in output dataset.}

\item{rest_name}{Name of 'failures' in column \code{response_name}.}
}
\description{
Convert binomial data to bernoulli data by expanding dataset.
}
\examples{

dat <- budworm
dat <- dat[dat$dose \%in\% c(1,2), ]
dat$ntotal <- 5
dat
dat.a <- dat |>
  binomial_to_bernoulli_data(ndead, ntotal, type="total")
dat.b <- dat |>
  dplyr::mutate(nalive=ntotal-ndead) |> dplyr::select(-ntotal) |>
  binomial_to_bernoulli_data(ndead, nalive, type="rest")

m0 <- glm(cbind(ndead, ntotal-ndead) ~ dose + sex, data=dat, family=binomial())
m1 <- glm(ndead / ntotal ~ dose + sex, data=dat, weight=ntotal, family=binomial())
ma <- glm(response ~ dose + sex, data=dat.a, family=binomial())
mb <- glm(response ~ dose + sex, data=dat.b, family=binomial())

dat.a$response
dat.b$response ## Not same and therefore the following do not match

all.equal(coef(m0), coef(ma))
all.equal(coef(m0), coef(mb))
all.equal(coef(m1), coef(ma))
all.equal(coef(m1), coef(mb))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_sampleBy.R
\name{by-sample}
\alias{by-sample}
\alias{sample_by}
\alias{sampleBy}
\title{Sampling from a data frame}
\usage{
sample_by(data, formula, frac = 0.1, replace = FALSE, systematic = FALSE)

sampleBy(
  formula,
  frac = 0.1,
  replace = FALSE,
  data = parent.frame(),
  systematic = FALSE
)
}
\arguments{
\item{data}{A data frame.}

\item{formula}{A formula defining the grouping of the data frame.}

\item{frac}{The part of data to be sampled.}

\item{replace}{Is the sampling with replacement.}

\item{systematic}{Should sampling be systematic.}
}
\value{
A dataframe.
}
\description{
A data frame is split according to some variables in a
formula, and a sample of a certain fraction of each is drawn.
}
\details{
If systematic=FALSE (default) then frac gives the fraction
of data sampled. If systematic=TRUE and frac=.2 then every 1/.2
i.e. every 5th observation is taken out.
}
\examples{
data(dietox)
sampleBy(formula = ~ Evit + Cu, frac=.1, data = dietox)
}
\seealso{
\code{\link{orderBy}}, \code{\link{order_by}},
\code{\link{splitBy}}, \code{\link{split_by}},
\code{\link{summaryBy}}, \code{\link{summary_by}},
\code{\link{transformBy}}, \code{\link{transform_by}}
}
\concept{byfunction}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vtools.R
\name{v}
\alias{v}
\title{Shorthand for vparse()}
\usage{
v(...)
}
\arguments{
\item{...}{Variable input in any accepted \code{vparse()} form}
}
\value{
A character vector of variable names
}
\description{
A short and convenient alias for \code{vparse()}. Accepts unquoted names, character vectors, or a formula.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linest_compute.R
\name{linest}
\alias{linest}
\alias{linest.lm}
\alias{linest.glm}
\alias{linest.geeglm}
\alias{linest.lmerMod}
\alias{linest.merMod}
\alias{linest.default}
\alias{confint.linest_class}
\alias{coef.linest_class}
\alias{summary.linest_class}
\title{Compute linear estimates}
\usage{
linest(object, L = NULL, level = 0.95, ...)

\method{confint}{linest_class}(object, parm, level = 0.95, ...)

\method{coef}{linest_class}(object, ...)

\method{summary}{linest_class}(object, ...)
}
\arguments{
\item{object}{Model object}

\item{L}{Either \code{NULL} or a matrix with p columns where p is
the number of parameters in the systematic effects in the
model. If \code{NULL} then \code{L} is taken to be the p times
p identity matrix}

\item{level}{The level of the (asymptotic) confidence interval.}

\item{\dots}{Additional arguments; currently not used.}

\item{parm}{Specification of the parameters estimates for which
confidence intervals are to be calculated.}

\item{confint}{Should confidence interval appear in output.}
}
\value{
A dataframe with results from computing the contrasts.
}
\description{
Compute linear estimates, i.e. \code{L \%*\% beta} for a range of models. One example of
linear estimates is population means (also known as LSMEANS).
}
\examples{

## Make balanced dataset
dat.bal <- expand.grid(list(AA=factor(1:2), BB=factor(1:3), CC=factor(1:3)))
dat.bal$y <- rnorm(nrow(dat.bal))

## Make unbalanced dataset
#   'BB' is nested within 'CC' so BB=1 is only found when CC=1
#   and BB=2,3 are found in each CC=2,3,4
dat.nst <- dat.bal
dat.nst$CC <-factor(c(1,1,2,2,2,2,1,1,3,3,3,3,1,1,4,4,4,4))

mod.bal  <- lm(y ~ AA + BB * CC, data=dat.bal)
mod.nst  <- lm(y ~ AA + BB : CC, data=dat.nst)

L <- LE_matrix(mod.nst, effect=c("BB", "CC"))
linest( mod.nst, L )

}
\seealso{
\code{\link{LSmeans}}, \code{\link{LE_matrix}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelling.R
\name{response}
\alias{response}
\title{Get response variable from model}
\usage{
response(object)
}
\arguments{
\item{object}{lm or glm object}
}
\description{
Get response variable from model
}
\examples{
data(cars)
lm1 <- lm(dist ~ speed + I(speed^2), data=cars)
lm1 |> response() |> head()
cars <- cars |> add_pred(lm1)
cars |> head()
cars <- cars |> add_resid(lm1)
cars 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lag_data.R
\name{lag_data}
\alias{lag_data}
\title{Construct lagged regressors and aligned response}
\usage{
lag_data(
  formula = NULL,
  data = NULL,
  y = NULL,
  x = NULL,
  lags = 0,
  include_intercept = TRUE,
  preserve_ts = FALSE
)
}
\arguments{
\item{formula}{An optional model formula. The left-hand side is
taken as the response and the right-hand side as the regressor(s).
Only one non-intercept regressor is currently allowed when
lagging is used.}

\item{data}{Optional data frame in which to evaluate \code{formula}.}

\item{y}{Optional numeric or time series object with the response.
Ignored if \code{formula} is supplied.}

\item{x}{Optional numeric vector with the regressor to be lagged.
Ignored if \code{formula} is supplied. If \code{x} is not \code{NULL}, it must
have the same length as \code{y} (when \code{y} is supplied).}

\item{lags}{Integer vector of lags to include. The default \code{lags = 0}
means that only the contemporaneous regressor is used. For example,
\code{lags = 0:2} gives columns corresponding to (x(t), x(t-1), x(t-2)).}

\item{include_intercept}{Logical; if \code{TRUE} (default) an intercept
column named \code{"Intercept"} is prepended to the design matrix.}

\item{preserve_ts}{Logical; if \code{TRUE} and \code{y} is a \code{ts} object, the
returned \code{y} will also be a \code{ts} object obtained via
\code{\link[stats]{window}} starting at the first retained time
point. Otherwise, \code{y} is returned as a numeric vector.}
}
\value{
A list with components
\describe{
\item{y}{The aligned response vector. If \code{preserve_ts = TRUE}
and \code{y} was a \code{ts} object, then this is a \code{ts} object; otherwise
a numeric vector.}
\item{X}{The design matrix of lagged regressors, including an
intercept column if \code{include_intercept = TRUE}. If \code{x} is \code{NULL},
then \code{X} is \code{NULL}.}
\item{rows}{Integer vector of row indices from the original data
that are retained after lagging.}
\item{max_lag}{The maximum lag used, i.e. \code{max(lags)}.}
}
}
\description{
Helper for building lagged design matrices for time series or
longitudinal data. The function aligns a response vector \code{y}
and a single regressor \code{x} (possibly with several lags) and
returns a list with the cleaned response, the design matrix and
the row indices used.

You can either:
\itemize{
\item supply \code{y} and \code{x} directly, or
\item use a formula of the form \code{y ~ x} together with \code{data}.
}
}
\details{
When using the formula interface, the function creates a model frame
internally. The first column is taken as the response. All remaining
columns are considered regressors. If the right-hand side contains
only an intercept, \code{x} is set to \code{NULL} and no lagged regressors are
constructed.

If multiple regressors are present on the right-hand side, they are
combined into a matrix, but in the current implementation this is
only allowed when no lagging is requested (i.e., \code{lags} is \code{0}).
Attempting to lag multiple regressors will result in an error.

For a given vector of lags, a lag matrix is built with one column
per lag. The column names are \code{"lag0"}, \code{"lag1"}, etc., corresponding
to the entries in \code{lags}. Rows which involve undefined values (due to
lagging) are dropped from both \code{y} and \code{X}. The indices of the rows
kept from the original series are returned in the \code{rows} component.
}
\examples{
## ------------------------------------------------------------
## 1. Basic usage with y/x interface
## ------------------------------------------------------------
set.seed(123)
n  <- 10
y  <- rnorm(n)
x  <- rnorm(n)

## Use current and one lag of x
ld1 <- lag_data(y = y, x = x, lags = 0:1)
ld1$y                # aligned response
ld1$X                # design matrix (Intercept, lag0, lag1)
ld1$rows             # indices retained


## ------------------------------------------------------------
## 2. Formula interface with a data frame
## ------------------------------------------------------------
dat <- data.frame(
  y = rnorm(20),
  x = rnorm(20)
)

## Use y ~ x with one lag
ld2 <- lag_data(y ~ x, data = dat, lags = 0:2)
head(ld2$X)
length(ld2$y)


## ------------------------------------------------------------
## 3. Using the result in a regression / ARX setting
## ------------------------------------------------------------
## Here we regress y_t on an intercept and lagged x's
ld3 <- lag_data(y = y, x = x, lags = 0:2)

## Remove intercept column when passing to lm()
fit_lm <- lm(ld3$y ~ ld3$X[, -1])
coef(fit_lm)


## ------------------------------------------------------------
## 4. Time series example with preserve_ts = TRUE
## ------------------------------------------------------------
y_ts <- Nile

## Regress Nile flow on its own lag-1 (simple ARX-like setup)
ld4 <- lag_data(y = y_ts, x = y_ts, lags = 1, preserve_ts = TRUE)

## y is now a ts object starting at the appropriate time
start(y_ts)
start(ld4$y)

## Fit an AR(1) model with lagged Nile as xreg
## (intercept in X, lag in the second column)
fit_arx <- stats::arima(ld4$y,
                        order = c(1, 0, 0),
                        xreg  = ld4$X[, -1, drop = FALSE])
fit_arx

}
\seealso{
\code{\link[stats]{lag}}, \code{\link[stats]{embed}},
and \code{\link[stats]{ts}} for related time series utilities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_stability.r
\name{cv_glm_fitlist}
\alias{cv_glm_fitlist}
\title{Cross-validation for list of glm objects}
\usage{
cv_glm_fitlist(data., fit_list, K = 10)
}
\arguments{
\item{data.}{A data frame}

\item{fit_list}{A list of glm objects}

\item{K}{Number of folds}
}
\description{
Cross-validation for list of glm objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.r
\name{interaction-plot}
\alias{interaction-plot}
\alias{interaction_plot}
\title{Two-way interaction plot}
\usage{
interaction_plot(.data, .formula, interval = "conf.int")
}
\arguments{
\item{.data}{A data frame}

\item{.formula}{A formula of the form \code{y ~ x1 + x2}}

\item{interval}{Either \code{conf.int}, \code{boxplot} or \code{none}}
}
\description{
Plots the mean of the response for
two-way combinations of factors, thereby illustrating possible
interactions.
}
\note{
This is a recent addition to the package and is subject to change.
}
\examples{

income$educf <- cut(income$educ, breaks=3)
income |> interaction_plot(inc ~ race + educf)
income |> interaction_plot(inc ~ race + educf, interval="conf.int")
income |> interaction_plot(inc ~ race + educf, interval="boxplot")
income |> interaction_plot(inc ~ race + educf, interval="none")

}
\concept{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_split_byrow_bycol.R
\name{split_byrow_bycol}
\alias{split_byrow_bycol}
\alias{split_bycol}
\alias{split_byrow}
\title{Split matrix or dataframe into list}
\usage{
split_bycol(x, idx = NULL, as.list = FALSE)

split_byrow(x, idx = NULL)
}
\arguments{
\item{x}{Matrix or dataframe.}

\item{idx}{Index to split by. If NULL, split by columns or rows.}

\item{as.list}{If TRUE, return list of dataframes. If FALSE, return list of matrices.}
}
\description{
Split matrix or dataframe into list by columns or by rows
}
\examples{
x <- mtcars[1:3, 1:6]
x  |> split_bycol()
x  |> split_bycol(as.list=TRUE)
x  |> split_bycol(as.list=FALSE)
x  |> split_bycol(idx=c(1,1,1,2,2,3,3,3))
## x  |> split_bycol(idx=c(1,1,7,2,2,3,3,3)) ## Gives error

x <- mtcars[1:6, 1:6]
x  |> split_byrow()
x  |> split_byrow(idx=c(1,1,2,2))

m <- as.matrix(x)
u <- x |> split_byrow(idx=c(1,1,2,2))
y <- m |> split_byrow(idx=c(1,1,2,2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_orderBy.R
\name{by-order}
\alias{by-order}
\alias{order_by}
\alias{orderBy}
\title{Ordering (sorting) rows of a data frame}
\usage{
order_by(data, formula)

orderBy(formula, data)
}
\arguments{
\item{data}{A dataframe}

\item{formula}{The right hand side of a formula}
}
\value{
The ordered data frame
}
\description{
Ordering (sorting) rows of a data frame by the certain
variables in the data frame. This function is essentially a
wrapper for the \code{order()} function - the important
difference being that variables to order by can be given by a
model formula.
}
\details{
The sign of the terms in the formula determines whether
sorting should be ascending or decreasing; see examples below
}
\examples{

orderBy(~ conc + Treatment, CO2)
## Sort decreasingly by conc
orderBy(~ - conc + Treatment, CO2)
## Same as:
order_by(CO2, c("conc", "Treatment"))
order_by(CO2, c("-conc", "Treatment"))
}
\seealso{
\code{\link{transformBy}}, \code{\link{transform_by}}, \code{\link{splitBy}}, \code{\link{split_by}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk} and Kevin
Wright
}
\concept{byfunction}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esticon.R
\name{esticon}
\alias{esticon}
\alias{coef.esticon_class}
\alias{summary.esticon_class}
\alias{confint.esticon_class}
\alias{vcov.esticon_class}
\title{Contrasts for lm, glm, lme, and geeglm objects}
\usage{
esticon(obj, L, beta0, conf.int = TRUE, level = 0.95, joint.test = FALSE, ...)

\method{coef}{esticon_class}(object, ...)

\method{summary}{esticon_class}(object, ...)

\method{confint}{esticon_class}(object, parm, level = 0.95, ...)

\method{vcov}{esticon_class}(object, ...)
}
\arguments{
\item{obj}{Regression object (of type lm, glm, lme, geeglm).}

\item{L}{Matrix (or vector) specifying linear functions of the regression
parameters (one
linear function per row).  The number of columns must match the number of
fitted regression parameters in the model. See 'details' below.}

\item{beta0}{A vector of numbers}

\item{conf.int}{TRUE}

\item{level}{The confidence level}

\item{joint.test}{Logical value. If TRUE a 'joint' Wald test for the
hypothesis L beta = beta0 is made. Default is that the 'row-wise' tests are
made, i.e. (L beta)i=beta0i.  If joint.test is TRUE, then no confidence
interval etc. is calculated.}

\item{\dots}{Additional arguments; currently not used.}

\item{object}{An \code{esticon_class} object.}

\item{parm}{a specification of which parameters are to be given
confidence intervals, either a vector of numbers or a vector
of names.  If missing, all parameters are considered.}
}
\value{
Returns a matrix with one row per linear function.  Columns contain
estimated coefficients, standard errors, t values, degrees of freedom,
two-sided p-values, and the lower and upper endpoints of the 1-alpha
confidence intervals.
}
\description{
Computes linear functions (i.e. weighted sums) of the
estimated regression parameters. Can also test the hypothesis,
that such a function is equal to a specific value.
}
\details{
Let the estimated parameters of the model be
\deqn{\beta_1, \beta_2, \dots, \beta_p}

A linear function of the estimates is of the form \deqn{l=\lambda_1
    \beta_1+\lambda_2 \beta_2+ \dots+\lambda_p \beta_p} where
\eqn{\lambda_1, \lambda_2, \dots,\lambda_p} is specified by the
user.

The esticon function calculates l, its standard error and by default also a
95 pct confidence interval.  It is sometimes of interest to test the
hypothesis \eqn{H_0: l=\beta_0} for some value \eqn{\beta_0}
given by the user. A test is provided for the hypothesis \eqn{H_0:
l=0} but other values of \eqn{\beta_0} can be specified.

In general, one can specify r such linear functions at one time by
specifying L to be an \eqn{r\times p} matrix where each row consists
of p numbers \eqn{\lambda_1,\lambda_2,\dots, \lambda_p}. Default is
then that \eqn{\beta_0} is a p vector of 0s but other values can be
given.

It is possible to test simultaneously that all specified linear functions
are equal to the corresponding values in \eqn{\beta_0}.

For computing contrasts among levels of a single factor, 'contrast.lm' may
be more convenient.
}
\examples{

data(iris)
lm1  <- lm(Sepal.Length ~ Sepal.Width + Species + Sepal.Width : Species, data=iris)
## Note that the setosa parameters are set to zero
coef(lm1)

## Estimate the intercept for versicolor
lambda1 <- c(1, 0, 1, 0, 0, 0)
esticon(lm1, L=lambda1)

## Estimate the difference between versicolor and virgica intercept
## and test if the difference is 1
lambda2 <- c(0, 1, -1, 0, 0, 0)
esticon(lm1, L=lambda2, beta0=1)

## Do both estimates at one time
esticon(lm1, L=rbind(lambda1, lambda2), beta0=c(0, 1))

## Make a combined test for that the difference between versicolor and virgica intercept
## and difference between versicolor and virginica slope is zero:
lambda3 <- c(0, 0, 0, 0, 1, -1)
esticon(lm1, L=rbind(lambda2, lambda3), joint.test=TRUE)

# Example using esticon on coxph objects (thanks to Alessandro A. Leidi).
# Using dataset 'veteran' in the survival package
# from the Veterans' Administration Lung Cancer study

if (require(survival)){
data(veteran)
sapply(veteran, class)
levels(veteran$celltype)
attach(veteran)
veteran.s <- Surv(time, status)
coxmod <- coxph(veteran.s ~ age + celltype + trt, method='breslow')
summary(coxmod)

# compare a subject 50 years old with celltype 1
# to a subject 70 years old with celltype 2
# both subjects on the same treatment
AvB <- c(-20, -1, 0, 0, 0)

# compare a subject 40 years old with celltype 2 on treat=0
# to a subject 35 years old with celltype 3 on treat=1
CvB <- c(5, 1, -1, 0, -1)

est <- esticon(coxmod, L=rbind(AvB, CvB))
est
##exp(est[, c(2, 7, 8)])
}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{shoes}
\alias{shoes}
\alias{shoes_long}
\title{Wear of shoes}
\format{
This data frame contains:
\describe{
\item{A:}{Wear, material A}
\item{B:}{Wear, material B}
\item{boy:}{Id of boy}
\item{footA:}{The foot with material A}
}

An object of class \code{data.frame} with 10 rows and 4 columns.

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 20 rows and 4 columns.
}
\usage{
shoes

shoes_long
}
\description{
Wear of soles of shoes of materials A and B for one foot each for of ten boys.
}
\details{
The shoes data are measurements of the amount wear of the
soles of shoes worn by 10 boys. The soles were made to two
different synthetic materials, a standard material A and a
cheaper material B.
}
\references{
Box, Hunter, Hunter (2005) Statistics for Experimenters, 2nd edition Wiley, p. 81.
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_transformBy.R
\name{by-transform}
\alias{by-transform}
\alias{transform_by}
\alias{transformBy}
\title{Function to make groupwise transformations}
\usage{
transform_by(data, formula, ...)

transformBy(formula, data, ...)
}
\arguments{
\item{data}{A data frame}

\item{formula}{A formula with only a right hand side, see examples below}

\item{\dots}{Further arguments of the form tag=value}
}
\value{
The modified value of the dataframe data.
}
\description{
Function to make groupwise transformations of data by applying
the transform function to subsets of data.
}
\details{
The ... arguments are tagged vector expressions, which are evaluated
in the data frame data. The tags are matched against names(data), and for
those that match, the value replace the corresponding variable in data,
and the others are appended to data.
}
\examples{
 
data(dietox)
transformBy(~Pig, data=dietox, minW=min(Weight), maxW=max(Weight), 
    gain=diff(range(Weight)))

}
\seealso{
\code{\link{orderBy}}, \code{\link{order_by}}, \code{\link{summaryBy}}, \code{\link{summary_by}},
\code{\link{splitBy}}, \code{\link{split_by}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{byfunction}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{crickets}
\alias{crickets}
\title{crickets data}
\format{
This data frame contains:
\describe{
\item{species:}{Species, (exis, nius), see details}
\item{temp:}{temperature}
\item{pps:}{pulse per second}
}
}
\usage{
crickets
}
\description{
Mating songs of male tree crickets.
}
\details{
Walker (1962) studied the mating songs of male tree crickets. Each
wingstroke by a cricket produces a pulse of song, and females may
use the number of pulses per second to identify males of the
correct species. Walker (1962) wanted to know whether the chirps
of the crickets Oecanthus exclamationis (abbreviated exis) and
Oecanthus niveus (abbreviated nius) had different pulse rates. See
the biostathandbook for details. (The
abbreviations are made from the the first two and last two letters
of the species.) Walker measured the pulse rate of the crickets
(variable \code{pps}) at a variety of temperatures (\code{temp}):
}
\examples{
	
data(crickets)
coplot(pps ~ temp | species, data=crickets) 
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_firstobs_lastobs.R
\name{firstlastobs}
\alias{firstlastobs}
\alias{lastobs}
\alias{firstobs}
\alias{firstobs.default}
\alias{lastobs.default}
\alias{firstobs.formula}
\alias{lastobs.formula}
\title{Locate the index of the first/last unique value}
\usage{
lastobs(x, ...)

firstobs(x, ...)

\method{lastobs}{default}(x, ...)

\method{firstobs}{default}(x, ...)

\method{lastobs}{formula}(formula, data = parent.frame(), ...)

\method{firstobs}{formula}(formula, data = parent.frame(), ...)
}
\arguments{
\item{x}{A vector}

\item{\dots}{Currently not used}

\item{formula}{A formula (only the first term is used, see 'details').}

\item{data}{A data frame}
}
\value{
A vector.
}
\description{
Locate the index of the first/last unique value in i) a vector
or of a variable in a data frame.
}
\details{
If writing ~a + b + c as formula, then only a is considered.
}
\examples{
x <- c(rep(1, 5), rep(2, 3), rep(3, 7), rep(1, 4))
firstobs(x)
lastobs(x)
data(dietox)
firstobs(~Pig, data=dietox)
lastobs(~Pig, data=dietox)

}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{data_handling}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{math_teachers}
\alias{math_teachers}
\title{Height of math teachers}
\format{
:
\describe{
\item{height}{Height in centimeters}
\item{sex}{Male or female}
}
}
\usage{
math_teachers
}
\description{
Height of a sample of math teachers in Danish high
schools collected at a continued education day at Mariager Fjord Gymnasium in 2019.
}
\examples{

aggregate(height ~  sex, data=math_teachers, FUN=mean)
aggregate(height ~  sex, data=math_teachers, FUN=function(x) {c(mean=mean(x), sd=sd(x))})

}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{which.maxn}
\alias{which.maxn}
\alias{which.minn}
\title{Where are the n largest or n smallest elements in a numeric vector ?}
\usage{
which.maxn(x, n = 1)
}
\arguments{
\item{x}{numeric vector}

\item{n}{integer >= 1}
}
\value{
A vector of length at most n with the indices of the n largest /
smaller elements. NAs are discarded and that can cause the vector to be
smaller than n.
}
\description{
Determines the locations, i.e., indices of the n largest or n smallest
elements of a numeric vector.
}
\examples{

x <- c(1:4, 0:5, 11, NA, NA)
ii <- which.minn(x, 5)

x <- c(1, rep(NA,10), 2)
ii <- which.minn(x, 5)

}
\seealso{
\code{\link{which.max}}, \code{\link{which.min}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{beets}
\alias{beets}
\title{beets data}
\format{
The format is: chr "beets"
}
\usage{
beets
}
\description{
Yield and sugar percentage in sugar beets from a split plot
experiment.  Data is obtained from a split plot experiment. There are 3
blocks and in each of these the harvest time defines the "whole plot" and
the sowing time defines the "split plot". Each plot was 25 square meters and
the yield is recorded in kg. See 'details' for the experimental layout.
}
\details{
\preformatted{  
Experimental plan
Sowing times            1        4. april
                        2       12. april
                        3       21. april
                        4       29. april
                        5       18. may
Harvest times           1        2. october
                        2       21. october
Plot allocation:
               Block 1     Block 2     Block 3
            +-----------|-----------|-----------+
      Plot  | 1 1 1 1 1 | 2 2 2 2 2 | 1 1 1 1 1 | Harvest time
       1-15 | 3 4 5 2 1 | 3 2 4 5 1 | 5 2 3 4 1 | Sowing time
            |-----------|-----------|-----------|
      Plot  | 2 2 2 2 2 | 1 1 1 1 1 | 2 2 2 2 2 | Harvest time
      16-30 | 2 1 5 4 3 | 4 1 3 2 5 | 1 4 3 2 5 | Sowing time
            +-----------|-----------|-----------+  
}
}
\examples{
data(beets)

beets$bh <- with(beets, interaction(block, harvest))
summary(aov(yield ~ block + sow + harvest + Error(bh), beets))
summary(aov(sugpct ~ block + sow + harvest + Error(bh), beets))

}
\references{
Ulrich Halekoh, Søren Højsgaard (2014)., A Kenward-Roger
Approximation and Parametric Bootstrap Methods for Tests in Linear Mixed
Models - The R Package pbkrtest., Journal of Statistical Software,
58(10), 1-30., \url{https://www.jstatsoft.org/v59/i09/}
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vtools.R
\name{vrename}
\alias{vrename}
\title{Rename columns in a data frame}
\usage{
vrename(df, rename_map)
}
\arguments{
\item{df}{A data frame}

\item{rename_map}{A named character vector (old_name = new_name)}
}
\value{
A data frame with renamed columns
}
\description{
Rename columns in a data frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{fatacid}
\alias{fatacid}
\title{Fish oil in pig food}
\format{
A dataframe.
}
\usage{
fatacid
}
\description{
Fish oil in pig food
}
\details{
A fish oil fatty acid \code{X14} has been added in
different concentrations to the food for pigs in a
study. Interest is in studying how much of the fatty acid can
be found in the tissue. The concentrations of \code{x14} in the
food are \code{verb+dose+=\{0.0, 4.4, 6.2, 9.3\}}.

The pigs are fed with this food until their weight is 60 kg.  From
thereof and until they are slaughtered at 100kg, their food does
not contain the fish oil.
At 60kg (sample=1) and 100kg (sample=2) muscle
biopsies are made and the concentration of x14 is
determined.
Measurements on the same pig are correlated, and pigs are additionally
related through litters.
}
\references{
Data courtesy of Charlotte Lauridsen, Department of
Animal Science, Aarhus University, Denmark.
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vtools.R
\name{vselect}
\alias{vselect}
\title{Select columns from a data frame using flexible input}
\usage{
vselect(df, ...)
}
\arguments{
\item{df}{A data frame}

\item{...}{Variable input (unquoted, character vector, or formula)}
}
\value{
A data frame with selected columns
}
\description{
Select columns from a data frame using flexible input
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_coerce_expr_to_function.R
\name{expr_to_fun}
\alias{expr_to_fun}
\title{Convert expression into function object.}
\usage{
expr_to_fun(expr_, order = NULL, vec_arg = FALSE)
}
\arguments{
\item{expr_}{R expression.}

\item{order}{desired order of function argument.}

\item{vec_arg}{should the function take vector valued argument.}
}
\description{
Convert expression into function object.
}
\examples{

ee  <- expression(b1 + (b0 - b1)*exp(-k*x) + b2*x)
ff1 <- expr_to_fun(ee)
formals(ff1)

ff2 <- expr_to_fun(ee, vec_arg=TRUE)
formals(ff2)
formals(ff2)$length_parm
formals(ff2)$names_parm |> eval()

ee <- expression(matrix(c(x1+x2, x1-x2, x1^2+x2^2, x1^3+x2^3), nrow=2))
ff1 <- expr_to_fun(ee)
ff2 <- expr_to_fun(ee, vec_arg=TRUE)

formals(ff2)
formals(ff2)$length_parm
formals(ff2)$names_parm |> eval()

}
\concept{functional}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_stability.r
\name{get_formulas}
\alias{get_formulas}
\title{Get formulas from model_stability_glm_class object}
\usage{
get_formulas(object, unique = TRUE, text = FALSE)
}
\arguments{
\item{object}{A model_stability_glm_class object}

\item{unique}{If TRUE, return unique models}

\item{text}{If TRUE, return text (rather than formula).}
}
\description{
Get formulas from model_stability_glm_class object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_splitBy.R
\name{by-split}
\alias{by-split}
\alias{split_by}
\alias{splitBy}
\alias{head.splitByData}
\alias{tail.splitByData}
\alias{split_by.legacy}
\alias{splitBy.legacy}
\title{Split a data frame into groups defined by variable(s)}
\usage{
split_by(data., ..., omit = TRUE)

splitBy(formula, data, omit = TRUE)

\method{head}{splitByData}(x, n = 6L, ...)

\method{tail}{splitByData}(x, n = 6L, ...)

split_by.legacy(data, formula, drop = TRUE)

splitBy.legacy(formula, data = parent.frame(), drop = TRUE)
}
\arguments{
\item{data.}{A data frame (or tibble) to split}

\item{...}{Variables defining the groups}

\item{omit}{If TRUE (default), group-defining variables are omitted in each split group}

\item{formula}{A right hand sided formula (for the old interface)}

\item{data}{A data frame  (for the old interface)}

\item{x}{A splitByData object.}

\item{n}{An integer vector.}

\item{drop}{Obsolete}
}
\value{
An object of class \"splitByData\" (a named list with group attributes)
}
\description{
Split a dataframe according to the levels of variables
in the dataframe. Uses vparse() to interpret flexible input.
}
\examples{

split_by(CO2, ~Treatment+Type)
split_by(CO2, Treatment, Type)
split_by(CO2, c("Treatment", "Type"))
split_by(CO2, "Treatment", "Type")

x <- split_by(CO2, "Treatment", "Type")
head(x, 3)
tail(x, 3)

## Via wrapper:
foo2 <- function(x) {
 x <- rlang::enquo(x)
 split_by(CO2, !!x)
}
foo2(~Treatment)

## The "Old" interface
splitBy(~Treatment + Type, CO2)
splitBy(~Treatment + Type, data=CO2)
splitBy(c("Treatment", "Type"), data=CO2)

}
\seealso{
\code{\link{orderBy}}, \code{\link{order_by}},
\code{\link{summaryBy}}, \code{\link{summary_by}},
\code{\link{transformBy}}, \code{\link{transform_by}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{byfunction}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{nir_milk}
\alias{nir_milk}
\alias{NIRmilk}
\title{Near infra red light (NIR) measurements in milk}
\format{
Data comes in two formats:

nir_milk: A list with two components
\itemize{
\item x Datafrane with infra red light
amount at different wavelengths (column names are the wavelengths;
just remove the leading X).
\item y Datafrane with response variables
fat, protein, lactose and dm (drymatter)
}

NIRmilk: This data frame contains 17 rows and 158 columns.
\itemize{
\item The first column is the sample number.
\item The columns named in the form \code{Xw} contains
the transmittance (fraction of electromagnetic power)
transmittance through the sample at wavelength \code{w}.
\item The response variables are fat, protein, lactose and dm (dry
matter).
}

An object of class \code{data.frame} with 17 rows and 158 columns.
}
\usage{
nir_milk

NIRmilk
}
\description{
Near infra red light (NIR) measurements are made at
152 wavelengths on 17 milk samples.  While milk runs through a
glass tube, infra red light is sent through the tube and the
amount of light passing though the tube is measured at
different wavelengths.  Each milk sample was additionally
analysed for fat, lactose, protein and dry matter.
}
\examples{
	
data(nir_milk)
data(NIRmilk)

}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.r
\name{plot_lm}
\alias{plot_lm}
\title{Plot linear model object}
\usage{
plot_lm(lm_fit, format = "2x2", global_aes = NULL)
}
\arguments{
\item{lm_fit}{An object of class 'lm'}

\item{format}{The format of the plot (or a list of plots if format is "list")}

\item{global_aes}{Currently no effect.}
}
\description{
Plot linear model object
}
\examples{
data(income)
m1 <- lm(inc ~ race + educ, data=income)
plot_lm(m1)
plot_lm(m1, "2x2")
plot_lm(m1, "1x4")
plot_lm(m1, "4x1")
plot_lm(m1, "list")

}
\concept{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{child_growth}
\alias{child_growth}
\title{Berkeley Growth Study data}
\format{
\describe{
\item{gender}{Gender of child}
\item{age}{Age at time of data recording}
\item{subject}{Identification for each child}
\item{height}{Height of child}
}
}
\usage{
child_growth
}
\description{
dataframe with heights of 39 boys and 54 girls from
age 1 to 18 and the ages at which they were collected.
}
\details{
Notice that the ages are not equally spaced. Data are
taken from the fda package (growth) but put in long format
here.
}
\references{
\if{html}{\out{<div class="sourceCode">}}\preformatted{Ramsay, James O., Hooker, Giles, and Graves, Spencer (2009),
_Functional data analysis with R and Matlab_, Springer, New York.

Ramsay, James O., and Silverman, Bernard W. (2005), _Functional
Data Analysis, 2nd ed._, Springer, New York.

Ramsay, James O., and Silverman, Bernard W. (2002), _Applied
Functional Data Analysis_, Springer, New York.

Tuddenham, R. D., and Snyder, M. M. (1954) "Physical growth of
California boys and girls from birth to age 18", _University of
California Publications in Child Development_, 1, 183-364.
}\if{html}{\out{</div>}}
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{NIRmilk}
\alias{NIRmilk}
\title{NIRmilk}
\format{
This data frame contains 17 rows and 158 columns.  The
first column is the sample number.  The columns \code{Xklm} contains
the transmittance (fraction of electromagnetic power)
transmittance through the sample at wavelength \code{klm}.  The
response variables are fat, protein, lactose and dm (dry
matter).
}
\usage{
NIRmilk
}
\description{
Near infra red light (NIR) measurements are made at
152 wavelengths on 17 milk samples.  While milk runs through a
glass tube, infra red light is sent through the tube and the
amount of light passing though the tube is measured at
different wavelengths.  Each milk sample was additionally
analysed for fat, lactose, protein and dry matter.
}
\examples{
	
data(NIRmilk)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_on_forecast.R
\name{transform_forecast}
\alias{transform_forecast}
\title{Transform forecasts from model scale to data scale by simulation}
\usage{
transform_forecast(fc_object, trans_fun, nsim = 2000L, level = 95, y0 = 1)
}
\arguments{
\item{fc_object}{A \code{"forecast"} object (from the \pkg{forecast} package)
for a univariate time series, typically obtained via
\code{forecast::forecast()}.}

\item{trans_fun}{A function of the form \code{function(x, y0) ...}
that takes a numeric vector \code{x} on the model scale and a scalar
starting value \code{y0} on the output scale and returns a numeric vector
of the same length on the output scale.}

\item{nsim}{Integer; number of simulated future paths to use. Larger
values give smoother prediction intervals but take longer to compute.}

\item{level}{Numeric; prediction interval coverage in percent.}

\item{y0}{Numeric; starting value on the output scale used to
reconstruct the historical series from \code{fc_object$x}. For percentage
change models it is often natural to set \code{y0 = 1} and interpret
the resulting series as an index.}
}
\value{
A \code{"forecast"} object similar to \code{fc_object}, but with the components
\code{x}, \code{mean}, \code{lower}, and \code{upper} defined on the output (data) scale.
}
\description{
\code{transform_forecast()} takes a univariate time series forecast fitted on a
transformed (model) scale and produces a new forecast object on the
original data scale. This is done by simulating future paths on the
model scale, transforming each path with a user-supplied function, and
then computing pointwise means and prediction intervals on the
transformed scale.
}
\details{
The function assumes that \code{fc_object} is a \code{"forecast"} object as
produced by the \pkg{forecast} package, and that \code{fc_object$model}
supports \code{simulate()} with arguments \code{nsim} and \code{future}.

The transformation function \code{trans_fun} must have the form
\code{trans_fun(x, y0)}, where \code{x} is a numeric vector representing a path on
the model scale (for example log-values or percentage changes), and
\code{y0} is a scalar "starting value" on the output scale. The function must
return a numeric vector of the same length as \code{x} giving the
corresponding path on the output scale.

Internally, \code{transform_forecast()} first reconstructs a historical series on
the output scale by applying \code{trans_fun()} to \code{fc_object$x} with the
supplied \code{y0}. It then simulates \code{nsim} future paths from
\code{fc_object$model} on the model scale, transforms each path to the
output scale using \code{trans_fun()} with \code{y0} equal to the last value of
the reconstructed historical series, and finally computes the pointwise
mean and prediction intervals (of nominal coverage \code{level}) across
the simulated paths. The result is returned as a new \code{"forecast"} object
with \code{x}, \code{mean}, \code{lower}, and \code{upper} on the output scale.
}
\examples{
## Example 1: Log-transform of the Canadian lynx data
if (requireNamespace("forecast", quietly = TRUE)) {

  llynx   <- log(lynx)
  fit_log <- forecast::auto.arima(llynx)
  fc_log  <- forecast::forecast(fit_log, h = 20)
  forecast::autoplot(fc_log)
  ## transformation: log -> original scale
  trans_log <- function(z, y0) {
    exp(z)
  }
  fc_lynx <- transform_forecast(fc_log, trans_fun = trans_log,
                            nsim = 20, level = 95, y0 = 1)
  plot(fc_lynx)
  forecast::autoplot(fc_lynx) + ggplot2::theme_minimal()
}


\dontrun{
if (requireNamespace("forecast", quietly = TRUE)) {
## Example 2 (variation): CO2 series, log-transform
  lco2    <- log(co2)
  fit_co2 <- forecast::auto.arima(lco2)
  fc_log_co2 <- forecast::forecast(fit_co2, h = 24)
  forecast::autoplot(fc_log_co2)
  trans_log <- function(z, y0) exp(z)

  fc_co2 <- transform_forecast(fc_log_co2, trans_fun = trans_log,
                           nsim = 20, level = 95, y0 = 1)
  forecast::autoplot(fc_co2) + ggplot2::theme_minimal()
}

# ## Example 3: Percentage change in income (uschange$Income)
if (requireNamespace("forecast", quietly = TRUE) &&
    requireNamespace("fpp2", quietly = TRUE)) {
  income <- uschange[, "Income"]  # quarterly percentage changes (\%)

  ## transformation: pct change -> index with base 1
  trans_pct <- function(r, y0) {
    y0 * cumprod(1 + r / 100)
  }

  fit_pct <- forecast::auto.arima(income)
  fc_pct  <- forecast::forecast(fit_pct, h = 24)
  forecast::autoplot(fc_pct)
  fc_idx  <- transform_forecast(fc_pct, trans_fun = trans_pct,
                            nsim = 200, level = 95, y0 = 1)

  plot(fc_idx)
  forecast::autoplot(fc_idx) + ggplot2::theme_minimal()
 }
 }
 
}
\seealso{
\code{\link[forecast]{forecast}}, \code{\link[forecast]{auto.arima}},
\code{\link[stats]{simulate}}, \code{\link[stats]{ts}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linest_tidy.R
\name{tidy-linest}
\alias{tidy-linest}
\alias{tidy.linest_class}
\title{Tidy a linest object}
\usage{
\method{tidy}{linest_class}(x, conf.int = FALSE, conf.level = 0.95, ...)
}
\arguments{
\item{x}{A 'linest_class' object (produced by \code{linest} methods).}

\item{conf.int}{Should confidence intervals be added.}

\item{conf.level}{Desired confidence level.}

\item{\dots}{Additional arguments; currently not used.}
}
\description{
Tidy summarizes information about the components of the object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/time_since_event.R
\name{timeSinceEvent}
\alias{timeSinceEvent}
\title{Calculate "time since event" in a vector.}
\usage{
timeSinceEvent(yvar, tvar = seq_along(yvar))
}
\arguments{
\item{yvar}{A numerical or logical vector specifying the events}

\item{tvar}{An optional vector specifying time}
}
\value{
A dataframe with columns 'yvar', 'tvar', 'abs.tse' (absolute time
since nearest event), 'sign.tse' (signed time since nearest event) and
'run' (indicator of the time window around each event).
}
\description{
Calculate "time since event" in a vector.
}
\details{
Events are coded as 1 in numeric vector (and non-events are
coded with values different from 1). \code{timeSinceEvent} will give the
time since event (with and without sign). In a logical vector, events are
coded as TRUE and all non-events as FALSE.
}
\note{
NA's in yvar are converted to zeros.
}
\examples{

## Events:
yvar <- c(0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0,
          0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0)

## Plot results:
tse <- timeSinceEvent(yvar)
plot(sign.tse ~ tvar, data=tse, type="b")
grid()
rug(tse$tvar[tse$yvar==1], col=4, lwd=4)
points(scale(tse$run), col=tse$run, lwd=2)
lines(abs.tse + .2 ~ tvar, data=tse, type="b", col=3)

## Find times for which time since an event is at most 1:
tse$tvar[tse$abs <= 1]

yvar <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 
0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
)

tvar <- c(207, 208, 208, 208, 209, 209, 209, 209, 210, 210, 211, 211, 
211, 212, 213, 213, 214, 214, 215, 216, 216, 216, 216, 217, 217, 
217, 218, 218, 219, 219, 219, 219, 220, 220, 221, 221, 221, 221, 
222, 222, 222)

timeSinceEvent(yvar, tvar)



}
\seealso{
\code{\link{subSeq}}, \code{\link{rle}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelling.R
\name{add_resid}
\alias{add_resid}
\title{Add residuals of different types to dataframe}
\usage{
add_resid(data, model, var = "resid", type)
}
\arguments{
\item{data}{dataframe or tibble}

\item{model}{model object}

\item{var}{name of new variable in dataframe / tibble}

\item{type}{type of residual value}
}
\value{
dataframe / tibble
}
\description{
Add residuals of different types to dataframe
}
\examples{
data(cars)
lm1 <- lm(dist ~ speed + I(speed^2), data=cars)
lm1 |> response() |> head()
cars <- cars |> add_pred(lm1)
cars |> head()
cars <- cars |> add_resid(lm1)
cars 

}
\author{
Søren Højsgaard
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{renameCol}
\alias{renameCol}
\title{Rename columns in a matrix or a dataframe.}
\usage{
renameCol(indata, src, tgt)
}
\arguments{
\item{indata}{A dataframe or a matrix}

\item{src}{Source: Vector of names of columns in \code{indata} to be
renamed. Can also be a vector of column numbers.}

\item{tgt}{Target: Vector with corresponding new names in the
output.}
}
\value{
A dataframe if \code{indata} is a dataframe; a matrix in
\code{indata} is a matrix.
}
\description{
Rename columns in a matrix or a dataframe.
}
\examples{
renameCol(CO2, 1:2, c("kk", "ll"))
renameCol(CO2, c("Plant", "Type"), c("kk", "ll"))

# These fail - as they should:
# renameCol(CO2, c("Plant", "Type", "conc"), c("kk", "ll"))
# renameCol(CO2, c("Plant", "Type", "Plant"), c("kk", "ll"))

}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utitlities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{data_breastcancer}
\alias{data_breastcancer}
\alias{breastcancer}
\title{Gene expression signatures for p53 mutation status in 250 breast cancer
samples}
\format{
A data frame with 250 observations on 1001 variables. The
first 1000 columns are numerical variables; the last column
(named \code{code}) is a factor with levels \code{case} and
\code{control}.
}
\source{
Chris Holmes, \email{c.holmes@stats.ox.ac.uk}
}
\usage{
breastcancer
}
\description{
Perturbations of the p53 pathway are associated with more aggressive and
therapeutically refractory tumours. We preprocessed the data using Robust
Multichip Analysis (RMA). Dataset has been truncated to the 1000 most
informative genes (as selected by Wilcoxon test statistics) to simplify
computation. The genes have been standardized to have zero mean and unit
variance (i.e. z-scored).
}
\details{
The factor \code{code} defines whether there was a mutation in the p53
sequence (code=case) or not (code=control).
}
\examples{

data(breastcancer)
bc <- breastcancer
pairs(bc[,1:5], col=bc$code)

train <- sample(1:nrow(bc), 50)
table(bc$code[train])
\dontrun{
library(MASS)
z <- lda(code ~ ., data=bc, prior = c(1, 1) / 2, subset = train)
pc <- predict(z, bc[-train, ])$class
pc
bc[-train, "code"]
table(pc, bc[-train, "code"])
}

}
\references{
Miller et al (2005, PubMed
ID:16141321)
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_stability.r
\name{model_stability_glm}
\alias{model_stability_glm}
\title{Model stability for glm objects}
\usage{
model_stability_glm(
  data.,
  model,
  M = 10,
  method = c("resample", "subgroups"),
  mc.cores = getOption("mc.cores", 2L),
  ...
)
}
\arguments{
\item{data.}{A data frame}

\item{model}{A glm object}

\item{M}{Number of searches}

\item{method}{Method for generating data. Either \code{subgroup} like in
bootstrapping or \code{resample} by resampling the entire datasets
with replications.}

\item{mc.cores}{Number of cores, defaults to getOption("mc.cores", 2L)}

\item{...}{Additional arguments to be passed to \code{\link{step}}}
}
\value{
A list with several slots of information. The most
important ones:
\itemize{
\item \code{rhs_matrix} : A matrix representatation of
formulas for the different models. As many rows as there are
predictors, as many columns as there are different models (the
same model might be selected on several datasets.)
\item \code{model_freq} : How many time each model is selected.
\item \code{pred_freq} : How many time does each predictor appear in a
model.
\item \code{model_parm} : How many parameters are in each model.
}
}
\description{
A way of assessing variability in selected models
across different datasets.
}
\details{
From \code{data.}, \code{M} datasets are generated. Options are
\itemize{
\item "resamples": M datasets with resamples of data with replacement, i.e. bootstrap samples.
\item "subgroups": M datasets with subgroups each consisting of n-M randomly selected observations.
}

On each of these datasets, a stepwise selection
(using \code{step}) is performed.
}
\note{
This is a recent addition. Suggestions and comments are
welcome. The addition may be removed without warning.
}
\examples{

set.seed(1234)
dat <- personality[,1:20]
idx <- sample(1:nrow(dat), 0.6*nrow(dat))
train <- dat[idx, ]
test <- dat[-idx, ]
fit <- stats::glm(agreebl ~ ., data = train)

M <- 8

#stab <- doBy::model_stability_glm(data=train, model=fit, M=M, method="resample", mc.cores=1)
stab <- doBy::model_stability_glm(data=train, model=fit, M=M, method="subgroups", mc.cores=1)
stab

formula_list <- formula(stab)
fit_list <- formula(stab, fit=TRUE)

formula_list
stab

(cv.train <- doBy::cv_glm_fitlist(train, fit_list, K=5))

(cv.test <- doBy::cv_glm_fitlist(test, fit_list, K=5))
ylim <- range(c(cv.train, cv.test))
par(mfrow=c(1,2))
plot(stab$model_parm, cv.train, ylim=ylim)
plot(stab$model_parm, cv.test, ylim=ylim)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_section_fun.R
\name{set_default}
\alias{set_default}
\title{Set default values in a functions arguments}
\usage{
set_default(fun, nms, vls = NULL)
}
\arguments{
\item{fun}{A function whose arguments will get new default values.}

\item{nms}{Character vector of argument names, or something coercible to that
(e.g. a call to \code{v()}).}

\item{vls}{Optional vector or list of values to use as defaults.
If \code{nms} is a named list, \code{vls} can be \code{NULL}.}
}
\value{
A new function with updated default values in its formals.
}
\description{
\code{set_default()} takes a function and returns a new version with
updated default values for specified arguments.
}
\details{
This is useful when you want to programmatically create specialized
versions of a function with certain arguments preset to default
values.
\itemize{
\item The specified arguments will be moved to the end of the formal argument list in the returned function.
\item You can supply arguments as a named list or as separate names and values.
}
}
\examples{
## Simple example
f1 <- function(x, y, z) { x + y + z }

## Add defaults for x and y
set_default(f1, list(x = 1, y = 2))
# function (z, x = 1, y = 2) { x + y + z }

## Same using separate vectors of names and values
set_default(f1, c("x", "y"), c(1, 2))

## Works with v() style if supported
# set_default(f1, v(x, y), c(1, 2))

## Another example with more arguments
f2 <- function(a, b, c, d) { a + b + c + d }
set_default(f2, list(b = 10, d = 5))

}
\concept{functional}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vtools.R
\name{vmap}
\alias{vmap}
\title{Apply a function to each parsed variable name}
\usage{
vmap(.vars, .f)
}
\arguments{
\item{.vars}{Variables to parse}

\item{.f}{Function to apply to each name}
}
\value{
A list of results
}
\description{
Apply a function to each parsed variable name
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_lmBy.R
\name{by-lmby}
\alias{by-lmby}
\alias{lm_by}
\alias{lmBy}
\alias{coef.lmBy}
\alias{coef.summary_lmBy}
\alias{summary.lmBy}
\alias{fitted.lmBy}
\alias{residuals.lmBy}
\alias{getBy}
\title{List of lm objects with a common model}
\usage{
lm_by(data., formula., id = NULL, ...)

lmBy(formula., data., id = NULL, ...)
}
\arguments{
\item{data.}{A dataframe}

\item{formula.}{A linear model formula object of the form \code{y ~ x1 + ... + xn | g1 + ... + gm}.  In the formula object, \code{y}
represents the response, \verb{x1, ..., xn} the covariates, and the
grouping factors specifying the partitioning of the data
according to which different lm fits should be performed.}

\item{id}{A formula describing variables from data which are to be
available also in the output.}

\item{\dots}{Additional arguments passed on to \code{lm()}.}
}
\value{
A list of lm fits.
}
\description{
The data is split into strata according to the levels
of the grouping factors and individual lm fits are obtained for
each stratum.
}
\examples{

lm_lst <- lmBy(1 / uptake ~ log(conc) | Treatment, data=CO2)
coef(lm_lst)

fitted(lm_lst)
residuals(lm_lst)

summary(lm_lst)
coef(summary(lm_lst))
coef(summary(lm_lst), simplify=TRUE)

}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{byfunction}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_bquote_list.R
\name{bquote_fun_list}
\alias{bquote_fun_list}
\title{Backquote a list of functions}
\usage{
bquote_fun_list(fun_list)
}
\arguments{
\item{fun_list}{List of functions}
}
\description{
Backquote a list of functions
}
\examples{

## Evaluate a list of functions
f1 <- function(x){x + 1}
f2 <- function(x){x + 8}

f1_ <- set_default(f1, list(x=10))
f2_ <- set_default(f2, list(x=10))

f1_(); f2_()

fn_list  <- list(f1_, f2_)
fn_list_ <- bquote_fun_list(fn_list)

eval(fn_list[[1]])     ## No
sapply(fn_list, eval)  ## No

eval(fn_list_[[1]])    ## Yes
sapply(fn_list_, eval) ## Yes

}
\seealso{
\code{\link[base:bquote]{base::bquote()}}, \code{\link[=set_default]{set_default()}}, \code{\link[=section_fun]{section_fun()}}
}
\concept{functional}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functional_section_fun.R
\name{section_fun}
\alias{section_fun}
\alias{section_fun_sub}
\alias{section_fun_env}
\alias{get_section}
\alias{get_fun}
\title{Section a function and set default values in function}
\usage{
section_fun(fun, nms, vls = NULL, method = "args")

section_fun_sub(fun, nms, vls = NULL, envir = parent.frame())

section_fun_env(fun, nms, vls = NULL)

get_section(object)

get_fun(object)
}
\arguments{
\item{fun}{Function to be sectioned}

\item{nms}{Either a named list of the form name=value where each
name is the name of an argument of the function (in which case
\code{vls} is ignored) or a character vector of names of arguments.}

\item{vls}{A vector or list of values of the arguments}

\item{method}{One of the following: 1) "args" (default); based on
substituting fixed values into the function argument list as
default values). For backward compatibility can also be "def".
2) "body" for substituting fixed values into the function
body. For backward compatibility can also be "sub". 3) "env":
(for environment); using an auxillary argument for storing
sectioned values.}

\item{envir}{Environment}

\item{object}{An object from section_fun (a scaffold object).}
}
\value{
A new function: The input function \code{fun} but with certain
arguments fixed at specific values.
}
\description{
Section a functions domain by fixing certain
arguments of a function call.
}
\details{
Let E be a subset of the cartesian product X x Y where X
and Y are some sets. Consider a function f(x,y) defined on
E. Then for any x in X, the section of E defined by x (denoted
Ex) is the set of $y$s in Y such that (x, y) is in
E. Correspondingly, the section of f(x,y) defined by x is the
function $f_x$ defined on Ex given by $f_x(y)=f(x,y)$.

\code{section_fun} is a wrapper for calling \code{set_default} (default
method), \code{section_fun_env} or \code{section_fun_sub}. Notice that
creating a sectioned function with \code{section_fun_sub} can be
time consuming.
}
\examples{

f  <- function(x, y){x + y}

f_ <- section_fun(f, list(y = 10),    method="args") ## "def"" is default
f_ <- section_fun(f, nms="y", vls=10, method="args") ## SAME AS ABOVE
f_
f_(x=1)

f_ <- section_fun(f, list(y = 10),    method="body") ## 
f_ <- section_fun(f, nms="y", vls=10, method="body") ## SAME AS ABOVE
f_
f_(x=1)

f_ <- section_fun(f, list(y = 10),    method="env")
f_ <- section_fun(f, nms="y", vls=10, method="env") ## SAME AS ABOVE
f_
f_(x=1)
get_section(f_)
get_fun(f_)

 
## With more complicated values:
g <- function(A, B) {
  A + B
}
g_ <- section_fun(g, list(A = matrix(1:4, nrow=2)))
g_ <- section_fun(g, "A", list(matrix(1:4, nrow=2)))
g_(diag(1, 2))

g_ <- section_fun(g, list(A = matrix(1:4, nrow=2)))

## Using built in function
set.seed(123)
rnorm5 <- section_fun(rnorm, list(n=5)) 
rnorm5(0, 1)

set.seed(123)
rnorm(5)


}
\seealso{
\code{\link[=bquote_fun_list]{bquote_fun_list()}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk} based on code
adapted from the curry package.
}
\concept{functional}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{data_cad}
\alias{data_cad}
\alias{cad1}
\alias{cad2}
\title{Coronary artery disease data}
\format{
A data frame with 236 observations on the following 14 variables.

\describe{
\item{\code{Sex}}{Sex; a factor with levels \code{Female} \code{Male}}

\item{\code{AngPec}}{Angina pectoris (chest pain attacks); a
factor with levels \code{Atypical} \code{None} \code{Typical}}

\item{\code{AMI}}{Acute myocardic infarct; a factor with
levels \code{Definite} \code{NotCertain}}

\item{\code{QWave}}{A reading from an electrocardiogram; a
factor with levels \code{No} \code{Yes}; Yes means pathological and is a sign of previous myocardial infarction. }

\item{\code{QWavecode}}{a factor with levels \code{Nonusable}
\code{Usable}. An assesment of whether QWave is reliable.}

\item{\code{STcode}}{a factor with levels
\code{Nonusable} \code{Usable}. An assesment of whether STchange is reliable.}

\item{\code{STchange}}{A reading from an electrocardiogram; a factor
with levels \code{No} \code{Yes}. An STchange indicates a blockage of the coronary artery.}

\item{\code{SuffHeartF}}{Sufficient heart frequency; a factor with levels \code{No}, \code{Yes}}

\item{\code{Hypertrophi}}{a factor with levels \code{No}, \code{Yes}. Hypertrophy refers to an
increased size of the heart muscle due to exercise. }

\item{\code{Hyperchol}}{a factor with levels \code{No} \code{Yes}. Hypercholesterolemia, also called high cholesterol,
is the presence of high levels of cholesterol in the blood.}

\item{\code{Smoker}}{Is the patient a smoker; a factor with levels \code{No}, \code{Yes}.}

\item{\code{Inherit}}{Hereditary predispositions for CAD; a factor with levels  \code{No}, \code{Yes}.}

\item{\code{Heartfail}}{Previous heart failures; a factor with  levels \code{No} \code{Yes}}

\item{\code{CAD}}{Coronary Artery Disease; a factor with levels
\code{No} \code{Yes}}.  CAD refers to a reduction of blood flow
to the heart muscle (commonly known as a heart attack). The
diagnosis made from biopsies.

}
}
\usage{
data(cad1)
}
\description{
A cross classified table with observational data from a Danish
heart clinic.  The response variable is CAD (coronary artery
disease, some times called heart attack).
}
\details{
Notice that data are collected at a heart clinic, so data do not
represent the population, but are conditional on patients having
ended up at the clinic.
\itemize{
\item cad1: Complete dataset, 236 cases.
\item cad2: Incomplete dataset, 67 cases. Information on (some of) the
variables 'Hyperchol', 'Smoker' and 'Inherit' is missing.
}
}
\examples{

data(cad1)
## maybe str(cad1) ; plot(cad1) ...

}
\references{
Hansen, J. F. (1980). The clinical diagnosis of
ischaemic heart disease due to coronary artery disease. Danish
Medical Bulletin

Højsgaard, Søren and Thiesson, Bo (1995). BIFROST - Block recursive
models Induced From Relevant knowledge, Observations and
Statistical Techniques. Computational Statistics and Data Analysis,
vol. 19, p. 155-175
#'
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{cropyield}
\alias{cropyield}
\title{Yield from Danish agricultural production of grain and root crop.}
\format{
A dataframe with 97 rows and 7 columns.
\describe{
\item{\code{year}}{From 1901 to 1997.}
\item{\code{precip}}{Milimeter precipitation.}
\item{\code{yield}}{Million feed units (see details).}
\item{\code{area}}{Area in 1000 ha for grains and root crop.}
\item{\code{fertil}}{1000 tons fertilizer.}
\item{\code{avgtmp1}}{Average temperature April-June (3 months).}
\item{\code{avgtmp2}}{Average temperature July-Octobre (4 months). }
}
}
\usage{
cropyield
}
\description{
Yield from Danish agricultural production of grain and root crop.
}
\details{
A feed unit is the amount of energy in a kg of barley.
}
\references{
Danmarks statistik (Statistics Denmark).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sequence_utilities.R
\name{sub_seq}
\alias{sub_seq}
\alias{subSeq}
\title{Find sub-sequences of identical elements in a vector.}
\usage{
sub_seq(x, item = NULL)

subSeq(x, item = NULL)
}
\arguments{
\item{x}{An atomic vector or a factor.}

\item{item}{Optionally a specific value to look for in \code{x}.}
}
\value{
A dataframe.
}
\description{
Find sub-sequences of identical elements in a vector.
}
\examples{

x <- c(1, 1, 1, 0, 0, 1, 1, 1, 2, 2, 2, 1, 2, 2, 2, 3)
sub_seq(x)
sub_seq(x, item=1)
}
\seealso{
\code{\link{rle}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pick.R
\name{head_matrix}
\alias{head_matrix}
\alias{head2}
\alias{tail2}
\title{head and tail for matrices}
\usage{
head2(x, n = 6, m = n)

tail2(x, n = 6, m = n)
}
\arguments{
\item{x}{matrix}

\item{n, m}{number of rows and columns}
}
\description{
head and tail for matrices
}
\examples{
M <- matrix(1:20, nrow=4)
head2(M)
head2(M, 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe_friendly.R
\name{pipe_arithmetic}
\alias{pipe_arithmetic}
\alias{reciprocal}
\alias{pow}
\alias{add}
\alias{subtract}
\alias{mult}
\alias{divide}
\title{Pipe-friendly arithmetic helpers}
\usage{
reciprocal(x)

pow(x, p)

add(x, k)

subtract(x, k)

mult(x, k)

divide(x, k)
}
\arguments{
\item{x}{A numeric vector or scalar.}

\item{p}{A numeric scalar exponent (for \code{pow}).}

\item{k}{A numeric scalar for addition, subtraction, multiplication, or division.}
}
\value{
A numeric vector or scalar resulting from the transformation.
}
\description{
A set of simple, vectorized, pipe-friendly arithmetic functions for
transforming numeric data in pipelines. These helpers make common
operations like multiplication, division, addition, subtraction,
exponentiation, and reciprocals clearer when using the native pipe \verb{|>}.
}
\details{
All functions are vectorized and support numeric vectors, scalars, or compatible objects.
They are designed to improve the readability of transformation pipelines.
}
\examples{
x <- c(1, 2, 3)

# Multiplication and division
x |> mult(10)
x |> divide(2)

# Addition and subtraction
x |> add(5)
x |> subtract(1)

# Reciprocal
x |> reciprocal()

# Power
x |> pow(2)

# Combined use in pipelines
x |>
  mult(2) |>
  add(3) |>
  reciprocal()

}
\concept{pipe_arithmetic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{potatoes}
\alias{potatoes}
\title{Weight and size of 20 potatoes}
\format{
A data frame with 20 observations on the following 3 variables.
\describe{
\item{\code{weight}}{a numeric vector}
\item{\code{length}}{a numeric vector}
\item{\code{width}}{a numeric vector}
}
}
\source{
My own garden; autumn 2015.
}
\usage{
potatoes
}
\description{
Weight and size of 20 potatoes. Weight in grams; size in millimeter. There
are two sizes: \code{length} is the longest length and \code{width} is the
shortest length across a potato.
}
\examples{

data(potatoes)
plot(potatoes) 

}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{fev}
\alias{fev}
\title{Forced expiratory volume in children}
\format{
A data frame with 654 observations on the following 5 variables.
\describe{
\item{\code{Age}}{Age in years.}
\item{\code{FEV}}{Forced expiratory volume in liters per second.}
\item{\code{Ht}}{Height in centimeters.}
\item{\code{Gender}}{Gender.}
\item{\code{Smoke}}{Smoking status.}
}
}
\usage{
fev
}
\description{
Dataset to examine if respiratory function in children
was influenced by smoking.
}
\examples{

data(fev)
summary(fev)

}
\references{
I. Tager and S. Weiss and B. Rosner and F. Speizer (1979). Effect
of Parental Cigarette Smoking on the Pulmonary Function of
Children.  American Journal of Epidemiology. 110:15-26
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{descStat}
\alias{descStat}
\title{Computing simple descriptive statistics of a numeric vector.}
\usage{
descStat(x, na.rm = TRUE)
}
\arguments{
\item{x}{A numeric vector}

\item{na.rm}{Should missing values be removed}
}
\value{
A vector with named elements.
}
\description{
Computing simple descriptive statistics of a numeric
vector - not unlike what proc means of SAS does
}
\examples{

x <- c(1, 2, 3, 4, NA, NaN)
descStat(x)

}
\seealso{
\code{\link{summaryBy}}, \code{\link{summary_by}}
}
\author{
Gregor Gorjanc; \email{gregor.gorjanc@bf.uni-lj.si}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{parseGroupFormula}
\alias{parseGroupFormula}
\title{Extract components from a formula with "conditioning bar"}
\usage{
parseGroupFormula(form)
}
\arguments{
\item{form}{A formula of the form \code{y ~ x1 + ... + xn | g1 + ... + gm}}
}
\value{
If the formula is \code{y ~ x1 + x2 | g1 + g2} the result is
\item{model}{\code{y ~ x1 + x2}} \item{groups}{\code{ g1 + g2}}
\item{groupFormula}{\code{~ g1 + g2}}
}
\description{
Extract components from a formula with the form
\code{y ~ x1 + ... + xn | g1 + ... + gm}
}
\examples{

gf1 <- parseGroupFormula(y ~ x1 + x2 | g1 + g2)
gf1

gf2 <- parseGroupFormula( ~ x1 + x2 | g1 + g2)
gf2

}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_lapplyBy.R
\name{by-lapply}
\alias{by-lapply}
\alias{lapply_by}
\alias{lapplyBy}
\alias{sapply_by}
\alias{sapplyBy}
\title{Formula based version of lapply and sapply}
\usage{
lapply_by(data, formula, FUN, ...)

lapplyBy(formula, data = parent.frame(), FUN, ...)

sapply_by(data, formula, FUN, ..., simplify = TRUE, USE.NAMES = TRUE)

sapplyBy(
  formula,
  data = parent.frame(),
  FUN,
  ...,
  simplify = TRUE,
  USE.NAMES = TRUE
)
}
\arguments{
\item{data}{A dataframe.}

\item{formula}{A formula describing how data should be split.}

\item{FUN}{A function to be applied to each element in the split
list, see 'Examples' below.}

\item{\dots}{optional arguments to FUN.}

\item{simplify}{Same as for \code{sapply}}

\item{USE.NAMES}{Same as for \code{sapply}}
}
\value{
A list.
}
\description{
This function is a wrapper for calling lapply on the
list resulting from first calling splitBy.
}
\examples{
fun <- function(x) range(x$uptake)
lapplyBy(~Treatment + Type, data=CO2, FUN=fun)
sapplyBy(~Treatment + Type, data=CO2, FUN=fun)

# Same as
lapply(splitBy(~Treatment + Type, data=CO2), FUN=fun)
}
\seealso{
\code{\link{splitBy}}, \code{\link{split_by}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{byfunction}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelling.R
\name{add_int}
\alias{add_int}
\title{Add interaction columns to data frame}
\usage{
add_int(.data, .formula)
}
\arguments{
\item{.data}{dataframe}

\item{.formula}{right hand sided formula}
}
\value{
dataframe
}
\description{
Add interaction columns to data frame
}
\author{
Søren Højsgaard
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_stability.r
\name{generate_data_list}
\alias{generate_data_list}
\title{Generate data list}
\usage{
generate_data_list(data., M, method = c("subgroups", "resample"))
}
\arguments{
\item{data.}{A data frame}

\item{M}{Number of folds}

\item{method}{Method for generating data}
}
\description{
Generate data list
}
\details{
There are n rows in data. The function returns a list of M
datasets. If method="subgroups" each dataset in the list has
(approximately) n-M randomly selected rows from data. If
method="resample" each dataset has n rows obtained by sampling
with replacement from the original data. The function is
heavily inspired by the boot::cv.glm function.
}
\note{
This is a recent addition. Suggestions and comments are
welcome. The addition may be removed without warning.
}
\examples{
dat <- cars[1:15,]
set.seed(1411)
dl1 <- generate_data_list(dat, 4)
dl1[[1]]
dl1 |> sapply(nrow)
dl2 <- generate_data_list(dat, 4, method="resample")
dl2 |> sapply(nrow)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{data_personality}
\alias{data_personality}
\alias{personality}
\title{Personality traits}
\format{
This dataframe has recordings on the following 32
variables: distant, talkatv, carelss, hardwrk, anxious,
agreebl, tense, kind, opposng, relaxed, disorgn, outgoin,
approvn, shy, discipl, harsh, persevr, friendl, worryin,
respnsi, contrar, sociabl, lazy, coopera, quiet, organiz,
criticl, lax, laidbck, withdrw, givinup, easygon
}
\usage{
data(personality)
}
\description{
The \code{peronality} dataframe has 240 rows and 32 columns
}
\examples{
data(personality)
str(personality)
}
\references{
Origin unclear
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_scale_df.R
\name{scale_df}
\alias{scale_df}
\title{Scale numeric variables in a data frame}
\usage{
scale_df(x, center = TRUE, scale = TRUE)
}
\arguments{
\item{x}{A data frame or matrix.}

\item{center}{Logical; if TRUE, center the variables.}

\item{scale}{Logical; if TRUE, scale the variables.}
}
\value{
An object of the same class as \code{x}.
}
\description{
Applies \code{base::scale()} to numeric, integer, or
logical columns in a data frame. Non-numeric columns are left
unchanged.
}
\details{
If \code{x} is not a data frame, \code{base::scale()} is applied directly.
}
\examples{
scale_df(iris)

}
\concept{data_handling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{.rhsf2list}
\alias{.rhsf2list}
\title{Convert right hand sided formula to a list}
\usage{
.rhsf2list(f)
}
\arguments{
\item{f}{A right hand sided formula}
}
\description{
Convert right hand sided formula to a list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/microbenchmark.R
\name{mb_summary}
\alias{mb_summary}
\alias{summary_mb}
\title{Fast summary of microbenchmark object}
\usage{
mb_summary(object, unit, add.unit = TRUE, ...)

summary_mb(object, unit, add.unit = TRUE, ...)
}
\arguments{
\item{object}{A microbenchmark object}

\item{unit}{The time unit to be used}

\item{add.unit}{Should time unit be added as column to resulting
dataframe.}

\item{...}{Additional arguments; currently not used.}
}
\description{
Fast summary of microbenchmark object. The default
summary method from the microbenchmark package is fairly slow
in producing a summary (due to a call to a function from the
multcomp package.)
}

\name{internal}
\alias{internal}

\alias{orderByOLD}
\alias{splitByOLD}
\alias{summaryByOLD}

\alias{linest_matrix_class-class}
\alias{print.linest_class}
\alias{print.esticon_class}
\alias{print.linest_matrix_class}
\alias{summary.linest_matrix_class}

\alias{asNumericMatrix2}
\alias{matrix2dataFrame2}
\alias{subsAttr2}
\alias{doby.xtabs}


\alias{print.lmBy}
\alias{print.summary_lmBy}
\alias{print.splitByData}

\alias{linest_class-class}


\alias{\%>\%}
 
\title{Internal functions for the doBy package}
\description{Internal functions for the doBy package}
\keyword{internal}% at least one, from doc/KEYWORDS





%%\alias{funBy}

%%\alias{pdiff}

%\alias{}
%% \alias{nullBasis}
%% \alias{nullBasis.lm}
%% \alias{nullBasis.glm}
%% \alias{nullBasis.geeglm}
%% \alias{nullBasis.lmerMod}
%% \alias{nullBasis.matrix}
%% \alias{nullBasis.Matrix}

%% \alias{null_basis}
%%\alias{is_estimable}
%% \alias{get_null_basis.matrix}
%% \alias{get_null_basis.Matrix}
%% \alias{get_null_basis.default}

% \alias{HTMLCommand2}
% \alias{HTMLdefault2}
% \alias{RweaveHTML2}
% \alias{RweaveHTMLRuncode2}

% \alias{RweaveHTMLreport}
% \alias{RweaveHTMLreportSetup}
% \alias{RweaveHTMLreportWritedoc}
% \alias{RweaveHTMLreportFinish}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linest_LSmeans.R
\name{ls-means}
\alias{ls-means}
\alias{LSmeans}
\alias{LSmeans.default}
\alias{LSmeans.lmerMod}
\alias{popMeans}
\alias{popMeans.default}
\alias{popMeans.lmerMod}
\title{Compute  LS-means (aka population means or
marginal means)}
\usage{
LSmeans(object, effect = NULL, at = NULL, level = 0.95, ...)

\method{LSmeans}{default}(object, effect = NULL, at = NULL, level = 0.95, ...)

\method{LSmeans}{lmerMod}(object, effect = NULL, at = NULL, level = 0.95, adjust.df = TRUE, ...)

popMeans(object, effect = NULL, at = NULL, level = 0.95, ...)

\method{popMeans}{default}(object, effect = NULL, at = NULL, level = 0.95, ...)

\method{popMeans}{lmerMod}(object, effect = NULL, at = NULL, level = 0.95, adjust.df = TRUE, ...)
}
\arguments{
\item{object}{Model object}

\item{effect}{A vector of variables. For each configuration of
these the estimate will be calculated.}

\item{at}{A list of values of covariates (including levels of some
factors) to be used in the calculations}

\item{level}{The level of the (asymptotic) confidence interval.}

\item{\dots}{Additional arguments; currently not used.}

\item{adjust.df}{Should denominator degrees of freedom be adjusted?}
}
\value{
A dataframe with results from computing the contrasts.
}
\description{
LS-means (least squares means, also known as
population means and as marginal means) for a range of model types.
}
\details{
There are restrictions on the formulas allowed in the model object.
For example having \code{y ~ log(x)} will cause an error. Instead one
must define the variable \code{logx = log(x)} and do \code{y ~ logx}.
}
\note{
\code{LSmeans} and \code{popMeans} are synonymous.
}
\section{Warning}{
 Notice that \code{LSmeans} and \code{LE_matrix}
fails if the model formula contains an offset (as one would
have in connection with e.g. Poisson regression.
}

\examples{

## Two way anova:

data(warpbreaks)

m0 <- lm(breaks ~ wool + tension, data=warpbreaks)
m1 <- lm(breaks ~ wool * tension, data=warpbreaks)
LSmeans(m0)
LSmeans(m1)

## same as:
K <- LE_matrix(m0);K
linest(m0, K)
K <- LE_matrix(m1);K
linest(m1, K)

LE_matrix(m0, effect="wool")
LSmeans(m0, effect="wool")

LE_matrix(m1, effect="wool")
LSmeans(m1, effect="wool")

LE_matrix(m0, effect=c("wool", "tension"))
LSmeans(m0, effect=c("wool", "tension"))

LE_matrix(m1, effect=c("wool", "tension"))
LSmeans(m1, effect=c("wool", "tension"))


## Regression; two parallel regression lines:

data(Puromycin)

m0 <- lm(rate ~ state + log(conc), data=Puromycin)
## Can not use LSmeans / LE_matrix here because of
## the log-transformation. Instead we must do:
Puromycin$lconc <- log( Puromycin$conc )
m1 <- lm(rate ~ state + lconc, data=Puromycin)

LE_matrix(m1)
LSmeans(m1)

LE_matrix(m1, effect="state")
LSmeans(m1, effect="state")

LE_matrix(m1, effect="state", at=list(lconc=3))
LSmeans(m1, effect="state", at=list(lconc=3))

## Non estimable contrasts

## ## Make balanced dataset
dat.bal <- expand.grid(list(AA=factor(1:2), BB=factor(1:3),
                            CC=factor(1:3)))
dat.bal$y <- rnorm(nrow(dat.bal))

## ## Make unbalanced dataset
#      'BB' is nested within 'CC' so BB=1 is only found when CC=1
#       and BB=2,3 are found in each CC=2,3,4
dat.nst <- dat.bal
dat.nst$CC <-factor(c(1, 1, 2, 2, 2, 2, 1, 1, 3, 3,
                      3, 3, 1, 1, 4, 4, 4, 4))

mod.bal  <- lm(y ~ AA + BB * CC,    data=dat.bal)
mod.nst  <- lm(y ~ AA + BB : CC, data=dat.nst)

LSmeans(mod.bal, effect=c("BB", "CC"))
LSmeans(mod.nst, effect=c("BB", "CC"))
LSmeans(mod.nst, at=list(BB=1, CC=1))

LSmeans(mod.nst, at=list(BB=1, CC=2))
## Above: NA's are correct; not an estimable function

if( require( lme4 )){
 warp.mm <- lmer(breaks ~ -1 + tension + (1|wool), data=warpbreaks)
 LSmeans(warp.mm, effect="tension")
 class(warp.mm)
 fixef(warp.mm)
 coef(summary(warp.mm))
 vcov(warp.mm)
 if (require(pbkrtest))
   vcovAdj(warp.mm)
}

LSmeans(warp.mm, effect="tension")

}
\seealso{
\code{\link{LE_matrix}}, \code{\link{linest}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_summaryBy.R
\name{by-summary}
\alias{by-summary}
\alias{summary_by}
\alias{summaryBy}
\title{Groupwise summary statistics}
\usage{
summary_by(
  data,
  formula,
  id = NULL,
  FUN = mean,
  keep.names = FALSE,
  p2d = FALSE,
  order = TRUE,
  full.dimension = FALSE,
  var.names = NULL,
  fun.names = NULL,
  ...
)

summaryBy(
  formula,
  data = parent.frame(),
  id = NULL,
  FUN = mean,
  keep.names = FALSE,
  p2d = FALSE,
  order = TRUE,
  full.dimension = FALSE,
  var.names = NULL,
  fun.names = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{formula}{A formula specifying response and grouping variables.}

\item{id}{A formula indicating variables to retain (not grouped by).}

\item{FUN}{A function or list of functions to apply to the response variables.}

\item{keep.names}{Logical; keep original variable names if only one function is applied.}

\item{p2d}{Replace parentheses in output names with dots?}

\item{order}{Logical; should result be ordered by grouping variables?}

\item{full.dimension}{Logical; if TRUE, repeat rows so output matches input size.}

\item{var.names}{Optional custom names for response variables.}

\item{fun.names}{Optional custom names for functions applied.}

\item{...}{Additional arguments passed to functions in \code{FUN}.}
}
\value{
A data frame of grouped summary statistics.
}
\description{
Computes summary statistics by groups, similar to the \code{summary} procedure in SAS.
A more flexible alternative to base R's \code{\link[stats]{aggregate}}.
}
\details{
Extra arguments in \code{...} are passed to all functions in \code{FUN}. If needed, wrap functions to handle these consistently (e.g., for \code{na.rm = TRUE}).
}
\examples{
data(CO2)

# Simple groupwise mean
summaryBy(uptake ~ Type + Treatment, data = CO2, FUN = mean)
summaryBy(cbind(uptake, conc) ~ Type + Treatment, data = CO2, FUN = mean)

# Compare with
aggregate(cbind(uptake, conc) ~ Type + Treatment, data = CO2, FUN = mean)

## Using '.' on the right hand side of a formula means to stratify by
## all variables not used elsewhere:
summaryBy(uptake ~ ., data = CO2, FUN = mean)

# Multiple functions using a custom summary function
myfun <- function(x, ...)
  c(m = mean(x, na.rm = TRUE), v = var(x, na.rm = TRUE), n = length(x))
summaryBy(uptake ~ Type + Treatment, data = CO2, FUN = myfun)

# Summary on transformed variables
# works:
summaryBy(cbind(lu=log(uptake), conc) ~ Type, data = CO2, FUN = mean)
# fails:
#summaryBy(cbind(log(uptake), conc) ~ Type, data = CO2, FUN = mean)
}
\seealso{
\code{\link[stats]{aggregate}}, \code{\link{orderBy}}, \code{\link{transformBy}}, \code{\link{splitBy}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{byfunction}
\keyword{grouping}
\keyword{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_scaleBy.R
\name{scaleBy}
\alias{scaleBy}
\alias{scale_by}
\title{Group-wise scaling of data}
\usage{
scaleBy(formula, data = parent.frame(), center = TRUE, scale = TRUE)

scale_by(data, formula, center = TRUE, scale = TRUE)
}
\arguments{
\item{formula}{Grouping structure: a formula, character vector, or variables as \code{as.quoted}.}

\item{data}{A data frame or matrix.}

\item{center}{Logical; if TRUE, center the variables.}

\item{scale}{Logical; if TRUE, scale the variables.}
}
\value{
A list of data frames or matrices (same class as input), one per group.
}
\description{
Splits a data frame or matrix by grouping variables and scales numeric variables within each group.
}
\examples{
scale_by(iris, ~Species)
scale_by(iris, ~1)

## Combine result into one data frame:
a <- scale_by(iris, ~Species)
d <- do.call(rbind, a)

## Old interface
scaleBy(~Species, data = iris, center = TRUE, scale = FALSE)
scaleBy(~1, data = iris)

}
\seealso{
\code{\link{summaryBy}}, \code{\link{transformBy}}, \code{\link{orderBy}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{byfunction}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linest_utilities.R
\name{is_estimable}
\alias{is_estimable}
\title{Determines if contrasts are estimable.}
\usage{
is_estimable(K, null.basis)
}
\arguments{
\item{K}{A matrix.}

\item{null.basis}{A basis for a null space (can be found with
\code{null_basis()}).}
}
\value{
A logical vector.
}
\description{
Determines if contrasts are estimable, that is, if the contrasts
can be written as a linear function of the data.
}
\details{
Consider the setting \eqn{E(Y)=Xb}. A linear function of \eqn{b},
say \eqn{l'b} is estimable if and only if there exists an \eqn{r} such
that \eqn{r'X=l'} or equivalently \eqn{l=X'r}. Hence \eqn{l} must be in
the column space of \eqn{X'}, i.e. in the orthogonal complement of the
null space of \eqn{X}. Hence, with a basis \eqn{B} for the null space,
\code{is_estimable()} checks if each row \eqn{l} of the matrix \eqn{K} is
perpendicular to each column basis vector in \eqn{B}.
}
\references{
\url{http://web.mit.edu/18.06/www/Essays/newpaper_ver3.pdf}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{income}
\alias{income}
\title{Income data, years of educations and ethnicity}
\format{
This data frame contains:
\describe{
\item{inc:}{Income: Yearly income (thousands of dollars).}
\item{educ:}{Education: Number of years of education (12=high school graduate, 16=college graduate).}
\item{race:}{Racial-Ethnic group: "b" (black), "h" (hispanic) and "w" (white).}
}
}
\usage{
income
}
\description{
Data on income, years of educations and ethnicity for a samle of adult Americans aged over 25. The year of sampling is not avalable in the source.
}
\details{
Variable names are as in the reference.
}
\references{
Agresti, A. (2024) Statistical Methods for the Social
Sciences, Global Edition (6th edition).  ISBN-13: 9781292449197. Table 13.1
}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linest_get.R
\name{linest-get}
\alias{linest-get}
\alias{get_xlevels}
\alias{get_xlevels.default}
\alias{get_xlevels.mer}
\alias{get_xlevels.merMod}
\alias{get_contrasts}
\alias{get_contrasts.default}
\alias{get_contrasts.merMod}
\alias{set_xlevels}
\alias{get_vartypes}
\alias{set_covariate_val}
\alias{get_X}
\alias{get_X.default}
\alias{get_X.merMod}
\title{Auxillary functions for computing lsmeans, contrasts etc}
\usage{
get_xlevels(obj)

\method{get_xlevels}{default}(obj)

\method{get_xlevels}{mer}(obj)

\method{get_xlevels}{merMod}(obj)

get_contrasts(obj)

\method{get_contrasts}{default}(obj)

\method{get_contrasts}{merMod}(obj)

set_xlevels(xlev, at)

get_vartypes(obj)

set_covariate_val(xlev, covariateVal)

get_X(obj, newdata, at = NULL)

\method{get_X}{default}(obj, newdata, at = NULL)

\method{get_X}{merMod}(obj, newdata, at = NULL)
}
\arguments{
\item{obj}{An R object}

\item{xlev}{FIXME: to be described}

\item{at}{FIXME: to be described}

\item{covariateVal}{FIXME: to be described}

\item{newdata}{FIXME: to be described}
}
\description{
Auxillary functions for computing lsmeans, contrasts etc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{codstom}
\alias{codstom}
\title{Diet of Atlantic cod in the Gulf of St. Lawrence (Canada)}
\format{
A data frame with 10000 observations on the following 10 variables.
\describe{
\item{\code{region}}{a factor with levels \code{SGSL} \code{NGSL}
representing the southern and northern Gulf of St. Lawrence, respectively}
\item{\code{ship.type}}{a factor with levels \code{2} \code{3} \code{31}
\code{34} \code{90} \code{99}}
\item{\code{ship.id}}{a factor with levels \code{11558} \code{11712}
\code{136148} \code{136885}
\code{136902} \code{137325} \code{151225} \code{151935} \code{99433}}
\item{\code{trip}}{a factor with levels \code{10} \code{11}
\code{12} \code{179} \code{1999}
\code{2} \code{2001} \code{20020808} \code{3} \code{4} \code{5}
\code{6} \code{7} \code{8}
\code{88} \code{9} \code{95}}
\item{\code{set}}{a numeric vector}
\item{\code{fish.id}}{a numeric vector}
\item{\code{fish.length}}{a numeric vector, length in mm}
\item{\code{prey.mass}}{a numeric vector, mass of item in stomach, in g}
\item{\code{prey.type}}{a factor with levels \code{Ammodytes_sp}
\code{Argis_dent}
\code{Chion_opil} \code{Detritus} \code{Empty} \code{Eualus_fab}
\code{Eualus_mac} \code{Gadus_mor} \code{Hyas_aran}
\code{Hyas_coar}
\code{Lebbeus_gro} \code{Lebbeus_pol} \code{Leptocl_mac}
\code{Mallot_vil}
\code{Megan_norv} \code{Ophiuroidea} \code{Other} \code{Paguridae}
\code{Pandal_bor} \code{Pandal_mon} \code{Pasiph_mult}
\code{Sabin_sept}
\code{Sebastes_sp} \code{Them_abys} \code{Them_comp} \code{Them_lib}}
}
}
\source{
Small subset from a larger dataset (more stomachs, more variables,
more \code{prey.types}) collected by D. Chabot and M. Hanson, Fisheries &
Oceans Canada \email{chabotd@dfo-mpo.gc.ca}.
}
\usage{
codstom
}
\description{
Stomach content data for Atlantic cod (\emph{Gadus morhua}) in the Gulf of
St.Lawrence, Eastern Canada.  Note: many prey items were of no interest for
this analysis and were regrouped into the "Other" category.
}
\details{
Cod are collected either by contracted commerical fishing vessels
(\code{ship.type} 90 or 99) or by research vessels.  Commercial vessels are
identified by a unique \code{ship.id}.

Either one research vessel or several commercial vessels conduct a survey
(\code{trip}), during which a trawl, gillnets or hooked lines are set
several times. Most trips are random stratified surveys (depth-based
stratification).

Each trip takes place within one of the \code{region}s.  The \code{trip}
label is only guaranteed to be unique within a region and the \code{set}
label is only guaranteed to be unique within a \code{trip}.

For each fish caught, the \code{fish.length} is recorded and the fish is
allocated a \code{fish.id}, but the \code{fish.id} is only guaranteed to be
unique within a \code{set}. A subset of the fish caught are selected for
stomach analysis (stratified random selection according to fish length; unit
of stratification is the set for research surveys, the combination ship.id
and stratum for surveys conducted by commercial vessels, although strata are
not shown in codstom).

The basic experimental unit in this data set is a cod stomach (one stomach
per fish).  Each stomach is uniquely identified by a combination of
\code{region}, \code{ship.type}, \code{ship.id}, \code{trip}, \code{set},
and \code{fish.id}.

For each prey item found in a stomach, the species and mass of the prey item
are recorded, so there can be multiple observations per stomach.  There may
also be several prey items with the same \code{prey.type} in the one stomach
(for example many \code{prey.types} have been recoded \code{Other}, which
produced many instances of \code{Other} in the same stomach).

If a stomach is empty, a single observation is recorded with
\code{prey.type} \code{Empty} and a \code{prey.mass} of zero.
}
\examples{

data(codstom)
str(codstom)
# removes multiple occurences of same prey.type in stomachs
codstom1 <- summaryBy(prey.mass ~ 
                      region + ship.type + ship.id + trip + set + fish.id + prey.type,
                      data = codstom, 
                      FUN = sum) 

# keeps a single line per stomach with the total mass of stomach content
codstom2 <- summaryBy(prey.mass ~ region + ship.type + ship.id + trip + set + fish.id,
                      data = codstom, 
                      FUN = sum) 

# mean prey mass per stomach for each trip
codstom3 <- summaryBy(prey.mass.sum ~ region + ship.type + ship.id + trip,
                      data = codstom2, FUN = mean) 

\dontrun{          
# wide version, one line per stomach, one column per prey type
library(reshape)
codstom4 <- melt(codstom, id = c(1:7, 9))
codstom5 <- cast(codstom4, 
                 region + ship.type + ship.id + trip + set + fish.id + fish.length ~ 
                 prey.type, sum)
k <- length(names(codstom5))
prey_col <- 8:k
out <- codstom5[,prey_col]
out[is.na(out)] <- 0
codstom5[,prey_col] <- out
codstom5$total.content <- rowSums(codstom5[, prey_col])
}

}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/by_subsetBy.R
\name{by-subset}
\alias{by-subset}
\alias{subset_by}
\alias{subsetBy}
\title{Finds subsets of a dataframe which is split by variables in
a formula.}
\usage{
subset_by(data, formula, subset, select, drop = FALSE, join = TRUE, ...)

subsetBy(
  formula,
  subset,
  data = parent.frame(),
  select,
  drop = FALSE,
  join = TRUE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{formula}{A right hand sided formula or a character vector of
variables to split by.}

\item{subset}{logical expression indicating elements or rows to
keep: missing values are taken as false.}

\item{select}{expression, indicating columns to select from a data
frame.}

\item{drop}{passed on to \code{[} indexing operator.}

\item{join}{If FALSE the result is a list of data frames (as
defined by 'formula'); if TRUE one data frame is returned.}

\item{\dots}{further arguments to be passed to or from other methods.}
}
\value{
A data frame.
}
\description{
A data frame is split by a formula into groups. Then
subsets are found within each group, and the result is
collected into a data frame.
}
\examples{

data(dietox)
subsetBy(~Evit, Weight < mean(Weight), data=dietox)
}
\seealso{
\code{\link{splitBy}}, \code{\link{split_by}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\concept{byfunction}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sequence_utilities.R
\name{time_since_event}
\alias{time_since_event}
\alias{timeSinceEvent}
\title{Calculate "time since event" in a vector}
\usage{
time_since_event(yvar, tvar = seq_along(yvar))

timeSinceEvent(...)
}
\arguments{
\item{yvar}{A numeric or logical vector indicating events.}

\item{tvar}{An optional numeric vector specifying time values. Defaults to the index.}

\item{...}{Arguments pased on to time_since_event}
}
\value{
A data frame with columns 'yvar', 'tvar', 'abs.tse' (absolute time since event),
'sign.tse' (signed time since event), and other helper columns.
}
\description{
Calculates the time since the nearest event in a sequence,
optionally using a custom time scale.
}
\details{
Events are coded as 1 (or TRUE). Non-events are anything else. The
result includes absolute and signed distances to events.
}
\examples{

## Example 1: Basic usage with default time index
y <- c(0, 0, 1, 0, 0, 1, 0)
tse <- time_since_event(y)
print(tse)

## Example 2: Custom (non-integer) time variable
y <- c(0, 0, 1, 0, 0, 0, 1, 0)
t <- seq(0.5, 3.5, length.out = length(y))
tse <- time_since_event(y, t)
print(tse)

## Example 3: Plotting the signed time since event
plot(sign.tse ~ tvar, data = tse, type = "b",
     main = "Signed time since event",
     xlab = "Time", ylab = "Signed time since event")
grid()
abline(h = 0, col = "red", lty = 2)

}
\seealso{
\code{\link{subSeq}}, \code{\link{rle}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{truncate0}
\alias{truncate0}
\title{Truncate values in a matrix / vector to zero if they are below a certain threshold.}
\usage{
truncate0(x, tol = 0.6, sparse = TRUE)
}
\arguments{
\item{x}{matrix / vector}

\item{tol}{threshold}

\item{sparse}{logical; if TRUE and \code{x} is a matrix, return a sparse matrix}
}
\description{
Truncate values in a matrix / vector to zero if they are below a certain threshold.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula_ops.R
\name{formula_ops}
\alias{formula_ops}
\alias{formula_add_str}
\alias{formula_add}
\alias{formula_poly}
\alias{formula_nth}
\alias{formula_to_interaction_matrix}
\alias{formula_chr_to_form}
\alias{to_str}
\alias{terms_labels}
\alias{simplify_rhs}
\alias{simplify_rhs.formula}
\alias{simplify_rhs.character}
\alias{as_rhs_frm}
\alias{as_lhs_frm}
\alias{as_rhs_chr}
\alias{as_lhs_chr}
\alias{unique_formula}
\title{Formula operations and coercion.}
\usage{
formula_add_str(frm1, terms, op = "+")

formula_add(frm1, frm2)

formula_poly(chr1, n, noint = FALSE, y = NULL)

formula_nth(frm1, n)

formula_to_interaction_matrix(frm1)

formula_chr_to_form(rhs, lhs = character(0))

to_str(chr1, collapse = "+")

terms_labels(frm1)

simplify_rhs(object)

\method{simplify_rhs}{formula}(object)

\method{simplify_rhs}{character}(object)

as_rhs_frm(object)

as_lhs_frm(object)

as_rhs_chr(object, string = FALSE)

as_lhs_chr(object, string = FALSE)

unique_formula(list_of_formulas)
}
\arguments{
\item{frm1, frm2}{Formulas to be coerced to character vectors.}

\item{terms}{Character string.}

\item{op}{Either "+" (default) or "-".}

\item{chr1}{Character vector to be coerced to formulas.}

\item{n}{Positive integer.}

\item{noint}{Boolean.}

\item{y}{Response}

\item{rhs, lhs}{right-hand-side and left-hand-side for formula (as characters)}

\item{collapse}{Character to use as separator.}

\item{object}{Character vector or formula.}

\item{string}{Boolean.}

\item{list_of_formulas}{list of formulas}
}
\description{
Formula operations and coercion as a supplement to \code{update.formula()}
}
\examples{

formula_poly("z", 2)
formula_poly("z", 2, noint=TRUE)

as_rhs_chr(c("a", "b", "z"))
as_rhs_chr(c("a*b", "z"))

as_rhs_chr(y~a+b+z)
as_rhs_chr(y~a+b+z, string=TRUE)
as_rhs_chr(y~a+b+z)
as_rhs_chr(y~a*b+z)
as_rhs_chr(y~a*b+z, string=TRUE)

as_lhs_chr(y~a*b+z)
as_lhs_chr(log(y)~a*b+z)      ## Not what one might expect
as_lhs_chr(cbind(y, u)~a*b+z) ## Not what one might expect

formula_chr_to_form(c("a*b", "z"))
formula_chr_to_form(c("a*b", "z"), "y")
formula_chr_to_form(c("a*b", "z"), "log(y)")

formula_add(y~a*b+z, ~-1)
formula_add(y~a*b+z, ~a:b)

formula_add_str(y~x1 + x2, "x3")
formula_add_str(y~x1 + x2, "x1")
formula_add_str(y~x1 + x2, "x1", op="-")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linest_matrix.R
\name{linest-matrix}
\alias{linest-matrix}
\alias{LE_matrix}
\alias{LE_matrix.default}
\alias{aggregate_linest_list}
\alias{get_linest_list}
\title{Linear estimates matrix}
\usage{
LE_matrix(object, effect = NULL, at = NULL)

\method{LE_matrix}{default}(object, effect = NULL, at = NULL)

aggregate_linest_list(linest_list)

get_linest_list(object, effect = NULL, at = NULL)
}
\arguments{
\item{object}{Model object}

\item{effect}{A vector of variables. For each configuration of
these the estimate will be calculated.}

\item{at}{Either NULL, a list or a dataframe. 1) If a list, then the list
must consist of covariates (including levels of some factors)
to be used in the calculations. 2) If a dataframe, the
dataframe is split rowwise and the function is invoked on each
row.}

\item{linest_list}{Linear estimate list (as generated by \code{get_linest_list}).}
}
\description{
Generate matrix specifying linear estimate.
}
\details{
Check this
}
\examples{

## Two way anova:

data(warpbreaks)

## An additive model
m0 <- lm(breaks ~ wool + tension, data=warpbreaks)

## Estimate mean for each wool type, for tension="M":
K <- LE_matrix(m0, at=list(wool=c("A", "B"), tension="M"))
K

## Vanilla computation:
K \%*\% coef(m0)

## Alternative; also providing standard errors etc:
linest(m0, K)
esticon(m0, K)

## Estimate mean for each wool type when averaging over tension;
# two ways of doing this
K <- LE_matrix(m0, at=list(wool=c("A", "B")))
K
K <- LE_matrix(m0, effect="wool")
K
linest(m0, K)

## The linear estimate is sometimes called to "least squares mean"
# (LSmeans) or popupulation means.
# Same as
LSmeans(m0, effect="wool")

## Without mentioning 'effect' or 'at' an average across all
#predictors are calculated:
K <- LE_matrix(m0)
K
linest(m0, K)

## Because the design is balanced (9 observations per combination
#of wool and tension) this is the same as computing the average. If
#the design is not balanced, the two quantities are in general not
#the same.
mean(warpbreaks$breaks)

## Same as 
LSmeans(m0)

## An interaction model 
m1 <- lm(breaks ~ wool * tension, data=warpbreaks)

K <- LE_matrix(m1, at=list(wool=c("A", "B"), tension="M"))
K
linest(m1, K)
K <- LE_matrix(m1, at=list(wool=c("A", "B")))
K
linest(m1, K)
K <- LE_matrix(m1, effect="wool")
K
linest(m1, K)
LSmeans(m1, effect="wool")

K <- LE_matrix(m1)
K
linest(m1, K)
LSmeans(m1)
}
\seealso{
\code{\link{LSmeans}}, \code{\link{linest}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doby_utilities.R
\name{recodeVar}
\alias{recodeVar}
\alias{recode_var}
\title{Recode values of a vector}
\usage{
recodeVar(x, src, tgt, default = NULL, keep.na = TRUE)

recode_var(x, src, tgt, default = NULL, keep.na = TRUE)
}
\arguments{
\item{x}{A vector; the variable to be recoded.}

\item{src}{The source values: a subset of the present values of x}

\item{tgt}{The target values: the corresponding new values of x}

\item{default}{Default target value for those values of x not listed in
\code{src}. When default=NULL, values of x which are not given in \code{src} will
be kept in the output.}

\item{keep.na}{If TRUE then NA's in x will be retained in the output}
}
\value{
A vector
}
\description{
Recodes a vector with values, say 1,2 to a variable with values, say 'a',
'b'
}
\section{Warning }{
 Care should be taken if x is a factor. A safe approach may
be to convert x to a character vector using as.character.
}

\examples{

x <- c("dec", "jan", "feb", "mar", "apr", "may")
src1 <- list(c("dec", "jan", "feb"), c("mar", "apr", "may"))
tgt1 <- list("winter", "spring")
recodeVar(x, src=src1, tgt=tgt1)
#[1] "winter" "winter" "winter" "spring" "spring" "spring"

x <- c(rep(1:3, 3))
#[1] 1 2 3 1 2 3 1 2 3

## Simple usage:
recodeVar(x, src=c(1, 2), tgt=c("A", "B"))
#[1] "A" "B" NA  "A" "B" NA  "A" "B" NA 

## Here we need to use lists
recodeVar(x, src=list(c(1, 2)), tgt=list("A"))
#[1] "A" "A" NA  "A" "A" NA  "A" "A" NA 
recodeVar(x, src=list(c(1, 2)), tgt=list("A"), default="L")
#[1] "A" "A" "L" "A" "A" "L" "A" "A" "L"
recodeVar(x, src=list(c(1, 2), 3), tgt=list("A", "B"), default="L")
#[1] "A" "A" "B" "A" "A" "B" "A" "A" "B"

## Dealing with NA's in x
x<-c(NA,rep(1:3, 3),NA)
#[1] NA  1  2  3  1  2  3  1  2  3 NA
recodeVar(x, src=list(c(1, 2)), tgt=list("A"))
#[1] NA  "A" "A" NA  "A" "A" NA  "A" "A" NA  NA 
recodeVar(x, src=list(c(1, 2)), tgt=list("A"), default="L")
#[1] NA  "A" "A" "L" "A" "A" "L" "A" "A" "L" NA 
recodeVar(x, src=list(c(1, 2)), tgt=list("A"), default="L", keep.na=FALSE)
#[1] "L" "A" "A" "L" "A" "A" "L" "A" "A" "L" "L"

x <- c("no", "yes", "not registered", "no", "yes", "no answer")
recodeVar(x, src = c("no", "yes"), tgt = c("0", "1"), default = NA)


}
\seealso{
\code{\link[base]{cut}}, \code{\link[base]{factor}},
\code{\link[doBy]{recodeVar}}
}
\author{
Søren Højsgaard, \email{sorenh@math.aau.dk}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{crime_rate}
\alias{crime_rate}
\title{crimeRate}
\format{
This data frame contains:
\describe{
\item{murder:}{crime of murder}
\item{rape:}{}
\item{robbery:}{}
\item{assault:}{}
\item{burglary:}{residential theft}
\item{larceny:}{unlawful taking of personal property (pocket picking)}
\item{autotheft:}{}
}
}
\usage{
crime_rate
}
\description{
Crime rates per 100,000 inhabitants in states of the USA for different crime
types in 1977.
}
\examples{
	
data(crime_rate)

}
\concept{dataset}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vtools.R
\name{vcheck}
\alias{vcheck}
\title{Check if variables exist in a data frame}
\usage{
vcheck(df, ...)
}
\arguments{
\item{df}{A data frame}

\item{...}{Variables to check}
}
\value{
TRUE if all variables exist, otherwise error
}
\description{
Check if variables exist in a data frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DATA_doby.R
\docType{data}
\name{dietox}
\alias{dietox}
\title{Growth curves of pigs in a 3x3 factorial experiment}
\format{
This data frame contains the following columns:

\describe{
\item{Weight}{Weight in Kg}
\item{Feed}{Cumulated feed intake in Kg}
\item{Time}{Time (in weeks) in the experiment}
\item{Pig}{Factor; id of each pig}
\item{Evit}{Factor; vitamin E dose; see 'details'.}
\item{Cu}{Factor, copper dose; see 'details'}
\item{Start}{Start weight in experiment, i.e. weight at week 1.}
\item{Litter}{Factor, id of litter of each pig}
}
}
\source{
Lauridsen, C., Højsgaard, S.,Sørensen, M.T. C. (1999) Influence of
Dietary Rapeseed Oli, Vitamin E, and Copper on Performance and
Antioxidant and Oxidative Status of Pigs. J. Anim. Sci.77:906-916
}
\usage{
dietox
}
\description{
The \code{dietox} data frame has 861 rows and 7 columns.
}
\details{
Data contains weight of slaughter pigs measured weekly for 12
weeks. Data also contains the start weight (i.e. the weight at week
1). The treatments are 3 different levels of Evit = vitamin E (dose: 0,
100, 200 mg dl-alpha-tocopheryl acetat /kg feed) in combination with 3
different levels of Cu=copper (dose: 0, 35, 175 mg/kg feed) in the feed.
The cumulated feed intake is also recorded. The pigs are litter mates.
}
\examples{

data(dietox)
head(dietox)
coplot(Weight ~ Time | Evit * Cu, data=dietox)

}
\concept{dataset}
\keyword{datasets}
